if exists (SELECT * FROM sysobjects WHERE name = 'RadSetLastUsed')
DROP PROCEDURE RadSetLastUsed
GO

if exists (SELECT * FROM sysobjects WHERE name = 'RadGetCacheUsers')
DROP PROCEDURE RadGetCacheUsers
GO

if exists (SELECT * FROM sysobjects WHERE name = 'RadGetUser')
DROP PROCEDURE RadGetUser
GO

if exists (SELECT * FROM sysobjects WHERE name = 'RadRoamCache')
DROP PROCEDURE RadRoamCache
GO

if exists (SELECT * FROM sysobjects WHERE name = 'RadGetRejects')
DROP PROCEDURE RadGetRejects
GO

if exists (SELECT * FROM sysobjects WHERE name = 'RadGetProxyAttributes')
DROP PROCEDURE RadGetProxyAttributes
GO

if exists (SELECT * FROM sysobjects WHERE name = 'RadAtCache')
DROP PROCEDURE RadAtCache
GO

if exists (SELECT * FROM sysobjects WHERE name = 'RadServerAccessCache')
DROP PROCEDURE RadServerAccessCache
GO

if exists (SELECT * FROM sysobjects WHERE name = 'RadDNISCache')
DROP PROCEDURE RadDNISCache
GO

if not exists (SELECT * FROM sysobjects WHERE name = 'RadProxyAttributeGroups')
EXEC('
CREATE TABLE RadProxyAttributeGroups
(
RadProxyAttributeGroupID int IDENTITY (1, 1) NOT NULL,
RadRoamServerID int NOT NULL,
Priority int NULL,
Description varchar (255) NULL,
CONSTRAINT PK_RadProxyAttributeGroups_ID PRIMARY KEY CLUSTERED (RadProxyAttributeGroupID)
)')
GO

if not exists (SELECT * FROM sysobjects WHERE name = 'RadProxyAttributes')
EXEC('
CREATE TABLE RadProxyAttributes
(
RadProxyAttributeID int IDENTITY (1, 1) NOT NULL,
RadProxyAttributeGroupID int NOT NULL,
RadAttributeID int NOT NULL,
SearchType tinyint NOT NULL,
String varchar (253) NOT NULL,
CONSTRAINT PK_RadProxyAttributes PRIMARY KEY NONCLUSTERED (RadProxyAttributeGroupID,RadAttributeID,SearchType,String),
CONSTRAINT FK_RadProxyAttributeGroups_ID FOREIGN KEY (RadProxyAttributeGroupID) REFERENCES RadProxyAttributeGroups (RadProxyAttributeGroupID)
)')
GO

if not exists (SELECT * FROM sysobjects so, syscolumns sc WHERE sc.id = so.id AND so.name = 'SubAccounts' AND sc.name = 'LastUsed')
ALTER TABLE SubAccounts Add LastUsed datetime NULL DEFAULT NULL
GO

if exists (SELECT * FROM sysobjects so, syscolumns sc WHERE sc.id = so.id AND so.name = 'Calls' AND sc.name = 'Username' AND sc.cdefault = 0)
ALTER TABLE Calls ADD CONSTRAINT DF_Calls_Username DEFAULT 'NULL' FOR Username
GO

CREATE PROCEDURE RadGetCacheUsers @date DATETIME, @flag TINYINT AS
IF (@flag = 1)
BEGIN
SELECT sa.AccountID, sa.Login, sa.Password, d.MailDomain, sa.AccountType, sa.LoginLimit, sa.TimeLeft, 
MasterExpire=CASE WHEN maExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,ma.Extension+ma.OverDue,maExpireDate)) END,
SubExpire=CASE WHEN saExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,sa.Extension+ma.OverDue,saExpireDate)) END,
OverLimit=CASE WHEN (ma.OverLimit > 0 AND ma.OverLimit < ma.Balance) THEN 1 ELSE 0 END
FROM SubAccounts sa, MasterAccounts ma, Domains d, Groups g
	WHERE sa.CustomerID = ma.CustomerID
	AND d.DomainID = g.DomainID
	AND ma.GroupID = g.GroupID
	AND sa.Active <> 0
	AND ma.Active <> 0
	AND sa.Login <> ''
	AND (sa.LastModifyDate > @date OR ma.LastModifyDate > @date)
UNION ALL
SELECT sa.AccountID, sa.Email, sa.Password, d.MailDomain, sa.AccountType, sa.LoginLimit, sa.TimeLeft,
MasterExpire=CASE WHEN maExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,ma.Extension+ma.OverDue,maExpireDate)) END,
SubExpire=CASE WHEN saExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,sa.Extension+ma.OverDue,saExpireDate)) END,
OverLimit=CASE WHEN (ma.OverLimit > 0 AND ma.OverLimit < ma.Balance) THEN 1 ELSE 0 END
FROM SubAccounts sa, MasterAccounts ma, Domains d, Groups g
	WHERE sa.CustomerID = ma.CustomerID
	AND d.DomainID = g.DomainID
	AND ma.GroupID = g.GroupID
	AND sa.Active <> 0
	AND ma.Active <> 0
	AND sa.Email <> ''
	AND (sa.LastModifyDate > @date OR ma.LastModifyDate > @date)

END ELSE IF (@flag = 2)
BEGIN
SELECT sa.AccountID, sa.Login, sa.Password, d.MailDomain, sa.AccountType, sa.LoginLimit, sa.TimeLeft, 
MasterExpire=CASE WHEN maExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,ma.Extension+ma.OverDue,maExpireDate)) END,
SubExpire=CASE WHEN saExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,sa.Extension+ma.OverDue,saExpireDate)) END,
OverLimit=CASE WHEN (ma.OverLimit > 0 AND ma.OverLimit < ma.Balance) THEN 1 ELSE 0 END
FROM SubAccounts sa, MasterAccounts ma, Domains d, Groups g
	WHERE sa.CustomerID = ma.CustomerID
	AND d.DomainID = g.DomainID
	AND ma.GroupID = g.GroupID
	AND sa.Active <> 0
	AND ma.Active <> 0
	AND sa.Login <> ''
	AND (sa.LastUsed > @date OR sa.LastUsed IS NULL)
UNION ALL
SELECT sa.AccountID, sa.Email, sa.Password, d.MailDomain, sa.AccountType, sa.LoginLimit, sa.TimeLeft,
MasterExpire=CASE WHEN maExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,ma.Extension+ma.OverDue,maExpireDate)) END,
SubExpire=CASE WHEN saExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,sa.Extension+ma.OverDue,saExpireDate)) END,
OverLimit=CASE WHEN (ma.OverLimit > 0 AND ma.OverLimit < ma.Balance) THEN 1 ELSE 0 END
FROM SubAccounts sa, MasterAccounts ma, Domains d, Groups g
	WHERE sa.CustomerID = ma.CustomerID
	AND d.DomainID = g.DomainID
	AND ma.GroupID = g.GroupID
	AND sa.Active <> 0 
	AND ma.Active <> 0
	AND sa.Email <> ''
	AND (sa.LastUsed > @date OR sa.LastUsed IS NULL)
END
GO

CREATE PROCEDURE RadGetUser @user VARCHAR(64) , @password VARCHAR(32) AS
SELECT sa.AccountID, sa.Login, sa.Password, d.MailDomain, sa.AccountType, sa.LoginLimit, sa.TimeLeft, 
MasterExpire=CASE WHEN maExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,ma.Extension+ma.OverDue,maExpireDate)) END,
SubExpire=CASE WHEN saExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,sa.Extension+ma.OverDue,saExpireDate)) END,
OverLimit=CASE WHEN (ma.OverLimit > 0 AND ma.OverLimit < ma.Balance) THEN 1 ELSE 0 END
FROM SubAccounts sa, MasterAccounts ma, Domains d, Groups g
	WHERE sa.CustomerID = ma.CustomerID
	AND d.DomainID = g.DomainID
	AND ma.GroupID = g.GroupID
	AND sa.Active <> 0 
	AND ma.Active <> 0
	AND sa.Login = @user
	AND (@password IS NULL OR sa.Password = @password)
UNION ALL
SELECT sa.AccountID, sa.Email, sa.Password, d.MailDomain, sa.AccountType, sa.LoginLimit, sa.TimeLeft, 
MasterExpire=CASE WHEN maExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,ma.Extension+ma.OverDue,maExpireDate)) END,
SubExpire=CASE WHEN saExpireDate IS NULL THEN NULL ELSE DATEDIFF(Day,'19700101',DATEADD(Day,sa.Extension+ma.OverDue,saExpireDate)) END,
OverLimit=CASE WHEN (ma.OverLimit > 0 AND ma.OverLimit < ma.Balance) THEN 1 ELSE 0 END
FROM SubAccounts sa, MasterAccounts ma, Domains d, Groups g
        WHERE sa.CustomerID = ma.CustomerID
        AND d.DomainID = g.DomainID
        AND ma.GroupID = g.GroupID
        AND sa.Active <> 0
        AND ma.Active <> 0
        AND sa.Email = @user
        AND (@password IS NULL OR sa.Password = @password)
GO

CREATE PROCEDURE RadRoamCache AS
Select Domain AS Label, Server, IPAddress, Secret, AuthPort, AcctPort,
Priority, Timeout, Retries, StripDomain, TreatAsLocal, AccountType
From RadRoamDomains rrd, RadRoamServers rrs
        Where rrd.RadRoamServerID = rrs.RadRoamServerID
UNION
Select rrd2.Domain AS Label, Server, IPAddress, Secret, AuthPort,
AcctPort, rrd.Priority, Timeout, Retries, StripDomain, TreatAsLocal,
rrd.AccountType
From RadRoamDomains rrd, RadRoamServers rrs, RadRoamDomains rrd2
        Where rrd.RadRoamServerID = rrs.RadRoamServerID
        AND rrd.Domain = 'DEFAULT'
UNION
Select CONVERT(VARCHAR(5),RadRoamServerID) AS Label, Server, IPAddress,
Secret, AuthPort, AcctPort, 0, Timeout, Retries, StripDomain,
TreatAsLocal, NULL
From RadRoamServers
Order By Label,Priority
GO

CREATE PROCEDURE RadGetRejects AS
Select ra.RadAttributeID, Name, Data, Value, Type, rr.RadVendorID,
rr.RadVendorType
From RadAttributes ra, RadRejects rr
Where ra.RadAttributeID=rr.RadAttributeID
        AND ra.RadVendorID = rr.RadVendorID
        AND ra.RadVendorType = rr.RadVendorType
ORDER BY Data,Value
GO

CREATE Procedure RadGetProxyAttributes AS
SELECT apg.RadProxyAttributeGroupID,apg.RadRoamServerID,ap.String,ap.SearchType,ap.RadAttributeID
FROM RadProxyAttributeGroups apg, RadProxyAttributes ap
	WHERE apg.RadProxyAttributeGroupID = ap.RadProxyAttributeGroupID
ORDER BY apg.Priority ASC, apg.RadProxyAttributeGroupID
GO

CREATE PROCEDURE RadSetLastUsed AS

CREATE TABLE #tmpusers
(
AccountID INT,
CallDate DATETIME
)

INSERT INTO #tmpusers (AccountID,CallDate)
SELECT AccountID,MAX(CallDate)
FROM Calls
	WHERE CallDate > DATEADD(dd,-1,getdate())
GROUP BY AccountID

UPDATE sa
SET sa.LastUsed = t.CallDate
FROM SubAccounts sa, #tmpusers t
	WHERE sa.AccountID = t.AccountID

DROP TABLE #tmpusers
GO

CREATE PROCEDURE RadAtCache @accounttype VARCHAR(16) AS
SELECT rc.AccountType, ra.RadAttributeID, Name, Data, Value, Type, rc.RadVendorID, rc.RadVendorType, rc.RadCheck
FROM RadATConfigs rc, RadAttributes ra
WHERE ra.RadAttributeID = rc.RadAttributeID
	AND (ra.RadVendorID = rc.RadVendorID OR rc.RadVendorID IS NULL)
	AND (ra.RadVendorType = rc.RadVendorType OR rc.RadVendorType IS NULL)
	AND (@accounttype IS NULL OR AccountType = @accounttype)
ORDER BY AccountType
GO

CREATE PROCEDURE RadServerAccessCache AS
Select MaxSessionLength, StartTime, StopTime, s.IPAddress, sa.Port, sa.AccountType
From Servers s, ServerAccess sa
	WHERE s.ServerID = sa.ServerID
GO

CREATE PROCEDURE RadDNISCache AS
Select at1.AccountType, dn.DNISNumber
FROM AccountTypes at1, DNISNumbers dn
	WHERE at1.DNISGroupID = dn.DNISGroupID
GO

GRANT SELECT,UPDATE,INSERT,DELETE ON RadProxyAttributeGroups TO EmeraldAdmin
GRANT SELECT,UPDATE,INSERT,DELETE ON RadProxyAttributes TO EmeraldAdmin
GRANT EXECUTE ON RadGetCacheUsers TO EmeraldApps
GRANT EXECUTE ON RadGetUser TO EmeraldApps
GRANT EXECUTE ON RadRoamCache TO EmeraldApps
GRANT EXECUTE ON RadGetRejects TO EmeraldApps
GRANT EXECUTE ON RadGetProxyAttributes TO EmeraldApps
GRANT EXECUTE ON RadSetLastUsed TO EmeraldApps
GRANT EXECUTE ON RadAtCache TO EmeraldApps
GRANT EXECUTE ON RadServerAccessCache TO EmeraldApps
GRANT EXECUTE ON RadDNISCache TO EmeraldApps
GO
