CREATE TABLE RadDestTypes (
       RadDestTypeID        int NOT NULL,
       RadDestType          varchar(64) NOT NULL
)
go

ALTER TABLE RadDestTypes
       ADD PRIMARY KEY (RadDestTypeID)
go

CREATE TABLE RadFilterGroups (
       RadFilterGroupID     int IDENTITY,
       RadFilterGroup       varchar(64) NOT NULL,
       RadSourceTypeID      int NOT NULL,
       Description          varchar(255) NULL,
       DestRadFilterGroupID int NULL,
       DestData             varchar(255) NULL,
       RadDestTypeID        int NOT NULL,
       SortOrder            smallint NOT NULL DEFAULT 1
)
go


ALTER TABLE RadFilterGroups
       ADD PRIMARY KEY (RadFilterGroupID)
go


CREATE TABLE RadFilters (
       RadFilterID          int IDENTITY,
       RadMergeTypeID       int NULL,
       RadFilterTypeID      int NULL,
       RadVendorID          int NULL,
       RadSearchTypeID      int NOT NULL,
       RadAttributeID       int NULL,
       RadFilterGroupID     int NOT NULL,
       Data                 varchar(255) NULL
)
go

ALTER TABLE RadFilters
       ADD PRIMARY KEY (RadFilterID)
go

CREATE TABLE RadFilterTypes (
       RadFilterTypeID      int NOT NULL,
       RadFilterType        varchar(64) NOT NULL
)
go

ALTER TABLE RadFilterTypes
       ADD PRIMARY KEY (RadFilterTypeID)
go

CREATE TABLE RadMergeTypes (
       RadMergeTypeID       int NOT NULL,
       RadMergeType         char(64) NOT NULL
)
go

ALTER TABLE RadMergeTypes
       ADD PRIMARY KEY (RadMergeTypeID)
go

CREATE TABLE RadSearchTypes (
       RadSearchTypeID      int NOT NULL,
       RadSearchType        varchar(64) NOT NULL
)
go

ALTER TABLE RadSearchTypes
       ADD PRIMARY KEY (RadSearchTypeID)
go

CREATE TABLE RadSourceTypes (
       RadSourceTypeID      int NOT NULL,
       RadSourceType        varchar(64) NOT NULL
)
go

ALTER TABLE RadSourceTypes
       ADD PRIMARY KEY (RadSourceTypeID)
go



ALTER TABLE RadFilterGroups
       ADD FOREIGN KEY (RadSourceTypeID)
                             REFERENCES RadSourceTypes
go


ALTER TABLE RadFilterGroups
       ADD FOREIGN KEY (RadDestTypeID)
                             REFERENCES RadDestTypes
go


ALTER TABLE RadFilters
       ADD FOREIGN KEY (RadMergeTypeID)
                             REFERENCES RadMergeTypes
go


ALTER TABLE RadFilters
       ADD FOREIGN KEY (RadFilterTypeID)
                             REFERENCES RadFilterTypes
go


ALTER TABLE RadFilters
       ADD FOREIGN KEY (RadSearchTypeID)
                             REFERENCES RadSearchTypes
go


ALTER TABLE RadFilters
       ADD FOREIGN KEY (RadFilterGroupID)
                             REFERENCES RadFilterGroups
go


IF object_id('RadGetFilterGroups') IS NOT NULL
	DROP PROCEDURE RadGetFilterGroups
GO
CREATE PROCEDURE RadGetFilterGroups AS
SELECT RadFilterGroupID, RadSourceTypeID, RadDestTypeID, DestRadFilterGroupID, DestData
FROM RadFilterGroups
ORDER BY SortOrder,RadFilterGroup DESC
GO

IF object_id('RadGetFilters') IS NOT NULL
	DROP PROCEDURE RadGetFilters
GO

CREATE PROCEDURE RadGetFilters @RadFilterGroupID INT AS
SELECT RadFilterTypeID, RadVendorID, RadAttributeID, RadSearchTypeID, RadMergeTypeID, Data
FROM RadFilters
	WHERE RadFilterGroupID = @RadFilterGroupID
ORDER BY RadFilterID DESC
GO


-- Populate RadDestTypes
INSERT INTO RadDestTypes (RadDestType,RadDestTypeID) VALUES('Filter Replace',1)
INSERT INTO RadDestTypes (RadDestType,RadDestTypeID) VALUES('Filter Merge',2)
INSERT INTO RadDestTypes (RadDestType,RadDestTypeID) VALUES('Nak',3)
INSERT INTO RadDestTypes (RadDestType,RadDestTypeID) VALUES('Ignore',4)
INSERT INTO RadDestTypes (RadDestType,RadDestTypeID) VALUES('Source Specific',5)
go

-- Populate RadFilterTypes
INSERT INTO RadFilterTypes (RadFilterTypeID,RadFilterType) VALUES(1,'Client IP')
INSERT INTO RadFilterTypes (RadFilterTypeID,RadFilterType) VALUES(2,'Host IP')
INSERT INTO RadFilterTypes (RadFilterTypeID,RadFilterType) VALUES(3,'Radius Attribute')
INSERT INTO RadFilterTypes (RadFilterTypeID,RadFilterType) VALUES(4,'Destination IP')
go

-- Populate RadMergeTypes
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Delete',1)
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Delete matching',2)
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Add',3)
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Replace value',4)
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Add or replace value',5)
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Add attributes from query',6)
go

-- Populate RadSearchTypes
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(1,'String')
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(2,'Substring')
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(3,'Equal')
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(4,'Less than')
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(5,'Greater than')
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(6,'Ends with')
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(7,'Starts with')
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(8,'Any value')
go

-- Populate RadSourceTypes
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Disable or Chain Dest',1)
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth In',10)
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth Out',11)
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth Proxy Out',12)
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth Proxy Req+Resp',13)
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth Req+Resp',14)
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Acct In',20)
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Acct Out',21)
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Acct Proxy Out',22)
go
