CREATE TABLE dbo.AccountTypes (
       AccountTypeID        int IDENTITY,
       AccountType          varchar(30) NOT NULL,
       Description          varchar(100) NOT NULL,
       DNISGroupID          int NULL,
       SortOrder            int DEFAULT 1  NOT NULL,
       BadAccountTypeID     int NULL,
       BadAck               int DEFAULT 10  NULL
) with identity_gap = 10
go


ALTER TABLE dbo.AccountTypes
       ADD PRIMARY KEY (AccountTypeID)
go


CREATE TABLE dbo.Calls (
       NASIdentifier        varchar(64) NOT NULL,
       NASPort              varchar(10) DEFAULT (-1)  NOT NULL,
       AcctSessionID        varchar(32) NOT NULL,
       AcctStatusType       smallint NOT NULL,
       CallDate             smalldatetime DEFAULT getdate()  NOT NULL,
       UserName             varchar(40) NULL,
       AcctDelayTime        int DEFAULT 0  NULL,
       AcctSessionTime      int DEFAULT 0  NULL,
       FramedAddress        varchar(64) NULL,
       AcctTerminateCause   smallint NULL,
       AcctInputOctets      numeric(18,0) NULL,
       AccountID            int NULL,
       AcctOutputOctets     numeric(18,0) NULL,
       NASPortType          tinyint NULL,
       ServerID             int NULL,
       NASPortDNIS          varchar(20) NULL,
       CallerID             varchar(20) NULL,
       ConnectInfo          varchar(32) NULL
)
go

CREATE INDEX iCallDate ON dbo.Calls
(
       CallDate
)
go

CREATE INDEX iUserName ON dbo.Calls
(
       UserName
)
go

CREATE INDEX iAccountID ON dbo.Calls
(
       AccountID
)
go

CREATE INDEX iServerID ON dbo.Calls
(
       ServerID
)
go


ALTER TABLE dbo.Calls
       ADD PRIMARY KEY (NASIdentifier, NASPort, AcctSessionID, 
              AcctStatusType)
go


CREATE TABLE dbo.DNISGroups (
       DNISGroupID          int IDENTITY,
       DNISGroup            varchar(25) NOT NULL,
       Description          varchar(45) NULL
) with identity_gap = 10
go


ALTER TABLE dbo.DNISGroups
       ADD PRIMARY KEY (DNISGroupID)
go


CREATE TABLE dbo.DNISNumbers (
       DNISGroupID          int NOT NULL,
       DNISNumber           varchar(20) NOT NULL
)
go


ALTER TABLE dbo.DNISNumbers
       ADD PRIMARY KEY (DNISGroupID, DNISNumber)
go


CREATE TABLE dbo.Licenses (
       LicenseID            varchar(50) NOT NULL,
       Company              varchar(80) NOT NULL
)
go


ALTER TABLE dbo.Licenses
       ADD PRIMARY KEY (LicenseID)
go


CREATE TABLE dbo.MapAttributes (
       MapAttribute         varchar(32) NOT NULL,
       MapType              int NOT NULL,
       RadAttributeID       int NOT NULL,
       RadVendorID          int NOT NULL
)
go


ALTER TABLE dbo.MapAttributes
       ADD PRIMARY KEY (MapAttribute, MapType)
go


CREATE TABLE dbo.MapValues (
       MapAttribute         varchar(32) NOT NULL,
       Value                varchar(32) NOT NULL,
       MapType              int NOT NULL,
       RadValue             int NOT NULL,
       Description          varchar(255) NULL
)
go


ALTER TABLE dbo.MapValues
       ADD PRIMARY KEY (MapAttribute, MapType, Value, RadValue)
go


CREATE TABLE dbo.MasterAccounts (
       CustomerID           int IDENTITY,
       FirstName            varchar(25) NULL,
       LastName             varchar(25) NULL,
       Active               smallint DEFAULT 1  NOT NULL,
       LastModifyUser       varchar(32) DEFAULT suser_name()  NULL,
       OverDue              smallint DEFAULT 0  NULL,
       Operator             varchar(32) DEFAULT suser_name()  NULL,
       StartDate            datetime DEFAULT getdate()  NULL,
       LastModifyDate       datetime DEFAULT getdate()  NULL,
       CreateDate           datetime DEFAULT getdate()  NULL,
       Comments             text NULL,
       CancelDate           datetime NULL
) with identity_gap = 10
go

CREATE INDEX i_MasterAccounts_LastName ON dbo.MasterAccounts
(
       LastName
)
go


ALTER TABLE dbo.MasterAccounts
       ADD PRIMARY KEY (CustomerID)
go


CREATE TABLE dbo.RadATConfigs (
       AccountTypeID        int NOT NULL,
       RadATConfigID        int IDENTITY,
       RadAttributeID       int NOT NULL,
       RadVendorID          int NOT NULL,
       Data                 varchar(255) NULL,
       Value                int NULL,
       RadCheck             smallint DEFAULT 0  NULL,
       Tag                  int NULL,
       LastModifyDate       datetime DEFAULT getdate()  NULL,
       LastModifyUser       varchar(32) DEFAULT suser_name()  NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadATConfigs
       ADD PRIMARY KEY (RadATConfigID)
go


CREATE TABLE dbo.RadAttributes (
       RadAttributeID       int NOT NULL,
       RadVendorID          int NOT NULL,
       Name                 varchar(40) NOT NULL,
       RadAttributeType     int NOT NULL,
       AliasAttributeID     int NULL,
       ReplyType            int DEFAULT 3  NULL,
       AliasVendorID        int NULL
)
go


ALTER TABLE dbo.RadAttributes
       ADD PRIMARY KEY (RadAttributeID, RadVendorID)
go


CREATE TABLE dbo.RadConfigs (
       RadConfigID          int IDENTITY,
       AccountID            int NOT NULL,
       RadAttributeID       int NOT NULL,
       RadVendorID          int NOT NULL,
       Data                 varchar(255) NULL,
       Value                int NULL,
       RadCheck             smallint DEFAULT 0  NULL,
       Tag                  int NULL,
       LastModifyDate       datetime DEFAULT getdate()  NULL,
       LastModifyUser       varchar(32) DEFAULT suser_name()  NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadConfigs
       ADD PRIMARY KEY (RadConfigID)
go


CREATE TABLE dbo.RadDestTypes (
       RadDestTypeID        int NOT NULL,
       RadDestType          varchar(64) NOT NULL
)
go


ALTER TABLE dbo.RadDestTypes
       ADD PRIMARY KEY (RadDestTypeID)
go


CREATE TABLE dbo.RadFilterGroups (
       RadFilterGroupID     int IDENTITY,
       RadFilterGroup       varchar(64) NOT NULL,
       RadSourceTypeID      int NOT NULL,
       Description          varchar(255) NULL,
       DestRadFilterGroupID int NULL,
       DestData             varchar(255) NULL,
       RadDestTypeID        int NOT NULL,
       SortOrder            smallint DEFAULT 1  NOT NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadFilterGroups
       ADD PRIMARY KEY (RadFilterGroupID)
go


CREATE TABLE dbo.RadFilters (
       RadFilterID          int IDENTITY,
       RadMergeTypeID       int NULL,
       RadFilterTypeID      int NULL,
       RadVendorID          int NULL,
       RadSearchTypeID      int NOT NULL,
       RadAttributeID       int NULL,
       RadFilterGroupID     int NOT NULL,
       Data                 varchar(255) NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadFilters
       ADD PRIMARY KEY (RadFilterID)
go


CREATE TABLE dbo.RadFilterTypes (
       RadFilterTypeID      int NOT NULL,
       RadFilterType        varchar(64) NOT NULL
)
go


ALTER TABLE dbo.RadFilterTypes
       ADD PRIMARY KEY (RadFilterTypeID)
go


CREATE TABLE dbo.RadIPAccountTypes (
       RadIPAccountTypeID   int NOT NULL,
       ServerGroupID        int NOT NULL,
       AccountTypeID        int NULL,
       RadIPGroupID         int NULL,
       Priority             int DEFAULT 0  NOT NULL
)
go


ALTER TABLE dbo.RadIPAccountTypes
       ADD PRIMARY KEY (RadIPAccountTypeID)
go


CREATE TABLE dbo.RadIPAddresses (
       IPAddress            varchar(16) NOT NULL,
       RadIPGroupID         int NOT NULL,
       NASIdentifier        varchar(16) NULL,
       NASPort              varchar(10) NULL,
       LastUsed             datetime DEFAULT getdate()  NULL,
       State                int DEFAULT 0  NOT NULL
)
go

CREATE INDEX i_RadIPAddresses_NAS ON dbo.RadIPAddresses
(
       NASIdentifier,
       NASPort
)
go


ALTER TABLE dbo.RadIPAddresses
       ADD PRIMARY KEY (IPAddress)
go


CREATE TABLE dbo.RadIPGroups (
       RadIPGroupID         int IDENTITY,
       RadIPGroup           varchar(32) NOT NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadIPGroups
       ADD PRIMARY KEY (RadIPGroupID)
go


CREATE TABLE dbo.RadLogMsgs (
       RadLogMsgID          int NOT NULL,
       Description          varchar(50) DEFAULT 'Undefined'  NOT NULL,
       Severity             int DEFAULT 0  NOT NULL
)
go


ALTER TABLE dbo.RadLogMsgs
       ADD PRIMARY KEY (RadLogMsgID)
go


CREATE TABLE dbo.RadLogs (
       RadLogMsgID          int NOT NULL,
       LogDate              smalldatetime DEFAULT getdate()  NOT NULL,
       UserName             varchar(40) NOT NULL,
       Data                 varchar(50) DEFAULT null  NULL,
       NASIdentifier        varchar(64) DEFAULT null  NULL,
       NASPort              varchar(10) NULL,
       CallerID             varchar(30) NULL
)
go


ALTER TABLE dbo.RadLogs
       ADD PRIMARY KEY (RadLogMsgID, LogDate, UserName)
go


CREATE TABLE dbo.RadMergeTypes (
       RadMergeTypeID       int NOT NULL,
       RadMergeType         char(64) NOT NULL
)
go


ALTER TABLE dbo.RadMergeTypes
       ADD PRIMARY KEY (RadMergeTypeID)
go


CREATE TABLE dbo.RadProxyAttributeGroups (
       RadProxyAttributeGroupID int IDENTITY,
       RadRoamServerID      int NULL,
       ProxyGroupName       varchar(32) NOT NULL,
       Priority             int DEFAULT 0  NOT NULL,
       Description          varchar(255) NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadProxyAttributeGroups
       ADD PRIMARY KEY (RadProxyAttributeGroupID)
go


CREATE TABLE dbo.RadProxyAttributes (
       RadProxyAttributeID  int IDENTITY,
       RadProxyAttributeGroupID int NULL,
       RadAttributeID       int NOT NULL,
       SearchType           smallint NOT NULL,
       String               varchar(253) NOT NULL,
       RadVendorID          int NOT NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadProxyAttributes
       ADD PRIMARY KEY NONCLUSTERED (RadProxyAttributeID)
go


CREATE TABLE dbo.RadRejects (
       RadRejectID          int IDENTITY,
       RadAttributeID       int NOT NULL,
       RadVendorID          int NOT NULL,
       Data                 varchar(255) NULL,
       Value                int NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadRejects
       ADD PRIMARY KEY (RadRejectID)
go


CREATE TABLE dbo.RadRoamDomains (
       RadRoamDomainID      int IDENTITY,
       RadRoamServerID      int NOT NULL,
       Domain               varchar(32) NOT NULL,
       Priority             int DEFAULT 0  NOT NULL,
       AccountTypeID        int NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadRoamDomains
       ADD PRIMARY KEY (RadRoamServerID, Domain)
go


CREATE TABLE dbo.RadRoamServers (
       RadRoamServerID      int IDENTITY,
       IPAddress            varchar(64) NULL,
       Server               varchar(32) NOT NULL,
       Secret               varchar(16) NOT NULL,
       Timeout              int NOT NULL,
       Retries              int NOT NULL,
       TreatAsLocal         smallint NOT NULL,
       StripDomain          smallint NOT NULL,
       AuthPort             int NULL,
       AcctPort             int NULL,
       RateTarget           int DEFAULT NULL  NULL,
       RateMax              int DEFAULT NULL  NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadRoamServers
       ADD PRIMARY KEY (RadRoamServerID)
go


CREATE TABLE dbo.RadSearchTypes (
       RadSearchTypeID      int NOT NULL,
       RadSearchType        varchar(64) NOT NULL
)
go


ALTER TABLE dbo.RadSearchTypes
       ADD PRIMARY KEY (RadSearchTypeID)
go


CREATE TABLE dbo.RadSourceTypes (
       RadSourceTypeID      int NOT NULL,
       RadSourceType        varchar(64) NOT NULL
)
go


ALTER TABLE dbo.RadSourceTypes
       ADD PRIMARY KEY (RadSourceTypeID)
go


CREATE TABLE dbo.RadTriggers (
       RadTriggerID         int IDENTITY,
       AccountID            int NOT NULL,
       TriggerType          int NULL,
       FileName             varchar(64) NOT NULL,
       Parameters           varchar(64) NULL,
       Directory            varchar(128) NULL
) with identity_gap = 10
go


ALTER TABLE dbo.RadTriggers
       ADD PRIMARY KEY (RadTriggerID)
go


CREATE TABLE dbo.RadValues (
       RadAttributeID       int NOT NULL,
       RadVendorID          int NOT NULL,
       Value                int NOT NULL,
       Name                 varchar(40) NOT NULL
)
go


ALTER TABLE dbo.RadValues
       ADD PRIMARY KEY (RadAttributeID, RadVendorID, Value)
go


CREATE TABLE dbo.RadVendors (
       RadVendorID          int NOT NULL,
       Name                 varchar(32) NOT NULL
)
go


ALTER TABLE dbo.RadVendors
       ADD PRIMARY KEY (RadVendorID)
go


CREATE TABLE dbo.ServerAccess (
       AccountTypeID        int NULL,
       ServerAccessID       int IDENTITY,
       ServerID             int NULL,
       StartTime            int DEFAULT 0  NULL,
       StopTime             int DEFAULT 0  NULL,
       MaxSessionLength     int DEFAULT 0  NULL,
       Port                 varchar(10) NULL
) with identity_gap = 10
go


ALTER TABLE dbo.ServerAccess
       ADD PRIMARY KEY (ServerAccessID)
go


CREATE TABLE dbo.ServerGroups (
       ServerGroupID        int IDENTITY,
       ServerGroup          varchar(32) NOT NULL
) with identity_gap = 10
go


ALTER TABLE dbo.ServerGroups
       ADD PRIMARY KEY (ServerGroupID)
go


CREATE TABLE dbo.ServerPorts (
       ServerID             int NOT NULL,
       Port                 varchar(10) NOT NULL,
       IPAddress            varchar(64) DEFAULT NULL  NULL,
       NASIdentifier        varchar(64) NOT NULL,
       MaxSessionTime       int DEFAULT 0  NULL,
       UserName             varchar(40) NULL,
       AcctSessionID        varchar(32) DEFAULT null  NULL,
       AcctStatusType       smallint DEFAULT null  NULL,
       CallDate             smalldatetime DEFAULT null  NULL,
       FramedAddress        varchar(64) DEFAULT null  NULL,
       AccountID            int NULL,
       SNMPUser             varchar(64) DEFAULT null  NULL,
       CallerID             varchar(20) DEFAULT null  NULL,
       ConnectInfo          varchar(32) DEFAULT null  NULL
)
go


ALTER TABLE dbo.ServerPorts
       ADD PRIMARY KEY (NASIdentifier, Port, ServerID)
go


CREATE TABLE dbo.Servers (
       ServerID             int IDENTITY,
       Server               varchar(25) NOT NULL,
       RadRoamServerID      int NULL,
       ServerType           int NULL,
       ServerGroupID        int NULL,
       IPAddress            varchar(64) NOT NULL,
       Comments             text NULL,
       Secret               varchar(16) DEFAULT null  NULL,
       Community            varchar(16) DEFAULT null  NULL
) with identity_gap = 10
go

CREATE UNIQUE INDEX u_Servers_IPAddress ON dbo.Servers
(
       IPAddress
)
go


ALTER TABLE dbo.Servers
       ADD PRIMARY KEY (ServerID)
go


CREATE TABLE dbo.ServerTypes (
       ServerType           int NOT NULL,
       Vendor               varchar(32) NOT NULL,
       Model                varchar(32) NULL,
       SNMPType             int NULL,
       SNMPUser             varchar(64) NULL
)
go


ALTER TABLE dbo.ServerTypes
       ADD PRIMARY KEY (ServerType)
go


CREATE TABLE dbo.SubAccounts (
       AccountID            int IDENTITY,
       CustomerID           int NOT NULL,
       Active               smallint DEFAULT 1  NOT NULL,
       Login                varchar(40) NULL,
       Email                varchar(64) NULL,
       FirstName            varchar(25) NULL,
       LastName             varchar(25) NULL,
       AccountTypeID        int NOT NULL,
       Password             varchar(32) NULL,
       ExpireDate           datetime DEFAULT null  NULL,
       Extension            int DEFAULT 0  NULL,
       LoginLimit           smallint DEFAULT 0  NULL,
       TimeLeft             int DEFAULT null  NULL,
       LastModifyUser       varchar(32) DEFAULT suser_name()  NULL,
       LastUsed             datetime DEFAULT getdate()  NULL,
       LastModifyDate       datetime DEFAULT getdate()  NULL,
       Operator             varchar(32) DEFAULT suser_name()  NULL
) with identity_gap = 10
go

CREATE INDEX i_SubAcconts_LastName ON dbo.SubAccounts
(
       LastName
)
go

CREATE INDEX i_SubAccounts_Login ON dbo.SubAccounts
(
       Login
)
go

CREATE INDEX i_SubAccounts_ExpireDate ON dbo.SubAccounts
(
       ExpireDate
)
go

CREATE INDEX i_SubAccounts_Email ON dbo.SubAccounts
(
       Email
)
go


ALTER TABLE dbo.SubAccounts
       ADD PRIMARY KEY (AccountID)
go


ALTER TABLE dbo.AccountTypes
       ADD FOREIGN KEY (AccountTypeID)
                             REFERENCES dbo.AccountTypes
go


ALTER TABLE dbo.AccountTypes
       ADD FOREIGN KEY (DNISGroupID)
                             REFERENCES dbo.DNISGroups
go


ALTER TABLE dbo.DNISNumbers
       ADD FOREIGN KEY (DNISGroupID)
                             REFERENCES dbo.DNISGroups
go


ALTER TABLE dbo.MapAttributes
       ADD FOREIGN KEY (RadAttributeID, RadVendorID)
                             REFERENCES dbo.RadAttributes
go


ALTER TABLE dbo.MapValues
       ADD FOREIGN KEY (MapAttribute, MapType)
                             REFERENCES dbo.MapAttributes
go


ALTER TABLE dbo.RadATConfigs
       ADD FOREIGN KEY (AccountTypeID)
                             REFERENCES dbo.AccountTypes
go


ALTER TABLE dbo.RadATConfigs
       ADD FOREIGN KEY (RadAttributeID, RadVendorID)
                             REFERENCES dbo.RadAttributes
go


ALTER TABLE dbo.RadAttributes
       ADD FOREIGN KEY (RadVendorID)
                             REFERENCES dbo.RadVendors
go


ALTER TABLE dbo.RadConfigs
       ADD FOREIGN KEY (RadAttributeID, RadVendorID)
                             REFERENCES dbo.RadAttributes
go


ALTER TABLE dbo.RadConfigs
       ADD FOREIGN KEY (AccountID)
                             REFERENCES dbo.SubAccounts
go


ALTER TABLE dbo.RadFilterGroups
       ADD FOREIGN KEY (RadSourceTypeID)
                             REFERENCES dbo.RadSourceTypes
go


ALTER TABLE dbo.RadFilterGroups
       ADD FOREIGN KEY (RadDestTypeID)
                             REFERENCES dbo.RadDestTypes
go


ALTER TABLE dbo.RadFilters
       ADD FOREIGN KEY (RadAttributeID, RadVendorID)
                             REFERENCES dbo.RadAttributes
go


ALTER TABLE dbo.RadFilters
       ADD FOREIGN KEY (RadMergeTypeID)
                             REFERENCES dbo.RadMergeTypes
go


ALTER TABLE dbo.RadFilters
       ADD FOREIGN KEY (RadFilterTypeID)
                             REFERENCES dbo.RadFilterTypes
go


ALTER TABLE dbo.RadFilters
       ADD FOREIGN KEY (RadSearchTypeID)
                             REFERENCES dbo.RadSearchTypes
go


ALTER TABLE dbo.RadFilters
       ADD FOREIGN KEY (RadFilterGroupID)
                             REFERENCES dbo.RadFilterGroups
go


ALTER TABLE dbo.RadIPAccountTypes
       ADD FOREIGN KEY (ServerGroupID)
                             REFERENCES dbo.ServerGroups
go


ALTER TABLE dbo.RadIPAccountTypes
       ADD FOREIGN KEY (AccountTypeID)
                             REFERENCES dbo.AccountTypes
go


ALTER TABLE dbo.RadIPAccountTypes
       ADD FOREIGN KEY (RadIPGroupID)
                             REFERENCES dbo.RadIPGroups
go


ALTER TABLE dbo.RadIPAddresses
       ADD FOREIGN KEY (RadIPGroupID)
                             REFERENCES dbo.RadIPGroups
go


ALTER TABLE dbo.RadLogs
       ADD FOREIGN KEY (RadLogMsgID)
                             REFERENCES dbo.RadLogMsgs
go


ALTER TABLE dbo.RadProxyAttributeGroups
       ADD FOREIGN KEY (RadRoamServerID)
                             REFERENCES dbo.RadRoamServers
go


ALTER TABLE dbo.RadProxyAttributes
       ADD FOREIGN KEY (RadProxyAttributeGroupID)
                             REFERENCES dbo.RadProxyAttributeGroups
go


ALTER TABLE dbo.RadProxyAttributes
       ADD FOREIGN KEY (RadAttributeID, RadVendorID)
                             REFERENCES dbo.RadAttributes
go


ALTER TABLE dbo.RadRejects
       ADD FOREIGN KEY (RadAttributeID, RadVendorID)
                             REFERENCES dbo.RadAttributes
go


ALTER TABLE dbo.RadRoamDomains
       ADD FOREIGN KEY (AccountTypeID)
                             REFERENCES dbo.AccountTypes
go


ALTER TABLE dbo.RadRoamDomains
       ADD FOREIGN KEY (RadRoamServerID)
                             REFERENCES dbo.RadRoamServers
go


ALTER TABLE dbo.RadTriggers
       ADD FOREIGN KEY (AccountID)
                             REFERENCES dbo.SubAccounts
go


ALTER TABLE dbo.RadValues
       ADD FOREIGN KEY (RadAttributeID, RadVendorID)
                             REFERENCES dbo.RadAttributes
go


ALTER TABLE dbo.ServerAccess
       ADD FOREIGN KEY (ServerID)
                             REFERENCES dbo.Servers
go


ALTER TABLE dbo.ServerAccess
       ADD FOREIGN KEY (AccountTypeID)
                             REFERENCES dbo.AccountTypes
go


ALTER TABLE dbo.ServerPorts
       ADD FOREIGN KEY (ServerID)
                             REFERENCES dbo.Servers
go


ALTER TABLE dbo.Servers
       ADD FOREIGN KEY (RadRoamServerID)
                             REFERENCES dbo.RadRoamServers
go


ALTER TABLE dbo.Servers
       ADD FOREIGN KEY (ServerType)
                             REFERENCES dbo.ServerTypes
go


ALTER TABLE dbo.Servers
       ADD FOREIGN KEY (ServerGroupID)
                             REFERENCES dbo.ServerGroups
go


ALTER TABLE dbo.SubAccounts
       ADD FOREIGN KEY (CustomerID)
                             REFERENCES dbo.MasterAccounts
go

CREATE PROCEDURE RadRoamCache AS
Select Domain AS Label, Server, IPAddress, Secret, AuthPort, AcctPort,
Priority, Timeout, Retries, StripDomain, TreatAsLocal, at1.AccountType, rrs.RateTarget, rrs.RateMax
From RadRoamDomains rrd JOIN RadRoamServers rrs ON (rrd.RadRoamServerID = rrs.RadRoamServerID)
	LEFT JOIN AccountTypes at1 ON (rrd.AccountTypeID = at1.AccountTypeID)
UNION
Select rrd2.Domain AS Label, Server, IPAddress, Secret, AuthPort,
AcctPort, rrd.Priority, Timeout, Retries, StripDomain, TreatAsLocal,
at1.AccountType, rrs.RateTarget, rrs.RateMax
From RadRoamDomains rrd, RadRoamServers rrs, RadRoamDomains rrd2, AccountTypes at1
        Where rrd.RadRoamServerID = rrs.RadRoamServerID
        AND rrd.Domain = 'DEFAULT'
	AND rrd.AccountTypeID = at1.AccountTypeID
UNION
Select CONVERT(VARCHAR(5),RadRoamServerID) AS Label, Server, IPAddress,
Secret, AuthPort, AcctPort, 0, Timeout, Retries, StripDomain,
TreatAsLocal, NULL, NULL, NULL
From RadRoamServers
Order By Label,Priority
GO

CREATE PROCEDURE RadGetRejects AS
Select ra.RadAttributeID AS AttributeID, ra.RadVendorID AS VendorID, Data, Value
From RadAttributes ra, RadRejects rr
Where ra.RadAttributeID = rr.RadAttributeID
        AND ra.RadVendorID = rr.RadVendorID
GO

CREATE Procedure RadGetProxyAttributes AS
SELECT apg.RadProxyAttributeGroupID,apg.RadRoamServerID,ap.String,ap.SearchType,ap.RadAttributeID, ap.RadVendorID
FROM RadProxyAttributeGroups apg, RadProxyAttributes ap
	WHERE apg.RadProxyAttributeGroupID = ap.RadProxyAttributeGroupID
ORDER BY apg.Priority ASC, apg.RadProxyAttributeGroupID
GO

CREATE PROCEDURE RadSetLastUsed AS
CREATE TABLE #tmpusers
(
AccountID INT,
CallDate DATETIME
)

INSERT INTO #tmpusers (AccountID,CallDate)
SELECT AccountID,MAX(CallDate) 
FROM Calls 
	WHERE CallDate > DATEADD(dd,-1,getdate())
GROUP BY AccountID

UPDATE sa
SET sa.LastUsed = t.CallDate
FROM SubAccounts sa, #tmpusers t
	WHERE sa.AccountID = t.AccountID

DROP TABLE #tmpusers
GO

CREATE PROCEDURE RadAtCache @accounttype VARCHAR(30) AS
SELECT at1.AccountType, ra.RadVendorID AS VendorID, ra.RadAttributeID AS AttributeID, Data, Value, rc.RadCheck, rc.Tag, at1.AccountTypeID
FROM RadATConfigs rc, RadAttributes ra, AccountTypes at1
	WHERE ra.RadAttributeID = rc.RadAttributeID
	AND rc.AccountTypeID = at1.AccountTypeID
	AND ra.RadVendorID = rc.RadVendorID
	AND (@accounttype IS NULL OR at1.AccountType = @accounttype)
ORDER BY AccountType, rc.RadATConfigID
GO

CREATE PROCEDURE RadServerAccessCache AS
Select MaxSessionLength, StartTime, StopTime, s.IPAddress, sa.Port, at1.AccountType
From Servers s, ServerAccess sa, AccountTypes at1
	WHERE s.ServerID = sa.ServerID
	AND sa.AccountTypeID = at1.AccountTypeID
GO

CREATE PROCEDURE RadDNISCache AS
Select at1.AccountType, dn.DNISNumber
FROM AccountTypes at1, DNISNumbers dn
	WHERE at1.DNISGroupID = dn.DNISGroupID
GO

CREATE PROCEDURE RadCheckTrigger @AccountID int AS
Select FileName, Parameters, Directory, TriggerType from RadTriggers Where AccountID = @AccountID
GO

CREATE PROCEDURE RadGetConfigs @AccountID int AS
Select Ra.RadVendorID AS VendorID, ra.RadAttributeID AS AttributeID, Data, Value, rc.RadCheck, rc.Tag
From RadConfigs rc, RadAttributes ra
Where ra.RadAttributeID = rc.RadAttributeID
	AND ra.RadVendorID = rc.RadVendorID
	AND rc.AccountID=@AccountID
ORDER BY rc.RadConfigID
GO

CREATE PROCEDURE RadGetCacheUsers @date DATETIME, @flag SMALLINT AS
IF (@flag = 1)
BEGIN
SELECT sa.AccountID, sa.Login, sa.Password, NULL, at1.AccountType, sa.LoginLimit, sa.TimeLeft * 60,
MasterExpire=CASE WHEN CancelDate IS NULL THEN NULL ELSE CancelDate END,
SubExpire=CASE WHEN ExpireDate IS NULL THEN NULL ELSE DATEADD(Day,COALESCE(sa.Extension,0) + COALESCE(ma.OverDue,0),ExpireDate) END,
0 AS OverLimit, CASE WHEN sa.Active = 0 OR ma.Active = 0 THEN NULL WHEN ma.StartDate IS NULL THEN getdate() ELSE ma.StartDate END AS StartDate,
(SELECT AccountType FROM AccountTypes at2 WHERE at1.BadAccountTypeID = at2.AccountTypeID) AS BadAccountType, at1.BadAck
FROM SubAccounts sa, MasterAccounts ma, AccountTypes at1
	WHERE sa.CustomerID = ma.CustomerID
	AND sa.AccountTypeID = at1.AccountTypeID
	AND sa.Login <> ''
	AND (sa.LastModifyDate > @date OR ma.LastModifyDate > @date)
END ELSE IF (@flag = 2)
BEGIN
SELECT sa.AccountID, sa.Login, sa.Password, NULL, at1.AccountType, sa.LoginLimit, sa.TimeLeft * 60,
CASE WHEN CancelDate IS NULL THEN NULL ELSE CancelDate END AS MasterExpire,
CASE WHEN ExpireDate IS NULL THEN NULL ELSE DATEADD(Day,COALESCE(sa.Extension,0) + COALESCE(ma.OverDue,0),ExpireDate) END AS SubExpire,
0 AS OverLimit, CASE WHEN sa.Active = 0 OR ma.Active = 0 THEN NULL WHEN ma.StartDate IS NULL THEN getdate() ELSE ma.StartDate END AS StartDate,
(SELECT AccountType FROM AccountTypes at2 WHERE at1.BadAccountTypeID = at2.AccountTypeID) AS BadAccountType, at1.BadAck
FROM SubAccounts sa, MasterAccounts ma, AccountTypes at1
	WHERE sa.CustomerID = ma.CustomerID
	AND sa.AccountTypeID = at1.AccountTypeID
	AND sa.Login <> ''
END
GO

CREATE PROCEDURE RadGetUser @user VARCHAR(64) AS
SELECT sa.AccountID, sa.Login, sa.Password, NULL AS Domain, at1.AccountType, sa.LoginLimit, sa.TimeLeft * 60 AS TimeLeft, 
CASE WHEN CancelDate IS NULL THEN NULL ELSE CancelDate END AS MasterExpire,
CASE WHEN ExpireDate IS NULL THEN NULL ELSE DATEADD(Day,COALESCE(sa.Extension,0) + COALESCE(ma.OverDue,0),ExpireDate) END AS SubExpire,
0 AS OverLimit, CASE WHEN sa.Active = 0 OR ma.Active = 0 THEN NULL WHEN ma.StartDate IS NULL THEN getdate() ELSE ma.StartDate END AS StartDate,
(SELECT AccountType FROM AccountTypes at2 WHERE at1.BadAccountTypeID = at2.AccountTypeID) AS BadAccountType, at1.BadAck,
CASE WHEN ExpireDate IS NULL THEN NULL ELSE DATEADD(Day,COALESCE(sa.Extension,0) + COALESCE(ma.OverDue,0),ExpireDate) END AS ExpireDate
FROM SubAccounts sa, MasterAccounts ma, AccountTypes at1
	WHERE sa.CustomerID = ma.CustomerID
	AND sa.AccountTypeID = at1.AccountTypeID
	AND sa.Login = @user
GO

CREATE PROCEDURE RadCheckOnline @UserName varchar(64), @accountid INT AS
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT COUNT(*) FROM ServerPorts WHERE (((AccountID IS NULL OR @accountid IS NULL) AND UserName = @UserName) OR (AccountID IS NOT NULL AND AccountID = @accountid)) AND AcctStatusType IN (1,3)
SET TRANSACTION ISOLATION LEVEL READ COMMITTED
GO

CREATE PROCEDURE RadCheckOnlineSNMP @UserName varchar(64), @accountid INT AS 
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED
SELECT s.IPAddress, st.SNMPType, s.Community, sp.SNMPUser, sp.AcctSessionID
FROM Servers s, ServerPorts sp, ServerTypes st
	WHERE (((sp.AccountID IS NULL OR @accountid IS NULL) AND sp.UserName = @UserName) 
		OR (sp.AccountID IS NOT NULL AND sp.AccountID = @accountid)) 
	AND sp.AcctStatusType IN (1,3)
	AND s.ServerType = st.ServerType
	AND s.ServerID = sp.ServerID
	AND sp.SNMPUser IS NOT NULL
	AND s.Community IS NOT NULL
SET TRANSACTION ISOLATION LEVEL READ COMMITTED
GO

CREATE PROCEDURE RadOnlineGetSNMP AS
SELECT s.IPAddress, st.SNMPType AS ServerType, s.Community, sp.SNMPUser, sp.AcctSessionID, sp.Username, sp.ServerID,sp.Port
FROM Servers s, ServerPorts sp, ServerTypes st
	WHERE s.ServerID = sp.ServerID
	AND s.ServerType = st.ServerType
	AND AcctStatusType=1
	AND DATALENGTH(sp.SNMPUser) > 5
	AND DATALENGTH(s.Community) > 1
	AND Username IS NOT NULL
ORDER BY sp.Port
GO

CREATE PROCEDURE RadGetFilterGroups AS
SELECT RadFilterGroupID, RadSourceTypeID, RadDestTypeID, DestRadFilterGroupID, DestData
FROM RadFilterGroups
ORDER BY SortOrder,RadFilterGroup DESC
GO

CREATE PROCEDURE RadGetFilters @RadFilterGroupID INT AS
SELECT RadFilterTypeID, RadVendorID, RadAttributeID, RadSearchTypeID, RadMergeTypeID, Data
FROM RadFilters
	WHERE RadFilterGroupID = @RadFilterGroupID
ORDER BY RadFilterID DESC
GO

CREATE PROCEDURE RadGetPoolConfigs (@AccountTypeID int, @ServerID int, @NASIdentifier varchar(16), @NASPort VARCHAR(10), @accountid INT, @callerid VARCHAR(20), @dnis VARCHAR(20)) AS
SET ROWCOUNT 1
SET NOCOUNT ON
DECLARE @ip VARCHAR(16)
UPDATE RadIPAddresses SET State = 0
	WHERE NASIdentifier = @NASIdentifier
	AND NASPort = @NASPort
	AND (State = 2
		OR (State = 1 AND LastUsed < DATEADD(mi,-2,getdate())))

-- Specific Account Type, Specific IP Group
SELECT @ip=ria.IPAddress
FROM ServerGroups sg, RadIPGroups rig, RadIPAddresses ria, RadIPAccountTypes riat, Servers s
	WHERE s.ServerGroupID = sg.ServerGroupID
	AND riat.ServerGroupID = sg.ServerGroupID
	AND riat.RadIPGroupID = rig.RadIPGroupID
	AND ria.RadIPGroupID = rig.RadIPGroupID
	AND riat.AccountTypeID = @AccountTypeID
	AND s.ServerID = @ServerID
	AND ria.State = 0
ORDER BY riat.Priority, ria.LastUsed ASC

-- All Account Types, Specific IP Group
IF (@ip IS NULL)
BEGIN
SELECT @ip=ria.IPAddress
FROM ServerGroups sg, RadIPGroups rig, RadIPAddresses ria, RadIPAccountTypes riat, Servers s
	WHERE s.ServerGroupID = sg.ServerGroupID
	AND riat.ServerGroupID = sg.ServerGroupID
	AND riat.RadIPGroupID = rig.RadIPGroupID
	AND ria.RadIPGroupID = rig.RadIPGroupID
	AND riat.AccountTypeID IS NULL
	AND s.ServerID = @ServerID
	AND State = 0
ORDER BY riat.Priority, ria.LastUsed ASC
END

-- Specific Account Type, All IP Groups
IF (@ip IS NULL)
BEGIN
SELECT @ip=ria.IPAddress
FROM ServerGroups sg, RadIPAddresses ria, RadIPAccountTypes riat, Servers s
	WHERE s.ServerGroupID = sg.ServerGroupID
	AND riat.ServerGroupID = sg.ServerGroupID
	AND riat.RadIPGroupID IS NULL
	AND riat.AccountTypeID = @AccountTypeID
	AND s.ServerID = @ServerID
	AND ria.State = 0
ORDER BY riat.Priority, ria.LastUsed ASC
END

-- All Account Types, All IP Groups
IF (@ip IS NULL)
BEGIN
SELECT @ip=ria.IPAddress
FROM ServerGroups sg, RadIPAddresses ria, RadIPAccountTypes riat, Servers s
	WHERE s.ServerGroupID = sg.ServerGroupID
	AND riat.ServerGroupID = sg.ServerGroupID
	AND riat.RadIPGroupID IS NULL
	AND riat.AccountTypeID IS NULL
	AND s.ServerID = @ServerID
	AND ria.State = 0
ORDER BY riat.Priority, ria.LastUsed ASC
END

IF @ip IS NOT NULL
	BEGIN
	Update RadIPAddresses
		Set LastUsed = getdate(), State = 1, NASIdentifier = @NASIdentifier, NASPort = @NASPort
		Where IPAddress = @ip
	Select 8 as AttributeID,0 as VendorID, 0 as Value, @ip as Data
	END
GO

CREATE PROCEDURE MapGetAttributes AS
SELECT MapAttribute,RadAttributeID,RadVendorID,MapType
FROM MapAttributes
go

CREATE PROCEDURE MapGetValues AS
SELECT MapAttribute,Value,RadValue,MapType
FROM MapValues
ORDER BY MapAttribute,MapType
go


CREATE TRIGGER MA_Update ON MasterAccounts
FOR UPDATE
AS
UPDATE MasterAccounts SET LastModifyDate = getdate()
FROM MasterAccounts ma, inserted i
	WHERE ma.CustomerID = i.CustomerID
GO

CREATE TRIGGER SA_Update ON SubAccounts
FOR UPDATE
AS
UPDATE SubAccounts SET LastModifyDate = GetDate()
FROM SubAccounts sa, inserted i
	WHERE sa.AccountID = i.AccountID
GO

CREATE TRIGGER SA_Update2 ON RadConfigs
FOR INSERT, UPDATE
AS
UPDATE SubAccounts SET LastModifyDate = GetDate()
FROM SubAccounts sa, inserted i
	WHERE sa.AccountID = i.AccountID
GO


CREATE TRIGGER calls_insert ON Calls
FOR INSERT
AS
DECLARE @rows INT
UPDATE RadIPAddresses SET State = CASE WHEN i.AcctStatusType IN (1,3) THEN 2 WHEN i.AcctStatusType = 2 THEN 0 ELSE State END
FROM RadIPAddresses ri, inserted i
	WHERE ri.NASPort = i.NASPort
	AND ri.NASIdentifier = i.NASIdentifier

SELECT @rows = 0
SELECT @rows = @rows + 1 
FROM inserted
GROUP BY NASIdentifier, NASPort, ServerID
UPDATE ServerPorts
        Set Username = i.Username, 
		AccountID = i.AccountID,
                AcctStatusType = CASE WHEN (i.CallDate = sp.CallDate AND i.AcctSessionID = sp.AcctSessionID AND i.AcctStatusType = 1 AND sp.AcctStatusType = 2) THEN 2 WHEN i.AcctStatusType in (1,3) THEN 1 ELSE i.AcctStatusType END,
		CallDate = i.CallDate,
		FramedAddress = i.FramedAddress,
		CallerID = i.CallerID,
		ConnectInfo = i.ConnectInfo,
		AcctSessionID = i.AcctSessionID
        FROM ServerPorts sp, inserted i
        WHERE sp.NASIdentifier = i.NASIdentifier
	AND sp.Port = i.NASPort
	AND (i.CallDate >= sp.CallDate OR sp.CallDate IS NULL)
IF (@@rowcount <> @rows)
	BEGIN
	/* SNMPUser should add the port for all except 10 (ascend) */
	INSERT INTO ServerPorts (ServerID,Port,NASIdentifier,IPAddress,MaxSessionTime,Username,AccountID,AcctSessionID,
	AcctStatusType,CallDate,FramedAddress,CallerID,ConnectInfo,SNMPUser)
	SELECT i.ServerID,i.NASPort,i.NASIdentifier,NULL,0,MIN(i.Username),MIN(i.AccountID),MIN(i.AcctSessionID),CASE WHEN MAX(i.AcctStatusType) in (1,3) THEN 1 ELSE MAX(i.AcctStatusType) END,
	MAX(i.CallDate),MIN(i.FramedAddress),MIN(i.CallerID),MIN(i.ConnectInfo),
	Case when MIN(st.SNMPType)=1 then MIN(st.SNMPUser) else MIN(st.SNMPUser) + convert(varchar(9), i.NASPort) END
	FROM inserted i, Servers s, ServerTypes st
		WHERE s.ServerType = st.ServerType
		AND s.ServerID = COALESCE(i.ServerID,(SELECT MAX(s.ServerID) FROM Servers s, inserted ii WHERE s.IPAddress = ii.NASIdentifier))
		AND NOT EXISTS (SELECT * FROM ServerPorts sp, inserted ii WHERE sp.NASIdentifier = ii.NASIdentifier AND sp.Port = ii.NASPort)
	GROUP BY i.NASIdentifier, i.NASPort, i.ServerID
	END
UPDATE SubAccounts
Set TimeLeft = sa.TimeLeft - (i.AcctSessionTime/60 + 1)
FROM SubAccounts sa, inserted i
WHERE ((i.AccountID IS NULL AND sa.Login = i.Username) OR (i.AccountID IS NOT NULL AND sa.AccountID = i.AccountID))
	and sa.TimeLeft IS NOT NULL
	and i.AcctStatusType = 2
GO
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 8741)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(8741,'3com')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8741 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,8741,'SS3-Firewall-User-Privilege',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8741 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8741,1,'Remote-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8741 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8741,2,'Bypass-Filters')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8741 AND RadAttributeID = 1 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8741,3,'VPN-Client-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8741 AND RadAttributeID = 1 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8741,4,'Access-To-VPN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8741 AND RadAttributeID = 1 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8741,5,'Limited-Management')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8741 AND RadAttributeID = 1 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8741,6,'L2TP-Client-Access')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 10415)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(10415,'3GPP')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,10415,'3GPP-IMSI',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,10415,'3GPP-Charging-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,10415,'3GPP-PDP-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,10415,'3GPP-Charging-Gateway-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,10415,'3GPP-GPRS-Negotiated-QoS-profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,10415,'3GPP-SGSN-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,10415,'3GPP-GGSN-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,10415,'3GPP-IMSI-MCC-MNC',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,10415,'3GPP-GGSN-MCC-MNC',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,10415,'3GPP-NSAPI',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,10415,'3GPP-Session-Stop-Indicator',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,10415,'3GPP-Selection-Mode',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,10415,'3GPP-Charging-Characteristics',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,10415,'3GPP-Charging-Gateway-IPv6-Address',5,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,10415,'3GPP-SGSN-IPv6-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,10415,'3GPP-GGSN-IPv6-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,10415,'3GPP-IPv6-DNS-Servers',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,10415,'3GPP-SGSN-MCC-MNC',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,10415,'3GPP-Teardown-Indicator',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,10415,'3GGP-IMEISV',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,10415,'3GPP-RAT-Type',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,10415,'3GPP-User-Location-Info',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,10415,'3GPP-MS-TimeZone',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,10415,'3GPP-Camel-Charging-Info',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,10415,'3GPP-Packet-Filter',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,10415,'3GPP-Negotiated-DSCP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10415 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,10415,'3GPP-Allocate-IP-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 3 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,10415,0,'IPv4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 3 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,10415,1,'PPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 3 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,10415,2,'IPv6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 3 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,10415,3,'IPv4v6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,0,'Reserved')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,1,'UTRAN')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,2,'GERAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,3,'WLAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,4,'GAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,5,'HSPA-Evolution')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,101,'IEEE-802-16e')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,102,'3GPP2-eHRPD')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 103)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,103,'3GPP2-HRPD')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 104)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,104,'3GPP2-1xRTT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 21 AND Value = 105)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,10415,105,'3GPP-EPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 27 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(27,10415,0,'Do-Not-Allocate-IPv4-IPv6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 27 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(27,10415,1,'Allocate-IPv4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 27 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(27,10415,2,'Allocate-IPv6-Prefix')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 10415 AND RadAttributeID = 27 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(27,10415,3,'Allocate-IPv4-IPv6-Prefix')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 5535)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(5535,'3GPP2')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,5535,'3GPP2-Ike-Preshared-Secret-Request',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,5535,'3GPP2-Security-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,5535,'3GPP2-Pre-Shared-Secret',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,5535,'3GPP2-Reverse-Tunnel-Spec',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,5535,'3GPP2-Diffserv-Class-Option',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,5535,'3GPP2-Accounting-Container',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,5535,'3GPP2-Home-Agent-IP-Address',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,5535,'3GPP2-KeyID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,5535,'3GPP2-PCF-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,5535,'3GPP2-BSID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,5535,'3GPP2-User-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,5535,'3GPP2-Forward-FCH-Mux-Option',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,5535,'3GPP2-Reverse-FCH-Mux-Option',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,5535,'3GPP2-Service-Option',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,5535,'3GPP2-Forward-Traffic-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,5535,'3GPP2-Reverse-Traffic-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,5535,'3GPP2-FCH-Frame-Size',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,5535,'3GPP2-Forward-FCH-RC',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,5535,'3GPP2-Reverse-FCH-RC',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,5535,'3GPP2-IP-Technology',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,5535,'3GPP2-Compulsory-Tunnel-Indicator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,5535,'3GPP2-Release-Indicator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,5535,'3GPP2-Bad-PPP-Frame-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,5535,'3GPP2-Number-Active-Transitions',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,5535,'3GPP2-Terminating-SDB-Octet-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,5535,'3GPP2-Originating-SDB-OCtet-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,5535,'3GPP2-Terminating-Number-SDBs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,5535,'3GPP2-Originating-Number-SDBs',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,5535,'3GPP2-IP-QoS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,5535,'3GPP2-Airlink-Priority',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,5535,'3GPP2-Airlink-Record-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,5535,'3GPP2-Airlink-Sequence-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,5535,'3GPP2-Received-HDLC-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,5535,'3GPP2-Correlation-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,5535,'3GPP2-Module-Orig-Term-Indicator',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,5535,'3GPP2-Inbound-Mobile-IP-Sig-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,5535,'3GPP2-Outbound-Mobile-IP-Sig-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,5535,'3GPP2-Session-Continue',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,5535,'3GPP2-Active-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,5535,'3GPP2-DCCH-Frame-Size',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,5535,'3GPP2-Begin-Session',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,5535,'3GPP2-ESN',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,5535,'3GPP2-S-Key',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,5535,'3GPP2-S-Request',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,5535,'3GPP2-S-Lifetime',3,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,5535,'3GPP2-MN-HA-SPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,5535,'3GPP2-MN-HA-Shared-Key',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,5535,'3GPP2-Remote-IP-Address',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,5535,'3GPP2-Remote-IPv6-Address',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,5535,'3GPP2-Remote-Address-Table-Index',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,5535,'3GPP2-Remote-IPv4-Addr-Octet-Count',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,5535,'3GPP2-Allowed-Diffserv-Marking',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,5535,'3GPP2-Service-Option-Profile',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,5535,'3GPP2-DNS-Update-Required',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,5535,'3GPP2-Foreign-Agent-Address',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,5535,'3GPP2-Last-User-Activity-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,5535,'3GPP2-MN-AAA-Removal-Indication',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,5535,'3GPP2-RN-Packet-Data-Inactivity-Timer',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,5535,'3GPP2-Forward-PDCH-RC',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 84)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(84,5535,'3GPP2-Forward-DCCH-Mux-Option',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 85)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(85,5535,'3GPP2-Reverse-DCCH-Mux-Option',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 86)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(86,5535,'3GPP2-Forward-DCCH-RC',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 87)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(87,5535,'3GPP2-Reverse-DHHC-RC',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 88)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(88,5535,'3GPP2-Session-Termination-Capability',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 89)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(89,5535,'3GPP2-Allowed-Persistent-TFTs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,5535,'3GPP2-Prepaid-Acct-Quota',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,5535,'3GPP2-Prepaid-acct-Capability',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,5535,'3GPP2-MIP-Lifetime',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,5535,'3GPP2-Acct-Stop-Trigger',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,5535,'3GPP2-Service-Reference-Id',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,5535,'3GPP2-DNS-Update-Capability',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,5535,'3GPP2-Disconnect-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,5535,'3GPP2-Remote-IPv6-Octet-Count',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,5535,'3GPP2-PrePaid-Tariff-Switching',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,5535,'3GPP2-Subnet',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,5535,'3GPP2-MEID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,5535,'3GPP2-DNS-Server-IP-Address',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,5535,'3GPP2-MIP6-Home-Agent-Address-From-BU',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(119,5535,'3GPP2-MIP6-Care-of-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 120)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(120,5535,'3GPP2-Home-Agent-Not-Authorized',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,5535,'3GPP2-MIP6-Session-Key',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,5535,'3GPP2-Hot-Line-Accounting-Information',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,5535,'3GPP2-MIP6-Mesg-ID',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,5535,'3GPP2-Filter-Rule',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,5535,'3GPP2-HTTP-Redirection-Rule',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,5535,'3GPP2-IP-Redirection-Rule',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,5535,'3GPP2-Hot-Line-Capability',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,5535,'3GPP2-MIP6-Home-Link-Prefix-Attr-A',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,5535,'3GPP2-Max-Authorized-Aggr-Bandwidth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,5535,'3GPP2-Authorized-Flow-Profile-IDs',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,5535,'3GPP2-Granted-QoS-Parameters',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,5535,'3GPP2-Maximum-Per-Flow-Priority',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 134)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(134,5535,'3GPP2-MIP6-Authenticator',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 138)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(138,5535,'3GPP2-MIP6-MAC-Mobility-Data',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 139)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(139,5535,'3GPP2-Inter-User-Priority',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,5535,'3GPP2-MIP6-Home-Agent-Address-Attr-B',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,5535,'3GPP2-MIP6-HOA-Received-From-BU',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,5535,'3GPP2-Carrier-ID',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,5535,'3GPP2-GMT-Time-Zone-Offset',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 144)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(144,5535,'3GPP2-Flow-ID-Parameter',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 145)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(145,5535,'3GPP2-Flow-Status',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 146)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(146,5535,'3GPP2-Filtered-Octet-Count-Terminating',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 147)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(147,5535,'3GPP2-Filtered-Octet-Count-Originating',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 162)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(162,5535,'3GPP2-RSVP-Inbound-Octet-Count',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 163)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(163,5535,'3GPP2-RSVP-Outbound-Octet-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 164)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(164,5535,'3GPP2-RSVP-Inbound-Packet-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 165)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(165,5535,'3GPP2-RSVP-Outbound-Packet-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 168)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(168,5535,'3GPP2-HA-Request',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 169)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(169,5535,'3GPP2-HA-Authorised',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 172)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(172,5535,'3GPP2-IP-Ver-Authorised',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 173)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(173,5535,'3GPP2-MIPv4-Mesg-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 179)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(179,5535,'3GPP2-MIP6-HA-Local-Assignment-Capblty',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 198)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(198,5535,'3GPP2-Accounting-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 203)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(203,5535,'3GPP2-HAAA-MIP6-HA-Protocol-Capblty-Ind',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 205)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(205,5535,'3GPP2-VAAA-Assigned-MIP6-HA',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 206)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(206,5535,'3GPP2-VAAA-Assigned-MIP6-HL',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5535 AND RadAttributeID = 207)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(207,5535,'3GPP2-VAAA-MIP6-HA-Protocol-Capblty-Ind',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5535 AND RadAttributeID = 40 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,5535,1,'Connection-Setup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5535 AND RadAttributeID = 40 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,5535,2,'Active-Start')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5535 AND RadAttributeID = 40 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,5535,3,'Active-Stop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5535 AND RadAttributeID = 40 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,5535,4,'Short-Data-Burst')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 0)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(0,'Standard')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,0,'User-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,0,'Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,0,'Challenge-Response',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,0,'NAS-Identifier',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,0,'NAS-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,0,'User-Service',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,0,'Framed-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,0,'Framed-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,0,'Framed-Netmask',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,0,'Framed-Routing',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,0,'Framed-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,0,'Framed-MTU',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,0,'Framed-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,0,'Login-Host',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,0,'Login-Service',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,0,'Login-TCP-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,0,'Change-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,0,'Reply-Message',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,0,'Callback-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,0,'Callback-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,0,'Expiration',3,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,0,'Framed-Route',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,0,'Framed-IPX-Network',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,0,'State',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,0,'Class',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,0,'Vendor-Specific',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,0,'Session-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,0,'Idle-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,0,'Termination-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,0,'NAS-Port-DNIS',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,0,'Caller-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,0,'NAS-Identifier',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,0,'Proxy-State',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,0,'Login-LAT-Service',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,0,'Login-LAT-Node',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,0,'Login-LAT-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,0,'Framed-AppleTalk-Link',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,0,'Framed-AppleTalk-Network',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,0,'Framed-AppleTalk-Zone',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,0,'Acct-Status-Type',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,0,'Acct-Delay-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,0,'Acct-Input-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,0,'Acct-Output-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,0,'Acct-Session-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,0,'Acct-Authentic',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,0,'Acct-Session-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,0,'Acct-Input-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,0,'Acct-Output-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,0,'Acct-Terminate-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,0,'Acct-Multi-Session-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,0,'Acct-Link-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,0,'Acct-Input-Gigawords',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,0,'Acct-Output-Gigawords',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,0,'Event-Time',3,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,0,'Egress-VLANID',11,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,0,'Ingress-Filters',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,0,'Egress-VLAN-Name',10,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,0,'User-Priority-Table',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,0,'CHAP-Challenge',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,0,'NAS-Port-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,0,'Port-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,0,'Login-LAT-Port',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,0,'Tunnel-Type',11,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,0,'Tunnel-Medium-Type',11,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,0,'Tunnel-Client-Endpoint',10,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,0,'Tunnel-Server-Endpoint',10,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,0,'Tunnel-Connection',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,0,'Tunnel-Password',10,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,0,'ARAP-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,0,'ARAP-Features',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,0,'ARAP-Zone-Access',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,0,'ARAP-Security',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,0,'ARAP-Security-Data',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,0,'Password-Retry',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,0,'Prompt',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 77)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(77,0,'Connect-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 78)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(78,0,'Configuration-Token',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,0,'EAP-Message',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,0,'Signature',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,0,'Tunnel-Private-Group-ID',10,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,0,'Tunnel-Assignment-ID',10,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,0,'Tunnel-Preference',11,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 84)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(84,0,'ARAP-Challenge-Response',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 85)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(85,0,'Acct-Interim-Interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 86)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(86,0,'Acct-Tunnel-Packets-Lost',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 87)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(87,0,'NAS-Port-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 88)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(88,0,'Framed-Pool',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 89)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(89,0,'Chargeable-User-Identity',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,0,'Tunnel-Client-Auth-ID',10,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,0,'Tunnel-Server-Auth-ID',10,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,0,'NAS-IPv6-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,0,'Framed-Interface-Id',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,0,'Framed-IPv6-Prefix',9,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,0,'Login-IPv6-Host',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 99)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(99,0,'Framed-IPv6-Route',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,0,'Framed-IPv6-Pool',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,0,'Error-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,0,'Sip-Response-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,0,'Digest-Response',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,0,'Digest-Realm',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,0,'Digest-Nonce',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,0,'Digest-Response-Auth',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,0,'Digest-Nextnonce',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,0,'Digest-Method',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,0,'Digest-URI',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,0,'Digest-Qop',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,0,'Digest-Algorithm',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,0,'Digest-Entity-Body-Hash',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,0,'Digest-CNonce',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,0,'Digest-Nonce-Count',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(115,0,'Digest-Username',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,0,'Digest-Opaque',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,0,'Digest-Auth-Param',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,0,'Digest-AKA-Auts',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(119,0,'Digest-Domain',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 120)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(120,0,'Digest-Stale',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,0,'Digest-HA1',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,0,'SIP-AOR',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,0,'Delegated-IPv6-Prefix',9,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,0,'Operator-Name',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,0,'Location-Information',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,0,'Location-Data',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,0,'Basic-Location-Policy-Rules',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,0,'Extended-Location-Policy-Rules',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,0,'Location-Capable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,0,'Requested-Location-Info',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,0,'Framed-Management-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 134)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(134,0,'Management-Transport-Protection',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 135)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(135,0,'Management-Policy-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 136)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(136,0,'Management-Privilege-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 137)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(137,0,'PKM-SS-Cert',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 138)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(138,0,'PKM-CA-Cert',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 139)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(139,0,'PKM-Config-Settings',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,0,'PKM-Cryptosuite-List',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,0,'PKM-SAID',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,0,'PKM-SA-Descriptor',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,0,'PKM-Auth-Key',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 144)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(144,0,'DS-Lite-Tunnel-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 145)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(145,0,'Mobile-Node-Identifier',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 146)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(146,0,'Service-Selection',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 147)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(147,0,'PMIP6-Home-LMA-IPv6-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 148)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(148,0,'PMIP6-Visited-LMA-IPv6-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 149)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(149,0,'PMIP6-Home-LMA-IPv4-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 150)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(150,0,'PMIP6-Visited-LMA-IPv4-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 151)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(151,0,'PMIP6-Home-HN-Prefix',9,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 152)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(152,0,'PMIP6-Visited-HN-Prefix',9,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 153)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(153,0,'PMIP6-Home-Interface-ID',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 154)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(154,0,'PMIP6-Visited-Interface-ID',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 155)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(155,0,'PMIP6-Home-IPv4-HoA',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 156)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(156,0,'PMIP6-Visited-IPv4-HoA',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 157)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(157,0,'PMIP6-Home-DHCP4-Server-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 158)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(158,0,'PMIP6-Visited-DHCP4-Server-Address',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 159)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(159,0,'PMIP6-Home-DHCP6-Server-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 160)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(160,0,'PMIP6-Visited-DHCP6-Server-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 161)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(161,0,'PMIP6-Home-IPv4-Gateway',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 162)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(162,0,'PMIP6-Visited-IPv4-Gateway',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 163)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(163,0,'EAP-Lower-Layer',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 164)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(164,0,'GSS-Acceptor-Service-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 165)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(165,0,'GSS-Acceptor-Host-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 166)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(166,0,'GSS-Acceptor-Service-Specifics',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 167)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(167,0,'GSS-Acceptor-Realm-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 168)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(168,0,'Framed-IPv6-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 169)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(169,0,'DNS-Server-IPv6-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 170)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(170,0,'Route-IPv6-Information',9,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 171)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(171,0,'Delegated-IPv6-Prefix-Pool',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 172)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(172,0,'Stateful-IPv6-Address-Pool',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 173)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(173,0,'IPv6-6rd-Configuration',20,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 44289)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44289,0,'IPv6-6rd-Config-IPv4-MaskLen',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 44290)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44290,0,'IPv6-6rd-Config-6rd-Prefix',9,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 44291)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44291,0,'IPv6-6rd-Config-Border-Relay',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 206)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(206,0,'Digest-Response',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 207)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(207,0,'Digest-Attributes',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 208)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(208,0,'Sip-Uri-User',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 211)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(211,0,'Sip-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 213)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(213,0,'Sip-Rpid',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 224)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(224,0,'Canopy-Shared-Secret',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 225)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(225,0,'Canopy-SULDR',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 226)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(226,0,'Canopy-SDLDR',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 227)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(227,0,'Canopy-ULBA',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 228)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(228,0,'Canopy-DLBA',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 229)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(229,0,'Canopy-Enable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 230)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(230,0,'Canopy-LPSULDR',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 231)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(231,0,'Canopy-LPSDLDR',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 232)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(232,0,'Canopy-HPCENABLE',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 233)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(233,0,'Canopy-HPSULDR',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 234)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(234,0,'Canopy-HPSDLDR',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 235)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(235,0,'Canopy-HIGHERBW',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 236)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(236,0,'Canopy-CIRENABLE',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 241)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(241,0,'Extended-Type-1',21,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 242)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(242,0,'Extended-Type-2',21,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 243)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(243,0,'Extended-Type-3',21,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 244)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(244,0,'Extended-Type-4',21,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 245)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(245,0,'Extended-Type-Flagged-1',22,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 246)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(246,0,'Extended-Type-Flagged-2',22,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 45 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,0,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 45 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,0,1,'RADIUS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 45 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,0,2,'Local')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 45 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,0,3,'Remote')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 45 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,0,4,'Diameter')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,1,'Start')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,2,'Stop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,3,'Alive')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,4,'Modem-Start')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,5,'Modem-Stop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,6,'Cancel')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,7,'Accounting-On')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,8,'Accounting-Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,9,'Tunnel-Start')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,10,'Tunnel-Stop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,11,'Tunnel-Reject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,12,'Tunnel-Link-Start')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,13,'Tunnel-Link-Stop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,14,'Tunnel-Link-Reject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 40 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(40,0,15,'Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,1,'User-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,2,'Lost-Carrier')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,3,'Lost-Service')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,4,'Idle-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,5,'Session-Timeout')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,6,'Admin-Reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,7,'Admin-Reboot')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,8,'Port-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,9,'NAS-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,10,'NAS-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,11,'NAS-Reboot')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,12,'Port-Unneeded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,13,'Port-Preempted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,14,'Port-Suspended')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,15,'Service-Unavailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,16,'Callback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,17,'User-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,18,'Host-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,19,'Supplicant-Restart')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,20,'Reauth-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,21,'Port-Reinit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 49 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,0,22,'Port-Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 72 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(72,0,1,'Default-Zone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 72 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(72,0,2,'Inclusive-Filter')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 72 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(72,0,3,'Exclusive-Filter')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 13 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,0,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 13 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,0,1,'Van-Jacobsen-TCP-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 13 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,0,2,'IPX-Header-Compression')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,1,'PPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,2,'SLIP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,3,'ARA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,4,'Gandalf')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,5,'Xylogics')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,6,'X.75-Synchronous')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,7,'GPRS-PDP-Context')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,9,'PPTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 256)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,256,'MPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 257)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,257,'EURAW')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 258)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,258,'EUUI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 259)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,259,'X25')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 260)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,260,'COMB')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 261)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,261,'FR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 10 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,0,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 10 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,0,1,'Broadcast')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 10 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,0,2,'Listen')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 10 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,0,3,'Broadcast-Listen')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 15 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,0,0,'Telnet')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 15 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,0,1,'Rlogin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 15 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,0,2,'TCP-Clear')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 15 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,0,3,'PortMaster')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 15 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,0,4,'LAT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 15 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,0,5,'X25-PAD')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 15 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,0,6,'X25-T3POS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 15 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,0,7,'TCP-Clear-Quiet')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,0,'Async')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,1,'Sync')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,2,'ISDN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,3,'ISDN-V120')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,4,'ISDN-V110')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,5,'Virtual')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,6,'PAIFS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,7,'HDLC-Clear-Channel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,8,'X25')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,9,'X75')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,10,'G.3-Fax')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,11,'SDSL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,12,'ADSL-CAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,13,'ADSL-DMT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,14,'IDSL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,15,'Ethernet')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,16,'xDSL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,17,'Cable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,18,'Wireless-Other')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,19,'Wireless-802.11')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,20,'Token-Ring')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,21,'FDDI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,22,'Wireless-CDMA2000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,23,'Wireless-UMTS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,24,'Wireless-1X-EV')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,25,'IAPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,26,'FTTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,27,'Wireless-802.16')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,28,'Wireless-802.20')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,29,'Wireless-802.22')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,30,'PPPoA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,31,'PPPoEoA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,32,'PPPoEoE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,33,'PPPoEoVLAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,34,'PPPoEoQinQ')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,35,'xPON')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 61 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,0,36,'Wireless-XGP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 76 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(76,0,1,'No-Echo')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 76 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(76,0,2,'Echo')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 29 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,0,0,'Default')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 29 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,0,1,'RADIUS-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,1,'IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,2,'IP6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,3,'NSAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,4,'HDLC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,5,'BBN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,6,'IEEE-802')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,7,'E-163')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,8,'E-164')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,9,'F-69')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,10,'X-121')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,11,'IPX')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,12,'Appletalk')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,13,'Decnet4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,14,'Vines')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 65 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,0,15,'E-164-NSAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,1,'PPTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,2,'L2F')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,3,'L2TP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,4,'ATMP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,5,'VTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,6,'AH')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,7,'IP-IP-Encap')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,8,'MIN-IP-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,9,'ESP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,10,'GRE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,11,'DVS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,12,'IP-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 64 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,0,13,'VLAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,1,'Login-User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,2,'Framed-User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,3,'Callback-Login')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,4,'Callback-Framed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,5,'Outbound')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,6,'Administrative')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,7,'NAS-Prompt')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,8,'Authenicate-Only')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,9,'Callback-NAS-Prompt')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,10,'Call-Check')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,11,'Callback-Administrative')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,12,'Voice')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,13,'Fax')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,14,'Modem-Relay')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,15,'IAPP-Register')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,16,'IAPP-AP-Check')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,17,'Authorize-Only')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,18,'Framed-Management')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 6 AND Value = 129)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,0,129,'Call-Check-User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 201)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,201,'Session-Context-Removed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 202)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,202,'Invalid-EAP-Packet-(Ignored)')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 401)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,401,'Unsupported-Attribute')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 402)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,402,'Missing-Attribute')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 403)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,403,'NAS-Identification-Mismatch')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 404)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,404,'Invalid-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 405)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,405,'Unsupported-Service')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 406)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,406,'Unsupported-Extension')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 407)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,407,'Invalid-Attribute-Value')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 501)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,501,'Administratively-Prohibited')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 502)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,502,'Request-Not-Routable-(Proxy)')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 503)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,503,'Session-Context-Not-Found')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 504)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,504,'Session-Context-Not-Removable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 505)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,505,'Other-Proxy-Processing-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 506)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,506,'Resources-Unavailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 507)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,507,'Request-Initiated')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 508)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,508,'Multi-Session-Unsupported')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 101 AND Value = 509)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,0,509,'Location-Info-Required')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 57 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(57,0,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 57 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(57,0,2,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 133 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,0,1,'SNMP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 133 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,0,2,'Web-based')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 133 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,0,3,'NETCONF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 133 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,0,4,'FTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 133 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,0,5,'TFTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 133 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,0,6,'SFTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 133 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,0,7,'RCP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 133 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,0,8,'SCP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 134 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(134,0,1,'No-Protection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 134 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(134,0,2,'Integrity-Protection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 134 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(134,0,3,'Integrity-Confidentiality-Protection')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 131 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(131,0,1,'Civic-Location')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 131 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(131,0,2,'Geo-Location')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 131 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(131,0,4,'User-Location')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 131 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(131,0,8,'NAS-Location')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 131 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(131,0,16,'Future-Requests')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 131 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(131,0,32,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 132 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(132,0,1,'Civic-Location')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 132 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(132,0,2,'Geo-Location')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 132 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(132,0,4,'User-Location')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 132 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(132,0,8,'NAS-Location')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 132 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(132,0,16,'Future-Requests')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 132 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(132,0,32,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 163 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,0,1,'Wired-IEEE-802.1X')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 163 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,0,2,'IEEE-802.11-(no-pre-auth)')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 163 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,0,3,'IEEE-802.11-(pre-authentication)')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 163 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,0,4,'IEEE-802.16e')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 163 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,0,5,'IKEv2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 163 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,0,6,'PPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 163 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,0,7,'PANA-(no-pre-authentication)')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 163 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,0,8,'GSS-API')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 163 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,0,9,'PANA-(pre-authentication)')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 5)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(5,'Acc')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,5,'Acc-Reason-Code',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,5,'Acc-Ccp-Option',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,5,'Acc-Input-Errors',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,5,'Acc-Output-Errors',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,5,'Acc-Access-Partition',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,5,'Acc-Customer-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,5,'Acc-Ip-Gateway-Pri',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,5,'Acc-Ip-Gateway-Sec',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,5,'Acc-Route-Policy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,5,'Acc-ML-MLX-Admin-State',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,5,'Acc-ML-Call-Threshold',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,5,'Acc-ML-Clear-Threshold',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,5,'Acc-ML-Damping-Factor',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,5,'Acc-Tunnel-Secret',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,5,'Acc-Clearing-Cause',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,5,'Acc-Clearing-Location',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,5,'Acc-Service-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,5,'Acc-Request-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,5,'Acc-Bridging-Support',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,5,'Acc-Apsm-Oversubscribed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,5,'Acc-Acct-On-Off-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,5,'Acc-Tunnel-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,5,'Acc-Dns-Server-Pri',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,5,'Acc-Dns-Server-Sec',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,5,'Acc-Nbns-Server-Pri',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,5,'Acc-Nbns-Server-Sec',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,5,'Acc-Dial-Port-Index',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,5,'Acc-Ip-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,5,'Acc-Ipx-Compression',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,5,'Acc-Connect-Tx-Speed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,5,'Acc-Connect-Rx-Speed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,5,'Acc-Modem-Modulation-Type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,5,'Acc-Modem-Error-Protocol',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,5,'Acc-Callback-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,5,'Acc-Callback-Num-Valid',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,5,'Acc-Callback-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,5,'Acc-Callback-CBCP-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,5,'Acc-Dialout-Auth-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,5,'Acc-Dialout-Auth-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,5,'Acc-Dialout-Auth-Username',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,5,'Acc-Access-Community',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,5,'Acc-Vpsm-Reject-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,5,'Acc-Ace-Token',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,5,'Acc-Ace-Token-Ttl',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,5,'Acc-Ip-Pool-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,5,'Acc-Igmp-Admin-State',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,5,'Acc-Igmp-Version',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,0,'No-reason-No-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,1,'Resource-shortage')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,2,'Session-already-open')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,3,'Too-many-RADIUS-users')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,4,'No-authentification-server')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,5,'No-authentification-response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,6,'No-accounting-server')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,7,'No-accounting-response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,8,'Access-Denied')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,9,'Temporary-buffer-shortage')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,10,'Protocol-error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,11,'Invalid-attribute')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,12,'Invalid-service-type')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,13,'Invalid-framed-protocol')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,14,'Invalid-attribute-value')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,15,'Invalid-user-information')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,16,'Invalid-IP-address')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,17,'Invalid-integer-syntax')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,18,'Invalid-NAS-port')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,19,'Requested-by-user')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,20,'Network-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,21,'Service-interruption')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,22,'Physical-port-error')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,23,'Idle-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,24,'Session-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,25,'Administrative-reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,26,'NAS-reload-or-reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,27,'NAS-error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,28,'NAS-request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,29,'Undefined-reason-given')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,30,'Conflicting-attributes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,31,'Port-limit-exceeded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,32,'Facility-not-available')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,33,'Internal-config-error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,34,'Bad-route-specification')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,35,'Access-Partition-bind-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,36,'Security-violation')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,37,'Request-type-conflict')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,38,'Configuration-disallowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,39,'Missing-attribute')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,40,'Invalid-request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,41,'Missing-parameter')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,42,'Invalid-parameter')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,43,'Call-cleared-with-cause')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,44,'Inopportune-config-request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,45,'Invalid-config-parameter')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,46,'Missing-config-parameter')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,47,'Incompatible-service-profile')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,48,'Administrative-reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,49,'Administrative-reload')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,50,'Port-unneeded')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,51,'Port-preempted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,52,'Port-suspended')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,53,'Service-unavailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,54,'Callback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,55,'User-error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 1 AND Value = 56)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5,56,'Host-request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 2 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,5,1,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 2 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,5,2,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 9 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,5,1,'Funnel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 9 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,5,2,'Direct')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 10 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,5,1,'Enabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 10 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,5,2,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,0,'Cause-unspecified')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,1,'Unassigned-number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,2,'No-route-to-transit-network')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,3,'No-route-to-destination')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,6,'Channel-unacceptable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,7,'Call-awarded-being-delivered')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,16,'Normal-clearing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,17,'User-busy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,18,'No-user-responding')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,19,'User-alerted-no-answer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,21,'Call-rejected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,22,'Number-changed')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,26,'Non-selected-user-clearing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,27,'Destination-out-of-order')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,28,'Invalid-or-incomplete-number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,29,'Facility-rejected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,30,'Response-to-status-inquiry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,31,'Normal-unspecified-cause')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,34,'No-circuit-or-channel-available')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,38,'Network-out-of-order')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,41,'Temporary-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,42,'Switching-equipment-congestion')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,43,'Access-information-discarded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,44,'Circuit-or-channel-unavailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,45,'Circuit-or-channed-preempted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,47,'Resources-unavailable')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,49,'Quality-of-service-unavailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,50,'Facility-not-subscribed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,52,'Outgoing-calls-barred')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,54,'Incoming-calls-barred')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,57,'Bearer-capability-unauthorized')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,58,'Bearer-capability-not-available')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,63,'Service-not-available')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 65)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,65,'Bearer-capablity-not-implmented')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 66)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,66,'Channel-type-not-implemented')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 69)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,69,'Facility-not-implemented')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 70)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,70,'Restrcted-digtal-infrmtion-only')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 79)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,79,'Service-not-implemented')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 81)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,81,'Invalid-call-reference')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 82)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,82,'Identified-channel-doesnt-exist')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 84)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,84,'Call-identify-in-use')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 85)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,85,'No-call-suspended')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 86)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,86,'Suspended-call-cleared')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 88)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,88,'Incompatible-destination')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 91)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,91,'Invalid-transit-network-selctin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 95)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,95,'Invalid-message')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 96)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,96,'Mandtory-infrmtion-elment-miss')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 97)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,97,'Message-not-implemented')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 98)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,98,'Inopportune-message')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 99)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,99,'Infrmtion-elemnt-not-implmented')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 100)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,100,'Invlid-infrmtion-element-contnt')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,101,'Message-incompatible-with-state')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,102,'Recovery-on-timer-expiration')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 103)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,103,'Mndtry-infrmtion-elmnt-lngt-err')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 111)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,111,'Protocol-error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 15 AND Value = 127)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,5,127,'Interworking')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 16 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,5,0,'Local-or-remote-user')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 16 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,5,1,'Prvte-ntwork-serving-local-user')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 16 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,5,2,'Pblic-ntwork-serving-local-user')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 16 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,5,3,'Transit-network')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 16 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,5,4,'Prvte-ntwork-serv-remote-user')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 16 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,5,5,'Pblic-ntwork-serv-remote-user')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 16 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,5,6,'International-network')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 16 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,5,10,'Beyond-interworking-point')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 18 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,5,1,'Ring-Indication')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 18 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,5,2,'Dial-Request')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 18 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,5,3,'User-Authentification')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 18 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,5,4,'Tunnel-Authentification')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 19 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,5,1,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 19 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,5,2,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 20 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,5,1,'False')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 20 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,5,2,'True')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 21 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,5,0,'NAS-Reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 21 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,5,1,'NAS-Reload')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 21 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,5,2,'Configuration-Reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 21 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,5,3,'Configuration-Reload')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 21 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,5,4,'Enabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 21 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,5,5,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 28 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(28,5,1,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 28 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(28,5,2,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 29 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,5,1,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 29 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,5,2,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 36 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36,5,0,'User-Auth')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 36 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36,5,3,'User-Specified-E-164')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 36 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36,5,6,'CBCP-Callback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 36 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36,5,7,'CLI-Callback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 37 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(37,5,1,'CBCP-None')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 37 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(37,5,2,'CBCP-User-Specified')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 37 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(37,5,3,'CBCP-Pre-Specified')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 38 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,5,1,'PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 38 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,5,2,'CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 38 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,5,3,'CHAP-PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 38 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,5,4,'NONE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 42 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,5,1,'PUBLIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 42 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,5,2,'NETMAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 43 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,5,1,'No-Access-Partition')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 43 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,5,2,'Access-Partition-Disabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 43 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,5,3,'Partition-Portlimit-Exceeded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 43 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,5,4,'License-Portlimit-Exceeded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 43 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,5,5,'Home-Server-Down')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 43 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,5,6,'Rejected-By-Home-Server')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 43 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,5,7,'NAS-Administratively-Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 47 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(47,5,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 47 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(47,5,2,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 48 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,5,1,'V1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5 AND RadAttributeID = 48 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,5,2,'V2')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 9148)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(9148,'Acme')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,9148,'Acme-FlowID-FS1-F',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,9148,'Acme-FlowType-FS1-F',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,9148,'Acme-Session-Ingress-CallId',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,9148,'Acme-Session-Egress-CallId',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,9148,'Acme-Flow-In-Realm-FS1-F',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,9148,'Acme-Flow-In-Src-Addr-FS1-F',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,9148,'Acme-Flow-In-Src-Port-FS1-F',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,9148,'Acme-Flow-In-Dst-Addr-FS1-F',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,9148,'Acme-Flow-In-Dst-Port-FS1-F',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,9148,'Acme-Flow-Out-Realm-FS1-F',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,9148,'Acme-Flow-Out-Src-Addr-FS1-F',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,9148,'Acme-Flow-Out-Src-Port-FS1-F',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,9148,'Acme-Flow-Out-Dst-Addr-FS1-F',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,9148,'Acme-Flow-Out-Dst-Port-FS1-F',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,9148,'Acme-Calling-Octets-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,9148,'Acme-Calling-Packets-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,9148,'Acme-Calling-RTCP-Packets-Lost-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,9148,'Acme-Calling-RTCP-Avg-Jitter-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,9148,'Acme-Calling-RTCP-Avg-Latency-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,9148,'Acme-Calling-RTCP-MaxJitter-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,9148,'Acme-Calling-RTCP-MaxLatency-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,9148,'Acme-Calling-RTP-Packets-Lost-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,9148,'Acme-Calling-RTP-Avg-Jitter-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,9148,'Acme-Calling-RTP-MaxJitter-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,9148,'Acme-Session-Generic-Id',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,9148,'Acme-Session-Ingress-Realm',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,9148,'Acme-Session-Egress-Realm',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,9148,'Acme-Session-Protocol-Type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,9148,'Acme-Called-Octets-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,9148,'Acme-Called-Packets-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,9148,'Acme-Called-RTCP-Packets-Lost-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,9148,'Acme-Called-RTCP-Avg-Jitter-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,9148,'Acme-Called-RTCP-Avg-Latency-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,9148,'Acme-Called-RTCP-MaxJitter-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,9148,'Acme-Called-RTCP-MaxLatency-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,9148,'Acme-Called-RTP-Packets-Lost-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,9148,'Acme-Called-RTP-Avg-Jitter-FS1',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,9148,'Acme-Called-RTP-MaxJitter-FS1',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,9148,'Acme-Session-Charging-Vector',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,9148,'Acme-Session-Charging-Function-Address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,9148,'Acme-Firmware-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,9148,'Acme-Local-Time-Zone',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,9148,'Acme-Post-Dial-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,9148,'Acme-CDR-Sequence-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,9148,'Acme-Session-Disposition',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,9148,'Acme-Disconnect-Initiator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,9148,'Acme-Disconnect-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,9148,'Acme-Intermediate-Time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,9148,'Acme-Primary-Routing-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,9148,'Acme-Originating-Trunk-Group',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,9148,'Acme-Terminating-Trunk-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,9148,'Acme-Originating-Trunk-Context',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,9148,'Acme-Terminating-Trunk-Context',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,9148,'Acme-P-Asserted-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,9148,'Acme-SIP-Diversion',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,9148,'Acme-SIP-Status',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,9148,'Acme-Ingress-Local-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,9148,'Acme-Ingress-Remote-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,9148,'Acme-Egress-Local-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 77)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(77,9148,'Acme-Egress-Remote-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 78)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(78,9148,'Acme-FlowID-FS1-R',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,9148,'Acme-FlowType-FS1-R',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,9148,'Acme-Flow-In-Realm-FS1-R',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,9148,'Acme-Flow-In-Src-Addr-FS1-R',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,9148,'Acme-Flow-In-Src-Port-FS1-R',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,9148,'Acme-Flow-In-Dst-Addr-FS1-R',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 84)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(84,9148,'Acme-Flow-In-Dst-Port-FS1-R',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 85)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(85,9148,'Acme-Flow-Out-Realm-FS1-R',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 86)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(86,9148,'Acme-Flow-Out-Src-Addr-FS1-R',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 87)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(87,9148,'Acme-Flow-Out-Src-Port-FS1-R',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 88)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(88,9148,'Acme-Flow-Out-Dst-Addr-FS1-R',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 89)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(89,9148,'Acme-Flow-Out-Dst-Port-FS1-R',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,9148,'Acme-FlowID-FS2-F',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,9148,'Acme-FlowType-FS2-F',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,9148,'Acme-Flow-In-Realm-FS2-F',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,9148,'Acme-Flow-In-Src-Addr-FS2-F',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,9148,'Acme-Flow-In-Src-Port-FS2-F',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,9148,'Acme-Flow-In-Dst-Addr-FS2-F',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,9148,'Acme-Flow-In-Dst-Port-FS2-F',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,9148,'Acme-Flow-Out-Realm-FS2-F',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,9148,'Acme-Flow-Out-Src-Addr-FS2-F',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 99)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(99,9148,'Acme-Flow-Out-Src-Port-FS2-F',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,9148,'Acme-Flow-Out-Dst-Addr-FS2-F',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,9148,'Acme-Flow-Out-Dst-Port-FS2-F',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,9148,'Acme-Calling-Octets-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,9148,'Acme-Calling-Packets-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,9148,'Acme-Calling-RTCP-Packets-Lost-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,9148,'Acme-Calling-RTCP-Avg-Jitter-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,9148,'Acme-Calling-RTCP-Avg-Latency-FS2',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,9148,'Acme-Calling-RTCP-MaxJitter-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,9148,'Acme-Calling-RTCP-MaxLatency-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,9148,'Acme-Calling-RTP-Packets-Lost-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,9148,'Acme-Calling-RTP-Avg-Jitter-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,9148,'Acme-Calling-RTP-MaxJitter-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,9148,'Acme-FlowID-FS2-R',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,9148,'Acme-FlowType-FS2-R',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,9148,'Acme-Flow-In-Realm-FS2-R',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(115,9148,'Acme-Flow-In-Src-Addr-FS2-R',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,9148,'Acme-Flow-In-Src-Port-FS2-R',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,9148,'Acme-Flow-In-Dst-Addr-FS2-R',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,9148,'Acme-Flow-In-Dst-Port-FS2-R',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(119,9148,'Acme-Flow-Out-Realm-FS2-R',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 120)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(120,9148,'Acme-Flow-Out-Src-Addr-FS2-R',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,9148,'Acme-Flow-Out-Src-Port-FS2-R',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,9148,'Acme-Flow-Out-Dst-Addr-FS2-R',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,9148,'Acme-Flow-Out-Dst-Port-FS2-R',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,9148,'Acme-Called-Octets-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,9148,'Acme-Called-Packets-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,9148,'Acme-Called-RTCP-Packets-Lost-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,9148,'Acme-Called-RTCP-Avg-Jitter-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,9148,'Acme-Called-RTCP-Avg-Latency-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,9148,'Acme-Called-RTCP-MaxJitter-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,9148,'Acme-Called-RTCP-MaxLatency-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,9148,'Acme-Called-RTP-Packets-Lost-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,9148,'Acme-Called-RTP-Avg-Jitter-FS2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,9148,'Acme-Called-RTP-MaxJitter-FS2',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 134)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(134,9148,'Acme-Egress-Final-Routing-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 135)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(135,9148,'Acme-Session-Ingress-RPH',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 136)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(136,9148,'Acme-Session-Egress-RPH',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 137)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(137,9148,'Acme-Ingress-Network-Interface-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 138)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(138,9148,'Acme-Ingress-Vlan-Tag-Value',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 139)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(139,9148,'Acme-Egress-Network-Interface-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,9148,'Acme-Egress-Vlan-Tag-Value',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,9148,'Acme-Refer-Call-Transfer-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,9148,'Acme-FlowMediaType-FS1-F',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,9148,'Acme-FlowMediaType-FS1-R',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 144)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(144,9148,'Acme-FlowMediaType-FS2-F',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 145)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(145,9148,'Acme-FlowMediaType-FS2-R',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 146)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(146,9148,'Acme-Flow-PTime-FS1-F',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 147)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(147,9148,'Acme-Flow-PTime-FS1-R',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 148)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(148,9148,'Acme-Flow-PTime-FS2-F',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 149)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(149,9148,'Acme-Flow-PTime-FS2-R',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 150)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(150,9148,'Acme-Session-Media-Process',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 151)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(151,9148,'Acme-Calling-R-Factor',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 152)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(152,9148,'Acme-Calling-MOS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 153)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(153,9148,'Acme-Called-R-Factor',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 154)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(154,9148,'Acme-Called-MOS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 155)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(155,9148,'Acme-Flow-In-Src-IPv6-Addr-FS1-F',5,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 156)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(156,9148,'Acme-Flow-In-Dst-IPv6-Addr-FS1-F',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 157)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(157,9148,'Acme-Flow-Out-Src-IPv6-Addr-FS1-F',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 158)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(158,9148,'Acme-Flow-Out-Dst-IPv6-Addr-FS1-F',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 159)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(159,9148,'Acme-Flow-In-Src-IPv6-Addr-FS1-R',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 160)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(160,9148,'Acme-Flow-In-Dst-IPv6-Addr-FS1-R',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 161)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(161,9148,'Acme-Flow-Out-Src-IPv6-Addr-FS1-R',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 162)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(162,9148,'Acme-Flow-Out-Dst-IPv6-Addr-FS1-R',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 163)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(163,9148,'Acme-Flow-In-Src-IPv6-Addr-FS2-F',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 164)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(164,9148,'Acme-Flow-In-Dst-IPv6-Addr-FS2-F',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 165)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(165,9148,'Acme-Flow-Out-Src-IPv6-Addr-FS2-F',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 166)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(166,9148,'Acme-Flow-Out-Dst-IPv6-Addr-FS2-F',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 167)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(167,9148,'Acme-Flow-In-Src-IPv6-Addr-FS2-R',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 168)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(168,9148,'Acme-Flow-In-Dst-IPv6-Addr-FS2-R',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 169)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(169,9148,'Acme-Flow-Out-Src-IPv6-Addr-FS2-R',5,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 170)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(170,9148,'Acme-Flow-Out-Dst-IPv6-Addr-FS2-R',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 171)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(171,9148,'Acme-Session-Forked-Call-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 200)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(200,9148,'Acme-Custom-VSA-200',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 201)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(201,9148,'Acme-Custom-VSA-201',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 202)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(202,9148,'Acme-Custom-VSA-202',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 203)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(203,9148,'Acme-Custom-VSA-203',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 204)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(204,9148,'Acme-Custom-VSA-204',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 205)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(205,9148,'Acme-Custom-VSA-205',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 206)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(206,9148,'Acme-Custom-VSA-206',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 207)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(207,9148,'Acme-Custom-VSA-207',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 208)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(208,9148,'Acme-Custom-VSA-208',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 209)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(209,9148,'Acme-Custom-VSA-209',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 210)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(210,9148,'Acme-Custom-VSA-210',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 211)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(211,9148,'Acme-Custom-VSA-211',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 212)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(212,9148,'Acme-Custom-VSA-212',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 213)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(213,9148,'Acme-Custom-VSA-213',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 214)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(214,9148,'Acme-Custom-VSA-214',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 215)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(215,9148,'Acme-Custom-VSA-215',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 216)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(216,9148,'Acme-Custom-VSA-216',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 217)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(217,9148,'Acme-Custom-VSA-217',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 218)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(218,9148,'Acme-Custom-VSA-218',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 219)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(219,9148,'Acme-Custom-VSA-219',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 220)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(220,9148,'Acme-Custom-VSA-220',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 221)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(221,9148,'Acme-Custom-VSA-221',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 222)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(222,9148,'Acme-Custom-VSA-222',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 223)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(223,9148,'Acme-Custom-VSA-223',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 224)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(224,9148,'Acme-Custom-VSA-224',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 225)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(225,9148,'Acme-Custom-VSA-225',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 226)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(226,9148,'Acme-Custom-VSA-226',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 227)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(227,9148,'Acme-Custom-VSA-227',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 228)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(228,9148,'Acme-Custom-VSA-228',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 229)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(229,9148,'Acme-Custom-VSA-229',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 230)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(230,9148,'Acme-Custom-VSA-230',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9148 AND RadAttributeID = 254)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(254,9148,'Acme-User-Class',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 26928)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(26928,'Aerohive')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 26928 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,26928,'AH-HM-Admin-Group-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 26928 AND RadAttributeID = 1 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,26928,0,'Read-Only-Admin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 26928 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,26928,1,'Super-Admin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 26928 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,26928,2,'Read-Write-Admin')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 14179)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(14179,'Airespace')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14179 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,14179,'Airespace-Wlan-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14179 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,14179,'Airespace-QOS-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14179 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,14179,'Airespace-DSCP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14179 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,14179,'Airespace-8021p-Tag',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14179 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,14179,'Airespace-Interface-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14179 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,14179,'Airespace-ACL-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14179 AND RadAttributeID = 2 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,14179,0,'Bronze')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14179 AND RadAttributeID = 2 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,14179,1,'Silver')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14179 AND RadAttributeID = 2 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,14179,2,'Gold')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14179 AND RadAttributeID = 2 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,14179,3,'Platinum')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14179 AND RadAttributeID = 2 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,14179,4,'Uranium')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 3041)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(3041,'Alcatel')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,3041,'AAT-Client-Primary-DNS',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,3041,'AAT-Client-Primary-WINS-NBNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,3041,'AAT-Client-Secondary-WINS-NBNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,3041,'AAT-Client-Primary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,3041,'AAT-PPP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,3041,'AAT-ATM-Direct',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,3041,'AAT-IP-TOS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,3041,'AAT-IP-TOS-Precedence',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,3041,'AAT-IP-TOS-Apply-To',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,3041,'AAT-MCast-Client',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,3041,'AAT-Modem-Port-No',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,3041,'AAT-Modem-Slot-No',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,3041,'AAT-Modem-Shelf-No',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,3041,'AAT-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,3041,'AAT-Vrouter-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,3041,'AAT-Require-Auth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,3041,'AAT-IP-Pool-Definition',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,3041,'AAT-Assign-IP-Pool',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,3041,'AAT-Data-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,3041,'AAT-Source-IP-Check',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,3041,'AAT-Modem-Answer-String',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,3041,'AAT-Auth-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,3041,'AAT-Qos',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,3041,'AAT-Qoa',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,3041,'AAT-Client-Assign-DNS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,3041,'AAT-ATM-VPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,3041,'AAT-ATM-VCI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,3041,'AAT-Input-Octets-Diff',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,3041,'AAT-Output-Octets-Diff',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,3041,'AAT-User-MAC-Address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3041 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,3041,'AAT-ATM-Traffic-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 27 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(27,3041,0,'Multicast-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 27 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(27,3041,1,'Multicast-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 62 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(62,3041,0,'Not-Require-Auth')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 62 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(62,3041,1,'Require-Auth')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 66 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,3041,0,'Source-IP-Check-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 66 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,3041,1,'Source-IP-Check-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 22 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3041,0,'IP-TOS-Normal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 22 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3041,1,'IP-TOS-Disabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 22 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3041,2,'IP-TOS-Cost')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 22 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3041,4,'IP-TOS-Reliability')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 22 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3041,8,'IP-TOS-Throughput')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 22 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3041,16,'IP-TOS-Latency')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 24 AND Value = 1024)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,3041,1024,'IP-TOS-Apply-To-Incoming')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 24 AND Value = 3072)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,3041,3072,'IP-TOS-Apply-To-Both')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 24 AND Value = 2048)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,3041,2048,'IP-TOS-Apply-To-Outgoing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 23 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,3041,0,'IP-TOS-Precedence-Pri-Normal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 23 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,3041,32,'IP-TOS-Precedence-Pri-One')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 23 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,3041,64,'IP-TOS-Precedence-Pri-Two')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 23 AND Value = 96)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,3041,96,'IP-TOS-Precedence-Pri-Three')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 23 AND Value = 128)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,3041,128,'IP-TOS-Precedence-Pri-Four')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 23 AND Value = 160)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,3041,160,'IP-TOS-Precedence-Pri-Five')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 23 AND Value = 192)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,3041,192,'IP-TOS-Precedence-Pri-Six')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 23 AND Value = 224)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,3041,224,'IP-TOS-Precedence-Pri-Seven')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 68 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,3041,0,'AAT-Auth-None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 68 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,3041,1,'AAT-Auth-Default')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 68 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,3041,2,'AAT-Auth-Any')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 68 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,3041,3,'AAT-Auth-PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 68 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,3041,4,'AAT-Auth-CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 68 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,3041,5,'AAT-Auth-MS-CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 72 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(72,3041,0,'DNS-Assign-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3041 AND RadAttributeID = 72 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(72,3041,1,'DNS-Assign-Yes')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 637)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(637,'AlcatelDN')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1792)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1792,637,'ISAM-VRF-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1793)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1793,637,'ISAM-VLAN-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1794)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1794,637,'ISAM-QoS-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1795)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1795,637,'ISAM-QoS-Parms',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1801)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1801,637,'ISAM-CLID-AAA',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1802)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1802,637,'ISAM-CLID-ATM',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1803)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1803,637,'ISAM-CLID-Alarm',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1804)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1804,637,'ISAM-CLID-DHCP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1805)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1805,637,'ISAM-CLID-EQP',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1806)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1806,637,'ISAM-CLID-IGMP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1807)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1807,637,'ISAM-CLID-CPEproxy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1808)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1808,637,'ISAM-CLID-IP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1809)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1809,637,'ISAM-CLID-PPPoE',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1810)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1810,637,'ISAM-CLID-QoS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1811)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1811,637,'ISAM-CLID-SWMgt',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1812)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1812,637,'ISAM-CLID-Transport',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1813)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1813,637,'ISAM-CLID-VLAN',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1814)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1814,637,'ISAM-CLID-XDSL',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1815)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1815,637,'ISAM-CLID-Security',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1816)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1816,637,'ISAM-CLID-Cluster',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1817)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1817,637,'ISAM-CLIP-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1818)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1818,637,'ISAM-CLIP-Password-Timeout',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 637 AND RadAttributeID = 1819)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1819,637,'ISAM-CLIP-Description',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1801 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1801,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1801 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1801,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1801 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1801,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1801 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1801,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1802 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1802,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1802 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1802,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1802 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1802,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1802 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1802,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1803 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1803,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1803 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1803,637,1,'Read-Access')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1803 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1803,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1803 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1803,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1804 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1804,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1804 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1804,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1804 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1804,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1804 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1804,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1805 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1805,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1805 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1805,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1805 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1805,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1805 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1805,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1806 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1806,637,0,'No-Access')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1806 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1806,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1806 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1806,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1806 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1806,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1807 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1807,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1807 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1807,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1807 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1807,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1807 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1807,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1808 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1808,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1808 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1808,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1808 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1808,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1808 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1808,637,3,'Read-Write-Access')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1809 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1809,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1809 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1809,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1809 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1809,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1809 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1809,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1810 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1810,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1810 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1810,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1810 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1810,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1810 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1810,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1811 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1811,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1811 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1811,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1811 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1811,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1811 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1811,637,3,'Read-Write-Access')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1812 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1812,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1812 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1812,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1812 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1812,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1812 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1812,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1813 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1813,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1813 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1813,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1813 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1813,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1813 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1813,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1814 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1814,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1814 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1814,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1814 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1814,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1814 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1814,637,3,'Read-Write-Access')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1815 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1815,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1815 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1815,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1815 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1815,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1815 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1815,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1816 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1816,637,0,'No-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1816 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1816,637,1,'Read-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1816 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1816,637,2,'Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 637 AND RadAttributeID = 1816 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1816,637,3,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 1872)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(1872,'Alteon')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1872 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,1872,'Alteon-Group-Mapping',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1872 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,1872,'Alteon-VPN-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1872 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,1872,'Alteon-Client-IP-Address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1872 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,1872,'Alteon-Client-Netmask',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1872 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,1872,'Alteon-Primary-NBNS-Server',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1872 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,1872,'Alteon-Secondary-NBNS-Server',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1872 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,1872,'Alteon-Primary-DNS-Server',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1872 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,1872,'Alteon-Secondary-DNS-Server',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1872 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,1872,'Alteon-Domain-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1872 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,1872,'Alteon-Service-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1872 AND RadAttributeID = 26 AND Value = 250)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(26,1872,250,'Alteon-L4admin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1872 AND RadAttributeID = 26 AND Value = 251)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(26,1872,251,'Alteon-Slbadmin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1872 AND RadAttributeID = 26 AND Value = 252)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(26,1872,252,'Alteon-Oper')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1872 AND RadAttributeID = 26 AND Value = 253)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(26,1872,253,'Alteon-L4oper')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1872 AND RadAttributeID = 26 AND Value = 254)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(26,1872,254,'Alteon-Slboper')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1872 AND RadAttributeID = 26 AND Value = 255)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(26,1872,255,'Alteon-User')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 3076)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(3076,'Altiga')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,3076,'Altiga-Min-Password-Length-G',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,3076,'Altiga-Allow-Alpha-Only-Passwords-G',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,3076,'Altiga-Primary-DNS-G',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,3076,'Altiga-Secondary-DNS-G',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,3076,'Altiga-Primary-WINS-G',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,3076,'Altiga-Secondary-WINS-G',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,3076,'Altiga-IPSec-Authentication-G',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,3076,'Altiga-IPSec-Banner-G',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,3076,'Altiga-PPTP-Encryption-G',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,3076,'Altiga-L2TP-Encryption-G',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,3076,'Altiga-IPSec-L2L-Keepalives-G',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,3076,'Altiga-IPSec-Split-Tunnel-List-G',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,3076,'Altiga-IPSec-Default-Domain-G',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,3076,'Altiga-IPSec-Secondary-Domains-G',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,3076,'Altiga-IPSec-Tunnel-Type-G',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,3076,'Altiga-IPSec-Mode-Config-G',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,3076,'Altiga-IPSec-User-Group-Lock-G',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,3076,'Altiga-IPSec-Over-NAT-G',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,3076,'Altiga-IPSec-Over-NAT-Port-Num-G',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 4 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,3076,1,'Allow')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 4 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,3076,0,'Disallow')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 13 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,3076,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 13 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,3076,1,'RADIUS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 13 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,3076,2,'LDAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 13 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,3076,3,'NTDomain')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 13 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,3076,4,'SDI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 13 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,3076,5,'Internal')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,2,'40bit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,3,'40-Encryption-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,4,'128')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,5,'128-Encryption-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,6,'40-or-128')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,7,'40-or-128-Encry-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,10,'40-Stateless-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,11,'40-Enc/Stateless-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,12,'128-Stateless-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,13,'128-Enc/Stateless-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,14,'40/128-Stateless-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 20 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,3076,15,'40/128-Enc/Statls-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,2,'40bit')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,3,'40-Encryption-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,4,'128')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,5,'128-Encryption-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,6,'40-or-128')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,7,'40-or-128-Encry-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,10,'40-Stateless-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,11,'40-Enc/Stateless-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,12,'128-Stateless-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,13,'128-Enc/Stateless-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,14,'40/128-Stateless-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 21 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,3076,15,'40/128-Enc/Statls-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 25 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(25,3076,1,'ON')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 25 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(25,3076,0,'OFF')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 30 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(30,3076,1,'LAN-to-LAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 30 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(30,3076,2,'Remote-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 31 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(31,3076,1,'ON')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 31 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(31,3076,0,'OFF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 33 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(33,3076,1,'ON')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 33 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(33,3076,0,'OFF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 34 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(34,3076,1,'ON')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 34 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(34,3076,0,'OFF')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 318)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(318,'APC')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 318 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,318,'APC-Service-Type',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 318 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,318,'APC-Outlets',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 318 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,318,1,'Admin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 318 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,318,2,'Device')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 318 AND RadAttributeID = 1 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,318,3,'ReadOnly')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 318 AND RadAttributeID = 1 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,318,4,'Outlet')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 2637)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(2637,'Aptis')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,2637,'CVX-Identification',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,2637,'CVX-VPOP-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,2637,'CVX-SS7-Session-ID-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,2637,'CVX-Radius-Redirect',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,2637,'CVX-IPSVC-AZNLVL',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,2637,'CVX-IPSVC-Mask',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,2637,'CVX-Multilink-Match-Info',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,2637,'CVX-Multilink-Group-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,2637,'CVX-PPP-Log-Mask',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,2637,'CVX-Modem-Begin-Modulation',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,2637,'CVX-Modem-End-Modulation',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,2637,'CVX-Modem-Error-Correction',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,2637,'CVX-Modem-Data-Compression',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,2637,'CVX-Modem-Tx-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,2637,'CVX-Modem-ReTx-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,2637,'CVX-Modem-SNR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,2637,'CVX-Modem-Local-Retrains',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,2637,'CVX-Modem-Remote-Retrains',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,2637,'CVX-Modem-Local-Rate-Negs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,2637,'CVX-Modem-Remote-Rate-Negs',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,2637,'CVX-Modem-Begin-Recv-Line-Lvl',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,2637,'CVX-Modem-End-Recv-Line-Lvl',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,2637,'CVX-Terminate-Component',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,2637,'CVX-Terminate-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,2637,'CVX-Reject-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 135)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(135,2637,'CVX-Primary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 136)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(136,2637,'CVX-Secondary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 137)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(137,2637,'CVX-Client-Assign-DNS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 152)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(152,2637,'CVX-Multicast-Rate-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 155)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(155,2637,'CVX-Multicast-Client',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 195)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(195,2637,'CVX-Disconnect-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 197)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(197,2637,'CVX-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 198)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(198,2637,'CVX-PreSession-Time',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 218)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(218,2637,'CVX-Assign-IP-Pool',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 235)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(235,2637,'CVX-Maximum-Channels',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 242)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(242,2637,'CVX-Data-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 244)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(244,2637,'CVX-Idle-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 253)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(253,2637,'CVX-PPP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2637 AND RadAttributeID = 255)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(255,2637,'CVX-Xmit-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 7564)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(7564,'Array-Networks')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7564 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,7564,'Accept-Acls',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7564 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,7564,'SourceNets',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7564 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,7564,'memberUid',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7564 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,7564,'memberGid',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7564 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,7564,'NetPool',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 529)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(529,'Ascend')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,529,'Ascend-Max-Shared-Users',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,529,'Ascend-UU-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,529,'Ascend-CIR-Timer',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,529,'Ascend-FR-08-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,529,'Ascend-Destination-Nas-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,529,'Ascend-FR-SVC-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,529,'Ascend-NAS-Port-Format',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,529,'Ascend-ATM-Fault-Management',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,529,'Ascend-ATM-Loopback-Cell-Loss',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,529,'Ascend-Ckt-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,529,'Ascend-SVC-Enabled',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,529,'Ascend-Session-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,529,'Ascend-H323-Gatekeeper',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,529,'Ascend-Global-Call-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,529,'Ascend-H323-Conference-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,529,'Ascend-H323-Fegw-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,529,'Ascend-H323-Dialed-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,529,'Ascend-Dialed-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,529,'Ascend-Inter-Arrival-Jitter',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,529,'Ascend-Dropped-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,529,'Ascend-Dropped-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,529,'Ascend-Auth-Delay',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,529,'Ascend-X25-Pad-X3-Profile',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,529,'Ascend-X25-Pad-X3-Parameters',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,529,'Ascend-Tunnel-VRouter-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,529,'Ascend-X25-Reverse-Charging',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,529,'Ascend-X25-Nui-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,529,'Ascend-X25-Nui-Password-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,529,'Ascend-X25-Cug',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,529,'Ascend-X25-Pad-Alias-1',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,529,'Ascend-X25-Pad-Alias-2',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,529,'Ascend-X25-Pad-Alias-3',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,529,'Ascend-X25-X121-Address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,529,'Ascend-X25-Nui',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,529,'Ascend-X25-Rpoa',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,529,'Ascend-X25-Pad-Prompt',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,529,'Ascend-X25-Pad-Banner',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,529,'Ascend-X25-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,529,'Ascend-Recv-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,529,'Ascend-Bi-Directional-Auth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,529,'Ascend-MTU',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,529,'Ascend-Call-Direction',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,529,'Ascend-Service-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,529,'Ascend-Filter-Required',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,529,'Ascend-Traffic-Shaper',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,529,'Ascend-Access-Intercept-LEA',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,529,'Ascend-Access-Intercept-Log',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,529,'Ascend-Private-Route-Table-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,529,'Ascend-Private-Route-Required',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,529,'Ascend-Cache-Refresh',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,529,'Ascend-Cache-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,529,'Ascend-Egress-Enabled',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,529,'Ascend-QOS-Upstream',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,529,'Ascend-QOS-Downstream',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,529,'Ascend-ATM-Connect-Vpi',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,529,'Ascend-ATM-Connect-Vci',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,529,'Ascend-ATM-Connect-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,529,'Ascend-ATM-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,529,'Ascend-IPX-Header-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,529,'Ascend-Calling-Id-Type-Of-Num',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,529,'Ascend-Calling-Id-Number-Plan',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,529,'Ascend-Calling-Id-Presentatn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,529,'Ascend-Calling-Id-Screening',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,529,'Ascend-BIR-Enable',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,529,'Ascend-BIR-Proxy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,529,'Ascend-BIR-Bridge-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,529,'Ascend-IPSEC-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,529,'Ascend-PPPoE-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,529,'Ascend-Bridge-Non-PPPoE',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,529,'Ascend-ATM-Direct',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 77)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(77,529,'Ascend-ATM-Direct-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 78)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(78,529,'Ascend-Client-Primary-WINS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,529,'Ascend-Client-Secondary-WINS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,529,'Ascend-Client-Assign-WINS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,529,'Ascend-Auth-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,529,'Ascend-Port-Redir-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,529,'Ascend-Port-Redir-Portnum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 84)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(84,529,'Ascend-Port-Redir-Server',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 85)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(85,529,'Ascend-IP-Pool-Chaining',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 86)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(86,529,'Ascend-Owner-IP-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 87)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(87,529,'Ascend-IP-TOS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 88)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(88,529,'Ascend-IP-TOS-Precedence',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 89)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(89,529,'Ascend-IP-TOS-Apply-To',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,529,'Ascend-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,529,'Ascend-Telnet-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,529,'Ascend-Dsl-Rate-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,529,'Ascend-Redirect-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,529,'Ascend-ATM-Vpi',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,529,'Ascend-ATM-Vci',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,529,'Ascend-Source-IP-Check',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,529,'Ascend-Dsl-Rate-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,529,'Ascend-Dsl-Upstream-Limit',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 99)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(99,529,'Ascend-Dsl-Downstream-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,529,'Ascend-Dsl-CIR-Recv-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,529,'Ascend-Dsl-CIR-Xmit-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,529,'Ascend-VRouter-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,529,'Ascend-Source-Auth',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,529,'Ascend-Private-Route',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,529,'Ascend-Numbering-Plan-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,529,'Ascend-FR-Link-Status-DLCI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,529,'Ascend-Calling-Subaddress',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,529,'Ascend-Callback-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,529,'Ascend-Endpoint-Disc',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,529,'Ascend-Remote-FW',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,529,'Ascend-Multicast-GLeave-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,529,'Ascend-CBCP-Enable',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,529,'Ascend-CBCP-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,529,'Ascend-CBCP-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(115,529,'Ascend-CBCP-Trunk-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,529,'Ascend-Appletalk-Route',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,529,'Ascend-Appletalk-Peer-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,529,'Ascend-Route-Appletalk',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(119,529,'Ascend-FCP-Parameter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 120)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(120,529,'Ascend-Modem-PortNo',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,529,'Ascend-Modem-SlotNo',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,529,'Ascend-Modem-ShelfNo',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,529,'Ascend-Call-Attempt-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,529,'Ascend-Call-Block-Duration',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,529,'Ascend-Maximum-Call-Duration',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,529,'Ascend-Temporary-Rtes',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,529,'Ascend-Tunneling-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,529,'Ascend-Shared-Profile-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,529,'Ascend-Primary-Home-Agent',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,529,'Ascend-Secondary-Home-Agent',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,529,'Ascend-Dialout-Allowed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,529,'Ascend-Client-Gateway',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,529,'Ascend-BACP-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 134)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(134,529,'Ascend-DHCP-Maximum-Leases',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 135)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(135,529,'Ascend-Client-Primary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 136)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(136,529,'Ascend-Client-Secondary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 137)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(137,529,'Ascend-Client-Assign-DNS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 138)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(138,529,'Ascend-User-Acct-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 139)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(139,529,'Ascend-User-Acct-Host',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,529,'Ascend-User-Acct-Port',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,529,'Ascend-User-Acct-Key',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,529,'Ascend-User-Acct-Base',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,529,'Ascend-User-Acct-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 144)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(144,529,'Ascend-Assign-IP-Client',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 145)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(145,529,'Ascend-Assign-IP-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 146)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(146,529,'Ascend-Assign-IP-Global-Pool',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 147)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(147,529,'Ascend-DHCP-Reply',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 148)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(148,529,'Ascend-DHCP-Pool-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 149)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(149,529,'Ascend-Expect-Callback',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 150)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(150,529,'Ascend-Event-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 151)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(151,529,'Ascend-Session-Svr-Key',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 152)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(152,529,'Ascend-Multicast-Rate-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 153)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(153,529,'Ascend-IF-Netmask',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 154)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(154,529,'Ascend-Remote-Addr',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 155)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(155,529,'Ascend-Multicast-Client',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 156)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(156,529,'Ascend-FR-Circuit-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 157)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(157,529,'Ascend-FR-LinkUp',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 158)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(158,529,'Ascend-FR-Nailed-Grp',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 159)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(159,529,'Ascend-FR-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 160)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(160,529,'Ascend-FR-Link-Mgt',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 161)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(161,529,'Ascend-FR-N391',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 162)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(162,529,'Ascend-FR-DCE-N392',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 163)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(163,529,'Ascend-FR-DTE-N392',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 164)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(164,529,'Ascend-FR-DCE-N393',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 165)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(165,529,'Ascend-FR-DTE-N393',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 166)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(166,529,'Ascend-FR-T391',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 167)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(167,529,'Ascend-FR-T392',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 168)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(168,529,'Ascend-Bridge-Address',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 169)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(169,529,'Ascend-TS-Idle-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 170)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(170,529,'Ascend-TS-Idle-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 171)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(171,529,'Ascend-DBA-Monitor',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 172)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(172,529,'Ascend-Base-Channel-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 173)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(173,529,'Ascend-Minimum-Channels',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 174)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(174,529,'Ascend-IPX-Route',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 175)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(175,529,'Ascend-FT1-Caller',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 176)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(176,529,'Ascend-Backup',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 177)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(177,529,'Ascend-Call-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 178)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(178,529,'Ascend-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 179)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(179,529,'Ascend-FR-DLCI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 180)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(180,529,'Ascend-FR-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 181)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(181,529,'Ascend-Ara-PW',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 182)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(182,529,'Ascend-IPX-Node-Addr',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 183)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(183,529,'Ascend-Home-Agent-IP-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 184)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(184,529,'Ascend-Home-Agent-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 185)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(185,529,'Ascend-Home-Network-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 186)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(186,529,'Ascend-Home-Agent-UDP-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 187)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(187,529,'Ascend-Multilink-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 188)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(188,529,'Ascend-Num-In-Multilink',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 189)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(189,529,'Ascend-First-Dest',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 190)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(190,529,'Ascend-Pre-Input-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 191)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(191,529,'Ascend-Pre-Output-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 192)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(192,529,'Ascend-Pre-Input-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 193)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(193,529,'Ascend-Pre-Output-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 194)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(194,529,'Ascend-Maximum-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 195)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(195,529,'Ascend-Disconnect-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 196)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(196,529,'Ascend-Connect-Progress',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 197)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(197,529,'Ascend-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 198)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(198,529,'Ascend-PreSession-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 199)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(199,529,'Ascend-Token-Idle',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 200)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(200,529,'Ascend-Token-Immediate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 201)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(201,529,'Ascend-Require-Auth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 202)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(202,529,'Ascend-Number-Sessions',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 203)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(203,529,'Ascend-Authen-Alias',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 204)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(204,529,'Ascend-Token-Expiry',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 205)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(205,529,'Ascend-Menu-Selector',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 206)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(206,529,'Ascend-Menu-Item',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 207)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(207,529,'Ascend-PW-Warntime',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 208)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(208,529,'Ascend-PW-Lifetime',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 209)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(209,529,'Ascend-IP-Direct',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 210)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(210,529,'Ascend-PPP-VJ-Slot-Comp',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 211)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(211,529,'Ascend-PPP-VJ-1172',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 212)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(212,529,'Ascend-PPP-Async-Map',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 213)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(213,529,'Ascend-Third-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 214)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(214,529,'Ascend-Send-Secret',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 215)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(215,529,'Ascend-Receive-Secret',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 216)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(216,529,'Ascend-IPX-Peer-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 217)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(217,529,'Ascend-IP-Pool-Definition',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 218)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(218,529,'Ascend-Assign-IP-Pool',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 219)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(219,529,'Ascend-FR-Direct',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 220)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(220,529,'Ascend-FR-Direct-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 221)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(221,529,'Ascend-FR-Direct-DLCI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 222)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(222,529,'Ascend-Handle-IPX',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 223)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(223,529,'Ascend-Netware-timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 224)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(224,529,'Ascend-IPX-Alias',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 225)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(225,529,'Ascend-Metric',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 226)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(226,529,'Ascend-PRI-Number-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 227)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(227,529,'Ascend-Dial-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 228)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(228,529,'Ascend-Route-IP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 229)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(229,529,'Ascend-Route-IPX',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 230)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(230,529,'Ascend-Bridge',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 231)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(231,529,'Ascend-Send-Auth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 232)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(232,529,'Ascend-Send-Passwd',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 233)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(233,529,'Ascend-Link-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 234)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(234,529,'Ascend-Target-Util',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 235)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(235,529,'Ascend-Maximum-Channels',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 236)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(236,529,'Ascend-Inc-Channel-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 237)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(237,529,'Ascend-Dec-Channel-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 238)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(238,529,'Ascend-Seconds-Of-History',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 239)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(239,529,'Ascend-History-Weigh-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 240)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(240,529,'Ascend-Add-Seconds',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 241)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(241,529,'Ascend-Remove-Seconds',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 242)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(242,529,'Ascend-Data-Filter',4,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 243)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(243,529,'Ascend-Call-Filter',4,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 244)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(244,529,'Ascend-Idle-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 245)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(245,529,'Ascend-Preempt-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 246)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(246,529,'Ascend-Callback',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 247)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(247,529,'Ascend-Data-Svc',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 248)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(248,529,'Ascend-Force-56',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 249)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(249,529,'Ascend-Billing-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 250)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(250,529,'Ascend-Call-By-Call',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 251)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(251,529,'Ascend-Transit-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 252)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(252,529,'Ascend-Host-Info',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 253)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(253,529,'Ascend-PPP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 254)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(254,529,'Ascend-MPP-Idle-Percent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 529 AND RadAttributeID = 255)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(255,529,'Ascend-Xmit-Rate',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 96 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(96,529,0,'Source-IP-Check-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 96 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(96,529,1,'Source-IP-Check-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 112 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,529,0,'CBCP-Not-Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 112 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,529,1,'CBCP-Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 113 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,529,1,'CBCP-No-Callback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 113 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,529,2,'CBCP-User-Callback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 113 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,529,3,'CBCP-Profile-Callback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 113 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,529,7,'CBCP-Any-Or-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 113 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,529,8,'CBCP-Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 219 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,529,0,'FR-Direct-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 219 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,529,1,'FR-Direct-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 222 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(222,529,0,'Handle-IPX-None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 222 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(222,529,1,'Handle-IPX-Client')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 222 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(222,529,2,'Handle-IPX-Server')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 216 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(216,529,0,'IPX-Peer-Router')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 216 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(216,529,1,'IPX-Peer-Dialin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 177 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(177,529,0,'Switched')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 177 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(177,529,1,'Nailed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 177 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(177,529,2,'Nailed/Mpp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 177 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(177,529,3,'Perm/Switched')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 177 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(177,529,6,'AO/DI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 177 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(177,529,7,'MegaMax')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 175 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(175,529,0,'FT1-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 175 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(175,529,1,'FT1-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 226 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(226,529,0,'Unknown-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 226 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(226,529,1,'Intl-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 226 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(226,529,2,'National-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 226 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(226,529,3,'Net-Specific-Number')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 226 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(226,529,4,'Local-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 226 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(226,529,5,'Abbrev-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 228 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(228,529,0,'Route-IP-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 228 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(228,529,1,'Route-IP-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 229 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(229,529,0,'Route-IPX-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 229 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(229,529,1,'Route-IPX-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 230 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(230,529,0,'Bridge-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 230 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(230,529,1,'Bridge-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 170 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(170,529,0,'TS-Idle-None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 170 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(170,529,1,'TS-Idle-Input')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 170 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(170,529,2,'TS-Idle-Input-Output')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 231 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(231,529,0,'Send-Auth-None')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 231 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(231,529,1,'Send-Auth-PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 231 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(231,529,2,'Send-Auth-CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 231 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(231,529,3,'Send-Auth-MS-CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 233 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(233,529,0,'Link-Comp-None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 233 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(233,529,1,'Link-Comp-Stac')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 233 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(233,529,2,'Link-Comp-Stac-Draft-9')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 233 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(233,529,3,'Link-Comp-MS-Stac')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 239 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(239,529,0,'History-Constant')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 239 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(239,529,1,'History-Linear')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 239 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(239,529,2,'History-Quadratic')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 246 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(246,529,0,'Callback-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 246 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(246,529,1,'Callback-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 149 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(149,529,0,'Expect-Callback-No')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 149 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(149,529,1,'Expect-Callback-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,0,'Switched-Voice-Bearer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,1,'Nailed-56KR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,2,'Nailed-64K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,3,'Switched-64KR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,4,'Switched-56K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,5,'Switched-384KR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,6,'Switched-384K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,7,'Switched-1536K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,8,'Switched-1536KR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,9,'Switched-128K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,10,'Switched-192K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,11,'Switched-256K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,12,'Switched-320K')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,13,'Switched-384K-MR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,14,'Switched-448K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,15,'Switched-512K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,16,'Switched-576K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,17,'Switched-640K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,18,'Switched-704K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,19,'Switched-768K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,20,'Switched-832K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,21,'Switched-896K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,22,'Switched-960K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,23,'Switched-1024K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,24,'Switched-1088K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,25,'Switched-1152K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,26,'Switched-1216K')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,27,'Switched-1280K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,28,'Switched-1344K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,29,'Switched-1408K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,30,'Switched-1472K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,31,'Switched-1600K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,32,'Switched-1664K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,33,'Switched-1728K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,34,'Switched-1792K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,35,'Switched-1856K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,36,'Switched-1920K')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,37,'Switched-inherited')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,38,'Switched-restricted-bearer-x30')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,39,'Switched-clear-bearer-v110')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,40,'Switched-restricted-64-x30')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,41,'Switched-clear-56-v110')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,42,'Switched-modem')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,43,'Switched-atmodem')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,45,'Switched-V110-24-56')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,46,'Switched-V110-48-56')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,47,'Switched-V110-96-56')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,48,'Switched-V110-192-56')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,49,'Switched-V110-384-56')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,50,'Switched-V110-24-56R')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,51,'Switched-V110-48-56R')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,52,'Switched-V110-96-56R')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,53,'Switched-V110-192-56R')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,54,'Switched-V110-384-56R')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,55,'Switched-V110-24-64')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 56)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,56,'Switched-V110-48-64')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,57,'Switched-V110-96-64')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,58,'Switched-V110-192-64')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 59)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,59,'Switched-V110-384-64')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,60,'Switched-V110-24-64R')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 61)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,61,'Switched-V110-48-64R')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 62)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,62,'Switched-V110-96-64R')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,64,'Switched-V110-384-64R')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,63,'Switched-V110-192-64R')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 68)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,68,'Switched-Pots')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 69)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,69,'Switched-ATM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 247 AND Value = 70)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(247,529,70,'Switched-FR')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 248 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(248,529,0,'Force-56-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 248 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(248,529,1,'Force-56-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 208 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(208,529,0,'Lifetime-In-Days')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 207 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(207,529,0,'Days-Of-Warning')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 211 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(211,529,1,'PPP-VJ-1172')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 210 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(210,529,1,'VJ-Slot-Comp-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 201 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(201,529,0,'Not-Require-Auth')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 201 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(201,529,1,'Require-Auth')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 200 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(200,529,0,'Tok-Imm-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 200 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(200,529,1,'Tok-Imm-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 171 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(171,529,0,'DBA-Transmit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 171 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(171,529,1,'DBA-Transmit-Recv')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 171 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(171,529,2,'DBA-None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 159 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(159,529,0,'Ascend-FR-DTE')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 159 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(159,529,1,'Ascend-FR-DCE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 159 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(159,529,2,'Ascend-FR-NNI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 160 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(160,529,0,'Ascend-FR-No-Link-Mgt')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 160 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(160,529,1,'Ascend-FR-T1-617D')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 160 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(160,529,2,'Ascend-FR-Q-933A')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 157 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(157,529,0,'Ascend-LinkUp-Default')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 157 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(157,529,1,'Ascend-LinkUp-AlwaysUp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 155 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,529,0,'Multicast-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 155 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,529,1,'Multicast-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 138 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(138,529,0,'Ascend-User-Acct-None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 138 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(138,529,1,'Ascend-User-Acct-User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 138 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(138,529,2,'Ascend-User-Acct-User-Default')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 142 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(142,529,0,'Base-10')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 142 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(142,529,1,'Base-16')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 147 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(147,529,0,'DHCP-Reply-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 147 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(147,529,1,'DHCP-Reply-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 137 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(137,529,0,'DNS-Assign-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 137 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(137,529,1,'DNS-Assign-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 150 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(150,529,1,'Ascend-ColdStart')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 150 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(150,529,2,'Ascend-Session-Event')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 133 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,529,0,'BACP-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 133 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,529,1,'BACP-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 131 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(131,529,0,'Dialout-Not-Allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 131 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(131,529,1,'Dialout-Allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 128 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,529,0,'Shared-Profile-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 128 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,529,1,'Shared-Profile-Yes')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 126 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(126,529,0,'Temp-Rtes-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 126 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(126,529,1,'Temp-Rtes-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,0,'No-Reason')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,1,'Not-Applicable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,2,'Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,3,'Call-Disconnected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,4,'CLID-Authentication-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,5,'CLID-RADIUS-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,10,'Modem-No-DCD')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,11,'DCD-Detected-Then-Inactive')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,12,'Modem-Invalid-Result-Codes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,20,'TermSrv-User-Quit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,21,'TermSrv-Idle-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,22,'TermSrv-Exit-Telnet')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,23,'TermSrv-No-IPaddr')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,24,'TermSrv-Exit-Raw-TCP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,25,'TermSrv-Exit-Login-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,26,'TermSrv-Exit-Raw-TCP-Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,27,'TermSrv-CTRL-C-In-Login')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,28,'TermSrv-Destroyed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,29,'TermSrv-User-Closed-VCon')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,30,'TermSrv-VCon-Destroyed')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,31,'TermSrv-Exit-Rlogin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,32,'TermSrv-Bad-Rlogin-Option')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,33,'TermSrv-Not-Enough-Resources')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,35,'MPP-No-NULL-Msg-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,40,'PPP-LCP-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,41,'PPP-LCP-Negotion-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,42,'PPP-PAP-Auth-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,43,'PPP-CHAP-Auth-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,44,'PPP-Rmt-Auth-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,45,'PPP-Rcv-Terminate-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,46,'PPP-Rcv-Close-Event')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,47,'PPP-No-NCPs-Open')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,48,'PPP-MP-Bundle-Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,49,'PPP-LCP-Close-MP-Add-Fail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,50,'Session-Table-Full')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,51,'Out-Of-Resources')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,52,'Invalid-IP-Address')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,53,'Hostname-Resolution-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,54,'Bad-Or-Missing-Port-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,60,'Host-Reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 61)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,61,'Connection-Refused')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 62)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,62,'Connection-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,63,'Connection-Closed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,64,'Network-Unreachable')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 65)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,65,'Host-Unreachable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 66)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,66,'Network-Unreachable-Admin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 67)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,67,'Host-Unreachable-Admin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 68)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,68,'Port-Unreachable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 100)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,100,'Session-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,101,'Invalid-Incoming-User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,102,'Disconnect-Due-To-Callback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 120)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,120,'Proto-Disabled-Or-Unsupported')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 150)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,150,'Disconnect-Req-By-RADIUS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 151)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,151,'Disconnect-Req-By-Local-Admin')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 160)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,160,'V110-Timeout-Sync-Retry-Exceed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 170)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,170,'PPP-Auth-Timeout-Exceeded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 180)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,180,'User-Executed-Do-Hangup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 185)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,185,'Remote-End-Hung-Up')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 190)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,190,'Resource-Has-Been-Quiesced')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 195 AND Value = 195)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,529,195,'Max-Call-Duration-Reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,0,'No-Progress')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,10,'Call-Up')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,30,'Modem-Up')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,31,'Modem-Awaiting-DCD')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,32,'Modem-Awaiting-Codes')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,40,'TermSrv-Started')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,41,'TermSrv-Raw-TCP-Started')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,42,'TermSrv-Telnet-Started')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,43,'TermSrv-Raw-TCP-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,44,'TermSrv-Telnet-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,45,'TermSrv-Rlogin-Started')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,46,'TermSrv-Rlogin-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,50,'Modem-Outdial-Call-Up')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,60,'LAN-Session-Up')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 61)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,61,'LCP-Opening')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 62)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,62,'CCP-Opening')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,63,'IPNCP-Opening')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,64,'BNCP-Opening')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 65)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,65,'LCP-Opened')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 66)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,66,'CCP-Opened')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 67)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,67,'IPNCP-Opened')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 68)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,68,'BNCP-Opened')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 69)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,69,'LCP-State-Initial')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 70)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,70,'LCP-State-Starting')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 71)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,71,'LCP-State-Closed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 72)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,72,'LCP-State-Stopped')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 73)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,73,'LCP-State-Closing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 74)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,74,'LCP-State-Stopping')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 75)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,75,'LCP-State-Request-Sent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 76)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,76,'LCP-State-Ack-Received')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 77)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,77,'LCP-State-Ack-Sent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 80)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,80,'IPXNCP-Opened')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 81)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,81,'ATNCP-Opened')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 82)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,82,'BACP-Opening')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 83)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,83,'BACP-Opened')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 90)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,90,'V110-Up')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 91)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,91,'V110-State-Opened')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 92)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,92,'V110-State-Carrier')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 93)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,93,'V110-State-Reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 196 AND Value = 94)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,529,94,'V110-State-Closed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 76 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(76,529,0,'ATM-Direct-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 76 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(76,529,1,'ATM-Direct-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 14 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,529,2,'VC-End-To-End-Loopback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 14 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,529,0,'VC-No-Loopback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 14 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,529,1,'VC-Segment-Loopback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 117 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(117,529,1,'Appletalk-Peer-Dialin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 117 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(117,529,0,'Appletalk-Peer-Router')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 81 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(81,529,2,'Auth-Any')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 81 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(81,529,4,'Auth-CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 81 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(81,529,1,'Auth-Default')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 81 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(81,529,5,'Auth-MS-CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 81 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(81,529,0,'Auth-None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 81 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(81,529,3,'Auth-PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 70 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(70,529,0,'BIR-Enable-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 70 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(70,529,1,'BIR-Enable-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 71 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(71,529,0,'BIR-Proxy-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 71 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(71,529,1,'BIR-Proxy-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 46 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(46,529,1,'Bi-Directional-Auth-Allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 46 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(46,529,0,'Bi-Directional-Auth-None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 46 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(46,529,2,'Bi-Directional-Auth-Required')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 75 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(75,529,0,'Bridge-Non-PPPoE-No')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 75 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(75,529,1,'Bridge-Non-PPPoE-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 56 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(56,529,0,'Refresh-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 56 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(56,529,1,'Refresh-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 48 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,529,0,'Ascend-Call-Direction-Incoming')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 48 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,529,1,'Ascend-Call-Direction-Outgoing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 67 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,529,3,'Data')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 67 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,529,1,'ISDN-Telephony')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 67 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,529,8,'National')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 67 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,529,9,'Private')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 67 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,529,4,'Telex')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 67 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,529,0,'Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 68 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,529,0,'Allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 68 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,529,2,'Number-Not-Available')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 68 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,529,1,'Restricted')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 69 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,529,3,'Network-Provided')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 69 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,529,0,'User-Not-Screened')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 69 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,529,2,'User-Provided-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 69 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,529,1,'User-Provided-Passed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 66 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,529,6,'Abbreviated-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 66 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,529,1,'International-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 66 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,529,2,'National-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 66 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,529,3,'Network-Specific')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 66 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,529,4,'Subscriber-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 66 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,529,0,'Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 16 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,529,0,'Ascend-PVC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 16 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,529,1,'Ascend-SVC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 80 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(80,529,0,'WINS-Assign-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 80 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(80,529,1,'WINS-Assign-Yes')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,10,'adslcap-dn-1280000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,9,'adslcap-dn-1600000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,8,'adslcap-dn-1920000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,7,'adslcap-dn-2240000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,6,'adslcap-dn-2560000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,5,'adslcap-dn-2688000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,4,'adslcap-dn-3200000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,3,'adslcap-dn-4480000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,2,'adslcap-dn-5120000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,1,'adslcap-dn-6272000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,12,'adslcap-dn-640000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,0,'adslcap-dn-7168000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,11,'adslcap-dn-960000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 121)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,121,'adsldmt-dn-128000')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 114)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,114,'adsldmt-dn-1280000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 113)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,113,'adsldmt-dn-1600000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 112)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,112,'adsldmt-dn-1920000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 111)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,111,'adsldmt-dn-2240000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 120)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,120,'adsldmt-dn-256000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 110)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,110,'adsldmt-dn-2560000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 109)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,109,'adsldmt-dn-2688000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 108)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,108,'adsldmt-dn-3200000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 119)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,119,'adsldmt-dn-384000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 107)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,107,'adsldmt-dn-4480000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 118)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,118,'adsldmt-dn-512000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 106)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,106,'adsldmt-dn-5120000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 105)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,105,'adsldmt-dn-6272000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 117)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,117,'adsldmt-dn-640000')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 104)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,104,'adsldmt-dn-7168000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 116)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,116,'adsldmt-dn-768000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 103)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,103,'adsldmt-dn-8000000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,102,'adsldmt-dn-8960000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,101,'adsldmt-dn-9504000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 115)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,115,'adsldmt-dn-960000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 99 AND Value = 100)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,529,100,'adsldmt-dn-auto')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 97 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(97,529,1,'Rate-Mode-AutoBaud')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 97 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(97,529,2,'Rate-Mode-Single')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 92 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(92,529,2,'Rate-Type-AdslCap')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 92 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(92,529,4,'Rate-Type-AdslDmt')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 92 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(92,529,3,'Rate-Type-AdslDmtCell')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 92 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(92,529,0,'Rate-Type-Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 92 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(92,529,1,'Rate-Type-Sdsl')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 153)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,153,'adsldmt-up-896000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,50,'adslcap-up-1088000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 56)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,56,'adslcap-up-272000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,55,'adslcap-up-408000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,54,'adslcap-up-544000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,53,'adslcap-up-680000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,52,'adslcap-up-816000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,51,'adslcap-up-952000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 151)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,151,'adsldmt-up-1088000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 160)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,160,'adsldmt-up-128000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 159)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,159,'adsldmt-up-256000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 158)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,158,'adsldmt-up-384000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 157)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,157,'adsldmt-up-512000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 156)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,156,'adsldmt-up-640000')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 155)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,155,'adsldmt-up-768000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 154)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,154,'adsldmt-up-800000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 152)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,152,'adsldmt-up-928000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 150)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,150,'adsldmt-up-auto')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,5,'sdsl-1168000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,0,'sdsl-144000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,6,'sdsl-1552000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,7,'sdsl-2320000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,1,'sdsl-272000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,2,'sdsl-400000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,3,'sdsl-528000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 98 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,529,4,'sdsl-784000')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 50 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,529,0,'Required-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 50 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,529,1,'Required-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 85 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(85,529,0,'IP-Pool-Chaining-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 85 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(85,529,1,'IP-Pool-Chaining-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 87 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(87,529,2,'IP-TOS-Cost')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 87 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(87,529,1,'IP-TOS-Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 87 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(87,529,16,'IP-TOS-Latency')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 87 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(87,529,0,'IP-TOS-Normal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 87 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(87,529,4,'IP-TOS-Reliability')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 87 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(87,529,8,'IP-TOS-Throughput')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 89 AND Value = 3072)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(89,529,3072,'IP-TOS-Apply-To-Both')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 89 AND Value = 1024)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(89,529,1024,'IP-TOS-Apply-To-Incoming')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 89 AND Value = 2048)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(89,529,2048,'IP-TOS-Apply-To-Outgoing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 88 AND Value = 160)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(88,529,160,'IP-TOS-Precedence-Pri-Five')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 88 AND Value = 128)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(88,529,128,'IP-TOS-Precedence-Pri-Four')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 88 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(88,529,0,'IP-TOS-Precedence-Pri-Normal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 88 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(88,529,32,'IP-TOS-Precedence-Pri-One')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 88 AND Value = 224)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(88,529,224,'IP-TOS-Precedence-Pri-Seven')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 88 AND Value = 192)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(88,529,192,'IP-TOS-Precedence-Pri-Six')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 88 AND Value = 96)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(88,529,96,'IP-TOS-Precedence-Pri-Three')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 88 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(88,529,64,'IP-TOS-Precedence-Pri-Two')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 65 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,529,0,'IPX-Header-Compression-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 65 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,529,1,'IPX-Header-Compression-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 13 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,529,3,'1_2_2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 13 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,529,2,'2_4_5_5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 13 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,529,1,'2_4_6_4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 13 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,529,0,'Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 105 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,529,1,'ISDN-Numbering-Plan')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 105 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,529,9,'Private-Numbering-Plan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 105 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,529,0,'Unknown-Numbering-Plan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 74 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(74,529,0,'PPPoE-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 74 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(74,529,1,'PPPoE-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 82 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(82,529,6,'Ascend-Proto-TCP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 82 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(82,529,17,'Ascend-Proto-UDP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 55 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(55,529,0,'Required-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 55 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(55,529,1,'Required-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 118 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(118,529,0,'Route-Appletalk-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 118 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(118,529,1,'Route-Appletalk-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 17 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(17,529,0,'Ascend-SVC-Enabled-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 17 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(17,529,1,'Ascend-SVC-Enabled-Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,20,'Ascend-Service-Type-ATM')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,7,'Ascend-Service-Type-Combinet')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,9,'Ascend-Service-Type-EuRaw')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,10,'Ascend-Service-Type-EuUi')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,8,'Ascend-Service-Type-FR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,21,'Ascend-Service-Type-HdlcNrm')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,19,'Ascend-Service-Type-IpFax')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,15,'Ascend-Service-Type-MP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,5,'Ascend-Service-Type-MPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,25,'Ascend-Service-Type-NetToNet')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,1,'Ascend-Service-Type-None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,0,'Ascend-Service-Type-NotUsed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,2,'Ascend-Service-Type-Other')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,3,'Ascend-Service-Type-PPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,18,'Ascend-Service-Type-PseuTunPPP')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,13,'Ascend-Service-Type-RawTcp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,4,'Ascend-Service-Type-Slip')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,11,'Ascend-Service-Type-Telnet')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,12,'Ascend-Service-Type-TelnetBin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,14,'Ascend-Service-Type-TermServer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,16,'Ascend-Service-Type-VirtualConn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,23,'Ascend-Service-Type-Visa2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,22,'Ascend-Service-Type-VoIp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,6,'Ascend-Service-Type-X25')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 49 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,529,17,'Ascend-Service-Type-X25DChan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 18 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,529,3,'Ascend-Session-G711-Alaw')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 18 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,529,2,'Ascend-Session-G711-Ulaw')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 18 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,529,4,'Ascend-Session-G723')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 18 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,529,6,'Ascend-Session-G723-64KPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 18 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,529,7,'Ascend-Session-G728')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 18 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,529,5,'Ascend-Session-G729')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 18 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,529,8,'Ascend-Session-RT24')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 18 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,529,1,'Ascend-Session-Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 18 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,529,0,'Ascend-Session-Unused')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 127 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(127,529,0,'ATMP-Tunnel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 127 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(127,529,1,'VTP-Tunnel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,4,'CC_SSP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,5,'CC_TSP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,0,'CRT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,11,'CUSTOM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,2,'DEFAULT')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,6,'HARDCOPY')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,7,'HDX')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,1,'INFONET')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,10,'NULL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,9,'POS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,3,'SCEN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 29 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,529,8,'SHARK')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 32 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(32,529,0,'Reverse-Charging-No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 529 AND RadAttributeID = 32 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(32,529,1,'Reverse-Charging-Yes')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 2167)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(2167,'Avaya')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2167 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,2167,'Avaya-Service-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2167 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,2167,'Avaya-StaticVlan-Type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2167 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,2167,'Avaya-PortPriority-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2167,1,'Avaya-Read-Only-User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2167,2,'Avaya-Read-Write-User')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 1 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2167,3,'Avaya-Admin-User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 13 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,2167,0,'Type-0')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 13 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,2167,1,'Type-1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 13 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,2167,2,'Type-2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 13 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,2167,3,'Type-3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 13 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,2167,4,'Type-4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 13 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,2167,5,'Type-5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 13 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,2167,6,'Type-6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2167 AND RadAttributeID = 13 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,2167,7,'Type-7')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 1584)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(1584,'Bay-Networks')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,1584,'Annex-Filter',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,1584,'Annex-CLI-Command',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,1584,'Annex-CLI-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,1584,'Annex-Host-Restrict',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,1584,'Annex-Host-Allow',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,1584,'Annex-Product-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,1584,'Annex-SW-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,1584,'Annex-Local-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,1584,'Annex-Callback-Portlist',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,1584,'Annex-Sec-Profile-Index',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,1584,'Annex-Tunnel-Authen-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,1584,'Annex-Tunnel-Authen-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,1584,'Annex-Authen-Servers',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,1584,'Annex-Acct-Servers',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,1584,'Annex-User-Server-Location',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,1584,'Annex-Local-Username',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,1584,'Annex-System-Disc-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,1584,'Annex-Modem-Disc-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,1584,'Annex-Disconnect-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,1584,'Annex-Addr-Resolution-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,1584,'Annex-Addr-Resolution-Servers',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,1584,'Annex-Domain-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,1584,'Annex-Transmit-Speed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,1584,'Annex-Receive-Speed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,1584,'Annex-Input-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,1584,'Annex-Output-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,1584,'Annex-Primary-DNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,1584,'Annex-Secondary-DNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,1584,'Annex-Primary-NBNS-Server',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,1584,'Annex-Secondary-NBNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,1584,'Annex-Syslog-Tap',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,1584,'Annex-Keypress-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,1584,'Annex-Unauthenticated-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,1584,'Annex-Re-CHAP-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,1584,'Annex-MRRU',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,1584,'Annex-EDO',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,1584,'Annex-PPP-Trace-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,1584,'Annex-Pre-Input-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,1584,'Annex-Pre-Output-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,1584,'Annex-Pre-Input-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,1584,'Annex-Pre-Output-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,1584,'Annex-Connect-Progress',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,1584,'Annex-Multicast-Rate-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,1584,'Annex-Maximum-Call-Duration',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,1584,'Annex-Multilink-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,1584,'Annex-Num-In-Multilink',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,1584,'Annex-Logical-Channel-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,1584,'Annex-Wan-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,1584,'Annex-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 85)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(85,1584,'Annex-Pool-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 86)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(86,1584,'Annex-Compression-Protocol',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 87)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(87,1584,'Annex-Transmitted-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 88)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(88,1584,'Annex-Retransmitted-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 89)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(89,1584,'Annex-Signal-to-Noise-Ratio',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,1584,'Annex-Retrain-Requests-Sent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,1584,'Annex-Retrain-Requests-Rcvd',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,1584,'Annex-Rate-Reneg-Req-Sent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,1584,'Annex-Rate-Reneg-Req-Rcvd',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,1584,'Annex-Begin-Receive-Line-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,1584,'Annex-End-Receive-Line-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,1584,'Annex-Begin-Modulation',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,1584,'Annex-Error-Correction-Prot',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,1584,'Annex-End-Modulation',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,1584,'Annex-User-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,1584,'Annex-Audit-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,1584,'CES-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 192)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(192,1584,'Passport-Access-Priority',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 193)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(193,1584,'Passport-Cli-Command',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 194)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(194,1584,'Passport-Command-Access',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1584 AND RadAttributeID = 195)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(195,1584,'Passport-Commands',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 192 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,1584,0,'None-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 192 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,1584,1,'Read-Only-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 192 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,1584,2,'L1-Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 192 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,1584,3,'L2-Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 192 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,1584,4,'L3-Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 192 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,1584,5,'Read-Write-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 192 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,1584,6,'Read-Write-All-Access')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 194 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(194,1584,0,'FALSE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 194 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(194,1584,1,'TRUE')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 38 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,1584,0,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 38 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,1584,1,'kmd5-128')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 39 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,1584,0,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 39 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,1584,1,'prefix-suffix')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 42 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,1584,1,'local')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 42 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,1584,2,'remote')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 47 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(47,1584,0,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 47 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(47,1584,1,'DHCP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,0,'Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,1,'Line-disconnected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,2,'Dial-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,3,'WAN-manager-error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,4,'Disconnect-reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,5,'Error-from-adm_notify')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,6,'Modem-down-adm_notify')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,7,'PPP-protocol-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,8,'Inactivity-timer')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,9,'CLI-Hangup-command')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,10,'CLI-last-job')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,11,'Session-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,12,'Slave-termination')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,13,'Abnormal-termination')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,14,'DCD-wait-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,15,'CLI-inactivity')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,16,'Admin-port-reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,17,'CLI-auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,18,'Slave-auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,19,'PAP-auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,20,'CHAP-auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,21,'Local-modem-reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,22,'Modem-dead')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,23,'PPP-LCP-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,24,'PPP-IPCP-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,25,'PPP-IPXCP-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,26,'PPP-ATCP-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,27,'PPP-CCP-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,28,'PPP-MP-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,29,'PPP-IPCP-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,30,'PPP-IPXCP-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,31,'PPP-ATCP-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,32,'PPP-CCP-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,33,'PPP-MP-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,34,'PPP-init-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,35,'PPP-Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,36,'PPP-Dialback-failed')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,37,'PPP-Address-In-Use')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,38,'PPP-No-device')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,39,'PPP-Modem-hangup-rcvd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,40,'PPP-Hangup-rcvd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,41,'PPP-Termination-rcvd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,42,'PPP-Kill-rcvd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,43,'PPP-Time-rcvd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,44,'PPP-No-memory')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,45,'PPP-Connection-Abort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,46,'PPP-VPN-LCP-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,47,'PPP-VPN-Auth-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,48,'PPP-MP-invalid-port')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,49,'PPP-Invalid-device')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,50,'PPP-MMP-bundle-failure')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,51,'DVS-Registration-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,52,'DVS-Home-agent-dereg')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,53,'DVS-Tunnel-no-renew')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 44 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(44,1584,54,'DVS-Tunnel-expired')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 45 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,1584,0,'Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 45 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,1584,1,'Local-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 45 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,1584,2,'CD-Timer-Expired')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 45 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,1584,4,'Remote-protocol-disc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 45 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,1584,5,'Clear-down')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 45 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,1584,6,'Long-Space-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 45 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,1584,7,'Carrier-Lost')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 45 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,1584,8,'Modem-Retrain-Timeout')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 100 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,1584,2,'Manager')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 100 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,1584,4,'User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 100 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,1584,8,'Operator')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 101 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,1584,2,'Manager')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 101 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,1584,4,'User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1584 AND RadAttributeID = 101 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,1584,8,'Operator')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 272)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(272,'BinTec')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825794)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825794,'X25')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825795)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825795,'X25-PPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825796)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825796,'IP-LAPB')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825798)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825798,'IP-HDLC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825799)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825799,'MPR-LAPB')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825800)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825800,'MPR-HDLC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825801)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825801,'FRAME-RELAY')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825802)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825802,'X31-BCHAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825803)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825803,'X75-PPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825804)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825804,'X75BTX-PPP')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825805)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825805,'X25-NOSIG')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 7 AND Value = 17825806)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,0,17825806,'X25-PPP-OPT')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 224)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(224,272,'BinTec-biboPPPTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 225)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(225,272,'BinTec-biboDialTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 226)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(226,272,'BinTec-ipExtIfTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 227)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(227,272,'BinTec-ipRouteTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 228)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(228,272,'BinTec-ipExtRtTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 229)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(229,272,'BinTec-ipNatPresetTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 230)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(230,272,'BinTec-ipxCircTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 231)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(231,272,'BinTec-ripCircTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 232)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(232,272,'BinTec-sapCircTable',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 233)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(233,272,'BinTec-ipxStaticRouteTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 234)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(234,272,'BinTec-ipxStaticServTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 235)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(235,272,'BinTec-ospfIfTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 236)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(236,272,'BinTec-pppExtIfTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 237)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(237,272,'BinTec-ipFilterTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 238)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(238,272,'BinTec-ipQoSTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 239)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(239,272,'BinTec-qosIfTable',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 272 AND RadAttributeID = 240)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(240,272,'BinTec-qosPolicyTable',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 4491)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(4491,'CableLabs')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 0)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(0,4491,'CableLabs-Reserved',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,4491,'CableLabs-Event-Message',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,4491,'CableLabs-MTA-Endpoint-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,4491,'CableLabs-Calling-Party-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,4491,'CableLabs-Called-Party-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,4491,'CableLabs-Database-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,4491,'CableLabs-Query-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,4491,'CableLabs-Returned-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,4491,'CableLabs-Call-Termination-Cause',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,4491,'CableLabs-Related-Call-Billing-Crl-ID',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,4491,'CableLabs-First-Call-Calling-Party-Num',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,4491,'CableLabs-Second-Call-Calling-Party-Num',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,4491,'CableLabs-Charge-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,4491,'CableLabs-Forwarded-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,4491,'CableLabs-Service-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,4491,'CableLabs-Intl-Code',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,4491,'CableLabs-Dial-Around-Code',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,4491,'CableLabs-Location-Routing-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,4491,'CableLabs-Carrier-Identification-Code',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,4491,'CableLabs-Trunk-Group-ID',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,4491,'CableLabs-Routing-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,4491,'CableLabs-MTA-UDP-Portnum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,4491,'CableLabs-Channel-State',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,4491,'CableLabs-SF-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,4491,'CableLabs-Error-Description',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,4491,'CableLabs-QoS-Descriptor',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,4491,'CableLabs-Direction-indicator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,4491,'CableLabs-Time-Adjustment',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,4491,'CableLabs-SDP-Upstream',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,4491,'CableLabs-SDP-Downstream',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,4491,'CableLabs-User-Input',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,4491,'CableLabs-Translation-Input',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,4491,'CableLabs-Redirected-From-Info',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,4491,'CableLabs-Electronic-Surveillance-Ind',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,4491,'CableLabs-Redirected-From-Party-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,4491,'CableLabs-Redirected-To-Party-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,4491,'CableLabs-El-Surveillance-DF-Security',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,4491,'CableLabs-CCC-ID',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,4491,'CableLabs-Financial-Entity-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,4491,'CableLabs-Flow-Direction',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,4491,'CableLabs-Signal-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,4491,'CableLabs-Alerting-Signal',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,4491,'CableLabs-Subject-Audible-Signal',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,4491,'CableLabs-Terminal-Display-Info',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,4491,'CableLabs-Switch-Hook-Flash',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,4491,'CableLabs-Dialed-Digits',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,4491,'CableLabs-Misc-Signaling-Information',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,4491,'CableLabs-AM-Opaque-Data',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,4491,'CableLabs-Subscriber-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,4491,'CableLabs-Volume-Usage-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,4491,'CableLabs-Gate-Usage-Info',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,4491,'CableLabs-Element-Requesting-QoS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,4491,'CableLabs-QoS-Release-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,4491,'CableLabs-Policy-Denied-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,4491,'CableLabs-Policy-Deleted-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,4491,'CableLabs-Policy-Update-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,4491,'CableLabs-Policy-Decision-Status',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,4491,'CableLabs-Application-Manager-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,4491,'CableLabs-Time-Usage-Limit',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,4491,'CableLabs-Gate-Time-Info',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,4491,'CableLabs-Account-Code',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4491 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,4491,'CableLabs-Authorization-Code',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,0,'Reserved')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,1,'Signaling-Start')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,2,'Signaling-Stop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,3,'Database-Query')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,4,'Intelligent-Peripheral-Usage-Start')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,5,'Intelligent-Peripheral-Usage-Stop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,6,'Service-Instance')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,7,'QoS-Reserve')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,8,'QoS-Release')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,9,'Service-Activation')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,10,'Service-Deactivation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,11,'Media-Report')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,12,'Signal-Instance')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,13,'Interconnect-Signaling-Start')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,14,'Interconnect-Signaling-Stop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,15,'Call-Answer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,16,'Call-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,17,'Time-Change')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,19,'QoS-Commit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,20,'Media-Alive')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,31,'Policy-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,32,'Policy-Delete')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 1 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,4491,33,'Policy-Update')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 7 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,4491,0,'Reserved')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 7 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,4491,1,'Toll-Free-Number-Looukp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 7 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,4491,2,'LNP-Number-Lookup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 7 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,4491,3,'Calling-Name-Delivery-Lookup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 29 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4491,0,'Reserved')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 29 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4491,1,'Open')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 29 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4491,2,'Change')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 29 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4491,3,'Close')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 37 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(37,4491,0,'Undefined')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 37 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(37,4491,1,'Originating')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 37 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(37,4491,2,'Terminating')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 50 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,4491,0,'Reserved')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 50 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,4491,1,'Upstream')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 50 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,4491,2,'Downstream')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 51 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(51,4491,0,'Reserved')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 51 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(51,4491,1,'Network-Signal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 51 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(51,4491,2,'Subject-Signal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,0,'Reserved-0')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,1,'Ringing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,2,'Distinctive-Ringing-2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,3,'Distinctive-Ringing-3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,4,'Distinctive-Ringing-4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,5,'Ringsplash')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,6,'Call-Waiting-Tone-1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,7,'Call-Waiting-Tone-2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,8,'Call-Waiting-Tone-3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,9,'Call-Waiting-Tone-4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,10,'Reserved-10')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,11,'Distinctive-Ringing-0')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,12,'Distinctive-Ringing-1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,13,'Distinctive-Ringing-5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,14,'Distinctive-Ringing-6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 52 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,4491,15,'Distinctive-Ringing-7')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 61 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,4491,0,'Reserved-0')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 61 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,4491,1,'Dial-Tone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 61 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,4491,2,'Stutter-Dial-Tone')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 61 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,4491,3,'Ring-Back-Tone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 61 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,4491,4,'Reorder-Tone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 61 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,4491,5,'Busy-Tone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 61 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,4491,6,'Confirmation-Tone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 61 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,4491,7,'Reserved-7')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 61 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,4491,8,'Message-Waiting-Indicator')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 61 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61,4491,9,'Off-hook-Warning-Tone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 65 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,4491,0,'Client')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 65 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,4491,1,'Policy-Server')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 65 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65,4491,2,'Embedded-Client')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 66 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,4491,1,'Gate-Closed-By-PS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 66 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,4491,2,'Inactivity-Resource-Recover-Timer-Exp')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 66 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,4491,3,'CM-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 66 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,4491,4,'Pre-Empted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 66 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,4491,5,'RSVP-PathTear-request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 66 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,4491,6,'CM-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 66 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,4491,7,'Admitted-Timer-Expiration')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 66 AND Value = 127)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(66,4491,127,'Other')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 67 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,4491,1,'Policy-Server-Admission-Control-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 67 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,4491,2,'Insufficient-Resources')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 67 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,4491,3,'Unknown-Subscriber')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 67 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,4491,4,'Unauthorized-AMID')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 67 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,4491,5,'Undefined-Service-Class-Name')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 67 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,4491,6,'Incompatible-Envelope')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 67 AND Value = 127)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,4491,127,'Other')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 68 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,4491,1,'Application-Manager-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 68 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,4491,2,'CMTS-Decistion')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 68 AND Value = 127)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,4491,127,'Other')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 69 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,4491,1,'Traffic-Profile')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 69 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,4491,2,'Classifier')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 69 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,4491,3,'Volume-Limit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 69 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,4491,4,'Time-Limit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 69 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,4491,5,'Opaque-Data')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 69 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,4491,6,'Multiple-Updates')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 69 AND Value = 127)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,4491,127,'Other')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 70 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(70,4491,1,'Policy-Approved')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4491 AND RadAttributeID = 70 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(70,4491,2,'Policy-Denied')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 52)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(52,'Cabletron')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 52 AND RadAttributeID = 201)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(201,52,'Cabletron-Protocol-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 52 AND RadAttributeID = 202)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(202,52,'Cabletron-Protocol-Callable',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 52 AND RadAttributeID = 201 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(201,52,1,'IP-Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 52 AND RadAttributeID = 201 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(201,52,2,'Bridge-Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 52 AND RadAttributeID = 201 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(201,52,3,'IP-BR-Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 52 AND RadAttributeID = 201 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(201,52,6,'BR-IPX-Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 52 AND RadAttributeID = 201 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(201,52,7,'IP-BR-IPX-Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 52 AND RadAttributeID = 202 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(202,52,1,'IP-Callable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 52 AND RadAttributeID = 202 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(202,52,2,'Bridge-Callable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 52 AND RadAttributeID = 202 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(202,52,3,'IP-BR-Callable')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 52 AND RadAttributeID = 202 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(202,52,6,'BR-IPX-Callable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 52 AND RadAttributeID = 202 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(202,52,7,'IP-BR-IPX-Callable')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 9)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(9,'Cisco')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,9,'Cisco-AVPair',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,9,'Cisco-NAS-Port',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,9,'h323-remote-address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,9,'h323-conf-id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,9,'h323-setup-time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,9,'h323-call-origin',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,9,'h323-call-type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,9,'h323-connect-time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,9,'h323-disconnect-time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,9,'h323-disconnect-cause',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,9,'h323-voice-quality',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,9,'h323-gw-id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,9,'h323-incoming-conf-id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,9,'Cisco-Encrypted-AVPair',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,9,'h323-credit-amount',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,9,'h323-credit-time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,9,'h323-return-code',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,9,'h323-prompt-id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,9,'h323-time-and-day',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,9,'h323-redirect-number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,9,'h323-preferred-lang',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,9,'h323-redirect-ip-address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,9,'h323-billing-model',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,9,'h323-currency',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,9,'subscriber',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,9,'gw-rxd-cdn',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,9,'gw-final-xlated-cdn',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,9,'remote-media-address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(115,9,'release-source',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,9,'gw-rxd-cgn',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,9,'gw-final-xlated-cgn',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,9,'call-id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,9,'session-protocol',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,9,'method',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 144)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(144,9,'prev-hop-via',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 145)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(145,9,'prev-hop-ip',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 146)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(146,9,'incoming-req-uri',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 147)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(147,9,'outgoing-req-uri',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 148)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(148,9,'next-hop-ip',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 149)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(149,9,'next-hop-dn',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 150)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(150,9,'sip-hdr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 187)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(187,9,'Cisco-Multilink-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 188)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(188,9,'Cisco-Num-In-Multilink',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 190)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(190,9,'Cisco-Pre-Input-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 191)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(191,9,'Cisco-Pre-Output-Octets',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 192)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(192,9,'Cisco-Pre-Input-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 193)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(193,9,'Cisco-Pre-Output-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 194)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(194,9,'Cisco-Maximum-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 195)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(195,9,'Cisco-Disconnect-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 197)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(197,9,'Cisco-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 198)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(198,9,'Cisco-PreSession-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 208)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(208,9,'Cisco-PW-Lifetime',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 209)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(209,9,'Cisco-IP-Direct',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 210)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(210,9,'Cisco-PPP-VJ-Slot-Comp',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 212)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(212,9,'Cisco-PPP-Async-Map',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 217)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(217,9,'Cisco-IP-Pool-Definition',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 218)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(218,9,'Cisco-Assign-IP-Pool',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 228)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(228,9,'Cisco-Route-IP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 233)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(233,9,'Cisco-Link-Compression',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 234)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(234,9,'Cisco-Target-Util',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 235)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(235,9,'Cisco-Maximum-Channels',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 242)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(242,9,'Cisco-Data-Filter',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 243)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(243,9,'Cisco-Call-Filter',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 244)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(244,9,'Cisco-Idle-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 250)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(250,9,'Cisco-Account-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 251)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(251,9,'Cisco-Service-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 252)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(252,9,'Cisco-Command-Code',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 253)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(253,9,'Cisco-Control-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9 AND RadAttributeID = 255)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(255,9,'Cisco-Xmit-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,2,'Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,4,'CLID-Authentication-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,10,'No-Carrier')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,11,'Lost-Carrier')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,12,'No-Detected-Result-Codes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,20,'User-Ends-Session')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,21,'Idle-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,22,'Exit-Telnet-Session')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,23,'No-Remote-IP-Addr')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,24,'Exit-Raw-TCP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,25,'Password-Fail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,26,'Raw-TCP-Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,27,'Control-C-Detected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,28,'EXEC-Program-Destroyed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,40,'Timeout-PPP-LCP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,41,'Failed-PPP-LCP-Negotiation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,42,'Failed-PPP-PAP-Auth-Fail')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,43,'Failed-PPP-CHAP-Auth')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,44,'Failed-PPP-Remote-Auth')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,45,'PPP-Remote-Terminate')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,46,'PPP-Closed-Event')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 100)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,100,'Session-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,101,'Session-Failed-Security')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,102,'Session-End-Callback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 9 AND RadAttributeID = 195 AND Value = 120)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(195,9,120,'Invalid-Protocol')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 5263)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(5263,'Cisco-BBSM')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5263 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,5263,'CBBSM-Bandwidth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,3076,'CVPN3000-Access-Hours',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,3076,'CVPN3000-Simultaneous-Logins',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,3076,'CVPN3000-SEP-Card-Assignment',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,3076,'CVPN3000-Priority-On-SEP',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,3076,'CVPN3000-Tunneling-Protocols',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,3076,'CVPN3000-IPSec-Sec-Association',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,3076,'CVPN3000-IPSec-Allow-Passwd-Store',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,3076,'CVPN3000-Use-Client-Address',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,3076,'CVPN3000-PPTP-Min-Auth-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,3076,'CVPN3000-L2TP-Min-Auth-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,3076,'CVPN3000-Auth-Server-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,3076,'CVPN3000-Auth-Server-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,3076,'CVPN3000-Request-Auth-Vector',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,3076,'CVPN3000-IPSec-Group-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,3076,'CVPN3000-Auth-Server-Priority',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,3076,'CVPN3000-IPSec-Banner2',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,3076,'CVPN3000-PPTP-MPPC-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,3076,'CVPN3000-L2TP-MPPC-Compression',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,3076,'CVPN3000-IPSec-IP-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,3076,'CVPN3000-IPSec-IKE-Peer-ID-Check',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,3076,'CVPN3000-IKE-Keep-Alives',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,3076,'CVPN3000-IPSec-Auth-On-Rekey',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,3076,'CVPN3000-Reqrd-Client-Fw-Vendor-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,3076,'CVPN3000-Reqrd-Client-Fw-Product-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,3076,'CVPN3000-Reqrd-Client-Fw-Description',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,3076,'CVPN3000-Require-HW-Client-Auth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,3076,'CVPN3000-Require-Individual-User-Auth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,3076,'CVPN3000-Authd-User-Idle-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,3076,'CVPN3000-Cisco-IP-Phone-Bypass',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,3076,'CVPN3000-User-Auth-Server-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,3076,'CVPN3000-User-Auth-Server-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,3076,'CVPN3000-User-Auth-Server-Secret',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,3076,'CVPN3000-IPSec-Split-Tunneling-Policy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,3076,'CVPN3000-IPSec-Reqrd-Client-Fw-Cap',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,3076,'CVPN3000-IPSec-Client-Fw-Filter-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,3076,'CVPN3000-IPSec-Client-Fw-Filter-Opt',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,3076,'CVPN3000-IPSec-Backup-Servers',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,3076,'CVPN3000-IPSec-Backup-Server-List',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,3076,'CVPN3000-DHCP-Network-Scope',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,3076,'CVPN3000-MS-Client-Icpt-DHCP-Conf-Msg',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,3076,'CVPN3000-MS-Client-Subnet-Mask',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,3076,'CVPN3000-Allow-Network-Extension-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,3076,'CVPN3000-IPSec-Authorization-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,3076,'CVPN3000-IPSec-Authorization-Required',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,3076,'CVPN3000-IPSec-DN-Field',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,3076,'CVPN3000-IPSec-Confidence-Level',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,3076,'CVPN3000-WebVPN-Content-Filter',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,3076,'CVPN3000-WebVPN-Enable-functions',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,3076,'CVPN3000-WebVPN-Exchange-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,3076,'CVPN3000-LEAP-Bypass',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 78)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(78,3076,'CVPN3000-WebVPN-Exchange-NETBIOS-name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,3076,'CVPN3000-Port-Forwarding-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,3076,'CVPN3000-Partition-Primary-DHCP',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,3076,'CVPN3000-Partition-Secondary-DHCP',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,3076,'CVPN3000-Partition-Premise-Router',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,3076,'CVPN3000-Partition-Max-Sessions',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,3076,'CVPN3000-Partition-Mobile-IP-Key',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 134)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(134,3076,'CVPN3000-Partition-Mobile-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 135)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(135,3076,'CVPN3000-Partition-Mobile-IP-SPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 136)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(136,3076,'CVPN3000-Strip-Realm',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3076 AND RadAttributeID = 137)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(137,3076,'CVPN3000-Group-Name',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 9 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,3076,1,'SEP1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 9 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,3076,2,'SEP2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 9 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,3076,4,'SEP3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 9 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,3076,8,'SEP4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 9 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,3076,15,'Any-SEP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 10 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,3076,1,'High')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 10 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,3076,2,'Med-High')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 10 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,3076,3,'Medium')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 10 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,3076,4,'Med-Low')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 10 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,3076,5,'Low')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 11 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(11,3076,1,'PPTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 11 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(11,3076,2,'L2TP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 11 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(11,3076,4,'IPSec')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 11 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(11,3076,5,'PPTP-and-IPSec')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 11 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(11,3076,8,'L2TP/IPSec')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 11 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(11,3076,15,'All')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 16 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,3076,0,'Disallow')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 16 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,3076,1,'Allow')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 17 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(17,3076,0,'Disallow')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 17 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(17,3076,1,'Allow')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 18 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,3076,1,'PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 18 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,3076,2,'CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 18 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,3076,4,'EAP-MD5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 18 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,3076,8,'EAP-GTC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 18 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,3076,16,'EAP-TLS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 18 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,3076,32,'MSCHAPv1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 18 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,3076,64,'MSCHAPv2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 18 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,3076,102,'Default')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 19 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,3076,1,'PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 19 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,3076,2,'CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 19 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,3076,4,'EAP-MD5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 19 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,3076,8,'EAP-GTC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 19 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,3076,16,'EAP-TLS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 19 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,3076,32,'MSCHAPv1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 19 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,3076,64,'MSCHAPv2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 19 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,3076,102,'Default')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 22 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3076,0,'First-Active-Server')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 22 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3076,1,'RADIUS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 22 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3076,2,'LDAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 22 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3076,3,'NT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 22 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3076,4,'SDI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 22 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,3076,5,'Internal')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 136 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(136,3076,0,'FALSE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3076 AND RadAttributeID = 136 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(136,3076,1,'TRUE')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 255)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(255,'Cisco-VPN5000')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 255 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,255,'CVPN5000-Tunnel-Throughput',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 255 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,255,'CVPN5000-Client-Assigned-IP',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 255 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,255,'CVPN5000-Client-Real-IP',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 255 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,255,'CVPN5000-VPN-GroupInfo',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 255 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,255,'CVPN5000-VPN-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 255 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,255,'CVPN5000-Echo',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 255 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,255,'CVPN5000-Client-Assigned-IPX',1,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 8744)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(8744,'Colubris')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8744 AND RadAttributeID = 0)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(0,8744,'Colubris-AVPair',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8744 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,8744,'Colubris-Intercept',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8744 AND RadAttributeID = 1 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8744,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8744 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8744,1,'Intercept-GRE')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 32584)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(32584,'DAN')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,32584,'DAN-CPE-MAC-ADDRESS',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,32584,'DAN-SF1-SFID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,32584,'DAN-SF1-DIRECTION',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,32584,'DAN-SF1-QOS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,32584,'DAN-SF1-CLS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,32584,'DAN-SF2-SFID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,32584,'DAN-SF2-DIRECTION',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,32584,'DAN-SF2-QOS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,32584,'DAN-SF2-CLS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,32584,'DAN-SF3-SFID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,32584,'DAN-SF3-DIRECTION',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,32584,'DAN-SF3-QOS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,32584,'DAN-SF3-CLS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,32584,'DAN-SF4-SFID',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,32584,'DAN-SF4-DIRECTION',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,32584,'DAN-SF4-QOS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,32584,'DAN-SF4-CLS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,32584,'DAN-SF5-SFID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,32584,'DAN-SF5-DIRECTION',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,32584,'DAN-SF5-QOS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,32584,'DAN-SF5-CLS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,32584,'DAN-SF6-SFID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,32584,'DAN-SF6-DIRECTION',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,32584,'DAN-SF6-QOS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,32584,'DAN-SF6-CLS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,32584,'DAN-SF7-SFID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,32584,'DAN-SF7-DIRECTION',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,32584,'DAN-SF7-QOS-PROF',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,32584,'DAN-SF7-CLS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,32584,'DAN-SF8-SFID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,32584,'DAN-SF8-DIRECTION',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,32584,'DAN-SF8-QOS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 32584 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,32584,'DAN-SF8-CLS-PROF',1,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 22736)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(22736,'Digium')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,22736,'Asterisk-Acc-Code',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,22736,'Asterisk-Src',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,22736,'Asterisk-Dst',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,22736,'Asterisk-Dst-Ctx',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,22736,'Asterisk-Clid',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,22736,'Asterisk-Chan',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,22736,'Asterisk-Dst-Chan',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,22736,'Asterisk-Last-App',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,22736,'Asterisk-Last-Data',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,22736,'Asterisk-Start-Time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,22736,'Asterisk-Answer-Time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,22736,'Asterisk-End-Time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,22736,'Asterisk-Duration',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,22736,'Asterisk-Bill-Sec',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(115,22736,'Asterisk-Disposition',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,22736,'Asterisk-AMA-Flags',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,22736,'Asterisk-Unique-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 22736 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,22736,'Asterisk-User-Field',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 6798)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(6798,'DS2')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6798 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,6798,'DS2-profile',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6798 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,6798,'DS2-fw',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6798 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,6798,'DS2-MAC',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6798 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,6798,1,'default')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6798 AND RadAttributeID = 2 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,6798,0,'MV')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6798 AND RadAttributeID = 2 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,6798,1,'LV')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6798 AND RadAttributeID = 2 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,6798,2,'EU')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6798 AND RadAttributeID = 3 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,6798,0,'HE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6798 AND RadAttributeID = 3 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,6798,1,'CPE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6798 AND RadAttributeID = 3 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,6798,2,'REP')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 3561)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(3561,'DSLForum')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,3561,'Agent-Circuit-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,3561,'Agent-Remote-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,3561,'Actual-Data-Rate-Upstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,3561,'Actual-Data-Rate-Downstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,3561,'Minimum-Data-Rate-Upstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,3561,'Minimum-Data-Rate-Downstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,3561,'Attainable-Data-Rate-Upstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 134)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(134,3561,'Attainable-Data-Rate-Downstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 135)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(135,3561,'Maximum-Data-Rate-Upstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 136)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(136,3561,'Maximum-Data-Rate-Downstream',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 137)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(137,3561,'Minimum-Data-Rate-Upstream-Low-Power',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 138)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(138,3561,'Minimum-Data-Rate-Downstream-Low-Power',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 139)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(139,3561,'Maximum-Interleaving-Delay-Upstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,3561,'Actual-Interleaving-Delay-Upstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,3561,'Maximum-Interleaving-Delay-Downstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,3561,'Actual-Interleaving-Delay-Downstream',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 144)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(144,3561,'Access-Loop-Encapsulation',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3561 AND RadAttributeID = 254)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(254,3561,'IWF-Session',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 193)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(193,'Ericsson')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,193,'Ericsson-ViG-Balance',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,193,'Ericsson-ViG-Codec',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,193,'Ericsson-ViG-Currency',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,193,'Ericsson-ViG-Currency-Quote',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,193,'Ericsson-ViG-Endpoint-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,193,'Ericsson-ViG-Sequence-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,193,'Ericsson-ViG-Access-Agent-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,193,'Ericsson-ViG-QoS-Class',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,193,'Ericsson-ViG-Digest-Response',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,193,'Ericsson-ViG-Digest-Attributes',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,193,'Ericsson-ViG-Business-Agreement-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,193,'Ericsson-ViG-Call-Role',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,193,'Ericsson-ViG-Remote-SK-UA-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,193,'Ericsson-ViG-Site',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,193,'Ericsson-ViG-TTL-relative',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,193,'Ericsson-ViG-Account-error-reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,193,'Ericsson-ViG-Layer-identity',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,193,'Ericsson-ViG-Major-protocol-version',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,193,'Ericsson-ViG-Minor-protocol-version',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,193,'Ericsson-ViG-Authentication-type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,193,'Ericsson-ViG-Trusted-access',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,193,'Ericsson-ViG-User-name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,193,'Ericsson-ViG-Global-unique-call-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,193,'Ericsson-ViG-Global-unique-service-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,193,'Ericsson-ViG-Interim-interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,193,'Ericsson-ViG-Alive-Indicator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,193,'Ericsson-ViG-TTL-Absolute',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,193,'Ericsson-ViG-TTL-Start-Event',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,193,'Ericsson-ViG-SK-IP-address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,193,'Ericsson-ViG-UA-IP-address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,193,'Ericsson-ViG-SA-IP-address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,193,'Ericsson-ViG-Calling-e164-number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,193,'Ericsson-ViG-Calling-H323Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,193,'Ericsson-ViG-Calling-Email-address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,193,'Ericsson-ViG-Dialled-e164-number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,193,'Ericsson-ViG-Dialled-H323Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,193,'Ericsson-ViG-Dialled-Email-address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,193,'Ericsson-ViG-Routed-e164-number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,193,'Ericsson-ViG-Routed-H323Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,193,'Ericsson-ViG-Routed-Email-address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,193,'Ericsson-ViG-SiteKeeper-name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,193,'Ericsson-ViG-Access-Group-name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,193,'Ericsson-ViG-Access-Agent-name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,193,'Ericsson-ViG-User-agent-group-name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,193,'Ericsson-ViG-User-agent-name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,193,'Ericsson-ViG-Routing-tariff',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,193,'Ericsson-ViG-Re-selection-counter',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,193,'Ericsson-ViG-CPN-digits',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,193,'Ericsson-ViG-CPN-TON',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,193,'Ericsson-ViG-CPN-NP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,193,'Ericsson-ViG-CPN-PI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,193,'Ericsson-ViG-CPN-SI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,193,'Ericsson-ViG-Dialled-num-digits',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,193,'Ericsson-ViG-Dialled-num-TON',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,193,'Ericsson-ViG-Dialled-num-NP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,193,'Ericsson-ViG-Routing-num-digits',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,193,'Ericsson-ViG-Routing-num-TON',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,193,'Ericsson-ViG-Routing-num-NP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,193,'Ericsson-ViG-Redirecting-num-digits',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 77)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(77,193,'Ericsson-ViG-Redirecting-num-TON',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 78)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(78,193,'Ericsson-ViG-Redirecting-num-NP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,193,'Ericsson-ViG-Redirecting-num-PI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,193,'Ericsson-ViG-Redirecting-num-RFD',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,193,'Ericsson-ViG-Time-stamp-UTC',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,193,'Ericsson-ViG-Time-stamp-TZ',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,193,'Ericsson-ViG-Time-stamp-DST',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 84)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(84,193,'Ericsson-ViG-Session-routing-duration',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 85)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(85,193,'Ericsson-ViG-Session-ringing-duration',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 86)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(86,193,'Ericsson-ViG-Access-type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 87)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(87,193,'Ericsson-ViG-Requested-bandwidth',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 88)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(88,193,'Ericsson-ViG-Allowed-bandwidth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 89)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(89,193,'Ericsson-ViG-Media-channel-count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,193,'Ericsson-ViG-Voice-media-rec-forward',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,193,'Ericsson-ViG-Voice-media-rec-backward',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,193,'Ericsson-ViG-Video-media-rec-forward',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,193,'Ericsson-ViG-Video-media-rec-backward',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,193,'Ericsson-ViG-Fax-media-rec-forward',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,193,'Ericsson-ViG-Fax-media-rec-backward',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,193,'Ericsson-ViG-Data-media-rec-forward',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,193,'Ericsson-ViG-Data-media-rec-backward',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,193,'Ericsson-ViG-Charging-Case',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 99)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(99,193,'Ericsson-ViG-Rel-cause-coding-std',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,193,'Ericsson-ViG-Rel-cause-location',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,193,'Ericsson-ViG-Rel-cause-class',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,193,'Ericsson-ViG-Rel-cause-value',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,193,'Ericsson-ViG-Rel-reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,193,'Ericsson-ViG-Internal-Rel-reason-val',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,193,'Ericsson-ViG-Internal-Rel-reason-orig',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,193,'Ericsson-ViG-Service-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,193,'Ericsson-ViG-User-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,193,'Ericsson-ViG-Service-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,193,'Ericsson-ViG-Test-Call-Indicator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,193,'Ericsson-ViG-Emergency-Call-Indicator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,193,'Ericsson-ViG-Calling-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,193,'Ericsson-ViG-Called-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,193,'Ericsson-ViG-Translated-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,193,'Ericsson-ViG-Calling-User-Group-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(115,193,'Ericsson-ViG-Calling-Usr-Sub-Group-ID',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,193,'Ericsson-ViG-Called-Usr-Group-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,193,'Ericsson-ViG-Called-Usr-Sub-Group-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,193,'Ericsson-ViG-Terminal-Type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(119,193,'Ericsson-ViG-Service-Duration',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 120)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(120,193,'Ericsson-ViG-Service-Execution-Result',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,193,'Ericsson-ViG-Service-Exe-Rslt-Desc',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,193,'Ericsson-ViG-Service-Description',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,193,'Ericsson-ViG-Service-Specific-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,193,'Ericsson-ViG-Proxy-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,193,'Ericsson-ViG-Auth-DataRequest',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,193,'Ericsson-ViG-IPT-Time-Stamp',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 193 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,193,'Ericsson-ViG-User-Name-Info',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 4874)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(4874,'ERX')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,4874,'ERX-Virtual-Router-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,4874,'ERX-Address-Pool-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,4874,'ERX-Local-Loopback-Interface',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,4874,'ERX-Primary-Dns',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,4874,'ERX-Secondary-Dns',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,4874,'ERX-Primary-Wins',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,4874,'ERX-Secondary-Wins',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,4874,'ERX-Tunnel-Virtual-Router',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,4874,'ERX-Tunnel-Password',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,4874,'ERX-Ingress-Policy-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,4874,'ERX-Egress-Policy-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,4874,'ERX-Ingress-Statistics',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,4874,'ERX-Egress-Statistics',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,4874,'ERX-Atm-Service-Category',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,4874,'ERX-Atm-PCR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,4874,'ERX-Atm-SCR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,4874,'ERX-Atm-MBS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,4874,'ERX-Cli-Initial-Access-Level',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,4874,'ERX-Cli-Allow-All-VR-Access',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,4874,'ERX-Alternate-Cli-Access-Level',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,4874,'ERX-Alternate-Cli-Vrouter-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,4874,'ERX-Sa-Validate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,4874,'ERX-Igmp-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,4874,'ERX-Pppoe-Description',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,4874,'ERX-Redirect-VR-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,4874,'ERX-Qos-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,4874,'ERX-Pppoe-Max-Sessions',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,4874,'ERX-Pppoe-Url',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,4874,'ERX-Qos-Profile-Interface-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,4874,'ERX-Tunnel-Nas-Port-Method',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,4874,'ERX-Service-Bundle',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,4874,'ERX-Tunnel-Tos',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,4874,'ERX-Tunnel-Maximum-Sessions',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,4874,'ERX-Framed-Ip-Route-Tag',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,4874,'ERX-Dial-Out-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,4874,'ERX-PPP-Username',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,4874,'ERX-PPP-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,4874,'ERX-PPP-Auth-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,4874,'ERX-Minimum-BPS',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,4874,'ERX-Maximum-BPS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,4874,'ERX-Bearer-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,4874,'ERX-Input-Gigapkts',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,4874,'ERX-Output-Gigapkts',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,4874,'ERX-Tunnel-Interface-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,4874,'ERX-IpV6-Virtual-Router',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,4874,'ERX-IpV6-Local-Interface',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,4874,'ERX-Ipv6-Primary-Dns',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,4874,'ERX-Ipv6-Secondary-Dns',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,4874,'Sdx-Service-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,4874,'Sdx-Session-Volume-Quota',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,4874,'Sdx-Tunnel-Disconnect-Cause-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,4874,'ERX-Radius-Client-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,4874,'ERX-Service-Description',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,4874,'ERX-L2tp-Recv-Window-Size',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,4874,'ERX-Dhcp-Options',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,4874,'ERX-Dhcp-Mac-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,4874,'ERX-Dhcp-Gi-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,4874,'ERX-LI-Action',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,4874,'ERX-Med-Dev-Handle',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,4874,'ERX-Med-Ip-Address',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,4874,'ERX-Med-Port-Number',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,4874,'ERX-MLPPP-Bundle-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,4874,'ERX-Interface-Desc',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,4874,'ERX-Tunnel-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,4874,'ERX-Service-Activate',10,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,4874,'ERX-Service-Deactivate',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,4874,'ERX-Service-Volume',11,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,4874,'ERX-Service-Timeout',11,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,4874,'ERX-Service-Statistics',11,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,4874,'ERX-DF-Bit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,4874,'ERX-IGMP-Access-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,4874,'ERX-IGMP-Access-Src-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,4874,'ERX-IGMP-OIF-Map-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,4874,'ERX-MLD-Access-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,4874,'ERX-MLD-Access-Src-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,4874,'ERX-MLD-OIF-Map-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 77)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(77,4874,'ERX-MLD-Version',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 78)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(78,4874,'ERX-IGMP-Version',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,4874,'ERX-IP-Mcast-Adm-Bw-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,4874,'ERX-IPv6-Mcast-Adm-Bw-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,4874,'ERX-Qos-Parameters',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,4874,'ERX-Service-Session',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,4874,'ERX-L2TP-Resynch-Method',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,4874,'ERX-Tunnel-Switch-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,4874,'ERX-L2c-Up-Stream-Data',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,4874,'ERX-L2c-Down-Stream-Data',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,4874,'ERX-Tunnel-Tx-Speed-Method',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,4874,'ERX-IGMP-Query-Interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,4874,'ERX-IGMP-Max-Resp-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,4874,'ERX-IGMP-Immediate-Leave',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,4874,'ERX-MLD-Query-Interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 99)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(99,4874,'ERX-MLD-Max-Resp-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,4874,'ERX-MLD-Immediate-Leave',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,4874,'ERX-IP-Block-Multicast',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,4874,'ERX-IGMP-Explicit-Tracking',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,4874,'ERX-IGMP-No-Tracking-V2-Grps',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,4874,'ERX-MLD-Explicit-Tracking',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,4874,'ERX-MLD-No-Tracking-V1-Grps',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,4874,'ERX-Acc-Loop-Cir-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,4874,'ERX-Acc-Aggr-Cir-Id-Bin',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,4874,'ERX-Acc-Aggr-Cir-Id-Asc',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,4874,'ERX-Act-Data-Rate-Up',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,4874,'ERX-Act-Data-Rate-Dn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(115,4874,'ERX-Min-Data-Rate-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,4874,'ERX-Min-Data-Rate-Dn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,4874,'ERX-Att-Data-Rate-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,4874,'ERX-Att-Data-Rate-Dn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(119,4874,'ERX-Max-Data-Rate-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 120)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(120,4874,'ERX-Max-Data-Rate-Dn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,4874,'ERX-Min-LP-Data-Rate-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,4874,'ERX-Min-LP-Data-Rate-Dn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,4874,'ERX-Max-Interlv-Delay-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,4874,'ERX-Act-Interlv-Delay-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,4874,'ERX-Max-Interlv-Delay-Dn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,4874,'ERX-Act-Interlv-Delay-Dn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,4874,'ERX-DSL-Line-State',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,4874,'ERX-DSL-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,4874,'ERX-IPv6-NdRa-Prefix',9,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,4874,'ERX-Qos-Set-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,4874,'ERX-Service-Acct-Interval',11,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,4874,'ERX-DownStream-Calc-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,4874,'ERX-UpStream-Calc-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,4874,'ERX-Max-Clients-Per-Interface',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4874 AND RadAttributeID = 144)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(144,4874,'ERX-PPP-Monitor-Ingress-Only',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 12 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(12,4874,0,'disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 12 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(12,4874,1,'enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 13 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,4874,0,'disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 13 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,4874,1,'enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 14 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,4874,1,'UBR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 14 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,4874,2,'UBRPCR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 14 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,4874,3,'nrtVBR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 14 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,4874,4,'CBR')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 22 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,4874,0,'disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 22 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,4874,1,'enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 23 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,4874,0,'disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 23 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,4874,1,'enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,1,'IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,2,'ATM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,3,'HDLC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,4,'ETHERNET')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,5,'SERVER-PORT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,6,'ATM-1483')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,7,'FRAME-RELAY')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,8,'MPLS-MINOR')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,9,'CBF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,10,'IP-TUNNEL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,11,'VLAN-SUB')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 29 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,4874,12,'PPPOE-SUB')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 30 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(30,4874,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 30 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(30,4874,1,'CISCO-CLID')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 38 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,4874,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 38 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,4874,1,'PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 38 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,4874,2,'CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 38 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,4874,3,'PAP-CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 38 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38,4874,4,'CHAP-PAP')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 41 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(41,4874,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 41 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(41,4874,1,'Analog')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 41 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(41,4874,2,'Digital')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 58 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(58,4874,0,'off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 58 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(58,4874,1,'on')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 58 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(58,4874,2,'noop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 70 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(70,4874,0,'dont-ignore-df-bit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 70 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(70,4874,1,'ignore-df-bit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 77 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(77,4874,1,'v1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 77 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(77,4874,2,'v2')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 78 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(78,4874,1,'v1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 78 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(78,4874,2,'v2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 78 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(78,4874,3,'v3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 69 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,4874,0,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 69 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,4874,1,'time')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 69 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,4874,2,'time-volume')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 90 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(90,4874,0,'disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 90 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(90,4874,1,'failover')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 90 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(90,4874,2,'silent-failover')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 90 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(90,4874,3,'failover-with-silent-backup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 94 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,4874,1,'static-layer2')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 94 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,4874,2,'dynamic-layer2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 94 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,4874,3,'qos')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 94 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,4874,4,'actual')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 97 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(97,4874,0,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 97 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(97,4874,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 100 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,4874,0,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 100 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,4874,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 101 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,4874,0,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 101 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(101,4874,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 102 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(102,4874,0,'disabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 102 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(102,4874,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 103 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(103,4874,0,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 103 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(103,4874,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 104 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(104,4874,0,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 104 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(104,4874,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 105 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,4874,0,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 105 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,4874,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 127 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(127,4874,1,'SHOWTIME')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 127 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(127,4874,2,'IDLE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 127 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(127,4874,3,'SILENT')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 128 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,4874,1,'ADSL1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 128 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,4874,2,'ADSL2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 128 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,4874,3,'ADSL2PLUS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 128 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,4874,4,'VDSL1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 128 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,4874,5,'VDSL2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 128 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,4874,6,'SDSL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 128 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,4874,7,'UNKNOWN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 144 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,4874,0,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 4874 AND RadAttributeID = 144 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,4874,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 1916)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(1916,'Extreme')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1916 AND RadAttributeID = 201)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(201,1916,'Extreme-CLI-Authorization',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1916 AND RadAttributeID = 202)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(202,1916,'Extreme-Shell-Command',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1916 AND RadAttributeID = 203)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(203,1916,'Extreme-Netlogin-Vlan',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1916 AND RadAttributeID = 204)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(204,1916,'Extreme-Netlogin-Url',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1916 AND RadAttributeID = 205)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(205,1916,'Extreme-Netlogin-Url-Desc',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1916 AND RadAttributeID = 206)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(206,1916,'Extreme-Netlogin-Only',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1916 AND RadAttributeID = 208)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(208,1916,'Extreme-User-Location',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1916 AND RadAttributeID = 209)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(209,1916,'Extreme-Netlogin-Vlan-Tag',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1916 AND RadAttributeID = 201 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(201,1916,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1916 AND RadAttributeID = 201 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(201,1916,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1916 AND RadAttributeID = 206 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(206,1916,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1916 AND RadAttributeID = 206 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(206,1916,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 1991)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(1991,'Foundry')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1991 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,1991,'Foundry-Privilege-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1991 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,1991,'Foundry-Command-String',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1991 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,1991,'Foundry-Command-Exception-Flag',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1991 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,1991,'Foundry-INM-Privilege',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1991 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,1991,'Foundry-Access-List',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1991 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,1991,'Foundry-MAC-Based-Vlan-QoS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1991 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,1991,'Foundry-INM-Role-Aor-List',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,0,'AAA_pri_0')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,1,'AAA_pri_1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,2,'AAA_pri_2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,3,'AAA_pri_3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,4,'AAA_pri_4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,5,'AAA_pri_5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,6,'AAA_pri_6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,7,'AAA_pri_7')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,8,'AAA_pri_8')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,9,'AAA_pri_9')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,10,'AAA_pri_10')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,11,'AAA_pri_11')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,12,'AAA_pri_12')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,13,'AAA_pri_13')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,14,'AAA_pri_14')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 4 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,1991,15,'AAA_pri_15')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 8 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,1991,0,'QoS_priority_0')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 8 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,1991,1,'QoS_priority_1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 8 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,1991,2,'QoS_priority_2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 8 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,1991,3,'QoS_priority_3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 8 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,1991,4,'QoS_priority_4')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 8 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,1991,5,'QoS_priority_5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 8 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,1991,6,'QoS_priority_6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 1991 AND RadAttributeID = 8 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,1991,7,'QoS_priority_7')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 27880)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(27880,'Freeswitch')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,27880,'Freeswitch-AVPair',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,27880,'Freeswitch-CLID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,27880,'Freeswitch-Dialplan',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,27880,'Freeswitch-Src',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,27880,'Freeswitch-Dst',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,27880,'Freeswitch-Src-Channel',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,27880,'Freeswitch-Dst-Channel',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,27880,'Freeswitch-Ani',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,27880,'Freeswitch-Aniii',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,27880,'Freeswitch-Lastapp',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,27880,'Freeswitch-Lastdata',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,27880,'Freeswitch-Disposition',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,27880,'Freeswitch-Hangupcause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,27880,'Freeswitch-Billusec',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,27880,'Freeswitch-AMAFlags',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,27880,'Freeswitch-RDNIS',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,27880,'Freeswitch-Context',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,27880,'Freeswitch-Source',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,27880,'Freeswitch-Callstartdate',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,27880,'Freeswitch-Callanswerdate',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,27880,'Freeswitch-Calltransferdate',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,27880,'Freeswitch-Callenddate',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 27880 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,27880,'Freeswitch-Signalbond',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,0,'None')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,1,'Unallocated-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,2,'No-Route-Transit-Net')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,3,'No-Route-Destination')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,6,'Channel-Unacceptable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,7,'Call-Awarded-Delivery')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,16,'Normal-Clearing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,17,'User-Busy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,18,'No-User-Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,19,'No-Answer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,20,'Subscriber-Absent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,21,'Call-Rejected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,22,'Number-Changed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,23,'Redirecto-To-New-Destination')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,25,'Exchange-Routing-Error')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,27,'Destination-Out-Of-Order')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,28,'Invalid-Number-Format')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,29,'Facility-Rejected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,30,'Response-To-Status-Enquiry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,31,'Normal-Unspecified')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,34,'Normal-Circuit-Congestion')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,38,'Network-Out-Of-Order')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,41,'Normal-Temporary-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,42,'Switch-Congestion')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,43,'Access-Info-Discarded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,44,'Requested-Chan-Unavail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,45,'Pre-Empted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,50,'Facility-Not-Subscribed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,52,'Outgoing-Call-Barred')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,54,'Incoming-Call-Barred')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,57,'Bearercapability-Notauth')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,58,'Bearercapability-Notavail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,63,'Service-Unavailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 65)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,65,'Bearercapability-Notimpl')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 66)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,66,'Chan-Not-Implemented')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 69)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,69,'Facility-Not-Implemented')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 79)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,79,'Service-Not-Implemented')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 81)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,81,'Invalid-Call-Reference')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 88)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,88,'Incompatible-Destination')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 95)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,95,'Invalid-Msg-Unspecified')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 96)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,96,'Mandatory-IE-Missing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 97)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,97,'Message-Type-Nonexist')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 98)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,98,'Wrong-Message')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 99)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,99,'IE-Nonexist')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 100)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,100,'Invalid-IE-Contents')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,101,'Wrong-Call-State')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,102,'Recovery-On-Timer-Expire')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 103)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,103,'Mandatory-IE-Length-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 111)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,111,'Protocol-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 127)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,127,'Interworking')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 142)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,142,'Success')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 487)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,487,'Originator-Cancel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 500)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,500,'Crash')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 501)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,501,'System-Shutdown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 502)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,502,'Lose-Race')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 503)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,503,'Manager-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 600)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,600,'Blind-Transfer')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 601)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,601,'Attended-Transfer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 602)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,602,'Allotted-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 603)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,603,'User-Challenge')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 604)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,604,'Media-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 605)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,605,'Picked-Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 27880 AND RadAttributeID = 13 AND Value = 606)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,27880,606,'User-Not-Registered')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 7221)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(7221,'Handlink')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7221 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,7221,'Traffic-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7221 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,7221,'SMTP-Redirect',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7221 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,7221,'BW-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7221 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,7221,'BW-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7221 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,7221,'Portable-Page-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 11)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(11,'HP')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 11 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,11,'HP-Privelege-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 11 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,11,'HP-Command-String',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 11 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,11,'HP-Command-Exception',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 11 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,11,'HP-Management-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 11 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,11,'HP-Port-Priority-Regeneration-Table',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 11 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,11,'HP-Bandwidth-Max-Ingress',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 11 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,11,'HP-EI-Status',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 11 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,11,'HP-Bandwidth-Max-Egress',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 11 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,11,'HP-Nas-Filter-Rule',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 11 AND RadAttributeID = 255)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(255,11,'HP-Capability-Advert',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 11 AND RadAttributeID = 3 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,11,0,'Permit-List')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 11 AND RadAttributeID = 3 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,11,1,'Deny-List')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 11 AND RadAttributeID = 26 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(26,11,5,'HTTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 11 AND RadAttributeID = 26 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(26,11,6,'HTTPS')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 2011)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(2011,'Huawei')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,2011,'Huawei-Input-Burst-Size',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,2011,'Huawei-Input-Average-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,2011,'Huawei-Input-Peak-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,2011,'Huawei-Output-Burst-Size',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,2011,'Huawei-Output-Average-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,2011,'Huawei-Output-Peak-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,2011,'Huawei-In-Kb-Before-T-Switch',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,2011,'Huawei-Out-Kb-Before-T-Switch',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,2011,'Huawei-In-Pkt-Before-T-Switch',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,2011,'Huawei-Out-Pkt-Before-T-Switch',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,2011,'Huawei-In-Kb-After-T-Switch',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,2011,'Huawei-Out-Kb-After-T-Switch',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,2011,'Huawei-In-Pkt-After-T-Switch',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,2011,'Huawei-Out-Pkt-After-T-Switch',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,2011,'Huawei-Remanent-Volume',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,2011,'Huawei-Tariff-Switch-Interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,2011,'Huawei-ISP-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,2011,'Huawei-Max-Users-Per-Logic-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,2011,'Huawei-Command',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,2011,'Huawei-Priority',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,2011,'Huawei-Control-Identifier',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,2011,'Huawei-Result-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,2011,'Huawei-Connect-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,2011,'Huawei-PortalURL',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,2011,'Huawei-FTP-Directory',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,2011,'Huawei-Exec-Privilege',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,2011,'Huawei-IP-Address',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,2011,'Huawei-Qos-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,2011,'Huawei-Destnation-IP-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,2011,'Huawei-Destnation-Volume',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,2011,'Huawei-Startup-Stamp',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,2011,'Huawei-IPHost-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,2011,'Huawei-PPP-NCP-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,2011,'Huawei-VSI-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,2011,'Huawei-Subnet-Mask',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,2011,'Huawei-Gateway-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,2011,'Huawei-Primary-WINS',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,2011,'Huawei-Secondary-WINS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 77)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(77,2011,'Huawei-Input-Peak-Burst-Size',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 78)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(78,2011,'Huawei-Output-Peak-Burst-Size',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,2011,'Huawei-Reduced-CIR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,2011,'Huawei-Tunnel-Session-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,2011,'Huawei-Zone-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,2011,'Huawei-Data-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,2011,'Huawei-Access-Service',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 84)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(84,2011,'Huawei-HW-Accounting-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 85)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(85,2011,'Huawei-HW-Portal-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 86)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(86,2011,'Huawei-HW-DPI-Policy-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 87)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(87,2011,'Huawei-Policy-Route',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 88)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(88,2011,'Huawei-Framed-Pool',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 89)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(89,2011,'Huawei-HW-L2TP-Terminate-Cause',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,2011,'Huawei-HW-Multi-Account-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,2011,'Huawei-Queue-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,2011,'Huawei-Layer4-Session-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,2011,'Huawei-Multicast-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,2011,'Huawei-VPN-Instance',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,2011,'Huawei-Policy-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,2011,'Huawei-Tunnel-Group-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,2011,'Huawei-Multicast-Source-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,2011,'Huawei-Multicast-Receive-Group',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 99)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(99,2011,'Huawei-User-Multicast-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,2011,'Huawei-Reduced-PIR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,2011,'Huawei-LI-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,2011,'Huawei-LI-Md-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,2011,'Huawei-LI-Md-Port',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,2011,'Huawei-LI-Md-VpnInstance',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,2011,'Huawei-Service-Chg-Cmd',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,2011,'Huawei-Acct-Packet-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,2011,'Huawei-Call-Reference',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,2011,'Huawei-PSTN-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,2011,'Huawei-Voip-Service-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,2011,'Huawei-Acct-Connection-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,2011,'Huawei-Error-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,2011,'Huawei-Remain-Monney',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,2011,'Huawei-Org-GK-ipaddr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,2011,'Huawei-Org-GW-ipaddr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,2011,'Huawei-Dst-GK-ipaddr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,2011,'Huawei-Dst-GW-ipaddr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,2011,'Huawei-Access-Num',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,2011,'Huawei-Remain-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,2011,'Huawei-Codec-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,2011,'Huawei-Transfer-Num',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,2011,'Huawei-New-User-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 134)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(134,2011,'Huawei-Transfer-Station-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 135)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(135,2011,'Huawei-Primary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 136)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(136,2011,'Huawei-Secondary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 137)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(137,2011,'Huawei-ONLY-Account-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 138)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(138,2011,'Huawei-Domain-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 139)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(139,2011,'Huawei-ANCP-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,2011,'Huawei-HTTP-Redirect-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,2011,'Huawei-Loopback-Address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,2011,'Huawei-QoS-Profile-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,2011,'Huawei-HW-Max-List-Num',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 254)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(254,2011,'Huawei-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2011 AND RadAttributeID = 255)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(255,2011,'Huawei-Product-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2011 AND RadAttributeID = 20 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,2011,1,'Trigger-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2011 AND RadAttributeID = 20 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,2011,2,'Terminate-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2011 AND RadAttributeID = 20 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,2011,3,'SetPolicy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2011 AND RadAttributeID = 20 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,2011,4,'Result')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 24023)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(24023,'IEA-Software')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,24023,'AM-Interrupt-HTMLFile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,24023,'AM-Interrupt-Interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,24023,'AM-Interrupt-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,24023,'AM-Status-HTMLFile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,24023,'AM-HTTP-Proxy-Port',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,24023,'AM-ACK-HTMLFile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,24023,'AM-NAK-HTMLFile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,24023,'AM-Bandwidth-Pool',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,24023,'AM-Bandwidth-Pool-Max-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,24023,'AM-Bandwidth-Pool-Max-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,24023,'AM-Mirroring',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24023 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,24023,'AM-Disconnect-Access',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24023 AND RadAttributeID = 11 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(11,24023,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24023 AND RadAttributeID = 11 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(11,24023,1,'Full-Packet-Local-PCAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24023 AND RadAttributeID = 12 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(12,24023,0,'Allow-All')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24023 AND RadAttributeID = 12 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(12,24023,1,'Deny-RADIUS')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 5586)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(5586,'IPWireless')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,5586,'Tier-Of-Service',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,5586,'Roaming-Indication',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,5586,'ISP-LNS-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,5586,'User-Trace-Indicator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,5586,'User-Name-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,5586,'L2TP-Tunnel-If-Index',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,5586,'IPW-Acct-Terminate-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,5586,'ISP-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,5586,'Node-B-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,5586,'IMEI-Status',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,5586,'SV-Status',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,5586,'L2TP-Tunnel-Local-Sid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,5586,'UMTS-Authentication-Vector',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,5586,'UMTS-Resynchronisation-Token',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,5586,'Additional-Contexts-Allowed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,5586,'Additional-Context-Accounting',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,5586,'UE-Software-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,5586,'UE-Hardware-Identity',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,5586,'GSM-Authentication-Vector',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,5586,'Serving-INC-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,5586,'UE-Application-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,5586,'Additional-Context-Configuration',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,5586,'UEInfo-RSCP',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,5586,'UEInfo-ISCP',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,5586,'UEInfo-RSCP-ISCP-Margin',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,5586,'L2TP-Tunnel-Remote-Tid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,5586,'L2TP-Tunnel-Remote-Sid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 5586 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,5586,'Test-Account-Indicator',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5586,1,'Business')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5586,2,'Gold')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 1 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5586,3,'Silver')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 1 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5586,4,'Bronze')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 1 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5586,5,'Compatibility')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 1 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,5586,6,'VoIP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 2 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,5586,1,'Home-Network-Subscriber')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 2 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,5586,2,'Roaming-Subscriber')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 4 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,5586,1,'Do-Not-Trace')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 4 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,5586,2,'Trace')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 5 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(5,5586,1,'IMSI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 5 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(5,5586,2,'IMEI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,1,'Normal-User-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,2,'UE-Fault')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,3,'Handover')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,4,'UE-Power-Down-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,5,'UE-Lost-Coverage')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,6,'No-Allocation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,7,'Layer-2-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,8,'Hardware-Unsupported')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,9,'Software-Image-Cut')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,11,'Manager-Initiated-Forced-User-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,12,'Forced-User-Disconnect-Due-To-Overload')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,13,'Forced-User-Disconnect-Due-To-Sys-Init')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,14,'Max-Reloc-Lu-Cxns-Exceeded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,15,'NodeB-Decommission')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,21,'L2TP-Session-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,22,'L2TP-Session-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,23,'Network-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,31,'SM-Timer-Expiry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,32,'UE_Unetectable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,33,'Air-Interface-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,34,'Requested-Sec-Alogs-Unsupported')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,35,'Change-Of-Security-Setup-Not-Supported')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,36,'Unknown-Security-Mode-Setup-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,37,'Lub-L1-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,38,'Lub-Out-Of-Sync')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,39,'Invalid-Cell-ID')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,40,'Downlink-TFCS-Unavailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,41,'Errored-Reloc-Related-RANAP-Signalling')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,42,'Test-UE-Not-Allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,51,'Network-USIM-Authentication-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,52,'USIM-Network-Autherntication-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,53,'USIM-Resynchronisation-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,60,'Unexpected-Lu-Release')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 61)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,61,'GTP-Path-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 79)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,79,'Administrative-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 80)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,80,'Normal-LCP-Terminate')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 81)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,81,'LCP-Echo-Request-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 82)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,82,'Unexpected-Endpoint-Discriminator')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 83)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,83,'Unexpected-MRRU')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 84)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,84,'Unexpected-SSN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 85)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,85,'LCP-Renegotiated-Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 86)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,86,'Compulsory-Encryption-Refused')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 87)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,87,'LCP-FSM-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 88)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,88,'LCP-Packets-Unrecognisable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 89)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,89,'LCP-Magic-Number-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 90)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,90,'Compulsory-Call-Back-Refused')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 91)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,91,'Auth-FSM-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 92)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,92,'Unexpected-Authenticated-name')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 93)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,93,'PPP-Authentication-Protocol-Unacceptable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 94)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,94,'PPP-Authentication-Failure')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 95)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,95,'NCP-FSM-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 96)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,96,'NCP-No-Available')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 97)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,97,'NCP-Address-Convergence-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 98)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,98,'NCP-Address-Not-Permitted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 8 AND Value = 99)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8,5586,99,'Unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 11 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(11,5586,1,'IMEI-OK')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 11 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(11,5586,2,'IMEI-Barred')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 12 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(12,5586,1,'SV-OK')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 12 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(12,5586,2,'SV-Out-Of-Date')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 29 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,5586,1,'Uplink-Interference')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 5586 AND RadAttributeID = 29 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,5586,2,'Downlink-Interference')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 1195)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(1195,'ITK')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,1195,'ITK-Auth-Serv-IP',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,1195,'ITK-Auth-Serv-Prot',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,1195,'ITK-Provider-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,1195,'ITK-Usergroup',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,1195,'ITK-Banner',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,1195,'ITK-Username-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,1195,'ITK-Password-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,1195,'ITK-Welcome-Message',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,1195,'ITK-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,1195,'ITK-IP-Pool',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,1195,'ITK-Tunnel-IP',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,1195,'ITK-Tunnel-Prot',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,1195,'ITK-Acct-Serv-IP',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,1195,'ITK-Acct-Serv-Prot',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,1195,'ITK-Filter-Rule',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(115,1195,'ITK-Channel-Binding',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,1195,'ITK-Start-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,1195,'ITK-NAS-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,1195,'ITK-ISDN-Prot',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(119,1195,'ITK-PPP-Auth-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 120)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(120,1195,'ITK-Dialout-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,1195,'ITK-Ftp-Auth-IP',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,1195,'ITK-Users-Default-Entry',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,1195,'ITK-Users-Default-Pw',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,1195,'ITK-Auth-Req-Type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,1195,'ITK-Modem-Pool-Id',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,1195,'ITK-Modem-Init-String',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,1195,'ITK-PPP-Client-Server-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,1195,'ITK-PPP-Compression-Prot',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,1195,'ITK-Username',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,1195,'ITK-Dest-No',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1195 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,1195,'ITK-DDI',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 2636)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(2636,'Juniper')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2636 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,2636,'Juniper-Local-User-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2636 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,2636,'Juniper-Allow-Commands',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2636 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,2636,'Juniper-Deny-Commands',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2636 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,2636,'Juniper-Allow-Configuration',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2636 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,2636,'Juniper-Deny-Configuration',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2636 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,2636,'Juniper-Interactive-Command',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2636 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,2636,'Juniper-Configuration-Change',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2636 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,2636,'Juniper-User-Permissions',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 762)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(762,'KarlNet')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 762 AND RadAttributeID = 151)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(151,762,'KarlNet-TurboCell-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 762 AND RadAttributeID = 152)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(152,762,'KarlNet-TurboCell-TxRate',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,0,'TxRate-Local')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,8,'TxRate-MaxSpeed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,11,'TxRate-11k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,12,'TxRate-12k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,13,'TxRate-13k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,14,'TxRate-14k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,15,'TxRate-15k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,16,'TxRate-16k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,17,'TxRate-17k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,18,'TxRate-18k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,19,'TxRate-19k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,20,'TxRate-20k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,21,'TxRate-21k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,22,'TxRate-22k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,23,'TxRate-23k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,24,'TxRate-24k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,25,'TxRate-25k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,26,'TxRate-26k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,27,'TxRate-27k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,28,'TxRate-28k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,29,'TxRate-29k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,30,'TxRate-30k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,31,'TxRate-31k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,32,'TxRate-32k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,33,'TxRate-33k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,34,'TxRate-34k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,35,'TxRate-35k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,36,'TxRate-36k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,37,'TxRate-37k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,38,'TxRate-38k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,39,'TxRate-39k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,40,'TxRate-40k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,41,'TxRate-41k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,42,'TxRate-42k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,43,'TxRate-43k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,44,'TxRate-44k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,45,'TxRate-45k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,46,'TxRate-46k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,47,'TxRate-47k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,48,'TxRate-48k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,49,'TxRate-49k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,50,'TxRate-50k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,51,'TxRate-51k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,52,'TxRate-52k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,53,'TxRate-53k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,54,'TxRate-54k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,55,'TxRate-55k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 56)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,56,'TxRate-56k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,57,'TxRate-57k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,58,'TxRate-58k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 59)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,59,'TxRate-59k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,60,'TxRate-60k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 61)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,61,'TxRate-61k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 62)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,62,'TxRate-62k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,63,'TxRate-63k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,64,'TxRate-64k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 65)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,65,'TxRate-65k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 66)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,66,'TxRate-66k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 67)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,67,'TxRate-67k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 68)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,68,'TxRate-68k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 69)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,69,'TxRate-69k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 70)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,70,'TxRate-70k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 71)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,71,'TxRate-71k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 72)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,72,'TxRate-72k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 73)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,73,'TxRate-73k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 74)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,74,'TxRate-74k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 75)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,75,'TxRate-75k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 76)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,76,'TxRate-76k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 77)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,77,'TxRate-77k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 78)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,78,'TxRate-78k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 79)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,79,'TxRate-79k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 80)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,80,'TxRate-80k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 81)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,81,'TxRate-81k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 82)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,82,'TxRate-82k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 83)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,83,'TxRate-83k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 84)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,84,'TxRate-84k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 85)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,85,'TxRate-85k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 86)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,86,'TxRate-86k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 87)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,87,'TxRate-87k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 88)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,88,'TxRate-88k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 89)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,89,'TxRate-89k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 90)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,90,'TxRate-90k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 91)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,91,'TxRate-91k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 92)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,92,'TxRate-92k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 93)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,93,'TxRate-93k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 94)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,94,'TxRate-94k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 95)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,95,'TxRate-95k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 96)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,96,'TxRate-96k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 97)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,97,'TxRate-97k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 98)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,98,'TxRate-98k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 99)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,99,'TxRate-99k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 100)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,100,'TxRate-100k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,101,'TxRate-101k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,102,'TxRate-102k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 103)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,103,'TxRate-103k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 104)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,104,'TxRate-104k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 105)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,105,'TxRate-105k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 106)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,106,'TxRate-106k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 107)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,107,'TxRate-107k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 108)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,108,'TxRate-108k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 109)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,109,'TxRate-109k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 110)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,110,'TxRate-110k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 111)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,111,'TxRate-111k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 112)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,112,'TxRate-112k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 113)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,113,'TxRate-113k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 114)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,114,'TxRate-114k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 115)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,115,'TxRate-115k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 116)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,116,'TxRate-116k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 117)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,117,'TxRate-117k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 118)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,118,'TxRate-118k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 119)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,119,'TxRate-119k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 120)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,120,'TxRate-120k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 121)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,121,'TxRate-121k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 122)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,122,'TxRate-122k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 123)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,123,'TxRate-123k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 124)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,124,'TxRate-124k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 125)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,125,'TxRate-125k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 126)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,126,'TxRate-126k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 127)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,127,'TxRate-127k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 128)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,128,'TxRate-128k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 129)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,129,'TxRate-129k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 130)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,130,'TxRate-130k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 131)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,131,'TxRate-131k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 132)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,132,'TxRate-132k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 133)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,133,'TxRate-133k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 134)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,134,'TxRate-134k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 135)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,135,'TxRate-135k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 136)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,136,'TxRate-136k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 137)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,137,'TxRate-137k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 138)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,138,'TxRate-138k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 139)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,139,'TxRate-139k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 140)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,140,'TxRate-140k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 141)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,141,'TxRate-141k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 142)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,142,'TxRate-142k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 143)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,143,'TxRate-143k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 144)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,144,'TxRate-144k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 145)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,145,'TxRate-145k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 146)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,146,'TxRate-146k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 147)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,147,'TxRate-147k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 148)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,148,'TxRate-148k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 149)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,149,'TxRate-149k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 150)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,150,'TxRate-150k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 151)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,151,'TxRate-151k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 152)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,152,'TxRate-152k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 153)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,153,'TxRate-153k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 154)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,154,'TxRate-154k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 155)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,155,'TxRate-155k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 156)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,156,'TxRate-156k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 157)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,157,'TxRate-157k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 158)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,158,'TxRate-158k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 159)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,159,'TxRate-159k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 160)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,160,'TxRate-160k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 161)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,161,'TxRate-161k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 162)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,162,'TxRate-162k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 163)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,163,'TxRate-163k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 164)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,164,'TxRate-164k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 165)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,165,'TxRate-165k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 166)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,166,'TxRate-166k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 167)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,167,'TxRate-167k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 168)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,168,'TxRate-168k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 169)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,169,'TxRate-169k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 170)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,170,'TxRate-170k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 171)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,171,'TxRate-171k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 172)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,172,'TxRate-172k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 173)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,173,'TxRate-173k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 174)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,174,'TxRate-174k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 175)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,175,'TxRate-175k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 176)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,176,'TxRate-176k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 177)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,177,'TxRate-177k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 178)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,178,'TxRate-178k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 179)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,179,'TxRate-179k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 180)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,180,'TxRate-180k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 181)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,181,'TxRate-181k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 182)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,182,'TxRate-182k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 183)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,183,'TxRate-183k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 184)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,184,'TxRate-184k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 185)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,185,'TxRate-185k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 186)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,186,'TxRate-186k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 187)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,187,'TxRate-187k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 188)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,188,'TxRate-188k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 189)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,189,'TxRate-189k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 190)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,190,'TxRate-190k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 191)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,191,'TxRate-191k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 192)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,192,'TxRate-192k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 193)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,193,'TxRate-193k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 194)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,194,'TxRate-194k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 195)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,195,'TxRate-195k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 196)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,196,'TxRate-196k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 197)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,197,'TxRate-197k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 198)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,198,'TxRate-198k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 199)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,199,'TxRate-199k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 200)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,200,'TxRate-200k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 201)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,201,'TxRate-201k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 202)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,202,'TxRate-202k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 203)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,203,'TxRate-203k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 204)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,204,'TxRate-204k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 205)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,205,'TxRate-205k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 206)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,206,'TxRate-206k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 207)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,207,'TxRate-207k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 208)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,208,'TxRate-208k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 209)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,209,'TxRate-209k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 210)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,210,'TxRate-210k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 211)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,211,'TxRate-211k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 212)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,212,'TxRate-212k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 213)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,213,'TxRate-213k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 214)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,214,'TxRate-214k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 215)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,215,'TxRate-215k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 216)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,216,'TxRate-216k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 217)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,217,'TxRate-217k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 218)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,218,'TxRate-218k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 219)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,219,'TxRate-219k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 220)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,220,'TxRate-220k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 221)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,221,'TxRate-221k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 222)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,222,'TxRate-222k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 223)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,223,'TxRate-223k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 224)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,224,'TxRate-224k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 225)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,225,'TxRate-225k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 226)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,226,'TxRate-226k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 227)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,227,'TxRate-227k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 228)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,228,'TxRate-228k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 229)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,229,'TxRate-229k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 230)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,230,'TxRate-230k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 231)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,231,'TxRate-231k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 232)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,232,'TxRate-232k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 233)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,233,'TxRate-233k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 234)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,234,'TxRate-234k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 235)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,235,'TxRate-235k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 236)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,236,'TxRate-236k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 237)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,237,'TxRate-237k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 238)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,238,'TxRate-238k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 239)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,239,'TxRate-239k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 240)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,240,'TxRate-240k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 241)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,241,'TxRate-241k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 242)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,242,'TxRate-242k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 243)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,243,'TxRate-243k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 244)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,244,'TxRate-244k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 245)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,245,'TxRate-245k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 246)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,246,'TxRate-246k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 247)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,247,'TxRate-247k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 248)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,248,'TxRate-248k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 249)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,249,'TxRate-249k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 250)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,250,'TxRate-250k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 251)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,251,'TxRate-251k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 252)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,252,'TxRate-252k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 253)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,253,'TxRate-253k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 254)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,254,'TxRate-254k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 255)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,255,'TxRate-255k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 256)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,256,'TxRate-256k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 257)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,257,'TxRate-257k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 258)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,258,'TxRate-258k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 259)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,259,'TxRate-259k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 260)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,260,'TxRate-260k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 261)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,261,'TxRate-261k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 262)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,262,'TxRate-262k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 263)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,263,'TxRate-263k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 264)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,264,'TxRate-264k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 265)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,265,'TxRate-265k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 266)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,266,'TxRate-266k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 267)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,267,'TxRate-267k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 268)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,268,'TxRate-268k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 269)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,269,'TxRate-269k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 270)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,270,'TxRate-270k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 271)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,271,'TxRate-271k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 272)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,272,'TxRate-272k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 273)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,273,'TxRate-273k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 274)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,274,'TxRate-274k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 275)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,275,'TxRate-275k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 276)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,276,'TxRate-276k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 277)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,277,'TxRate-277k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 278)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,278,'TxRate-278k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 279)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,279,'TxRate-279k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 280)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,280,'TxRate-280k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 281)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,281,'TxRate-281k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 282)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,282,'TxRate-282k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 283)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,283,'TxRate-283k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 284)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,284,'TxRate-284k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 285)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,285,'TxRate-285k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 286)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,286,'TxRate-286k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 287)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,287,'TxRate-287k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 288)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,288,'TxRate-288k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 289)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,289,'TxRate-289k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 290)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,290,'TxRate-290k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 291)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,291,'TxRate-291k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 292)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,292,'TxRate-292k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 293)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,293,'TxRate-293k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 294)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,294,'TxRate-294k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 295)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,295,'TxRate-295k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 296)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,296,'TxRate-296k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 297)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,297,'TxRate-297k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 298)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,298,'TxRate-298k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 299)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,299,'TxRate-299k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 300)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,300,'TxRate-300k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 301)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,301,'TxRate-301k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 302)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,302,'TxRate-302k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 303)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,303,'TxRate-303k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 304)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,304,'TxRate-304k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 305)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,305,'TxRate-305k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 306)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,306,'TxRate-306k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 307)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,307,'TxRate-307k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 308)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,308,'TxRate-308k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 309)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,309,'TxRate-309k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 310)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,310,'TxRate-310k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 311)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,311,'TxRate-311k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 312)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,312,'TxRate-312k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 313)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,313,'TxRate-313k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 314)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,314,'TxRate-314k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 315)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,315,'TxRate-315k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 316)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,316,'TxRate-316k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 317)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,317,'TxRate-317k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 318)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,318,'TxRate-318k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 319)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,319,'TxRate-319k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 320)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,320,'TxRate-320k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 321)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,321,'TxRate-321k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 322)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,322,'TxRate-322k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 323)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,323,'TxRate-323k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 324)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,324,'TxRate-324k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 325)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,325,'TxRate-325k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 326)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,326,'TxRate-326k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 327)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,327,'TxRate-327k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 328)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,328,'TxRate-328k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 329)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,329,'TxRate-329k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 330)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,330,'TxRate-330k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 331)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,331,'TxRate-331k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 332)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,332,'TxRate-332k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 333)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,333,'TxRate-333k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 334)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,334,'TxRate-334k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 335)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,335,'TxRate-335k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 336)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,336,'TxRate-336k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 337)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,337,'TxRate-337k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 338)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,338,'TxRate-338k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 339)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,339,'TxRate-339k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 340)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,340,'TxRate-340k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 341)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,341,'TxRate-341k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 342)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,342,'TxRate-342k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 343)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,343,'TxRate-343k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 344)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,344,'TxRate-344k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 345)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,345,'TxRate-345k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 346)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,346,'TxRate-346k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 347)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,347,'TxRate-347k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 348)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,348,'TxRate-348k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 349)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,349,'TxRate-349k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 350)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,350,'TxRate-350k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 351)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,351,'TxRate-351k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 352)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,352,'TxRate-352k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 353)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,353,'TxRate-353k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 354)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,354,'TxRate-354k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 355)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,355,'TxRate-355k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 356)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,356,'TxRate-356k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 357)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,357,'TxRate-357k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 358)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,358,'TxRate-358k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 359)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,359,'TxRate-359k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 360)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,360,'TxRate-360k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 361)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,361,'TxRate-361k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 362)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,362,'TxRate-362k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 363)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,363,'TxRate-363k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 364)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,364,'TxRate-364k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 365)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,365,'TxRate-365k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 366)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,366,'TxRate-366k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 367)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,367,'TxRate-367k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 368)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,368,'TxRate-368k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 369)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,369,'TxRate-369k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 370)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,370,'TxRate-370k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 371)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,371,'TxRate-371k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 372)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,372,'TxRate-372k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 373)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,373,'TxRate-373k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 374)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,374,'TxRate-374k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 375)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,375,'TxRate-375k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 376)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,376,'TxRate-376k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 377)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,377,'TxRate-377k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 378)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,378,'TxRate-378k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 379)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,379,'TxRate-379k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 380)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,380,'TxRate-380k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 381)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,381,'TxRate-381k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 382)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,382,'TxRate-382k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 383)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,383,'TxRate-383k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 384)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,384,'TxRate-384k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 385)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,385,'TxRate-385k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 386)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,386,'TxRate-386k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 387)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,387,'TxRate-387k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 388)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,388,'TxRate-388k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 389)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,389,'TxRate-389k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 390)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,390,'TxRate-390k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 391)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,391,'TxRate-391k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 392)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,392,'TxRate-392k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 393)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,393,'TxRate-393k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 394)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,394,'TxRate-394k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 395)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,395,'TxRate-395k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 396)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,396,'TxRate-396k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 397)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,397,'TxRate-397k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 398)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,398,'TxRate-398k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 399)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,399,'TxRate-399k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 400)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,400,'TxRate-400k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 401)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,401,'TxRate-401k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 402)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,402,'TxRate-402k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 403)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,403,'TxRate-403k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 404)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,404,'TxRate-404k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 405)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,405,'TxRate-405k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 406)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,406,'TxRate-406k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 407)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,407,'TxRate-407k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 408)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,408,'TxRate-408k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 409)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,409,'TxRate-409k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 410)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,410,'TxRate-410k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 411)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,411,'TxRate-411k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 412)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,412,'TxRate-412k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 413)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,413,'TxRate-413k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 414)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,414,'TxRate-414k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 415)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,415,'TxRate-415k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 416)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,416,'TxRate-416k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 417)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,417,'TxRate-417k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 418)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,418,'TxRate-418k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 419)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,419,'TxRate-419k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 420)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,420,'TxRate-420k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 421)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,421,'TxRate-421k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 422)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,422,'TxRate-422k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 423)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,423,'TxRate-423k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 424)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,424,'TxRate-424k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 425)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,425,'TxRate-425k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 426)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,426,'TxRate-426k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 427)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,427,'TxRate-427k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 428)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,428,'TxRate-428k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 429)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,429,'TxRate-429k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 430)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,430,'TxRate-430k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 431)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,431,'TxRate-431k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 432)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,432,'TxRate-432k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 433)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,433,'TxRate-433k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 434)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,434,'TxRate-434k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 435)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,435,'TxRate-435k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 436)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,436,'TxRate-436k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 437)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,437,'TxRate-437k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 438)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,438,'TxRate-438k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 439)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,439,'TxRate-439k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 440)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,440,'TxRate-440k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 441)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,441,'TxRate-441k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 442)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,442,'TxRate-442k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 443)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,443,'TxRate-443k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 444)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,444,'TxRate-444k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 445)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,445,'TxRate-445k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 446)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,446,'TxRate-446k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 447)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,447,'TxRate-447k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 448)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,448,'TxRate-448k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 449)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,449,'TxRate-449k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 450)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,450,'TxRate-450k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 451)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,451,'TxRate-451k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 452)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,452,'TxRate-452k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 453)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,453,'TxRate-453k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 454)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,454,'TxRate-454k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 455)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,455,'TxRate-455k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 456)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,456,'TxRate-456k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 457)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,457,'TxRate-457k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 458)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,458,'TxRate-458k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 459)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,459,'TxRate-459k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 460)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,460,'TxRate-460k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 461)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,461,'TxRate-461k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 462)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,462,'TxRate-462k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 463)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,463,'TxRate-463k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 464)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,464,'TxRate-464k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 465)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,465,'TxRate-465k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 466)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,466,'TxRate-466k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 467)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,467,'TxRate-467k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 468)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,468,'TxRate-468k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 469)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,469,'TxRate-469k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 470)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,470,'TxRate-470k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 471)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,471,'TxRate-471k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 472)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,472,'TxRate-472k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 473)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,473,'TxRate-473k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 474)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,474,'TxRate-474k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 475)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,475,'TxRate-475k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 476)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,476,'TxRate-476k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 477)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,477,'TxRate-477k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 478)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,478,'TxRate-478k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 479)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,479,'TxRate-479k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 480)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,480,'TxRate-480k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 481)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,481,'TxRate-481k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 482)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,482,'TxRate-482k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 483)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,483,'TxRate-483k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 484)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,484,'TxRate-484k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 485)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,485,'TxRate-485k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 486)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,486,'TxRate-486k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 487)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,487,'TxRate-487k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 488)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,488,'TxRate-488k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 489)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,489,'TxRate-489k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 490)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,490,'TxRate-490k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 491)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,491,'TxRate-491k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 492)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,492,'TxRate-492k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 493)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,493,'TxRate-493k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 494)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,494,'TxRate-494k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 495)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,495,'TxRate-495k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 496)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,496,'TxRate-496k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 497)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,497,'TxRate-497k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 498)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,498,'TxRate-498k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 499)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,499,'TxRate-499k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 500)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,500,'TxRate-500k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 501)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,501,'TxRate-501k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 502)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,502,'TxRate-502k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 503)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,503,'TxRate-503k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 504)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,504,'TxRate-504k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 505)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,505,'TxRate-505k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 506)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,506,'TxRate-506k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 507)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,507,'TxRate-507k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 508)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,508,'TxRate-508k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 509)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,509,'TxRate-509k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 510)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,510,'TxRate-510k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 511)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,511,'TxRate-511k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 512)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,512,'TxRate-512k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 513)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,513,'TxRate-513k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 514)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,514,'TxRate-514k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 515)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,515,'TxRate-515k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 516)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,516,'TxRate-516k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 517)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,517,'TxRate-517k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 518)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,518,'TxRate-518k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 519)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,519,'TxRate-519k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 520)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,520,'TxRate-520k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 521)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,521,'TxRate-521k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 522)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,522,'TxRate-522k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 523)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,523,'TxRate-523k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 524)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,524,'TxRate-524k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 525)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,525,'TxRate-525k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 526)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,526,'TxRate-526k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 527)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,527,'TxRate-527k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 528)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,528,'TxRate-528k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 529)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,529,'TxRate-529k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 530)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,530,'TxRate-530k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 531)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,531,'TxRate-531k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 532)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,532,'TxRate-532k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 533)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,533,'TxRate-533k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 534)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,534,'TxRate-534k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 535)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,535,'TxRate-535k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 536)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,536,'TxRate-536k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 537)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,537,'TxRate-537k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 538)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,538,'TxRate-538k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 539)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,539,'TxRate-539k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 540)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,540,'TxRate-540k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 541)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,541,'TxRate-541k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 542)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,542,'TxRate-542k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 543)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,543,'TxRate-543k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 544)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,544,'TxRate-544k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 545)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,545,'TxRate-545k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 546)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,546,'TxRate-546k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 547)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,547,'TxRate-547k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 548)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,548,'TxRate-548k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 549)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,549,'TxRate-549k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 550)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,550,'TxRate-550k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 551)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,551,'TxRate-551k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 552)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,552,'TxRate-552k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 553)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,553,'TxRate-553k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 554)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,554,'TxRate-554k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 555)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,555,'TxRate-555k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 556)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,556,'TxRate-556k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 557)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,557,'TxRate-557k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 558)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,558,'TxRate-558k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 559)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,559,'TxRate-559k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 560)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,560,'TxRate-560k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 561)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,561,'TxRate-561k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 562)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,562,'TxRate-562k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 563)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,563,'TxRate-563k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 564)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,564,'TxRate-564k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 565)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,565,'TxRate-565k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 566)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,566,'TxRate-566k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 567)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,567,'TxRate-567k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 568)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,568,'TxRate-568k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 569)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,569,'TxRate-569k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 570)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,570,'TxRate-570k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 571)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,571,'TxRate-571k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 572)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,572,'TxRate-572k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 573)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,573,'TxRate-573k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 574)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,574,'TxRate-574k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 575)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,575,'TxRate-575k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 576)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,576,'TxRate-576k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 577)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,577,'TxRate-577k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 578)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,578,'TxRate-578k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 579)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,579,'TxRate-579k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 580)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,580,'TxRate-580k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 581)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,581,'TxRate-581k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 582)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,582,'TxRate-582k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 583)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,583,'TxRate-583k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 584)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,584,'TxRate-584k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 585)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,585,'TxRate-585k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 586)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,586,'TxRate-586k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 587)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,587,'TxRate-587k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 588)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,588,'TxRate-588k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 589)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,589,'TxRate-589k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 590)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,590,'TxRate-590k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 591)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,591,'TxRate-591k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 592)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,592,'TxRate-592k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 593)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,593,'TxRate-593k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 594)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,594,'TxRate-594k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 595)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,595,'TxRate-595k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 596)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,596,'TxRate-596k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 597)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,597,'TxRate-597k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 598)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,598,'TxRate-598k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 599)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,599,'TxRate-599k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 600)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,600,'TxRate-600k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 601)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,601,'TxRate-601k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 602)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,602,'TxRate-602k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 603)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,603,'TxRate-603k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 604)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,604,'TxRate-604k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 605)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,605,'TxRate-605k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 606)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,606,'TxRate-606k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 607)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,607,'TxRate-607k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 608)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,608,'TxRate-608k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 609)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,609,'TxRate-609k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 610)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,610,'TxRate-610k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 611)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,611,'TxRate-611k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 612)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,612,'TxRate-612k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 613)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,613,'TxRate-613k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 614)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,614,'TxRate-614k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 615)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,615,'TxRate-615k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 616)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,616,'TxRate-616k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 617)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,617,'TxRate-617k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 618)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,618,'TxRate-618k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 619)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,619,'TxRate-619k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 620)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,620,'TxRate-620k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 621)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,621,'TxRate-621k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 622)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,622,'TxRate-622k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 623)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,623,'TxRate-623k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 624)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,624,'TxRate-624k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 625)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,625,'TxRate-625k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 626)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,626,'TxRate-626k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 627)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,627,'TxRate-627k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 628)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,628,'TxRate-628k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 629)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,629,'TxRate-629k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 630)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,630,'TxRate-630k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 631)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,631,'TxRate-631k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 632)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,632,'TxRate-632k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 633)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,633,'TxRate-633k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 634)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,634,'TxRate-634k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 635)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,635,'TxRate-635k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 636)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,636,'TxRate-636k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 637)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,637,'TxRate-637k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 638)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,638,'TxRate-638k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 639)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,639,'TxRate-639k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 640)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,640,'TxRate-640k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 641)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,641,'TxRate-641k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 642)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,642,'TxRate-642k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 643)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,643,'TxRate-643k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 644)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,644,'TxRate-644k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 645)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,645,'TxRate-645k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 646)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,646,'TxRate-646k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 647)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,647,'TxRate-647k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 648)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,648,'TxRate-648k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 649)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,649,'TxRate-649k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 650)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,650,'TxRate-650k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 651)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,651,'TxRate-651k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 652)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,652,'TxRate-652k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 653)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,653,'TxRate-653k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 654)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,654,'TxRate-654k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 655)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,655,'TxRate-655k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 656)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,656,'TxRate-656k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 657)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,657,'TxRate-657k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 658)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,658,'TxRate-658k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 659)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,659,'TxRate-659k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 660)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,660,'TxRate-660k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 661)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,661,'TxRate-661k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 662)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,662,'TxRate-662k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 663)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,663,'TxRate-663k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 664)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,664,'TxRate-664k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 665)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,665,'TxRate-665k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 666)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,666,'TxRate-666k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 667)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,667,'TxRate-667k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 668)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,668,'TxRate-668k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 669)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,669,'TxRate-669k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 670)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,670,'TxRate-670k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 671)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,671,'TxRate-671k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 672)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,672,'TxRate-672k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 673)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,673,'TxRate-673k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 674)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,674,'TxRate-674k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 675)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,675,'TxRate-675k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 676)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,676,'TxRate-676k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 677)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,677,'TxRate-677k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 678)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,678,'TxRate-678k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 679)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,679,'TxRate-679k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 680)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,680,'TxRate-680k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 681)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,681,'TxRate-681k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 682)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,682,'TxRate-682k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 683)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,683,'TxRate-683k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 684)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,684,'TxRate-684k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 685)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,685,'TxRate-685k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 686)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,686,'TxRate-686k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 687)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,687,'TxRate-687k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 688)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,688,'TxRate-688k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 689)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,689,'TxRate-689k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 690)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,690,'TxRate-690k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 691)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,691,'TxRate-691k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 692)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,692,'TxRate-692k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 693)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,693,'TxRate-693k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 694)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,694,'TxRate-694k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 695)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,695,'TxRate-695k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 696)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,696,'TxRate-696k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 697)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,697,'TxRate-697k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 698)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,698,'TxRate-698k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 699)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,699,'TxRate-699k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 700)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,700,'TxRate-700k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 701)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,701,'TxRate-701k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 702)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,702,'TxRate-702k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 703)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,703,'TxRate-703k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 704)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,704,'TxRate-704k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 705)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,705,'TxRate-705k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 706)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,706,'TxRate-706k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 707)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,707,'TxRate-707k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 708)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,708,'TxRate-708k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 709)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,709,'TxRate-709k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 710)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,710,'TxRate-710k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 711)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,711,'TxRate-711k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 712)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,712,'TxRate-712k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 713)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,713,'TxRate-713k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 714)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,714,'TxRate-714k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 715)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,715,'TxRate-715k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 716)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,716,'TxRate-716k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 717)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,717,'TxRate-717k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 718)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,718,'TxRate-718k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 719)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,719,'TxRate-719k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 720)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,720,'TxRate-720k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 721)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,721,'TxRate-721k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 722)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,722,'TxRate-722k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 723)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,723,'TxRate-723k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 724)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,724,'TxRate-724k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 725)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,725,'TxRate-725k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 726)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,726,'TxRate-726k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 727)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,727,'TxRate-727k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 728)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,728,'TxRate-728k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 729)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,729,'TxRate-729k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 730)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,730,'TxRate-730k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 731)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,731,'TxRate-731k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 732)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,732,'TxRate-732k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 733)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,733,'TxRate-733k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 734)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,734,'TxRate-734k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 735)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,735,'TxRate-735k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 736)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,736,'TxRate-736k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 737)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,737,'TxRate-737k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 738)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,738,'TxRate-738k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 739)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,739,'TxRate-739k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 740)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,740,'TxRate-740k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 741)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,741,'TxRate-741k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 742)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,742,'TxRate-742k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 743)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,743,'TxRate-743k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 744)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,744,'TxRate-744k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 745)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,745,'TxRate-745k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 746)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,746,'TxRate-746k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 747)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,747,'TxRate-747k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 748)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,748,'TxRate-748k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 749)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,749,'TxRate-749k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 750)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,750,'TxRate-750k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 751)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,751,'TxRate-751k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 752)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,752,'TxRate-752k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 753)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,753,'TxRate-753k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 754)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,754,'TxRate-754k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 755)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,755,'TxRate-755k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 756)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,756,'TxRate-756k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 757)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,757,'TxRate-757k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 758)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,758,'TxRate-758k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 759)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,759,'TxRate-759k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 760)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,760,'TxRate-760k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 761)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,761,'TxRate-761k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 762)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,762,'TxRate-762k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 763)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,763,'TxRate-763k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 764)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,764,'TxRate-764k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 765)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,765,'TxRate-765k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 766)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,766,'TxRate-766k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 767)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,767,'TxRate-767k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 768)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,768,'TxRate-768k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 769)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,769,'TxRate-769k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 770)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,770,'TxRate-770k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 771)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,771,'TxRate-771k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 772)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,772,'TxRate-772k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 773)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,773,'TxRate-773k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 774)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,774,'TxRate-774k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 775)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,775,'TxRate-775k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 776)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,776,'TxRate-776k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 777)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,777,'TxRate-777k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 778)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,778,'TxRate-778k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 779)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,779,'TxRate-779k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 780)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,780,'TxRate-780k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 781)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,781,'TxRate-781k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 782)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,782,'TxRate-782k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 783)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,783,'TxRate-783k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 784)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,784,'TxRate-784k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 785)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,785,'TxRate-785k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 786)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,786,'TxRate-786k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 787)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,787,'TxRate-787k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 788)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,788,'TxRate-788k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 789)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,789,'TxRate-789k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 790)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,790,'TxRate-790k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 791)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,791,'TxRate-791k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 792)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,792,'TxRate-792k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 793)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,793,'TxRate-793k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 794)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,794,'TxRate-794k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 795)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,795,'TxRate-795k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 796)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,796,'TxRate-796k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 797)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,797,'TxRate-797k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 798)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,798,'TxRate-798k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 799)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,799,'TxRate-799k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 800)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,800,'TxRate-800k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 801)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,801,'TxRate-801k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 802)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,802,'TxRate-802k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 803)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,803,'TxRate-803k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 804)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,804,'TxRate-804k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 805)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,805,'TxRate-805k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 806)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,806,'TxRate-806k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 807)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,807,'TxRate-807k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 808)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,808,'TxRate-808k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 809)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,809,'TxRate-809k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 810)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,810,'TxRate-810k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 811)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,811,'TxRate-811k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 812)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,812,'TxRate-812k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 813)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,813,'TxRate-813k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 814)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,814,'TxRate-814k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 815)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,815,'TxRate-815k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 816)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,816,'TxRate-816k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 817)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,817,'TxRate-817k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 818)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,818,'TxRate-818k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 819)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,819,'TxRate-819k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 820)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,820,'TxRate-820k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 821)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,821,'TxRate-821k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 822)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,822,'TxRate-822k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 823)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,823,'TxRate-823k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 824)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,824,'TxRate-824k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 825)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,825,'TxRate-825k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 826)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,826,'TxRate-826k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 827)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,827,'TxRate-827k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 828)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,828,'TxRate-828k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 829)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,829,'TxRate-829k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 830)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,830,'TxRate-830k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 831)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,831,'TxRate-831k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 832)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,832,'TxRate-832k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 833)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,833,'TxRate-833k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 834)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,834,'TxRate-834k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 835)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,835,'TxRate-835k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 836)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,836,'TxRate-836k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 837)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,837,'TxRate-837k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 838)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,838,'TxRate-838k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 839)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,839,'TxRate-839k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 840)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,840,'TxRate-840k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 841)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,841,'TxRate-841k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 842)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,842,'TxRate-842k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 843)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,843,'TxRate-843k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 844)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,844,'TxRate-844k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 845)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,845,'TxRate-845k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 846)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,846,'TxRate-846k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 847)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,847,'TxRate-847k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 848)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,848,'TxRate-848k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 849)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,849,'TxRate-849k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 850)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,850,'TxRate-850k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 851)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,851,'TxRate-851k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 852)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,852,'TxRate-852k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 853)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,853,'TxRate-853k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 854)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,854,'TxRate-854k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 855)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,855,'TxRate-855k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 856)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,856,'TxRate-856k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 857)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,857,'TxRate-857k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 858)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,858,'TxRate-858k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 859)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,859,'TxRate-859k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 860)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,860,'TxRate-860k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 861)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,861,'TxRate-861k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 862)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,862,'TxRate-862k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 863)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,863,'TxRate-863k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 864)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,864,'TxRate-864k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 865)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,865,'TxRate-865k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 866)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,866,'TxRate-866k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 867)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,867,'TxRate-867k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 868)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,868,'TxRate-868k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 869)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,869,'TxRate-869k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 870)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,870,'TxRate-870k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 871)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,871,'TxRate-871k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 872)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,872,'TxRate-872k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 873)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,873,'TxRate-873k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 874)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,874,'TxRate-874k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 875)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,875,'TxRate-875k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 876)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,876,'TxRate-876k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 877)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,877,'TxRate-877k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 878)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,878,'TxRate-878k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 879)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,879,'TxRate-879k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 880)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,880,'TxRate-880k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 881)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,881,'TxRate-881k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 882)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,882,'TxRate-882k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 883)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,883,'TxRate-883k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 884)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,884,'TxRate-884k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 885)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,885,'TxRate-885k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 886)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,886,'TxRate-886k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 887)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,887,'TxRate-887k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 888)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,888,'TxRate-888k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 889)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,889,'TxRate-889k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 890)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,890,'TxRate-890k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 891)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,891,'TxRate-891k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 892)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,892,'TxRate-892k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 893)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,893,'TxRate-893k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 894)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,894,'TxRate-894k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 895)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,895,'TxRate-895k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 896)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,896,'TxRate-896k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 897)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,897,'TxRate-897k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 898)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,898,'TxRate-898k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 899)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,899,'TxRate-899k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 900)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,900,'TxRate-900k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 901)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,901,'TxRate-901k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 902)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,902,'TxRate-902k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 903)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,903,'TxRate-903k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 904)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,904,'TxRate-904k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 905)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,905,'TxRate-905k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 906)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,906,'TxRate-906k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 907)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,907,'TxRate-907k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 908)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,908,'TxRate-908k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 909)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,909,'TxRate-909k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 910)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,910,'TxRate-910k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 911)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,911,'TxRate-911k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 912)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,912,'TxRate-912k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 913)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,913,'TxRate-913k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 914)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,914,'TxRate-914k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 915)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,915,'TxRate-915k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 916)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,916,'TxRate-916k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 917)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,917,'TxRate-917k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 918)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,918,'TxRate-918k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 919)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,919,'TxRate-919k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 920)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,920,'TxRate-920k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 921)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,921,'TxRate-921k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 922)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,922,'TxRate-922k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 923)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,923,'TxRate-923k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 924)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,924,'TxRate-924k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 925)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,925,'TxRate-925k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 926)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,926,'TxRate-926k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 927)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,927,'TxRate-927k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 928)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,928,'TxRate-928k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 929)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,929,'TxRate-929k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 930)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,930,'TxRate-930k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 931)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,931,'TxRate-931k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 932)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,932,'TxRate-932k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 933)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,933,'TxRate-933k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 934)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,934,'TxRate-934k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 935)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,935,'TxRate-935k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 936)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,936,'TxRate-936k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 937)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,937,'TxRate-937k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 938)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,938,'TxRate-938k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 939)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,939,'TxRate-939k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 940)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,940,'TxRate-940k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 941)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,941,'TxRate-941k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 942)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,942,'TxRate-942k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 943)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,943,'TxRate-943k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 944)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,944,'TxRate-944k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 945)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,945,'TxRate-945k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 946)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,946,'TxRate-946k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 947)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,947,'TxRate-947k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 948)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,948,'TxRate-948k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 949)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,949,'TxRate-949k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 950)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,950,'TxRate-950k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 951)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,951,'TxRate-951k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 952)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,952,'TxRate-952k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 953)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,953,'TxRate-953k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 954)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,954,'TxRate-954k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 955)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,955,'TxRate-955k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 956)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,956,'TxRate-956k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 957)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,957,'TxRate-957k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 958)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,958,'TxRate-958k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 959)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,959,'TxRate-959k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 960)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,960,'TxRate-960k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 961)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,961,'TxRate-961k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 962)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,962,'TxRate-962k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 963)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,963,'TxRate-963k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 964)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,964,'TxRate-964k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 965)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,965,'TxRate-965k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 966)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,966,'TxRate-966k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 967)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,967,'TxRate-967k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 968)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,968,'TxRate-968k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 969)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,969,'TxRate-969k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 970)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,970,'TxRate-970k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 971)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,971,'TxRate-971k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 972)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,972,'TxRate-972k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 973)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,973,'TxRate-973k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 974)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,974,'TxRate-974k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 975)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,975,'TxRate-975k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 976)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,976,'TxRate-976k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 977)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,977,'TxRate-977k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 978)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,978,'TxRate-978k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 979)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,979,'TxRate-979k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 980)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,980,'TxRate-980k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 981)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,981,'TxRate-981k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 982)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,982,'TxRate-982k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 983)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,983,'TxRate-983k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 984)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,984,'TxRate-984k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 985)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,985,'TxRate-985k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 986)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,986,'TxRate-986k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 987)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,987,'TxRate-987k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 988)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,988,'TxRate-988k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 989)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,989,'TxRate-989k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 990)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,990,'TxRate-990k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 991)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,991,'TxRate-991k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 992)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,992,'TxRate-992k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 993)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,993,'TxRate-993k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 994)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,994,'TxRate-994k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 995)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,995,'TxRate-995k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 996)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,996,'TxRate-996k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 997)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,997,'TxRate-997k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 998)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,998,'TxRate-998k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 999)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,999,'TxRate-999k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1000)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1000,'TxRate-1000k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1001)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1001,'TxRate-1001k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1002)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1002,'TxRate-1002k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1003)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1003,'TxRate-1003k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1004)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1004,'TxRate-1004k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1005)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1005,'TxRate-1005k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1006)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1006,'TxRate-1006k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1007)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1007,'TxRate-1007k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1008)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1008,'TxRate-1008k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1009)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1009,'TxRate-1009k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1010)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1010,'TxRate-1010k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1011)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1011,'TxRate-1011k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1012)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1012,'TxRate-1012k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1013)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1013,'TxRate-1013k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1014)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1014,'TxRate-1014k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1015)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1015,'TxRate-1015k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1016)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1016,'TxRate-1016k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1017)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1017,'TxRate-1017k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1018)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1018,'TxRate-1018k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1019)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1019,'TxRate-1019k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1020)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1020,'TxRate-1020k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1021)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1021,'TxRate-1021k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1022)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1022,'TxRate-1022k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1023)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1023,'TxRate-1023k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1024)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1024,'TxRate-1024k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1025)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1025,'TxRate-1152k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1026)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1026,'TxRate-1280k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1027)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1027,'TxRate-1408k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1028)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1028,'TxRate-1536k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1029)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1029,'TxRate-1664k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1030)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1030,'TxRate-1792k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1031)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1031,'TxRate-1920k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1032)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1032,'TxRate-2048k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1033)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1033,'TxRate-2176k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1034)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1034,'TxRate-2304k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1035)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1035,'TxRate-2432k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1036)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1036,'TxRate-2560k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1037)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1037,'TxRate-2688k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1038)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1038,'TxRate-2816k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1039)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1039,'TxRate-2944k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1040)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1040,'TxRate-3072k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1041)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1041,'TxRate-3200k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1042)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1042,'TxRate-3328k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1043)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1043,'TxRate-3456k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1044)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1044,'TxRate-3584k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1045)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1045,'TxRate-3712k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1046)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1046,'TxRate-3840k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1047)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1047,'TxRate-3968k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1048)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1048,'TxRate-4096k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1049)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1049,'TxRate-4224k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1050)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1050,'TxRate-4352k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1051)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1051,'TxRate-4480k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1052)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1052,'TxRate-4608k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1053)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1053,'TxRate-4736k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1054)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1054,'TxRate-4864k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1055)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1055,'TxRate-4992k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1056)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1056,'TxRate-5120k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1057)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1057,'TxRate-5248k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1058)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1058,'TxRate-5376k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1059)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1059,'TxRate-5504k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1060)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1060,'TxRate-5632k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1061)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1061,'TxRate-5760k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1062)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1062,'TxRate-5888k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1063)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1063,'TxRate-6016k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1064)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1064,'TxRate-6144k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1065)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1065,'TxRate-6272k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1066)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1066,'TxRate-6400k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1067)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1067,'TxRate-6528k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1068)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1068,'TxRate-6656k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1069)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1069,'TxRate-6784k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1070)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1070,'TxRate-6912k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1071)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1071,'TxRate-7040k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1072)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1072,'TxRate-7168k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1073)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1073,'TxRate-7296k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1074)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1074,'TxRate-7424k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1075)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1075,'TxRate-7552k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1076)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1076,'TxRate-7680k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1077)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1077,'TxRate-7808k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1078)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1078,'TxRate-7936k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1079)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1079,'TxRate-8064k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1080)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1080,'TxRate-8192k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1081)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1081,'TxRate-8320k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1082)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1082,'TxRate-8448k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1083)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1083,'TxRate-8576k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1084)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1084,'TxRate-8704k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1085)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1085,'TxRate-8832k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1086)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1086,'TxRate-8960k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1087)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1087,'TxRate-9088k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1088)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1088,'TxRate-9216k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1089)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1089,'TxRate-9344k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1090)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1090,'TxRate-9472k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1091)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1091,'TxRate-9600k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1092)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1092,'TxRate-9728k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1093)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1093,'TxRate-9856k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1094)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1094,'TxRate-9984k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1095)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1095,'TxRate-10112k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1096)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1096,'TxRate-10240k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1097)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1097,'TxRate-10368k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1098)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1098,'TxRate-10496k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1099)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1099,'TxRate-10624k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1100)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1100,'TxRate-10752k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1101,'TxRate-10880k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1102,'TxRate-11008k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1103)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1103,'TxRate-11136k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1104)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1104,'TxRate-11264k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1105)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1105,'TxRate-11392k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1106)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1106,'TxRate-11520k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1107)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1107,'TxRate-11648k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1108)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1108,'TxRate-11776k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1109)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1109,'TxRate-11904k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1110)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1110,'TxRate-12032k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1111)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1111,'TxRate-12160k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1112)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1112,'TxRate-12288k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1113)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1113,'TxRate-12416k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1114)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1114,'TxRate-12544k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1115)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1115,'TxRate-12672k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1116)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1116,'TxRate-12800k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1117)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1117,'TxRate-12928k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1118)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1118,'TxRate-13056k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1119)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1119,'TxRate-13184k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1120)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1120,'TxRate-13312k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1121)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1121,'TxRate-13440k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1122)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1122,'TxRate-13568k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1123)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1123,'TxRate-13696k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1124)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1124,'TxRate-13824k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1125)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1125,'TxRate-13952k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1126)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1126,'TxRate-14080k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1127)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1127,'TxRate-14208k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1128)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1128,'TxRate-14336k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1129)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1129,'TxRate-14464k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1130)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1130,'TxRate-14592k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1131)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1131,'TxRate-14720k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1132)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1132,'TxRate-14848k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1133)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1133,'TxRate-14976k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1134)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1134,'TxRate-15104k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1135)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1135,'TxRate-15232k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1136)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1136,'TxRate-15360k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1137)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1137,'TxRate-15488k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1138)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1138,'TxRate-15616k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1139)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1139,'TxRate-15744k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1140)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1140,'TxRate-15872k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1141)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1141,'TxRate-16000k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1142)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1142,'TxRate-16128k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1143)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1143,'TxRate-16256k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1144)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1144,'TxRate-16384k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1145)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1145,'TxRate-16512k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1146)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1146,'TxRate-16640k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1147)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1147,'TxRate-16768k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1148)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1148,'TxRate-16896k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1149)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1149,'TxRate-17024k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1150)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1150,'TxRate-17152k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1151)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1151,'TxRate-17280k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1152)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1152,'TxRate-17408k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1153)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1153,'TxRate-17536k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1154)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1154,'TxRate-17664k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1155)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1155,'TxRate-17792k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1156)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1156,'TxRate-17920k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1157)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1157,'TxRate-18048k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1158)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1158,'TxRate-18176k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1159)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1159,'TxRate-18304k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1160)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1160,'TxRate-18432k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1161)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1161,'TxRate-18560k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1162)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1162,'TxRate-18688k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1163)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1163,'TxRate-18816k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1164)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1164,'TxRate-18944k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1165)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1165,'TxRate-19072k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1166)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1166,'TxRate-19200k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1167)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1167,'TxRate-19328k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1168)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1168,'TxRate-19456k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1169)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1169,'TxRate-19584k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1170)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1170,'TxRate-19712k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1171)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1171,'TxRate-19840k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1172)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1172,'TxRate-19968k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1173)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1173,'TxRate-20096k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1174)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1174,'TxRate-20224k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1175)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1175,'TxRate-20352k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1176)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1176,'TxRate-20480k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1177)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1177,'TxRate-20608k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1178)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1178,'TxRate-20736k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1179)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1179,'TxRate-20864k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1180)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1180,'TxRate-20992k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1181)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1181,'TxRate-21120k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1182)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1182,'TxRate-21248k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1183)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1183,'TxRate-21376k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1184)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1184,'TxRate-21504k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1185)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1185,'TxRate-21632k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1186)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1186,'TxRate-21760k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1187)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1187,'TxRate-21888k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1188)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1188,'TxRate-22016k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1189)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1189,'TxRate-22144k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1190)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1190,'TxRate-22272k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1191)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1191,'TxRate-22400k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1192)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1192,'TxRate-22528k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1193)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1193,'TxRate-22656k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1194)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1194,'TxRate-22784k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1195)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1195,'TxRate-22912k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1196)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1196,'TxRate-23040k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1197)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1197,'TxRate-23168k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1198)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1198,'TxRate-23296k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1199)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1199,'TxRate-23424k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1200)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1200,'TxRate-23552k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1201)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1201,'TxRate-23680k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1202)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1202,'TxRate-23808k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1203)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1203,'TxRate-23936k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1204)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1204,'TxRate-24064k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1205)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1205,'TxRate-24192k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1206)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1206,'TxRate-24320k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1207)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1207,'TxRate-24448k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1208)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1208,'TxRate-24576k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1209)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1209,'TxRate-24704k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1210)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1210,'TxRate-24832k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1211)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1211,'TxRate-24960k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1212)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1212,'TxRate-25088k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1213)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1213,'TxRate-25216k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1214)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1214,'TxRate-25344k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1215)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1215,'TxRate-25472k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1216)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1216,'TxRate-25600k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1217)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1217,'TxRate-25728k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1218)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1218,'TxRate-25856k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1219)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1219,'TxRate-25984k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1220)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1220,'TxRate-26112k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1221)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1221,'TxRate-26240k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1222)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1222,'TxRate-26368k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1223)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1223,'TxRate-26496k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1224)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1224,'TxRate-26624k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1225)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1225,'TxRate-26752k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1226)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1226,'TxRate-26880k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1227)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1227,'TxRate-27008k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1228)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1228,'TxRate-27136k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1229)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1229,'TxRate-27264k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1230)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1230,'TxRate-27392k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1231)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1231,'TxRate-27520k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1232)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1232,'TxRate-27648k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1233)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1233,'TxRate-27776k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1234)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1234,'TxRate-27904k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1235)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1235,'TxRate-28032k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1236)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1236,'TxRate-28160k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1237)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1237,'TxRate-28288k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1238)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1238,'TxRate-28416k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1239)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1239,'TxRate-28544k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1240)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1240,'TxRate-28672k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1241)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1241,'TxRate-28800k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1242)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1242,'TxRate-28928k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1243)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1243,'TxRate-29056k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1244)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1244,'TxRate-29184k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1245)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1245,'TxRate-29312k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1246)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1246,'TxRate-29440k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1247)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1247,'TxRate-29568k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1248)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1248,'TxRate-29696k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1249)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1249,'TxRate-29824k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1250)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1250,'TxRate-29952k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1251)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1251,'TxRate-30080k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1252)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1252,'TxRate-30208k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1253)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1253,'TxRate-30336k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1254)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1254,'TxRate-30464k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1255)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1255,'TxRate-30592k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1256)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1256,'TxRate-30720k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1257)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1257,'TxRate-30848k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1258)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1258,'TxRate-30976k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1259)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1259,'TxRate-31104k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1260)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1260,'TxRate-31232k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1261)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1261,'TxRate-31360k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1262)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1262,'TxRate-31488k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1263)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1263,'TxRate-31616k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1264)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1264,'TxRate-31744k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1265)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1265,'TxRate-31872k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1266)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1266,'TxRate-32000k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1267)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1267,'TxRate-32128k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1268)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1268,'TxRate-32256k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1269)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1269,'TxRate-32384k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1270)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1270,'TxRate-32512k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1271)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1271,'TxRate-32640k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1272)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1272,'TxRate-32768k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1273)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1273,'TxRate-32896k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1274)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1274,'TxRate-33024k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1275)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1275,'TxRate-33152k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1276)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1276,'TxRate-33280k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1277)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1277,'TxRate-33408k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1278)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1278,'TxRate-33536k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1279)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1279,'TxRate-33664k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1280)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1280,'TxRate-33792k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1281)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1281,'TxRate-33920k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1282)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1282,'TxRate-34048k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1283)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1283,'TxRate-34176k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1284)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1284,'TxRate-34304k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1285)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1285,'TxRate-34432k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1286)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1286,'TxRate-34560k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1287)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1287,'TxRate-34688k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1288)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1288,'TxRate-34816k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1289)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1289,'TxRate-34944k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1290)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1290,'TxRate-35072k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1291)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1291,'TxRate-35200k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1292)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1292,'TxRate-35328k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1293)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1293,'TxRate-35456k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1294)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1294,'TxRate-35584k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1295)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1295,'TxRate-35712k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1296)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1296,'TxRate-35840k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1297)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1297,'TxRate-35968k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1298)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1298,'TxRate-36096k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1299)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1299,'TxRate-36224k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1300)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1300,'TxRate-36352k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1301)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1301,'TxRate-36480k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1302)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1302,'TxRate-36608k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1303)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1303,'TxRate-36736k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1304)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1304,'TxRate-36864k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1305)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1305,'TxRate-36992k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1306)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1306,'TxRate-37120k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1307)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1307,'TxRate-37248k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1308)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1308,'TxRate-37376k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1309)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1309,'TxRate-37504k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1310)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1310,'TxRate-37632k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1311)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1311,'TxRate-37760k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1312)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1312,'TxRate-37888k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1313)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1313,'TxRate-38016k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1314)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1314,'TxRate-38144k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1315)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1315,'TxRate-38272k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1316)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1316,'TxRate-38400k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1317)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1317,'TxRate-38528k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1318)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1318,'TxRate-38656k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1319)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1319,'TxRate-38784k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1320)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1320,'TxRate-38912k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1321)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1321,'TxRate-39040k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1322)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1322,'TxRate-39168k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1323)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1323,'TxRate-39296k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1324)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1324,'TxRate-39424k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1325)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1325,'TxRate-39552k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1326)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1326,'TxRate-39680k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1327)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1327,'TxRate-39808k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1328)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1328,'TxRate-39936k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1329)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1329,'TxRate-40064k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1330)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1330,'TxRate-40192k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1331)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1331,'TxRate-40320k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1332)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1332,'TxRate-40448k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1333)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1333,'TxRate-40576k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1334)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1334,'TxRate-40704k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1335)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1335,'TxRate-40832k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1336)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1336,'TxRate-40960k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1337)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1337,'TxRate-41088k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1338)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1338,'TxRate-41216k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1339)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1339,'TxRate-41344k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1340)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1340,'TxRate-41472k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1341)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1341,'TxRate-41600k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1342)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1342,'TxRate-41728k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1343)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1343,'TxRate-41856k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1344)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1344,'TxRate-41984k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1345)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1345,'TxRate-42112k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1346)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1346,'TxRate-42240k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1347)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1347,'TxRate-42368k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1348)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1348,'TxRate-42496k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1349)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1349,'TxRate-42624k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1350)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1350,'TxRate-42752k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1351)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1351,'TxRate-42880k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1352)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1352,'TxRate-43008k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1353)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1353,'TxRate-43136k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1354)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1354,'TxRate-43264k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1355)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1355,'TxRate-43392k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1356)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1356,'TxRate-43520k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1357)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1357,'TxRate-43648k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1358)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1358,'TxRate-43776k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1359)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1359,'TxRate-43904k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1360)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1360,'TxRate-44032k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1361)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1361,'TxRate-44160k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1362)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1362,'TxRate-44288k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1363)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1363,'TxRate-44416k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1364)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1364,'TxRate-44544k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1365)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1365,'TxRate-44672k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1366)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1366,'TxRate-44800k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1367)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1367,'TxRate-44928k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1368)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1368,'TxRate-45056k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1369)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1369,'TxRate-45184k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1370)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1370,'TxRate-45312k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1371)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1371,'TxRate-45440k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1372)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1372,'TxRate-45568k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1373)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1373,'TxRate-45696k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1374)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1374,'TxRate-45824k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1375)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1375,'TxRate-45952k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1376)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1376,'TxRate-46080k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1377)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1377,'TxRate-46208k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1378)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1378,'TxRate-46336k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1379)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1379,'TxRate-46464k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1380)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1380,'TxRate-46592k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1381)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1381,'TxRate-46720k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1382)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1382,'TxRate-46848k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1383)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1383,'TxRate-46976k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1384)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1384,'TxRate-47104k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1385)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1385,'TxRate-47232k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1386)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1386,'TxRate-47360k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1387)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1387,'TxRate-47488k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1388)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1388,'TxRate-47616k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1389)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1389,'TxRate-47744k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1390)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1390,'TxRate-47872k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1391)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1391,'TxRate-48000k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1392)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1392,'TxRate-48128k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1393)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1393,'TxRate-48256k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1394)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1394,'TxRate-48384k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1395)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1395,'TxRate-48512k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1396)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1396,'TxRate-48640k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1397)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1397,'TxRate-48768k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1398)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1398,'TxRate-48896k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1399)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1399,'TxRate-49024k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1400)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1400,'TxRate-49152k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1401)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1401,'TxRate-49280k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1402)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1402,'TxRate-49408k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1403)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1403,'TxRate-49536k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1404)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1404,'TxRate-49664k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1405)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1405,'TxRate-49792k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1406)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1406,'TxRate-49920k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1407)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1407,'TxRate-50048k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1408)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1408,'TxRate-50176k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1409)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1409,'TxRate-50304k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1410)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1410,'TxRate-50432k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1411)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1411,'TxRate-50560k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1412)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1412,'TxRate-50688k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1413)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1413,'TxRate-50816k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1414)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1414,'TxRate-50944k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1415)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1415,'TxRate-51072k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1416)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1416,'TxRate-51200k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1417)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1417,'TxRate-51328k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1418)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1418,'TxRate-51456k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1419)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1419,'TxRate-51584k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1420)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1420,'TxRate-51712k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1421)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1421,'TxRate-51840k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1422)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1422,'TxRate-51968k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1423)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1423,'TxRate-52096k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1424)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1424,'TxRate-52224k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1425)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1425,'TxRate-52352k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1426)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1426,'TxRate-52480k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1427)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1427,'TxRate-52608k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1428)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1428,'TxRate-52736k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1429)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1429,'TxRate-52864k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1430)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1430,'TxRate-52992k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1431)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1431,'TxRate-53120k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1432)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1432,'TxRate-53248k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1433)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1433,'TxRate-53376k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1434)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1434,'TxRate-53504k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1435)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1435,'TxRate-53632k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1436)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1436,'TxRate-53760k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1437)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1437,'TxRate-53888k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1438)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1438,'TxRate-54016k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1439)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1439,'TxRate-54144k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1440)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1440,'TxRate-54272k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1441)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1441,'TxRate-54400k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1442)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1442,'TxRate-54528k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1443)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1443,'TxRate-54656k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1444)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1444,'TxRate-54784k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1445)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1445,'TxRate-54912k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1446)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1446,'TxRate-55040k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1447)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1447,'TxRate-55168k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1448)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1448,'TxRate-55296k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1449)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1449,'TxRate-55424k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1450)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1450,'TxRate-55552k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1451)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1451,'TxRate-55680k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1452)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1452,'TxRate-55808k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1453)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1453,'TxRate-55936k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1454)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1454,'TxRate-56064k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1455)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1455,'TxRate-56192k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1456)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1456,'TxRate-56320k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1457)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1457,'TxRate-56448k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1458)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1458,'TxRate-56576k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1459)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1459,'TxRate-56704k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1460)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1460,'TxRate-56832k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1461)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1461,'TxRate-56960k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1462)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1462,'TxRate-57088k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1463)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1463,'TxRate-57216k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1464)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1464,'TxRate-57344k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1465)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1465,'TxRate-57472k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1466)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1466,'TxRate-57600k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1467)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1467,'TxRate-57728k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1468)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1468,'TxRate-57856k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1469)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1469,'TxRate-57984k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1470)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1470,'TxRate-58112k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1471)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1471,'TxRate-58240k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1472)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1472,'TxRate-58368k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1473)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1473,'TxRate-58496k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1474)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1474,'TxRate-58624k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1475)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1475,'TxRate-58752k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1476)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1476,'TxRate-58880k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1477)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1477,'TxRate-59008k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1478)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1478,'TxRate-59136k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1479)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1479,'TxRate-59264k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1480)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1480,'TxRate-59392k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1481)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1481,'TxRate-59520k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1482)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1482,'TxRate-59648k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1483)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1483,'TxRate-59776k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1484)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1484,'TxRate-59904k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1485)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1485,'TxRate-60032k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1486)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1486,'TxRate-60160k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1487)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1487,'TxRate-60288k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1488)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1488,'TxRate-60416k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1489)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1489,'TxRate-60544k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1490)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1490,'TxRate-60672k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1491)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1491,'TxRate-60800k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1492)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1492,'TxRate-60928k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1493)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1493,'TxRate-61056k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1494)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1494,'TxRate-61184k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1495)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1495,'TxRate-61312k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1496)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1496,'TxRate-61440k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1497)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1497,'TxRate-61568k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1498)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1498,'TxRate-61696k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1499)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1499,'TxRate-61824k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1500)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1500,'TxRate-61952k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1501)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1501,'TxRate-62080k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1502)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1502,'TxRate-62208k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1503)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1503,'TxRate-62336k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1504)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1504,'TxRate-62464k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1505)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1505,'TxRate-62592k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1506)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1506,'TxRate-62720k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1507)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1507,'TxRate-62848k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1508)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1508,'TxRate-62976k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1509)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1509,'TxRate-63104k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1510)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1510,'TxRate-63232k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1511)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1511,'TxRate-63360k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1512)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1512,'TxRate-63488k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1513)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1513,'TxRate-63616k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1514)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1514,'TxRate-63744k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1515)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1515,'TxRate-63872k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1516)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1516,'TxRate-64000k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1517)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1517,'TxRate-64128k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1518)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1518,'TxRate-64256k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1519)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1519,'TxRate-64384k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1520)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1520,'TxRate-64512k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1521)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1521,'TxRate-64640k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1522)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1522,'TxRate-64768k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1523)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1523,'TxRate-64896k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1524)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1524,'TxRate-65024k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1525)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1525,'TxRate-65152k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1526)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1526,'TxRate-65280k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1527)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1527,'TxRate-65408k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1528)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1528,'TxRate-65536k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1529)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1529,'TxRate-65664k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1530)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1530,'TxRate-65792k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1531)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1531,'TxRate-65920k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1532)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1532,'TxRate-66048k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1533)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1533,'TxRate-66176k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1534)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1534,'TxRate-66304k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1535)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1535,'TxRate-66432k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1536)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1536,'TxRate-66560k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1537)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1537,'TxRate-66688k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1538)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1538,'TxRate-66816k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1539)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1539,'TxRate-66944k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1540)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1540,'TxRate-67072k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1541)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1541,'TxRate-67200k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1542)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1542,'TxRate-67328k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1543)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1543,'TxRate-67456k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1544)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1544,'TxRate-67584k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1545)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1545,'TxRate-67712k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1546)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1546,'TxRate-67840k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1547)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1547,'TxRate-67968k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1548)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1548,'TxRate-68096k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1549)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1549,'TxRate-68224k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1550)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1550,'TxRate-68352k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1551)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1551,'TxRate-68480k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1552)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1552,'TxRate-68608k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1553)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1553,'TxRate-68736k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1554)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1554,'TxRate-68864k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1555)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1555,'TxRate-68992k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1556)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1556,'TxRate-69120k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1557)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1557,'TxRate-69248k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1558)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1558,'TxRate-69376k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1559)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1559,'TxRate-69504k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1560)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1560,'TxRate-69632k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1561)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1561,'TxRate-69760k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1562)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1562,'TxRate-69888k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1563)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1563,'TxRate-70016k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1564)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1564,'TxRate-70144k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1565)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1565,'TxRate-70272k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1566)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1566,'TxRate-70400k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1567)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1567,'TxRate-70528k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1568)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1568,'TxRate-70656k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1569)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1569,'TxRate-70784k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1570)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1570,'TxRate-70912k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1571)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1571,'TxRate-71040k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1572)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1572,'TxRate-71168k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1573)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1573,'TxRate-71296k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1574)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1574,'TxRate-71424k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1575)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1575,'TxRate-71552k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1576)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1576,'TxRate-71680k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1577)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1577,'TxRate-71808k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1578)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1578,'TxRate-71936k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1579)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1579,'TxRate-72064k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1580)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1580,'TxRate-72192k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1581)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1581,'TxRate-72320k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1582)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1582,'TxRate-72448k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1583)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1583,'TxRate-72576k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1584)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1584,'TxRate-72704k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1585)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1585,'TxRate-72832k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1586)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1586,'TxRate-72960k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1587)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1587,'TxRate-73088k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1588)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1588,'TxRate-73216k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1589)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1589,'TxRate-73344k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1590)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1590,'TxRate-73472k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1591)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1591,'TxRate-73600k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1592)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1592,'TxRate-73728k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1593)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1593,'TxRate-73856k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1594)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1594,'TxRate-73984k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1595)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1595,'TxRate-74112k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1596)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1596,'TxRate-74240k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1597)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1597,'TxRate-74368k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1598)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1598,'TxRate-74496k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1599)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1599,'TxRate-74624k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1600)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1600,'TxRate-74752k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1601)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1601,'TxRate-74880k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1602)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1602,'TxRate-75008k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1603)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1603,'TxRate-75136k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1604)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1604,'TxRate-75264k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1605)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1605,'TxRate-75392k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1606)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1606,'TxRate-75520k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1607)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1607,'TxRate-75648k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1608)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1608,'TxRate-75776k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1609)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1609,'TxRate-75904k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1610)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1610,'TxRate-76032k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1611)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1611,'TxRate-76160k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1612)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1612,'TxRate-76288k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1613)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1613,'TxRate-76416k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1614)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1614,'TxRate-76544k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1615)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1615,'TxRate-76672k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1616)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1616,'TxRate-76800k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1617)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1617,'TxRate-76928k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1618)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1618,'TxRate-77056k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1619)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1619,'TxRate-77184k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1620)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1620,'TxRate-77312k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1621)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1621,'TxRate-77440k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1622)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1622,'TxRate-77568k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1623)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1623,'TxRate-77696k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1624)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1624,'TxRate-77824k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1625)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1625,'TxRate-77952k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1626)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1626,'TxRate-78080k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1627)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1627,'TxRate-78208k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1628)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1628,'TxRate-78336k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1629)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1629,'TxRate-78464k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1630)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1630,'TxRate-78592k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1631)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1631,'TxRate-78720k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1632)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1632,'TxRate-78848k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1633)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1633,'TxRate-78976k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1634)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1634,'TxRate-79104k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1635)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1635,'TxRate-79232k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1636)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1636,'TxRate-79360k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1637)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1637,'TxRate-79488k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1638)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1638,'TxRate-79616k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1639)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1639,'TxRate-79744k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1640)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1640,'TxRate-79872k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1641)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1641,'TxRate-80000k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1642)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1642,'TxRate-80128k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1643)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1643,'TxRate-80256k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1644)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1644,'TxRate-80384k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1645)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1645,'TxRate-80512k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1646)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1646,'TxRate-80640k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1647)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1647,'TxRate-80768k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1648)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1648,'TxRate-80896k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1649)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1649,'TxRate-81024k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1650)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1650,'TxRate-81152k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1651)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1651,'TxRate-81280k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1652)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1652,'TxRate-81408k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1653)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1653,'TxRate-81536k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1654)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1654,'TxRate-81664k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1655)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1655,'TxRate-81792k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1656)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1656,'TxRate-81920k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1657)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1657,'TxRate-82048k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1658)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1658,'TxRate-82176k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1659)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1659,'TxRate-82304k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1660)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1660,'TxRate-82432k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1661)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1661,'TxRate-82560k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1662)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1662,'TxRate-82688k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1663)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1663,'TxRate-82816k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1664)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1664,'TxRate-82944k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1665)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1665,'TxRate-83072k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1666)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1666,'TxRate-83200k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1667)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1667,'TxRate-83328k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1668)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1668,'TxRate-83456k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1669)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1669,'TxRate-83584k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1670)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1670,'TxRate-83712k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1671)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1671,'TxRate-83840k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1672)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1672,'TxRate-83968k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1673)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1673,'TxRate-84096k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1674)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1674,'TxRate-84224k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1675)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1675,'TxRate-84352k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1676)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1676,'TxRate-84480k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1677)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1677,'TxRate-84608k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1678)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1678,'TxRate-84736k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1679)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1679,'TxRate-84864k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1680)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1680,'TxRate-84992k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1681)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1681,'TxRate-85120k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1682)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1682,'TxRate-85248k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1683)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1683,'TxRate-85376k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1684)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1684,'TxRate-85504k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1685)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1685,'TxRate-85632k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1686)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1686,'TxRate-85760k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1687)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1687,'TxRate-85888k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1688)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1688,'TxRate-86016k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1689)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1689,'TxRate-86144k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1690)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1690,'TxRate-86272k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1691)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1691,'TxRate-86400k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1692)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1692,'TxRate-86528k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1693)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1693,'TxRate-86656k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1694)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1694,'TxRate-86784k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1695)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1695,'TxRate-86912k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1696)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1696,'TxRate-87040k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1697)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1697,'TxRate-87168k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1698)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1698,'TxRate-87296k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1699)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1699,'TxRate-87424k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1700)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1700,'TxRate-87552k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1701)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1701,'TxRate-87680k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1702)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1702,'TxRate-87808k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1703)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1703,'TxRate-87936k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1704)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1704,'TxRate-88064k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1705)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1705,'TxRate-88192k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1706)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1706,'TxRate-88320k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1707)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1707,'TxRate-88448k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1708)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1708,'TxRate-88576k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1709)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1709,'TxRate-88704k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1710)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1710,'TxRate-88832k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1711)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1711,'TxRate-88960k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1712)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1712,'TxRate-89088k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1713)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1713,'TxRate-89216k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1714)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1714,'TxRate-89344k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1715)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1715,'TxRate-89472k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1716)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1716,'TxRate-89600k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1717)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1717,'TxRate-89728k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1718)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1718,'TxRate-89856k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1719)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1719,'TxRate-89984k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1720)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1720,'TxRate-90112k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1721)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1721,'TxRate-90240k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1722)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1722,'TxRate-90368k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1723)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1723,'TxRate-90496k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1724)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1724,'TxRate-90624k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1725)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1725,'TxRate-90752k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1726)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1726,'TxRate-90880k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1727)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1727,'TxRate-91008k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1728)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1728,'TxRate-91136k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1729)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1729,'TxRate-91264k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1730)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1730,'TxRate-91392k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1731)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1731,'TxRate-91520k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1732)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1732,'TxRate-91648k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1733)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1733,'TxRate-91776k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1734)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1734,'TxRate-91904k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1735)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1735,'TxRate-92032k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1736)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1736,'TxRate-92160k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1737)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1737,'TxRate-92288k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1738)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1738,'TxRate-92416k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1739)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1739,'TxRate-92544k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1740)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1740,'TxRate-92672k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1741)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1741,'TxRate-92800k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1742)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1742,'TxRate-92928k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1743)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1743,'TxRate-93056k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1744)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1744,'TxRate-93184k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1745)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1745,'TxRate-93312k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1746)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1746,'TxRate-93440k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1747)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1747,'TxRate-93568k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1748)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1748,'TxRate-93696k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1749)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1749,'TxRate-93824k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1750)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1750,'TxRate-93952k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1751)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1751,'TxRate-94080k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1752)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1752,'TxRate-94208k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1753)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1753,'TxRate-94336k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1754)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1754,'TxRate-94464k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1755)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1755,'TxRate-94592k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1756)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1756,'TxRate-94720k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1757)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1757,'TxRate-94848k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1758)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1758,'TxRate-94976k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1759)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1759,'TxRate-95104k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1760)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1760,'TxRate-95232k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1761)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1761,'TxRate-95360k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1762)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1762,'TxRate-95488k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1763)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1763,'TxRate-95616k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1764)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1764,'TxRate-95744k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1765)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1765,'TxRate-95872k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1766)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1766,'TxRate-96000k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1767)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1767,'TxRate-96128k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1768)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1768,'TxRate-96256k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1769)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1769,'TxRate-96384k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1770)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1770,'TxRate-96512k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1771)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1771,'TxRate-96640k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1772)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1772,'TxRate-96768k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1773)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1773,'TxRate-96896k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1774)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1774,'TxRate-97024k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1775)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1775,'TxRate-97152k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1776)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1776,'TxRate-97280k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1777)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1777,'TxRate-97408k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1778)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1778,'TxRate-97536k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1779)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1779,'TxRate-97664k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1780)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1780,'TxRate-97792k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1781)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1781,'TxRate-97920k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1782)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1782,'TxRate-98048k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1783)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1783,'TxRate-98176k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1784)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1784,'TxRate-98304k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1785)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1785,'TxRate-98432k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1786)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1786,'TxRate-98560k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1787)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1787,'TxRate-98688k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1788)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1788,'TxRate-98816k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1789)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1789,'TxRate-98944k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1790)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1790,'TxRate-99072k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1791)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1791,'TxRate-99200k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1792)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1792,'TxRate-99328k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1793)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1793,'TxRate-99456k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1794)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1794,'TxRate-99584k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1795)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1795,'TxRate-99712k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1796)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1796,'TxRate-99840k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1797)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1797,'TxRate-99968k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1798)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1798,'TxRate-100096k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1799)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1799,'TxRate-100224k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1800)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1800,'TxRate-100352k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1801)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1801,'TxRate-100480k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1802)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1802,'TxRate-100608k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1803)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1803,'TxRate-100736k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1804)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1804,'TxRate-100864k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1805)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1805,'TxRate-100992k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1806)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1806,'TxRate-101120k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1807)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1807,'TxRate-101248k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1808)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1808,'TxRate-101376k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1809)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1809,'TxRate-101504k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1810)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1810,'TxRate-101632k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1811)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1811,'TxRate-101760k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1812)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1812,'TxRate-101888k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1813)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1813,'TxRate-102016k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1814)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1814,'TxRate-102144k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1815)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1815,'TxRate-102272k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1816)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1816,'TxRate-102400k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1817)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1817,'TxRate-102528k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1818)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1818,'TxRate-102656k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1819)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1819,'TxRate-102784k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1820)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1820,'TxRate-102912k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1821)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1821,'TxRate-103040k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1822)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1822,'TxRate-103168k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1823)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1823,'TxRate-103296k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1824)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1824,'TxRate-103424k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1825)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1825,'TxRate-103552k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1826)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1826,'TxRate-103680k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1827)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1827,'TxRate-103808k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1828)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1828,'TxRate-103936k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1829)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1829,'TxRate-104064k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1830)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1830,'TxRate-104192k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1831)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1831,'TxRate-104320k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1832)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1832,'TxRate-104448k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1833)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1833,'TxRate-104576k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1834)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1834,'TxRate-104704k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1835)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1835,'TxRate-104832k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1836)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1836,'TxRate-104960k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1837)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1837,'TxRate-105088k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1838)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1838,'TxRate-105216k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1839)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1839,'TxRate-105344k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1840)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1840,'TxRate-105472k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1841)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1841,'TxRate-105600k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1842)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1842,'TxRate-105728k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1843)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1843,'TxRate-105856k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1844)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1844,'TxRate-105984k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1845)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1845,'TxRate-106112k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1846)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1846,'TxRate-106240k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1847)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1847,'TxRate-106368k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1848)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1848,'TxRate-106496k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1849)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1849,'TxRate-106624k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1850)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1850,'TxRate-106752k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1851)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1851,'TxRate-106880k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1852)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1852,'TxRate-107008k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1853)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1853,'TxRate-107136k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1854)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1854,'TxRate-107264k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1855)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1855,'TxRate-107392k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1856)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1856,'TxRate-107520k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1857)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1857,'TxRate-107648k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1858)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1858,'TxRate-107776k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1859)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1859,'TxRate-107904k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1860)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1860,'TxRate-108032k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1861)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1861,'TxRate-108160k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1862)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1862,'TxRate-108288k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1863)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1863,'TxRate-108416k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1864)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1864,'TxRate-108544k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1865)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1865,'TxRate-108672k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1866)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1866,'TxRate-108800k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1867)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1867,'TxRate-108928k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1868)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1868,'TxRate-109056k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1869)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1869,'TxRate-109184k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1870)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1870,'TxRate-109312k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1871)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1871,'TxRate-109440k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1872)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1872,'TxRate-109568k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1873)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1873,'TxRate-109696k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1874)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1874,'TxRate-109824k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1875)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1875,'TxRate-109952k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1876)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1876,'TxRate-110080k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1877)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1877,'TxRate-110208k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1878)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1878,'TxRate-110336k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1879)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1879,'TxRate-110464k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1880)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1880,'TxRate-110592k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1881)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1881,'TxRate-110720k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1882)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1882,'TxRate-110848k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1883)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1883,'TxRate-110976k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1884)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1884,'TxRate-111104k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1885)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1885,'TxRate-111232k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1886)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1886,'TxRate-111360k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1887)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1887,'TxRate-111488k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1888)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1888,'TxRate-111616k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1889)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1889,'TxRate-111744k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1890)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1890,'TxRate-111872k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1891)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1891,'TxRate-112000k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1892)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1892,'TxRate-112128k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1893)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1893,'TxRate-112256k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1894)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1894,'TxRate-112384k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1895)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1895,'TxRate-112512k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1896)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1896,'TxRate-112640k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1897)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1897,'TxRate-112768k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1898)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1898,'TxRate-112896k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1899)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1899,'TxRate-113024k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1900)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1900,'TxRate-113152k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1901)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1901,'TxRate-113280k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1902)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1902,'TxRate-113408k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1903)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1903,'TxRate-113536k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1904)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1904,'TxRate-113664k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1905)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1905,'TxRate-113792k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1906)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1906,'TxRate-113920k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1907)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1907,'TxRate-114048k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1908)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1908,'TxRate-114176k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1909)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1909,'TxRate-114304k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1910)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1910,'TxRate-114432k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1911)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1911,'TxRate-114560k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1912)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1912,'TxRate-114688k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1913)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1913,'TxRate-114816k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1914)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1914,'TxRate-114944k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1915)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1915,'TxRate-115072k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1916)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1916,'TxRate-115200k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1917)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1917,'TxRate-115328k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1918)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1918,'TxRate-115456k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1919)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1919,'TxRate-115584k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1920)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1920,'TxRate-115712k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1921)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1921,'TxRate-115840k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1922)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1922,'TxRate-115968k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1923)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1923,'TxRate-116096k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1924)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1924,'TxRate-116224k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1925)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1925,'TxRate-116352k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1926)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1926,'TxRate-116480k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1927)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1927,'TxRate-116608k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1928)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1928,'TxRate-116736k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1929)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1929,'TxRate-116864k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1930)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1930,'TxRate-116992k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1931)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1931,'TxRate-117120k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1932)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1932,'TxRate-117248k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1933)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1933,'TxRate-117376k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1934)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1934,'TxRate-117504k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1935)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1935,'TxRate-117632k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1936)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1936,'TxRate-117760k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1937)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1937,'TxRate-117888k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1938)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1938,'TxRate-118016k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1939)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1939,'TxRate-118144k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1940)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1940,'TxRate-118272k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1941)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1941,'TxRate-118400k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1942)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1942,'TxRate-118528k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1943)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1943,'TxRate-118656k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1944)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1944,'TxRate-118784k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1945)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1945,'TxRate-118912k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1946)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1946,'TxRate-119040k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1947)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1947,'TxRate-119168k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1948)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1948,'TxRate-119296k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1949)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1949,'TxRate-119424k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1950)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1950,'TxRate-119552k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1951)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1951,'TxRate-119680k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1952)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1952,'TxRate-119808k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1953)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1953,'TxRate-119936k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1954)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1954,'TxRate-120064k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1955)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1955,'TxRate-120192k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1956)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1956,'TxRate-120320k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1957)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1957,'TxRate-120448k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1958)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1958,'TxRate-120576k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1959)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1959,'TxRate-120704k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1960)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1960,'TxRate-120832k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1961)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1961,'TxRate-120960k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1962)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1962,'TxRate-121088k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1963)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1963,'TxRate-121216k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1964)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1964,'TxRate-121344k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1965)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1965,'TxRate-121472k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1966)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1966,'TxRate-121600k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1967)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1967,'TxRate-121728k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1968)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1968,'TxRate-121856k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1969)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1969,'TxRate-121984k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1970)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1970,'TxRate-122112k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1971)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1971,'TxRate-122240k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1972)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1972,'TxRate-122368k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1973)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1973,'TxRate-122496k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1974)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1974,'TxRate-122624k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1975)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1975,'TxRate-122752k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1976)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1976,'TxRate-122880k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1977)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1977,'TxRate-123008k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1978)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1978,'TxRate-123136k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1979)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1979,'TxRate-123264k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1980)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1980,'TxRate-123392k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1981)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1981,'TxRate-123520k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1982)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1982,'TxRate-123648k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1983)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1983,'TxRate-123776k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1984)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1984,'TxRate-123904k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1985)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1985,'TxRate-124032k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1986)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1986,'TxRate-124160k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1987)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1987,'TxRate-124288k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1988)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1988,'TxRate-124416k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1989)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1989,'TxRate-124544k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1990)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1990,'TxRate-124672k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1991)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1991,'TxRate-124800k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1992)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1992,'TxRate-124928k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1993)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1993,'TxRate-125056k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1994)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1994,'TxRate-125184k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1995)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1995,'TxRate-125312k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1996)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1996,'TxRate-125440k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1997)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1997,'TxRate-125568k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1998)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1998,'TxRate-125696k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 1999)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,1999,'TxRate-125824k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2000)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2000,'TxRate-125952k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2001)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2001,'TxRate-126080k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2002)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2002,'TxRate-126208k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2003)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2003,'TxRate-126336k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2004)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2004,'TxRate-126464k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2005)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2005,'TxRate-126592k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2006)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2006,'TxRate-126720k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2007)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2007,'TxRate-126848k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2008)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2008,'TxRate-126976k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2009)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2009,'TxRate-127104k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2010)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2010,'TxRate-127232k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2011)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2011,'TxRate-127360k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2012)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2012,'TxRate-127488k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2013)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2013,'TxRate-127616k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2014)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2014,'TxRate-127744k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2015)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2015,'TxRate-127872k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2016)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2016,'TxRate-128000k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2017)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2017,'TxRate-128128k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2018)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2018,'TxRate-128256k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2019)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2019,'TxRate-128384k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2020)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2020,'TxRate-128512k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2021)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2021,'TxRate-128640k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2022)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2022,'TxRate-128768k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2023)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2023,'TxRate-128896k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2024)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2024,'TxRate-129024k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2025)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2025,'TxRate-129152k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2026)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2026,'TxRate-129280k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2027)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2027,'TxRate-129408k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2028)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2028,'TxRate-129536k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2029)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2029,'TxRate-129664k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2030)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2030,'TxRate-129792k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2031)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2031,'TxRate-129920k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2032)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2032,'TxRate-130048k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2033)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2033,'TxRate-130176k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2034)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2034,'TxRate-130304k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2035)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2035,'TxRate-130432k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2036)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2036,'TxRate-130560k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2037)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2037,'TxRate-130688k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2038)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2038,'TxRate-130816k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2039)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2039,'TxRate-130944k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2040)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2040,'TxRate-131072k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2041)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2041,'TxRate-131200k')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2042)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2042,'TxRate-131328k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2043)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2043,'TxRate-131456k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2044)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2044,'TxRate-131584k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2045)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2045,'TxRate-131712k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2046)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2046,'TxRate-131840k')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 152 AND Value = 2047)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(152,762,2047,'TxRate-131968k')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 762 AND RadAttributeID = 153)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(153,762,'KarlNet-TurboCell-OpState',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 153 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,762,0,'Up')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 153 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,762,1,'Down')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 762 AND RadAttributeID = 154)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(154,762,'KarlNet-TurboCell-OpMode',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 154 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(154,762,0,'Peer-to-Peer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 154 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(154,762,1,'Base')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 154 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(154,762,2,'Base-Polling')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 762 AND RadAttributeID = 154 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(154,762,3,'Satellite-NT')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 16445)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(16445,'Kineto')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,16445,'Kineto-UMA-Release-Indicator',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,16445,'Kineto-UMA-AP-Radio-Identity',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,16445,'Kineto-UMA-Cell-Identity',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,16445,'Kineto-UMA-Location-Area-Identification',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,16445,'Kineto-UMA-Coverage-Indicator',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,16445,'Kineto-UMA-Classmark',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,16445,'Kineto-UMA-Geographical-Location',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,16445,'Kineto-UMA-SGW-IP-Address',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,16445,'Kineto-UMA-SGW-FQDN',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,16445,'Kineto-UMA-Redirection-Counter',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,16445,'Kineto-UMA-Discovery-Reject-Cause',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,16445,'Kineto-UMA-RRC-State',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,16445,'Kineto-UMA-Register-Reject-Cause',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,16445,'Kineto-UMA-Routing-Area-Code',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,16445,'Kineto-UMA-AP-Location',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,16445,'Kineto-UMA-Location-Status',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,16445,'Kineto-UMA-Utran-Cell-Identity',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,16445,'Kineto-UMA-Location-Blacklist-Indicator',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,16445,'Kineto-UMA-AP-Service-Name',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,16445,'Kineto-UMA-Service-Zone-Information',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,16445,'Kineto-UMA-Serving-UNC-Table-Indicator',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,16445,'Kineto-UMA-Registration-Indicators',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,16445,'Kineto-UMA-UMA-PLMN-List',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,16445,'Kineto-UMA-Required-UMA-Services',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,16445,'Kineto-UMA-3G-Cell-Identity',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,16445,'Kineto-UMA-MS-Radio-Identity',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,16445,'Kineto-UMA-UNC-IP-Address',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,16445,'Kineto-UMA-UNC-FQDN',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65281)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65281,16445,'Kineto-URR-Transaction-Type',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65282)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65282,16445,'Kineto-Location-Key',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65283)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65283,16445,'Kineto-UP-Client-Remote-Address',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65284)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65284,16445,'Kineto-Hand-In-Control-Flag',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65285)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65285,16445,'Kineto-Hand-Out-Control-Flag',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65286)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65286,16445,'Kineto-Billing-Rate-Indicator',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65289)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65289,16445,'Kineto-Service-Area-Code',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65408)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65408,16445,'KW-IUH-MESSAGE-TYPE',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65409)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65409,16445,'KW-HNB-REMOTE-ADDRESS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65410)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65410,16445,'KW-HNB-IDENTITY',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65411)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65411,16445,'KW-HNB-LOC-INFO-MACRO-COVERAGE-IND',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65412)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65412,16445,'KW-HNB-LOC-INFO-GERAN-CELL-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65413)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65413,16445,'KW-HNB-LOC-INFO-UTRAN-CELL-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65414)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65414,16445,'KW-HNB-LOC-INFO-GEO-COORDINATES',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65415)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65415,16445,'KW-HNB-LOC-INFO-ALTITUDE-DIRECTION',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65416)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65416,16445,'KW-HNB-LOC-INFO-IP-ADDRESS',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65417)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65417,16445,'KW-HNB-PLMN-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65418)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65418,16445,'KW-HNB-CELL-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65419)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65419,16445,'KW-HNB-LAC',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65420)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65420,16445,'KW-HNB-RAC',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65421)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65421,16445,'KW-HNB-SAC',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65422)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65422,16445,'KW-HNB-CSG-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65423)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65423,16445,'KW-UE-CAPABILITIES',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65424)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65424,16445,'KW-HNB-LOCATION-AREA-IND',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65425)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65425,16445,'KW-IUH-BILLING-RATE-INDICATOR',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65426)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65426,16445,'KW-REGISTRATION-REJECT-CAUSE',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65427)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65427,16445,'KW-HNB-LOCATION-BLACKLIST-IND',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65428)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65428,16445,'KW-HNB-CELL-ACCESS-MODE',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16445 AND RadAttributeID = 65429)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65429,16445,'KW-UE-MEMBERSHIP-STATUS',6,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65411 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65411,16445,0,'GERAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65411 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65411,16445,1,'UTRAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65411 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65411,16445,2,'NONE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65414 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65414,16445,0,'North')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65414 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65414,16445,1,'South')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65415 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65415,16445,0,'Height')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65415 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65415,16445,1,'Depth')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65423 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65423,16445,0,'R99')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65423 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65423,16445,1,'Rel-4')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65423 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65423,16445,2,'Rel-5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65423 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65423,16445,3,'Rel-6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65423 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65423,16445,4,'Rel-7')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 16445 AND RadAttributeID = 65423 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65423,16445,5,'Rel-8')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 2356)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(2356,'Lancom')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2356 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,2356,'LCS-Traffic-Limit',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2356 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,2356,'LCS-Mac-Address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2356 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,2356,'LCS-Redirection-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2356 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,2356,'LCS-Comment',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2356 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,2356,'LCS-Account-End',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2356 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,2356,'LCS-WPA-Passphrase',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2356 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,2356,'LCS-PbSpotUserName',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2356 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,2356,'LCS-TxRateLimit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2356 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,2356,'LCS-RxRateLimit',1,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 307)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(307,'Livingston')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,307,'LE-Terminate-Detail',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,307,'LE-Advice-of-Charge',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,307,'LE-Connect-Detail',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,307,'LE-IP-Pool',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,307,'LE-IP-Gateway',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,307,'LE-Modem-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,307,'LE-IPSec-Log-Options',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,307,'LE-IPSec-Deny-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,307,'LE-IPSec-Active-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,307,'LE-IPSec-Outsource-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,307,'LE-IPSec-Passive-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,307,'LE-NAT-TCP-Session-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,307,'LE-NAT-Other-Session-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,307,'LE-NAT-Log-Options',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,307,'LE-NAT-Sess-Dir-Fail-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,307,'LE-NAT-Inmap',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,307,'LE-NAT-Outmap',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,307,'LE-NAT-Outsource-Inmap',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,307,'LE-NAT-Outsource-Outmap',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,307,'LE-Admin-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 307 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,307,'LE-Multicast-Client',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 10 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,307,1,'Drop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 10 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,307,2,'ICMP-Reject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 10 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(10,307,3,'Pass-Through')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 9 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,307,1,'SA-Success-On')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 9 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,307,2,'SA-Failure-On')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 9 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,307,3,'Console-On')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 9 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,307,4,'Syslog-On')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 9 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,307,5,'SA-Success-Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 9 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,307,6,'SA-Failure-Off')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 9 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,307,7,'Console-Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 9 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,307,8,'Syslog-Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 17 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(17,307,1,'Drop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 17 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(17,307,2,'ICMP-Reject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 17 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(17,307,3,'Pass-Through')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 16 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,307,1,'Session-Success-On')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 16 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,307,2,'Session-Failure-On')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 16 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,307,3,'Console-On')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 16 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,307,4,'Syslog-On')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 16 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,307,5,'Success-Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 16 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,307,6,'Failure-Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 16 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,307,7,'Console-Off')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 16 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(16,307,8,'Syslog-Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 307 AND RadAttributeID = 23 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,307,1,'On')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 4846)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(4846,'Lucent')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,4846,'Lucent-Max-Shared-Users',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,4846,'Lucent-IP-DSCP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,4846,'Lucent-X25-X121-Source-Address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,4846,'Lucent-PPP-Circuit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,4846,'Lucent-PPP-Circuit-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,4846,'Lucent-UU-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,4846,'Lucent-User-Priority',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,4846,'Lucent-CIR-Timer',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,4846,'Lucent-FR-08-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,4846,'Lucent-Destination-NAS-Port',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,4846,'Lucent-FR-SVC-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,4846,'Lucent-NAS-Port-Format',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,4846,'Lucent-ATM-Fault-Management',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,4846,'Lucent-ATM-Loopback-Cell-Loss',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,4846,'Lucent-Ckt-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,4846,'Lucent-SVC-Enabled',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,4846,'Lucent-Session-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,4846,'Lucent-H323-Gatekeeper',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,4846,'Lucent-Global-Call-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,4846,'Lucent-H323-Conference-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,4846,'Lucent-H323-Destination-NAS-ID',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,4846,'Lucent-H323-Dialed-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,4846,'Lucent-Dialed-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,4846,'Lucent-Inter-Arrival-Jitter',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,4846,'Lucent-Dropped-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,4846,'Lucent-Dropped-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,4846,'Lucent-Auth-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,4846,'Lucent-X25-Pad-X3-Profile',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,4846,'Lucent-X25-Pad-X3-Parameters',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,4846,'Lucent-Tunnel-VRouter-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,4846,'Lucent-X25-Reverse-Charging',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,4846,'Lucent-X25-Nui-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,4846,'Lucent-X25-Nui-Password-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,4846,'Lucent-X25-Cug',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,4846,'Lucent-X25-Pad-Alias-1',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,4846,'Lucent-X25-Pad-Alias-2',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,4846,'Lucent-X25-Pad-Alias-3',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,4846,'Lucent-X25-X121-Address',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,4846,'Lucent-X25-Nui',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,4846,'Lucent-X25-Rpoa',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,4846,'Lucent-X25-Pad-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,4846,'Lucent-X25-Pad-Banner',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,4846,'Lucent-X25-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,4846,'Lucent-Recv-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,4846,'Lucent-Bi-Directional-Auth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,4846,'Lucent-MTU',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,4846,'Lucent-Call-Direction',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,4846,'Lucent-Service-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,4846,'Lucent-Filter-Required',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,4846,'Lucent-Traffic-Shaper',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,4846,'Lucent-Access-Intercept-LEA',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,4846,'Lucent-Access-Intercept-Log',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,4846,'Lucent-Private-Route-Table-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,4846,'Lucent-Private-Route-Required',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,4846,'Lucent-Cache-Refresh',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,4846,'Lucent-Cache-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,4846,'Lucent-Egress-Enabled',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,4846,'Lucent-QOS-Upstream',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,4846,'Lucent-QOS-Downstream',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,4846,'Lucent-ATM-Connect-Vpi',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,4846,'Lucent-ATM-Connect-Vci',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,4846,'Lucent-ATM-Connect-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,4846,'Lucent-ATM-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,4846,'Lucent-IPX-Header-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,4846,'Lucent-Calling-Id-Type-Of-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,4846,'Lucent-Calling-Id-Numbering-Plan',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,4846,'Lucent-Calling-Id-Presentation',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,4846,'Lucent-Calling-Id-Screening',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,4846,'Lucent-BIR-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,4846,'Lucent-BIR-Proxy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,4846,'Lucent-BIR-Bridge-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,4846,'Lucent-IPSEC-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,4846,'Lucent-PPPoE-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,4846,'Lucent-Bridge-Non-PPPoE',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,4846,'Lucent-ATM-Direct',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 77)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(77,4846,'Lucent-ATM-Direct-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 78)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(78,4846,'Lucent-Client-Primary-WINS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,4846,'Lucent-Client-Secondary-WINS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,4846,'Lucent-Client-Assign-WINS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,4846,'Lucent-Auth-Type',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,4846,'Lucent-Port-Redir-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,4846,'Lucent-Port-Redir-Portnum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 84)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(84,4846,'Lucent-Port-Redir-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 85)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(85,4846,'Lucent-IP-Pool-Chaining',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 86)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(86,4846,'Lucent-Owner-IP-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 87)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(87,4846,'Lucent-IP-TOS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 88)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(88,4846,'Lucent-IP-TOS-Precedence',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 89)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(89,4846,'Lucent-IP-TOS-Apply-To',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,4846,'Lucent-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,4846,'Lucent-Telnet-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,4846,'Lucent-Dsl-Rate-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,4846,'Lucent-Redirect-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,4846,'Lucent-ATM-Vpi',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,4846,'Lucent-ATM-Vci',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,4846,'Lucent-Source-IP-Check',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,4846,'Lucent-Dsl-Rate-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,4846,'Lucent-Dsl-Upstream-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 99)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(99,4846,'Lucent-Dsl-Downstream-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,4846,'Lucent-Dsl-CIR-Recv-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,4846,'Lucent-Dsl-CIR-Xmit-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,4846,'Lucent-VRouter-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,4846,'Lucent-Source-Auth',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,4846,'Lucent-Private-Route',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,4846,'Lucent-Numbering-Plan-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,4846,'Lucent-FR-Link-Status-DLCI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,4846,'Lucent-Calling-Subaddress',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,4846,'Lucent-Callback-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,4846,'Lucent-Endpoint-Disc',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,4846,'Lucent-Remote-FW',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,4846,'Lucent-Multicast-GLeave-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,4846,'Lucent-CBCP-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,4846,'Lucent-CBCP-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,4846,'Lucent-CBCP-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(115,4846,'Lucent-CBCP-Trunk-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,4846,'Lucent-Appletalk-Route',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,4846,'Lucent-Appletalk-Peer-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,4846,'Lucent-Route-Appletalk',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(119,4846,'Lucent-FCP-Parameter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 120)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(120,4846,'Lucent-Modem-PortNo',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,4846,'Lucent-Modem-SlotNo',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,4846,'Lucent-Modem-ShelfNo',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,4846,'Lucent-Call-Attempt-Limit',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,4846,'Lucent-Call-Block-Duration',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,4846,'Lucent-Maximum-Call-Duration',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,4846,'Lucent-Route-Preference',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,4846,'Lucent-Tunneling-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,4846,'Lucent-Shared-Profile-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,4846,'Lucent-Primary-Home-Agent',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,4846,'Lucent-Secondary-Home-Agent',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,4846,'Lucent-Dialout-Allowed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,4846,'Lucent-Client-Gateway',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,4846,'Lucent-BACP-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 134)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(134,4846,'Lucent-DHCP-Maximum-Leases',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 135)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(135,4846,'Lucent-Client-Primary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 136)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(136,4846,'Lucent-Client-Secondary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 137)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(137,4846,'Lucent-Client-Assign-DNS',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 138)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(138,4846,'Lucent-User-Acct-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 139)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(139,4846,'Lucent-User-Acct-Host',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,4846,'Lucent-User-Acct-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,4846,'Lucent-User-Acct-Key',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,4846,'Lucent-User-Acct-Base',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,4846,'Lucent-User-Acct-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 144)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(144,4846,'Lucent-Assign-IP-Client',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 145)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(145,4846,'Lucent-Assign-IP-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 146)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(146,4846,'Lucent-Assign-IP-Global-Pool',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 147)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(147,4846,'Lucent-DHCP-Reply',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 148)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(148,4846,'Lucent-DHCP-Pool-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 149)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(149,4846,'Lucent-Expect-Callback',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 150)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(150,4846,'Lucent-Event-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 151)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(151,4846,'Lucent-Session-Svr-Key',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 152)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(152,4846,'Lucent-Multicast-Rate-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 153)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(153,4846,'Lucent-IF-Netmask',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 154)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(154,4846,'Lucent-Remote-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 155)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(155,4846,'Lucent-Multicast-Client',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 156)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(156,4846,'Lucent-FR-Circuit-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 157)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(157,4846,'Lucent-FR-LinkUp',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 158)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(158,4846,'Lucent-FR-Nailed-Grp',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 159)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(159,4846,'Lucent-FR-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 160)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(160,4846,'Lucent-FR-Link-Mgt',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 161)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(161,4846,'Lucent-FR-N391',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 162)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(162,4846,'Lucent-FR-DCE-N392',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 163)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(163,4846,'Lucent-FR-DTE-N392',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 164)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(164,4846,'Lucent-FR-DCE-N393',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 165)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(165,4846,'Lucent-FR-DTE-N393',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 166)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(166,4846,'Lucent-FR-T391',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 167)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(167,4846,'Lucent-FR-T392',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 168)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(168,4846,'Lucent-Bridge-Address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 169)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(169,4846,'Lucent-TS-Idle-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 170)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(170,4846,'Lucent-TS-Idle-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 171)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(171,4846,'Lucent-DBA-Monitor',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 172)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(172,4846,'Lucent-Base-Channel-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 173)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(173,4846,'Lucent-Minimum-Channels',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 174)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(174,4846,'Lucent-IPX-Route',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 175)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(175,4846,'Lucent-FT1-Caller',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 176)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(176,4846,'Lucent-Backup',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 177)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(177,4846,'Lucent-Call-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 178)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(178,4846,'Lucent-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 179)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(179,4846,'Lucent-FR-DLCI',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 180)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(180,4846,'Lucent-FR-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 181)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(181,4846,'Lucent-Ara-PW',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 182)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(182,4846,'Lucent-IPX-Node-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 183)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(183,4846,'Lucent-Home-Agent-IP-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 184)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(184,4846,'Lucent-Home-Agent-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 185)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(185,4846,'Lucent-Home-Network-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 186)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(186,4846,'Lucent-Home-Agent-UDP-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 187)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(187,4846,'Lucent-Multilink-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 188)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(188,4846,'Lucent-Num-In-Multilink',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 189)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(189,4846,'Lucent-First-Dest',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 190)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(190,4846,'Lucent-Pre-Input-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 191)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(191,4846,'Lucent-Pre-Output-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 192)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(192,4846,'Lucent-Pre-Input-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 193)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(193,4846,'Lucent-Pre-Output-Packets',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 194)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(194,4846,'Lucent-Maximum-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 195)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(195,4846,'Lucent-Disconnect-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 196)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(196,4846,'Lucent-Connect-Progress',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 197)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(197,4846,'Lucent-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 198)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(198,4846,'Lucent-PreSession-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 199)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(199,4846,'Lucent-Token-Idle',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 200)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(200,4846,'Lucent-Token-Immediate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 201)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(201,4846,'Lucent-Require-Auth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 202)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(202,4846,'Lucent-Number-Sessions',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 203)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(203,4846,'Lucent-Authen-Alias',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 204)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(204,4846,'Lucent-Token-Expiry',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 205)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(205,4846,'Lucent-Menu-Selector',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 206)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(206,4846,'Lucent-Menu-Item',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 207)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(207,4846,'Lucent-PW-Warntime',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 208)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(208,4846,'Lucent-PW-Lifetime',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 209)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(209,4846,'Lucent-IP-Direct',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 210)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(210,4846,'Lucent-PPP-VJ-Slot-Comp',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 211)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(211,4846,'Lucent-PPP-VJ-1172',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 212)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(212,4846,'Lucent-PPP-Async-Map',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 213)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(213,4846,'Lucent-Third-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 214)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(214,4846,'Lucent-Send-Secret',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 215)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(215,4846,'Lucent-Receive-Secret',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 216)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(216,4846,'Lucent-IPX-Peer-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 217)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(217,4846,'Lucent-IP-Pool-Definition',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 218)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(218,4846,'Lucent-Assign-IP-Pool',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 219)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(219,4846,'Lucent-FR-Direct',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 220)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(220,4846,'Lucent-FR-Direct-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 221)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(221,4846,'Lucent-FR-Direct-DLCI',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 222)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(222,4846,'Lucent-Handle-IPX',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 223)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(223,4846,'Lucent-Netware-timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 224)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(224,4846,'Lucent-IPX-Alias',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 225)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(225,4846,'Lucent-Metric',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 226)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(226,4846,'Lucent-PRI-Number-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 227)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(227,4846,'Lucent-Dial-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 228)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(228,4846,'Lucent-Route-IP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 229)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(229,4846,'Lucent-Route-IPX',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 230)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(230,4846,'Lucent-Bridge',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 231)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(231,4846,'Lucent-Send-Auth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 232)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(232,4846,'Lucent-Send-Passwd',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 233)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(233,4846,'Lucent-Link-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 234)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(234,4846,'Lucent-Target-Util',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 235)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(235,4846,'Lucent-Maximum-Channels',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 236)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(236,4846,'Lucent-Inc-Channel-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 237)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(237,4846,'Lucent-Dec-Channel-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 238)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(238,4846,'Lucent-Seconds-Of-History',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 239)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(239,4846,'Lucent-History-Weigh-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 240)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(240,4846,'Lucent-Add-Seconds',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 241)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(241,4846,'Lucent-Remove-Seconds',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 242)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(242,4846,'Lucent-Data-Filter',4,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 243)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(243,4846,'Lucent-Call-Filter',4,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 244)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(244,4846,'Lucent-Idle-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 245)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(245,4846,'Lucent-Preempt-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 246)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(246,4846,'Lucent-Callback',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 247)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(247,4846,'Lucent-Data-Svc',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 248)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(248,4846,'Lucent-Force-56',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 249)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(249,4846,'Lucent-Billing-Number',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 250)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(250,4846,'Lucent-Call-By-Call',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 251)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(251,4846,'Lucent-Transit-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 252)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(252,4846,'Lucent-Host-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 253)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(253,4846,'Lucent-PPP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 254)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(254,4846,'Lucent-MPP-Idle-Percent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 255)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(255,4846,'Lucent-Xmit-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 256)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(256,4846,'Lucent-Fr05-Traffic-Shaper',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 257)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(257,4846,'Lucent-Fr05-Vpi',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 258)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(258,4846,'Lucent-Fr05-Vci',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 259)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(259,4846,'Lucent-Fr05-Enabled',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 260)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(260,4846,'Lucent-Tunnel-Auth-Type',11,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 261)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(261,4846,'Lucent-MOH-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 262)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(262,4846,'Lucent-ATM-Circuit-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 263)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(263,4846,'Lucent-Priority-For-PPP',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 264)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(264,4846,'Lucent-Max-RTP-Delay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 265)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(265,4846,'Lucent-RTP-Port-Range',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 266)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(266,4846,'Lucent-TOS-Copying',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 267)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(267,4846,'Lucent-Packet-Classification',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 268)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(268,4846,'Lucent-No-High-Prio-Pkt-Duratio',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 269)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(269,4846,'Lucent-AT-Answer-String',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 270)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(270,4846,'Lucent-IP-OUTGOING-TOS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 271)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(271,4846,'Lucent-IP-OUTGOING-TOS-Precedence',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 272)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(272,4846,'Lucent-IP-OUTGOING-DSCP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 273)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(273,4846,'Lucent-TermSrv-Login-Prompt',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 274)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(274,4846,'Lucent-Multicast-Service-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 275)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(275,4846,'Lucent-Multicast-Max-Groups',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 276)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(276,4846,'Lucent-Multicast-Service-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 277)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(277,4846,'Lucent-Multicast-Service-Active',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 278)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(278,4846,'Lucent-Multicast-Service-Snmp-Trap',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 279)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(279,4846,'Lucent-Multicast-Service-Filter-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 280)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(280,4846,'Lucent-Multicast-Filter-Active',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 281)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(281,4846,'Lucent-Multicast-Filter-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 282)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(282,4846,'Lucent-Tunnel-TOS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 283)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(283,4846,'Lucent-Tunnel-TOS-Precedence',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 284)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(284,4846,'Lucent-Tunnel-DSCP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 285)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(285,4846,'Lucent-Tunnel-TOS-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 286)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(286,4846,'Lucent-Tunnel-TOS-Copy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 287)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(287,4846,'Lucent-Http-Redirect-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 288)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(288,4846,'Lucent-Http-Redirect-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 289)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(289,4846,'Lucent-L2TP-DCI-Disconnect-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 290)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(290,4846,'Lucent-L2TP-DCI-Protocol-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 291)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(291,4846,'Lucent-L2TP-DCI-Direction',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 292)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(292,4846,'Lucent-L2TP-DCI-Message',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 293)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(293,4846,'Lucent-L2TP-Q931-Cause-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 294)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(294,4846,'Lucent-L2TP-Q931-Cause-Message',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 295)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(295,4846,'Lucent-L2TP-Q931-Advisory-Message',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 296)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(296,4846,'Lucent-L2TP-RC-Result-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 297)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(297,4846,'Lucent-L2TP-RC-Error-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 298)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(298,4846,'Lucent-L2TP-RC-Error-Message',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 299)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(299,4846,'Lucent-L2TP-Disconnect-Scenario',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 300)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(300,4846,'Lucent-L2TP-Peer-Disconnect-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 301)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(301,4846,'Lucent-L2TP-Peer-Connect-Progress',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 302)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(302,4846,'Lucent-QuickConnect-Attempted',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 303)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(303,4846,'Lucent-Num-Moh-Sessions',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 304)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(304,4846,'Lucent-Cumulative-Hold-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 305)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(305,4846,'Lucent-Modem-Modulation',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 306)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(306,4846,'Lucent-User-Acct-Expiration',3,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 307)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(307,4846,'Lucent-User-Login-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 308)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(308,4846,'Lucent-First-Level-User',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 309)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(309,4846,'Lucent-IP-Source-If',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 310)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(310,4846,'Lucent-Reverse-Path-Check',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 321)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(321,4846,'Lucent-LCP-Keepalive-Period',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 322)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(322,4846,'Lucent-LCP-Keepalive-Missed-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10000)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10000,4846,'Lucent-Dsl-Atuc-Chan-Uncorrect-Blks',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10001)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10001,4846,'Lucent-Dsl-Atuc-Chan-Corrected-Blks',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10002)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10002,4846,'Lucent-Dsl-Atuc-Chan-Xmit-Blks',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10003)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10003,4846,'Lucent-Dsl-Atuc-Chan-Recd-Blks',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10004)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10004,4846,'Lucent-Dsl-Atuc-Perf-Inits',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10005)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10005,4846,'Lucent-Dsl-Atuc-Perf-ESs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10006)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10006,4846,'Lucent-Dsl-Atuc-Perf-Lprs',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10007)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10007,4846,'Lucent-Dsl-Atuc-Perf-Lols',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10008)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10008,4846,'Lucent-Dsl-Atuc-Perf-Loss',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10009)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10009,4846,'Lucent-Dsl-Atuc-Perf-Lofs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10010)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10010,4846,'Lucent-Dsl-Atuc-Curr-Attainable-Rate-Dn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10011)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10011,4846,'Lucent-Dsl-Atuc-Curr-Output-Pwr-Dn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10012)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10012,4846,'Lucent-Dsl-Atuc-Curr-Atn-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10013)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10013,4846,'Lucent-Dsl-Atuc-Curr-Snr-Mgn-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10014)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10014,4846,'Lucent-Dsl-Atuc-PS-Fast-Retrains',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10015)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10015,4846,'Lucent-Dsl-Atuc-PS-Failed-Fast-Retrains',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10016)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10016,4846,'Lucent-Dsl-Code-Violations',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10017)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10017,4846,'Lucent-Line-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10018)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10018,4846,'Lucent-Dsl-Curr-Up-Rate',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10019)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10019,4846,'Lucent-Dsl-Curr-Dn-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10020)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10020,4846,'Lucent-Dsl-Physical-Slot',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10021)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10021,4846,'Lucent-Dsl-Physical-Line',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10022)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10022,4846,'Lucent-Dsl-If-Index',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10023)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10023,4846,'Lucent-Dsl-Oper-Status',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10024)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10024,4846,'Lucent-Dsl-Related-If-Index',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10025)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10025,4846,'Lucent-Dsl-Atuc-Curr-Attainable-Rate-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10026)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10026,4846,'Lucent-Dsl-Atuc-Curr-Output-Pwr-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10027)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10027,4846,'Lucent-Dsl-Atuc-Curr-Atn-Dn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10028)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10028,4846,'Lucent-Dsl-Atuc-Curr-Snr-Mgn-D',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10029)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10029,4846,'Lucent-Dsl-Related-Slot',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10030)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10030,4846,'Lucent-Dsl-Related-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10031)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10031,4846,'Lucent-Dsl-Sparing-Role',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10032)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10032,4846,'Lucent-Absolute-Time',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10033)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10033,4846,'Lucent-Configured-Rate-Up-Min',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10034)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10034,4846,'Lucent-Configured-Rate-Up-Max',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10035)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10035,4846,'Lucent-Configured-Rate-Dn-Min',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10036)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10036,4846,'Lucent-Configured-Rate-Dn-Max',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10037)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10037,4846,'Lucent-Dsl-Physical-Channel',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10100,4846,'Lucent-Sonet-Section-ESs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10101,4846,'Lucent-Sonet-Section-SESs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10102,4846,'Lucent-Sonet-Section-SEFSs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10103,4846,'Lucent-Sonet-Section-CVs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10104,4846,'Lucent-Sonet-Line-ESs-Near',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10105,4846,'Lucent-Sonet-Line-SESs-Near',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10106,4846,'Lucent-Sonet-Line-CVs-Near',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10107,4846,'Lucent-Sonet-Line-USs-Near',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10108,4846,'Lucent-Sonet-Line-ESs-Far',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10109,4846,'Lucent-Sonet-Line-SESs-Far',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10110,4846,'Lucent-Sonet-Line-CVs-Far',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10111,4846,'Lucent-Sonet-Line-USs-Far',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10112,4846,'Lucent-Sonet-Path-ESs-Near',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10113,4846,'Lucent-Sonet-Path-SESs-Near',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10114,4846,'Lucent-Sonet-Path-CVs-Near',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10115,4846,'Lucent-Sonet-Path-USs-Near',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10116,4846,'Lucent-Sonet-Path-ESs-Far',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10117,4846,'Lucent-Sonet-Path-SESs-Far',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10118,4846,'Lucent-Sonet-Path-CVs-Far',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10119,4846,'Lucent-Sonet-Path-USs-Far',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10200)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10200,4846,'Lucent-Ds3-F-Bit-Err',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10201)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10201,4846,'Lucent-Ds3-P-Bit-Err',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10202)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10202,4846,'Lucent-Ds3-CCVs',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10203)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10203,4846,'Lucent-Ds3-PESs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10204)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10204,4846,'Lucent-Ds3-PSESs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10205)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10205,4846,'Lucent-Ds3-SEFs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10206)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10206,4846,'Lucent-Ds3-UASs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10207)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10207,4846,'Lucent-Ds3-LCVs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10208)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10208,4846,'Lucent-Ds3-PCVs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10209)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10209,4846,'Lucent-Ds3-LESs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10210)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10210,4846,'Lucent-Ds3-CESs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10211)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10211,4846,'Lucent-Ds3-CSESs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10300)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10300,4846,'Lucent-Rtp-Local-Number-Of-Samples',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10301)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10301,4846,'Lucent-Rtp-Remote-Number-Of-Samples',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10302)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10302,4846,'Lucent-Rtp-Local-Jitter-Minimum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10303)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10303,4846,'Lucent-Rtp-Local-Jitter-Maximum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10304)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10304,4846,'Lucent-Rtp-Local-Jitter-Mean',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10305)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10305,4846,'Lucent-Rtp-Local-Jitter-Variance',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10306)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10306,4846,'Lucent-Rtp-Local-Delay-Minimum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10307)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10307,4846,'Lucent-Rtp-Local-Delay-Maximum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10308)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10308,4846,'Lucent-Rtp-Local-Delay-Mean',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10309)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10309,4846,'Lucent-Rtp-Local-Delay-Variance',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10310)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10310,4846,'Lucent-Rtp-Local-Packets-Sent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10311)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10311,4846,'Lucent-Rtp-Local-Packets-Lost',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10312)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10312,4846,'Lucent-Rtp-Local-Packets-Late',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10313)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10313,4846,'Lucent-Rtp-Local-Bytes-Sent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10314)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10314,4846,'Lucent-Rtp-Local-Silence-Percent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10315)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10315,4846,'Lucent-Rtp-Remote-Jitter-Minimum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10316)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10316,4846,'Lucent-Rtp-Remote-Jitter-Maximum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10317)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10317,4846,'Lucent-Rtp-Remote-Jitter-Mean',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10318)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10318,4846,'Lucent-Rtp-Remote-Jitter-Variance',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10319)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10319,4846,'Lucent-Rtp-Remote-Delay-Minimum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10320)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10320,4846,'Lucent-Rtp-Remote-Delay-Maximum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10321)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10321,4846,'Lucent-Rtp-Remote-Delay-Mean',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10322)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10322,4846,'Lucent-Rtp-Remote-Delay-Variance',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10323)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10323,4846,'Lucent-Rtp-Remote-Packets-Sent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10324)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10324,4846,'Lucent-Rtp-Remote-Packets-Lost',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10325)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10325,4846,'Lucent-Rtp-Remote-Packets-Late',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10326)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10326,4846,'Lucent-Rtp-Remote-Bytes-Sent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 10327)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10327,4846,'Lucent-Rtp-Remote-Silence-Percent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 19999)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19999,4846,'Lucent-Tunnel-Auth-Type2',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20000)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20000,4846,'Lucent-Multi-Packet-Separator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20100,4846,'Lucent-Min-Xmit-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20101,4846,'Lucent-Max-Xmit-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20102,4846,'Lucent-Min-Recv-Rate',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20103,4846,'Lucent-Max-Recv-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20104,4846,'Lucent-Error-Correction-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20105,4846,'Lucent-Compression-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20106,4846,'Lucent-Modulation',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20107,4846,'Lucent-Xmit-Symbol-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20108,4846,'Lucent-Recv-Symbol-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20109,4846,'Lucent-Current-Xmit-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20110,4846,'Lucent-Current-Recv-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20111,4846,'Lucent-Current-Line-Quality',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20112,4846,'Lucent-Current-SNR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20113,4846,'Lucent-Min-SNR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20114,4846,'Lucent-Max-SNR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20115,4846,'Lucent-Local-Retrain-Requested',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20116,4846,'Lucent-Remote-Retrain-Requested',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20117,4846,'Lucent-Connection-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20118,4846,'Lucent-Modem-Disconnect-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 4846 AND RadAttributeID = 20119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20119,4846,'Lucent-Retrain-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 616)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(616,'MCI')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,616,'BillClass',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,616,'TermClass',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,616,'TermNodePort',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,616,'TimeZoneDesc',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,616,'WhoByte',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,616,'TCPPortNum',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,616,'ExtendedBill',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,616,'TelnetSourceIP',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,616,'NodeNumber',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,616,'TCPSourcePort',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 616 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,616,'ISOCountry',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,0,'Continental-USA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,1,'Unknown-/-Default')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,2,'1-800-USA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,6,'Alaska-Hawaii')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,7,'United-Kingdom')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,8,'Germany')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,9,'Canada')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,10,'Japan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,12,'Switzerland')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,18,'Wireless-USA')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,23,'Brazil')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,24,'Netherlands')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,25,'France')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,26,'Belgium')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,27,'Hong-Kong')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,28,'NWCanada')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,29,'Sweden')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,30,'Austria')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,31,'Italy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,32,'Spain')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,34,'Mexico')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,35,'Singapore')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,36,'Denmark')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,37,'Ireland')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,38,'Taiwan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,39,'Hungary')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,40,'Luxembourg')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,41,'New-Zealand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,42,'Australia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,43,'1-800-Mexico')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,44,'Norway')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,45,'Finland')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,46,'Caribbean')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,47,'Venezuela')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,48,'Colombia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,49,'Thailand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,50,'Malaysia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,51,'Poland')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,52,'India')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,53,'Puerto-Rico')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,54,'Argentina')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,55,'Chili')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,57,'Panama')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,58,'South-Korea')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 59)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,59,'Bermuda')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,60,'South-Africa')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 61)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,61,'Portugal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 62)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,62,'Indonesia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,63,'Philippines')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,64,'China')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 65)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,65,'Czechre-Republic')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 66)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,66,'Greece')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 67)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,67,'United-Arabemirates')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 68)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,68,'Saudia-Arabia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 69)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,69,'Russia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 70)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,70,'Turkey')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 1 AND Value = 71)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,616,71,'Israel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,0,'Normal-async-Dialup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,2,'Normal-WATS-dialup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,3,'Normal-gateway-interconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,5,'Restricted-WATS-dialup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,7,'Auto-connect-port-(TRW)')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,8,'Normal-async-POS-dialup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,9,'SYNC-ISDN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,10,'SYNC-ISDN-WATS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,11,'Normal-gateway-POS-interconne')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,12,'Normal-ISDN-connect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,13,'Normal-ISDN-WATS-connect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,16,'Pseudo-TTY-(Independent-Proce')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,17,'Pseudo-TTY-(Independent-Proce')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,18,'Pseudo-TTY-(Independent-Proce')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,19,'Pseudo-TTY-(Independent-Proce')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,20,'Pseudo-TTY-(Independent-Proce')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,21,'Pseudo-TTY-(Independent-Proce')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,22,'Pseudo-TTY-(Independent-Proce')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,23,'Pseudo-TTY-(Independent-Proce')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,32,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,33,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,34,'Hard-wired-semi-privileged-te')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,35,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,36,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,37,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,38,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,39,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,40,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,41,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,42,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,43,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,44,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,45,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,46,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,47,'Hard-wired-semi-privileged-te')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,48,'Host-console-terminals')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,49,'Host-console-terminals')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,50,'Host-console-terminals')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,51,'Host-console-terminals')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,52,'Host-console-terminals')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,53,'Host-console-terminals')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,54,'Host-console-terminals')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 6 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,616,55,'Host-console-terminals')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,0,'Non-Gateway')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1,'IP-Gateway')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,9,'Bell-Atlantic-Mobile')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,63,'Equant')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,64,'MCI-WAN-X25')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 67)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,67,'Private-Customers-X25')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 88)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,88,'MCI-(tymnet)')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 103)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,103,'Guam/POS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 110)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,110,'VISA-POS-Dev')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 118)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,118,'VISA-VPL-Gateway')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 128)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,128,'InfoNet-World')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 140)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,140,'Venezuela-Net-in-Columbia-<BO')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 142)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,142,'Global-One')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 184)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,184,'Verizon-(CDPD)-<AXV>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 239)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,239,'dead-MCII')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 262)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,262,'MCI-WAN-PPP-Plant')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 263)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,263,'AOL-Host-Server-Nodes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 295)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,295,'Visa-VPL-Project')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 379)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,379,'dead-Global-One')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 391)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,391,'TTN-NET-<TIX-YVR>')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 399)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,399,'dead-Bell-South')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 408)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,408,'Venezuela-Net-<CUJ-CTZ>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 486)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,486,'AOL-Host-Server-Nodes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 580)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,580,'dead-Trendline')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 614)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,614,'dead-EIRPAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 615)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,615,'dead-SEVA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 703)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,703,'Argentina-Advance-<MIA>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 721)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,721,'GRIC-(IP-Gateway)')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 722)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,722,'GRIC-POLAND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 723)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,723,'IFX-CHILE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 725)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,725,'GRIC-PHILIPPINES')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 726)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,726,'GRIC-BAHRAIN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 727)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,727,'Toll-Free-Satyam-India')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 728)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,728,'GRIC-TURKEY')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 729)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,729,'GRIC-HONDURAS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 730)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,730,'GRIC-BERMUDA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 731)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,731,'GRIC-PAPUA-NEW-GUINEA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 732)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,732,'GRIC-ANGOLA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 733)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,733,'GRIC-ARGENTINA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 734)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,734,'GRIC-CHINA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 735)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,735,'GRIC-COLOMBIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 736)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,736,'GRIC-EGYPT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 737)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,737,'GRIC-ESTONIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 738)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,738,'GRIC-PERU')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 739)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,739,'GRIC-GUAM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 740)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,740,'GRIC-ICELAND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 741)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,741,'GRIC-INDIA')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 742)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,742,'GRIC-ECUADOR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 743)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,743,'GRIC-ISRAEL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 744)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,744,'GRIC-JAMAICA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 745)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,745,'GRIC-KAZAKHSTAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 746)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,746,'GRIC-KENYA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 747)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,747,'GRIC-LITHUANIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 748)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,748,'GRIC-UNITED-ARAB-EMIRATES')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 749)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,749,'GRIC-RUSSIAN-FEDERATION')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 750)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,750,'GRIC-SLOVENIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 751)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,751,'GRIC-BOLIVIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 752)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,752,'GRIC-ROMANIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 753)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,753,'GRIC-UKRAINE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 754)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,754,'GRIC-UZBEKISTAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 755)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,755,'GRIC-VENEZUELA')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 756)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,756,'GRIC-Vietnam')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 757)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,757,'GRIC-COSTA-RICA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 758)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,758,'GRIC-DOMINICAN-REPUBLIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 759)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,759,'GRIC-LEBANON')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 760)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,760,'GRIC-GUATEMALA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 761)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,761,'GRIC-BELARUS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 762)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,762,'GRIC-CROATIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 763)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,763,'GRIC-SAUDI-ARABIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 764)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,764,'GRIC-BULGARIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 765)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,765,'GRIC-NIGERIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 766)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,766,'GRIC-ALGERIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 767)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,767,'GRIC-BAHAMAS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 768)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,768,'GRIC-HUNGARY')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 769)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,769,'GRIC-KUWAIT')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 770)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,770,'GRIC-NEPAL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 771)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,771,'GRIC-YUGOSLAVIA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 773)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,773,'GRIC-JORDAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 774)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,774,'GRIC-BANGLADESH')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 775)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,775,'GRIC-COTE-DLVOIRE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 776)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,776,'GRIC-GHANA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 777)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,777,'GRIC-MACAO')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 778)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,778,'GRIC-PAKISTAN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 779)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,779,'GRIC-UGANDA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 780)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,780,'GRIC-Cyprus')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 781)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,781,'GRIC-Northern-Mariana-Islands')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 782)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,782,'GRIC-Japan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 874)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,874,'Satyam-<SRE>')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 907)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,907,'Transpac-dead')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 913)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,913,'UUNET-Germany-<10-Nodes>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 915)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,915,'Equant-CNS-<PAX>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 981)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,981,'BT-Spain-<BCT-MRL>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 982)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,982,'UUNET-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 983)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,983,'JiTong-<KNH-TJI>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 984)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,984,'Gridnet-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 985)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,985,'NetONE-<TLC>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 986)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,986,'Infonet-Europe-IP-<FRK-FRL>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 987)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,987,'Infonet-Russia-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 988)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,988,'Infonet-Asia-IP-<KOS-KOT>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 989)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,989,'Infonet-Latin-America-IP-<GIK')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 990)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,990,'MCI-WORLDCOM-IRC-<MUG-MUH>')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 991)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,991,'Embratel-Brazil')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 992)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,992,'Avantel-Mexico')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 993)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,993,'800-Embratel-Brazil')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 994)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,994,'800-Aventel-Mexico')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1000)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1000,'Equant-IP-Bolivia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1001)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1001,'Equant-IP-Columbia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1002)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1002,'Equant-IP-Costa-Rica')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1003)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1003,'Equant-IP-Ecuador')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1004)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1004,'Equant-IP-El-Salvador')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1005)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1005,'Equant-IP-Guatemala')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1006)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1006,'Equant-IP-Honduras')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1007)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1007,'Equant-IP-Nicaragua')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1008)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1008,'Equant-IP-Uruguay')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1009)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1009,'Equant-IP-Bahrain')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1010)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1010,'Equant-IP-Bulgaria')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1011)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1011,'Equant-IP-Croatia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1012)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1012,'Equant-IP-Cyprus')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1013)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1013,'Equant-IP-Egypt')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1014)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1014,'Equant-IP-Estonia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1015)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1015,'Equant-IP-Iceland')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1016)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1016,'Equant-IP-Israel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1017)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1017,'Equant-IP-Latvia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1018)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1018,'Equant-IP-Malta')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1019)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1019,'Equant-IP-Pakistan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1020)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1020,'Equant-IP-Poland')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1021)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1021,'Equant-IP-Romania')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1022)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1022,'Equant-IP-Russia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1023)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1023,'Equant-IP-Slovakia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1024)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1024,'Equant-IP-Slovenia')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1025)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1025,'Equant-IP-Turkey')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1026)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1026,'Equant-IP-UAE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1027)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1027,'Equant-IP-Ukraine')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1028)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1028,'Equant-IP-Philippines')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1029)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1029,'Equant-IP-Sri-Lanka')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1030)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1030,'Equant-IP-Ivory-Coast')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1031)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1031,'Equant-IP-Kazakhstan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1032)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1032,'IFX-Venezuela')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1033)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1033,'IFX-Colombia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1034)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1034,'IFX-Uruguay')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1035)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1035,'Wayport-USA-Wireless-was-Htl')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1036)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1036,'Wayport-USA-Wireless-was-Air')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1037)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1037,'800-JiTong')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1038)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1038,'Wayport-USA-Wireless-was-Cfe')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1039)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1039,'Wayport-USA-LAN-was-Htl')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1040)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1040,'Wayport-USA-LAN-was-Air')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1041)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1041,'Wayport-USA-LAN-was-cafe')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1042)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1042,'DAN-India')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1043)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1043,'Toll-Free-Equant-IP-Malaysia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1044)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1044,'Prima-Argentina')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1045)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1045,'Toll-Free-Equant-IP-Israel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1046)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1046,'Toll-Free-Equant-IP-S-Korea')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1047)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1047,'Toll-Free-Equant-IP-Australia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1048)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1048,'Toll-Free-Equant-IP-Nw-Zealand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1049)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1049,'Toll-Free-Equant-IP-Singapore')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1050)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1050,'Golden-Telecom-Russia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1051)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1051,'Equant-IP-USA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1052)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1052,'Toll-Free-Equant-IP-Poland')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1053)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1053,'Toll-Free-Eqt-IP-So-Africa')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1054)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1054,'Equant-IP-Lithuania')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1055)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1055,'Equant-IP-Jordan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1056)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1056,'Equant-IP-Saudi-Arabia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1057)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1057,'Equant-IP-Kenya')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1058)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1058,'Boingo-Wireless-Argentina')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1059)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1059,'Boingo-Wireless-Australia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1060)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1060,'Boingo-Wireless-Brazil')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1061)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1061,'Boingo-Wireless-Canada')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1062)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1062,'Boingo-Wireless-India')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1063)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1063,'Boingo-Wireless-Mexico')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1064)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1064,'Boingo-Wireless-Portugal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1065)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1065,'Boingo-Wireless-UK')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1066)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1066,'Boingo-Wireless-USA')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1067)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1067,'Errant-Billing-Whobyte')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1068)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1068,'Boingo-Wireless-Poland')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1069)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1069,'Boingo-Wireless-Japan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1070)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1070,'Boingo-Wireless-Austria')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1071)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1071,'Boingo-Wireless-China')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1072)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1072,'Boingo-Wireless-France')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1073)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1073,'Boingo-Wireless-Germany')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1074)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1074,'Boingo-Wireless-Italy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1075)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1075,'Boingo-Wireless-Kazakhstan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1076)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1076,'Boingo-Wireless-Korea')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1077)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1077,'Boingo-Wireless-Malaysia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1078)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1078,'Boingo-Wireless-Netherlands')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1079)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1079,'Boingo-Wireless-Norway')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1080)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1080,'Boingo-Wireless-Philippines')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1081)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1081,'Boingo-Wireless-Russia')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1082)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1082,'Boingo-Wireless-Singapore')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1083)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1083,'Boingo-Wireless-South-Africa')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1084)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1084,'Boingo-Wireless-Switzerland')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1085)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1085,'Boingo-Wireless-Taiwan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1086)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1086,'Boingo-Wireless-Turkey')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1087)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1087,'Boingo-Wireless-United-Arab-Em')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1088)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1088,'Equant-IP-Nigeria')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1089)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1089,'Equant-IP-Brazil')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1090)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1090,'Toll-Free-Equant-IP-Brazil')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1091)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1091,'Equant-IP-Senegal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1092)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1092,'Equant-IP-Uzbekistan')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1093)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1093,'IFX-Honduras')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1094)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1094,'IFX-El-Salvador')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1095)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1095,'IFX-Guatemala')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1096)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1096,'Equant-IP-Oman')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1097)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1097,'Equant-IP-Kuwait')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1098)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1098,'Boingo-Wireless-Aruba')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1099)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1099,'Boingo-Wireless-Denmark')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1100)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1100,'Boingo-Wireless-Ireland')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1101,'Boingo-Wireless-Kuwait')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1102,'Boingo-Wireless-Spain')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1103)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1103,'Boingo-Wireless-Tanzania')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1104)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1104,'Boingo-Hotel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1105)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1105,'Boingo-Other-Wireless')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1106)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1106,'Boingo-Premium-A')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1107)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1107,'Boingo-Premium-B')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1108)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1108,'AirPath-Hotel')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1109)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1109,'AirPath-Other-Wireless')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1110)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1110,'Wayport-Hotel-Wired')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1111)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1111,'Wayport-Hotel-Common-Area')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1112)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1112,'Wayport-Hotel-All-Wireless')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 616 AND RadAttributeID = 13 AND Value = 1113)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,616,1113,'Wayport-Other-Wireless')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 61)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(61,'Merit')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 61 AND RadAttributeID = 211)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(211,61,'Merit-Proxy-Action',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 61 AND RadAttributeID = 222)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(222,61,'Merit-User-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 61 AND RadAttributeID = 223)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(223,61,'Merit-User-Realm',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 311)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(311,'Microsoft')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,311,'MS-CHAP-Response',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,311,'MS-CHAP-Error',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,311,'MS-CHAP-CPW-1',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,311,'MS-CHAP-CPW-2',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,311,'MS-CHAP-LM-Enc-PW',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,311,'MS-CHAP-NT-Enc-PW',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,311,'MS-MPPE-Encryption-Policy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,311,'MS-MPPE-Encryption-Type',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,311,'MS-RAS-Vendor',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,311,'MS-CHAP-Domain',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,311,'MS-CHAP-Challenge',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,311,'MS-CHAP-MPPE-Keys',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,311,'MS-BAP-Usage',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,311,'MS-Link-Utilization-Threshold',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,311,'MS-Link-Drop-Time-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,311,'MS-MPPE-Send-Key',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,311,'MS-MPPE-Recv-Key',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,311,'MS-RAS-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,311,'MS-Old-ARAP-Password',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,311,'MS-New-ARAP-Password',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,311,'MS-ARAP-PW-Change-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,311,'MS-Filter',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,311,'MS-Acct-Auth-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,311,'MS-Acct-EAP-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,311,'MS-CHAP2-Response',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,311,'MS-CHAP2-Success',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,311,'MS-CHAP2-CPW',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,311,'MS-Primary-DNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,311,'MS-Secondary-DNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,311,'MS-Primary-NBNS-Server',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,311,'MS-Secondary-NBNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,311,'MS-RAS-Client-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,311,'MS-RAS-Client-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,311,'MS-Quarantine-IPFilter',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,311,'MS-Quarantine-Session-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,311,'MS-User-Security-Identity',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,311,'MS-Identity-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,311,'MS-Service-Class',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,311,'MS-Quarantine-User-Class',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,311,'MS-Quarantine-State',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,311,'MS-Quarantine-Grace-Time',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,311,'MS-Network-Access-Server-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,311,'MS-AFW-Zone',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 48 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,311,1,'MS-AFW-Zone-Boundary-Policy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 48 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,311,2,'MS-AFW-Zone-Unprotected-Policy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 48 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,311,3,'MS-AFW-Zone-Protected-Policy')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,311,'MS-AFW-Protection-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 49 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,311,1,'HECP-Response-Sign-Only')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 49 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,311,2,'HECP-Response-Sign-And-Encrypt')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,311,'MS-Machine-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,311,'MS-IPv6-Filter',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,311,'MS-IPv4-Remediation-Servers',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,311,'MS-IPv6-Remediation-Servers',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,311,'MS-RNAP-Not-Quarantine-Capable',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 54 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(54,311,0,'SoH-Sent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 54 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(54,311,1,'SoH-Not-Sent')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,311,'MS-Quarantine-SOH',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,311,'MS-RAS-Correlation',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,311,'MS-Extended-Quarantine-State',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,311,'MS-HCAP-User-Groups',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,311,'MS-HCAP-Location-Group-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,311,'MS-HCAP-User-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,311,'MS-User-IPv4-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,311,'MS-User-IPv6-Address',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 311 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,311,'MS-TSG-Device-Redirection',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 13 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,311,0,'Not-Allowed')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 13 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,311,1,'Allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 13 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(13,311,2,'Required')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 21 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,311,1,'Just-Change-Password')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 21 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,311,2,'Expired-Password')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 21 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,311,3,'Admin-Requires-Password-Change')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 21 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,311,4,'Password-Too-Short')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 23 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,311,1,'PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 23 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,311,2,'CHAP')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 23 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,311,3,'MS-CHAP-1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 23 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,311,4,'MS-CHAP-2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 23 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,311,5,'EAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 24 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,311,4,'MD5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 24 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,311,5,'OTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 24 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,311,6,'Generic-Token-Card')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 24 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,311,13,'TLS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 7 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,311,1,'Encryption-Allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 311 AND RadAttributeID = 7 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,311,2,'Encryption-Required')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 14988)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(14988,'Mikrotik')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,14988,'Mikrotik-Recv-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,14988,'Mikrotik-Xmit-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,14988,'Mikrotik-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,14988,'Mikrotik-Wireless-Forward',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,14988,'Mikrotik-Wireless-Skip-Dot1x',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,14988,'Mikrotik-Wireless-Enc-Algo',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,14988,'Mikrotik-Wireless-Enc-Key',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,14988,'Mikrotik-Rate-Limit',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,14988,'Mikrotik-Realm',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,14988,'Mikrotik-Host-IP',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,14988,'Mikrotik-Mark-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,14988,'Mikrotik-Advertise-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,14988,'Mikrotik-Advertise-Interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,14988,'Mikrotik-Recv-Limit-Gigawords',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,14988,'Mikrotik-Xmit-Limit-Gigawords',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,14988,'Mikrotik-Wireless-PSK',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,14988,'Mikrotik-Total-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,14988,'Mikrotik-Total-Limit-Gigawords',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,14988,'Mikrotik-Address-List',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,14988,'Mikrotik-Wireless-MPKey',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,14988,'Mikrotik-Wireless-Comment',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14988 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,14988,'Mikrotik-Delegated-IPv6-Pool',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14988 AND RadAttributeID = 6 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,14988,0,'No-encryption')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14988 AND RadAttributeID = 6 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,14988,1,'40-bit-WEP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14988 AND RadAttributeID = 6 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,14988,2,'104-bit-WEP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14988 AND RadAttributeID = 6 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,14988,3,'AES-CCM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14988 AND RadAttributeID = 6 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(6,14988,4,'TKIP')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 161)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(161,'Motorola')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,161,'Motorola-Canopy-LPULCIR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,161,'Motorola-Canopy-LPDLCIR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,161,'Motorola-Canopy-HPULCIR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,161,'Motorola-Canopy-HPDLCIR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,161,'Motorola-Canopy-HPENABLE',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,161,'Motorola-Canopy-ULBR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,161,'Motorola-Canopy-ULBL',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,161,'Motorola-Canopy-DLBR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,161,'Motorola-Canopy-DLBL',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,161,'Motorola-Canopy-VLLEARNEN',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,161,'Motorola-Canopy-VLFRAMES',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,161,'Motorola-Canopy-VLIDSET',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,161,'Motorola-Canopy-VLIDSET2',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,161,'Motorola-Canopy-VLIDSET3',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,161,'Motorola-Canopy-VLIDSET4',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,161,'Motorola-Canopy-VLAGETO',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,161,'Motorola-Canopy-VLIGVID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,161,'Motorola-Canopy-VLMGVID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,161,'Motorola-Canopy-VLSMMGPASS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,161,'Motorola-Canopy-BCASTMIR',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,161,'Motorola-Canopy-Gateway',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,161,'Motorola-Canopy-UserLevel',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,161,'Motorola-WiMAX-Network-Domain-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,161,'Motorola-WiMAX-EMS-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,161,'Motorola-WiMAX-Provisioning-Server',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,161,'Motorola-WiMAX-NTP-Server',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,161,'Motorola-WiMAX-HO-SVC-CLASS',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,161,'Motorola-WiMAX-Max-Total-Bandwidth',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,161,'Motorola-WiMAX-Max-Commit-Bandwidth',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,161,'Motorola-WiMAX-Convergence-Sublayer',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,161,'Motorola-WiMAX-Service-Flows',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,161,'Motorola-WiMAX-VLAN-ID',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,161,'Motorola-WiMAX-MIP-MN-HOME-ADDRESS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,161,'Motorola-WiMAX-MIP-KEY',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,161,'Motorola-WiMAX-MIP-SPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 161 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,161,'Motorola-WiMAX-MN-HA',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 5 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(5,161,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 5 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(5,161,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 14 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,161,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 14 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,161,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 15 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,161,0,'All')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 15 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,161,1,'Tagged')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 15 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(15,161,2,'Untagged')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 23 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,161,0,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 23 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,161,1,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 50 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,161,0,'GUEST')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 50 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,161,1,'TECH')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 50 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,161,2,'INSTALL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 161 AND RadAttributeID = 50 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,161,3,'ADMIN')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 6504)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(6504,'Navini')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6504 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,6504,'Navini-AVPair',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 3224)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(3224,'Netscreen')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3224 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,3224,'NS-Admin-Privilege',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3224 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,3224,'NS-VSYS-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3224 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,3224,'NS-User-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3224 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,3224,'NS-Primary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3224 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,3224,'NS-Secondary-DNS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3224 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,3224,'NS-Primary-WINS',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3224 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,3224,'NS-Secondary-WINS',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3224 AND RadAttributeID = 220)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(220,3224,'NS-NSM-User-Domain-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3224 AND RadAttributeID = 221)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(221,3224,'NS-NSM-User-Role-Mapping',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3224 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,3224,1,'Root-Admin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3224 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,3224,2,'All-VSYS-Root-Admin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3224 AND RadAttributeID = 1 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,3224,3,'VSYS-Admin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3224 AND RadAttributeID = 1 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,3224,4,'Read-Only-Admin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3224 AND RadAttributeID = 1 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,3224,5,'Read-Only-VSYS-Admin')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 3309)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(3309,'Nomadix')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,3309,'Nomadix-Bw-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,3309,'Nomadix-Bw-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,3309,'Nomadix-URL-Redirection',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,3309,'Nomadix-IP-Upsell',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,3309,'Nomadix-Expiration',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,3309,'Nomadix-Subnet',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,3309,'Nomadix-MaxBytesUp',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,3309,'Nomadix-MaxBytesDown',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,3309,'Nomadix-EndofSession',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,3309,'Nomadix-Logoff-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,3309,'Nomadix-Net-VLAN',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,3309,'Nomadix-Config-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3309 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,3309,'Nomadix-Goodbye-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3309 AND RadAttributeID = 4 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,3309,0,'PrivatePool')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3309 AND RadAttributeID = 4 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,3309,1,'PublicPool')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 9048)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(9048,'OSC')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9048 AND RadAttributeID = 0)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(0,9048,'OSC-AVPAIR',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9048 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,9048,'OSC-Uid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9048 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,9048,'OSC-Gid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9048 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,9048,'OSC-Home',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 9048 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,9048,'OSC-Shell',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 2334)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(2334,'Packeteer')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2334 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,2334,'Packeteer-AVPair',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 14895)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(14895,'Propel')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14895 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,14895,'Propel-Accelerate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14895 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,14895,'Propel-Dialed-Digits',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14895 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,14895,'Propel-Client-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14895 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,14895,'Propel-Client-NAS-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14895 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,14895,'Propel-Client-Source-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14895 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,14895,'Propel-Content-Filter-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 20024)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(20024,'Proxyconn')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 20024 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,20024,'Proxyconn-Package',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 20024 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,20024,'Proxyconn-Server-PrimaryID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 20024 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,20024,'Proxyconn-Server-SecondaryID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 20024 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,20024,'Proxyconn-Server-PrimaryIP',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 20024 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,20024,'Proxyconn-Server-SecondaryIP',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 20024 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,20024,'Proxyconn-ClusterID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 20024 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,20024,'Proxyconn-Locality',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 20024 AND RadAttributeID = 1 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,20024,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 20024 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,20024,1,'Basic')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 20024 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,20024,2,'Premium')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 21074)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(21074,'Purewave')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 21074 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,21074,'Purewave-Client-Profile',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 21074 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,21074,'Purewave-CS-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 21074 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,21074,'Purewave-Max-Downlink-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 21074 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,21074,'Purewave-Max-Uplink-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 21074 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,21074,'Purewave-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 21074 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,21074,'Purewave-IP-Netmask',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 21074 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,21074,'Purewave-Service-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 2 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,21074,1,'ETHERNET_CS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 2 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2,21074,0,'IPV4_CS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 3 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,21074,3,'QPSK_1/2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 3 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,21074,4,'QPSK_3/4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 3 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,21074,5,'QAM16_1/2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 3 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,21074,6,'QAM16_3/4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 3 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,21074,7,'QAM64_1/2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 3 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,21074,8,'QAM64_2/3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 3 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,21074,9,'QAM64_3/4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 3 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,21074,10,'QAM64_5/6')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 4 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,21074,3,'QPSK_1/2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 4 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,21074,4,'QPSK_3/4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 4 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,21074,5,'QAM16_1/2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 4 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,21074,6,'QAM16_3/4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 4 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,21074,7,'QAM64_1/2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 4 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,21074,8,'QAM64_2/3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 4 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,21074,9,'QAM64_3/4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 4 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,21074,10,'QAM64_5/6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 7 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,21074,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 21074 AND RadAttributeID = 7 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,21074,0,'Disabled')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 6618)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(6618,'Quintum')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,6618,'Quintum-AVPair',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,6618,'Quintum-NAS-Port',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,6618,'Quintum-h323-remote-address',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,6618,'Quintum-h323-conf-id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,6618,'Quintum-h323-setup-time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,6618,'Quintum-h323-call-origin',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,6618,'Quintum-h323-call-type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,6618,'Quintum-h323-connect-time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,6618,'Quintum-h323-disconnect-time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,6618,'Quintum-h323-disconnect-cause',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,6618,'Quintum-h323-voice-quality',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,6618,'Quintum-h323-gw-id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,6618,'Quintum-h323-incoming-conf-id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,6618,'Quintum-h323-credit-amount',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,6618,'Quintum-h323-credit-time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,6618,'Quintum-h323-return-code',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,6618,'Quintum-h323-prompt-id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,6618,'Quintum-h323-time-and-day',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,6618,'Quintum-h323-redirect-number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,6618,'Quintum-h323-preferred-lang',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,6618,'Quintum-h323-redirect-ip-address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,6618,'Quintum-h323-billing-model',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,6618,'Quintum-h323-currency-type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 230)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(230,6618,'Quintum-Trunkid-In',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6618 AND RadAttributeID = 231)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(231,6618,'Quintum-Trunkid-Out',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 2352)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(2352,'Redback')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,2352,'Client-DNS-Pri',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,2352,'Client-DNS-Sec',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,2352,'DHCP-Max-Leases',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,2352,'Context-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,2352,'Bridge-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,2352,'BG-Aging-Time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,2352,'BG-Path-Cost',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,2352,'BG-Span-Dis',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,2352,'BG-Trans-BPDU',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,2352,'Rate-Limit-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,2352,'Rate-Limit-Burst',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,2352,'Police-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,2352,'Police-Burst',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,2352,'Source-Validation',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,2352,'Tunnel-Domain',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,2352,'Tunnel-Local-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,2352,'Tunnel-Remote-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,2352,'Tunnel-Function',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,2352,'Tunnel-Max-Sessions',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,2352,'Tunnel-Max-Tunnels',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,2352,'Tunnel-Session-Auth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,2352,'Tunnel-Window',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,2352,'Tunnel-Retransmit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,2352,'Tunnel-Cmd-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,2352,'PPPOE-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,2352,'PPPOE-MOTM',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,2352,'Tunnel-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,2352,'Tunnel-Context',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,2352,'Tunnel-Algorithm',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,2352,'Tunnel-Deadtime',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,2352,'Mcast-Send',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,2352,'Mcast-Receive',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,2352,'Mcast-MaxGroups',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,2352,'Ip-Address-Pool-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,2352,'Tunnel-DNIS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,2352,'Medium-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,2352,'PVC-Encapsulation-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,2352,'PVC-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,2352,'PVC-Circuit-Padding',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,2352,'Bind-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,2352,'Bind-Auth-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,2352,'Bind-Auth-Max-Sessions',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,2352,'Bind-Bypass-Bypass',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,2352,'Bind-Auth-Context',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,2352,'Bind-Auth-Service-Grp',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,2352,'Bind-Bypass-Context',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,2352,'Bind-Int-Context',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,2352,'Bind-Tun-Context',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,2352,'Bind-Ses-Context',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,2352,'Bind-Dot1q-Slot',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,2352,'Bind-Dot1q-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,2352,'Bind-Dot1q-Vlan-Tag-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,2352,'Bind-Int-Interface-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,2352,'Bind-L2TP-Tunnel-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,2352,'Bind-L2TP-Flow-Control',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,2352,'Bind-Sub-User-At-Context',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,2352,'Bind-Sub-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,2352,'Ip-Host-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,2352,'IP-TOS-Field',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,2352,'NAS-Real-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,2352,'Tunnel-Session-Auth-Ctx',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,2352,'Tunnel-Session-Auth-Service-Grp',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,2352,'Tunnel-Rate-Limit-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,2352,'Tunnel-Rate-Limit-Burst',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,2352,'Tunnel-Police-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,2352,'Tunnel-Police-Burst',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,2352,'Tunnel-L2F-Second-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,2352,'ACL-Definition',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,2352,'PPPoE-IP-Route-Add',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,2352,'TTY-Level-Max',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,2352,'TTY-Level-Start',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,2352,'Tunnel-Checksum',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,2352,'Tunnel-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 78)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(78,2352,'Tunnel-Client-VPN',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,2352,'Tunnel-Server-VPN',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,2352,'Tunnel-Client-Rhost',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,2352,'Tunnel-Server-Rhost',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,2352,'Tunnel-Client-Int-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,2352,'Tunnel-Server-Int-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 84)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(84,2352,'PPP-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 85)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(85,2352,'Tunnel-Hello-Timer',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 86)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(86,2352,'Redback-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 87)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(87,2352,'Qos-Policy-Policing',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 88)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(88,2352,'Qos-Policy-Metering',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 89)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(89,2352,'Qos-Policy-Queuing',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,2352,'Igmp-Service-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,2352,'Sub-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,2352,'Forward-Policy',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,2352,'Remote-Port',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,2352,'Reauth-String',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,2352,'Reauth-More',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,2352,'Agent-Remote-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,2352,'Agent-Circuit-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,2352,'Platform-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 99)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(99,2352,'Client-NBNS-Pri',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,2352,'Client-NBNS-Sec',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,2352,'Shaping-Profile-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,2352,'BG-Cct-Addr-Max',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,2352,'IP-Interface-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,2352,'NAT-Policy-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,2352,'RB-NPM-Service-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,2352,'HTTP-Redirect-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,2352,'Bind-Auto-Sub-User',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,2352,'Bind-Auto-Sub-Context',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,2352,'Bind-Auto-Sub-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,2352,'Circuit-Protocol-Encap',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,2352,'OS-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,2352,'Session-Traffic-Limit',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,2352,'QOS-Reference',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,2352,'Rate-Limit-Excess-Burst',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,2352,'Police-Excess-Burst',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,2352,'Tunnel-Rate-Limit-Excess-Burst',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,2352,'Tunnel-Police-Excess-Burst',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,2352,'DHCP-Vendor-Class-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,2352,'Qos-Rate',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,2352,'DHCP-Vendor-Encap-Option',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,2352,'Acct-Input-Octets',8,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,2352,'Acct-Output-Octets',8,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,2352,'Acct-Input-Packets',8,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,2352,'Acct-Output-Packets',8,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,2352,'Assigned-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,2352,'Acct-Mcast-In-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 134)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(134,2352,'Acct-Mcast-Out-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 135)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(135,2352,'Acct-Mcast-In-Packets',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 136)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(136,2352,'Acct-Mcast-Out-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 137)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(137,2352,'LAC-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 138)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(138,2352,'LAC-Real-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 139)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(139,2352,'LAC-Port-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,2352,'LAC-Real-Port-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,2352,'Acct-Dyn-Ac-Ent',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,2352,'Session-Error-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,2352,'Session-Error-Msg',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 144)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(144,2352,'Acct-Update-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 145)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(145,2352,'Mac-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 146)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(146,2352,'Vlan-Source-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 147)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(147,2352,'Acct-Mcast-In-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 148)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(148,2352,'Acct-Mcast-Out-Octets',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 149)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(149,2352,'Acct-Mcast-In-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 150)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(150,2352,'Acct-Mcast-Out-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 151)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(151,2352,'Reauth-Session-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 156)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(156,2352,'QOS-Rate-Inbound',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 157)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(157,2352,'QOS-Rate-Outbound',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 158)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(158,2352,'Route-Tag',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 159)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(159,2352,'LI-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 160)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(160,2352,'LI-Md-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 161)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(161,2352,'LI-Md-Port',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 162)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(162,2352,'LI-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 163)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(163,2352,'LI-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 164)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(164,2352,'Dynamic-Policy-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 165)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(165,2352,'HTTP-Redirect-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 166)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(166,2352,'DSL-Actual-Rate-Up',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 167)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(167,2352,'DSL-Actual-Rate-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 168)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(168,2352,'DSL-Min-Rate-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 169)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(169,2352,'DSL-Min-Rate-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 170)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(170,2352,'DSL-Attainable-Rate-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 171)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(171,2352,'DSL-Attainable-Rate-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 172)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(172,2352,'DSL-Max-Rate-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 173)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(173,2352,'DSL-Max-Rate-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 174)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(174,2352,'DSL-Min-Low-Power-Rate-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 175)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(175,2352,'DSL-Min-Low-Power-Rate-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 176)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(176,2352,'DSL-Max-Inter-Delay-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 177)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(177,2352,'DSL-Actual-Inter-Delay-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 178)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(178,2352,'DSL-Max-Inter-Delay-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 179)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(179,2352,'DSL-Actual-Inter-Delay-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 180)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(180,2352,'DSL-Line-State',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 181)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(181,2352,'DSL-L2-Encapsulation',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 182)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(182,2352,'DSL-Transmission-System',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 183)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(183,2352,'DSL-PPPOA-PPPOE-Inter-Work-Flag',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 185)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(185,2352,'DSL-Actual-Rate-Down-Factor',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 184)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(184,2352,'DSL-Combined-Line-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 186)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(186,2352,'Class-Volume-limit',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 187)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(187,2352,'Class-Volume-In-Counter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 188)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(188,2352,'Class-Volume-Out-Counter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 189)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(189,2352,'Flow-FAC-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 190)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(190,2352,'Service-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 191)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(191,2352,'Service-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 192)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(192,2352,'Service-Parameter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 193)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(193,2352,'Service-Error-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 194)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(194,2352,'Deactivate-Service-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 195)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(195,2352,'Qos-Profile-Overhead',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 196)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(196,2352,'Dynamic-QoS-Param',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 197)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(197,2352,'Acct-Alt-Session-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 198)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(198,2352,'Idle-Timeout-Threshold',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 199)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(199,2352,'Double-Authentication',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 200)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(200,2352,'SBC-Adjacency',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 201)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(201,2352,'DHCP-Field',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 202)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(202,2352,'DHCP-Option',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 203)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(203,2352,'Security-Service',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 204)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(204,2352,'Reauth-Service-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 205)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(205,2352,'Flow-IP-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2352 AND RadAttributeID = 206)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(206,2352,'Radius-Throttle-Watermark',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,1,'AAA-ENCAPS-ATM-RAW')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,2,'AAA-ENCAPS-ATM-ROUTE1483')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,3,'AAA-ENCAPS-ATM-AUTO1483')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,4,'AAA-ENCAPS-ATM-MULTI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,5,'AAA-ENCAPS-ATM-BRIDGE1483')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,6,'AAA-ENCAPS-ATM-PPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,7,'AAA-ENCAPS-ATM-PPP-SERIAL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,8,'AAA-ENCAPS-ATM-PPP-NLPID')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,9,'AAA-ENCAPS-ATM-PPP-AUTO')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,10,'AAA-ENCAPS-ATM-PPPOE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,11,'AAA-ENCAPS-ATM-L2TP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,12,'AAA-ENCAPS-ATM-PPP-LLC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,13,'AAA-ENCAPS-FRAME-AUTO1490')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,14,'AAA-ENCAPS-FRAME-MULTI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,15,'AAA-ENCAPS-FRAME-BRIDGE1490')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,16,'AAA-ENCAPS-FRAME-PPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,17,'AAA-ENCAPS-FRAME-PPP-AUTO')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,18,'AAA-ENCAPS-FRAME-PPPOE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,19,'AAA-ENCAPS-FRAME-ROUTE1490')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,20,'AAA-ENCAPS-FRAME-L2TP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,21,'AAA-ENCAPS-L2TP-VC-MUXED')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,22,'AAA-ENCAPS-ETH')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,23,'AAA-ENCAPS-ETH-PPPOE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 39 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39,2352,24,'AAA-ENCAPS-ETH-MULTI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 41 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(41,2352,1,'AAA-CIRCUIT-PADDING')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 41 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(41,2352,2,'AAA-CIRCUIT-NO-PADDING')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 42 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,2352,1,'AAA-AUTH-BIND')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 42 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,2352,2,'AAA-BYPASS-BIND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 42 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,2352,3,'AAA-INTERFACE-BIND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 42 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,2352,4,'AAA-SUBSCRIBE-BIND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 42 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,2352,5,'AAA-TUNNEL-BIND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 42 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,2352,6,'AAA-SESSION-BIND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 42 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,2352,7,'AAA-Q8021-BIND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 42 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(42,2352,8,'AAA-MULTI-BIND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 43 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,2352,1,'AAA-PPP-PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 43 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,2352,2,'AAA-PPP-CHAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 43 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,2352,3,'AAA-PPP-CHAP-WAIT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 43 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,2352,4,'AAA-PPP-CHAP-PAP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 43 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,2352,5,'AAA-PPP-CHAP-WAIT-PAP')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 18 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,2352,1,'LAC-Only')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 18 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,2352,2,'LNS-Only')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 18 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(18,2352,3,'LAC-LNS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 33 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(33,2352,1,'NO-SEND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 33 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(33,2352,2,'SEND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 33 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(33,2352,3,'UNSOLICITED-SEND')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 34 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(34,2352,1,'NO-RECEIVE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 34 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(34,2352,2,'RECEIVE')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 37 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(37,2352,1,'DNIS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 37 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(37,2352,2,'DNIS-Only')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,40,'NAS-PORT-TYPE-10BT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,41,'NAS-PORT-TYPE-100BT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,42,'NAS-PORT-TYPE-DS3-FR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,43,'NAS-PORT-TYPE-DS3-ATM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,44,'NAS-PORT-TYPE-OC3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,45,'NAS-PORT-TYPE-HSSI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,46,'NAS-PORT-TYPE-EIA530')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,47,'NAS-PORT-TYPE-T1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,48,'NAS-PORT-TYPE-CHAN-T3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,49,'NAS-PORT-TYPE-DS1-FR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,50,'NAS-PORT-TYPE-E3-ATM')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,51,'NAS-PORT-TYPE-IMA-ATM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,52,'NAS-PORT-TYPE-DS3-ATM-2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,53,'NAS-PORT-TYPE-OC3-ATM-2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,54,'NAS-PORT-TYPE-1000BSX')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,55,'NAS-PORT-TYPE-E1-FR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 56)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,56,'NAS-PORT-TYPE-E1-ATM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,57,'NAS-PORT-TYPE-E3-FR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,58,'NAS-PORT-TYPE-OC3-POS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 59)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,59,'NAS-PORT-TYPE-OC12-POS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 139 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(139,2352,60,'NAS-PORT-TYPE-PPPOE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,40,'NAS-PORT-TYPE-10BT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,41,'NAS-PORT-TYPE-100BT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,42,'NAS-PORT-TYPE-DS3-FR')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,43,'NAS-PORT-TYPE-DS3-ATM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,44,'NAS-PORT-TYPE-OC3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,45,'NAS-PORT-TYPE-HSSI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,46,'NAS-PORT-TYPE-EIA530')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,47,'NAS-PORT-TYPE-T1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,48,'NAS-PORT-TYPE-CHAN-T3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,49,'NAS-PORT-TYPE-DS1-FR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,50,'NAS-PORT-TYPE-E3-ATM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,51,'NAS-PORT-TYPE-IMA-ATM')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,52,'NAS-PORT-TYPE-DS3-ATM-2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,53,'NAS-PORT-TYPE-OC3-ATM-2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,54,'NAS-PORT-TYPE-1000BSX')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,55,'NAS-PORT-TYPE-E1-FR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 56)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,56,'NAS-PORT-TYPE-E1-ATM')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,57,'NAS-PORT-TYPE-E3-FR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,58,'NAS-PORT-TYPE-OC3-POS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 59)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,59,'NAS-PORT-TYPE-OC12-POS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 140 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,2352,60,'NAS-PORT-TYPE-PPPOE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,1,'AAA_LOAD_ACCT_SESSION_UP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,2,'AAA_LOAD_ACCT_SESSION_DOWN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,3,'AAA_LOAD_ACCT_PERIODIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,4,'AAA_LOAD_ACCT_DYN_AC_ENT_START')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,5,'AAA_LOAD_ACCT_DYN_AC_ENT_STOP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,6,'AAA_LOAD_ACCT_DYN_AC_ENT_TIMEOUT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,7,'AAA_LOAD_ACCT_SUBSCRIBER_REAUTHOR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,8,'AAA_LOAD_ACCT_PPP_IPCP_UP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,9,'AAA_LOAD_ACCT_PPP_MP_LINK_UP')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,10,'AAA_LOAD_ACCT_DHCP_IP_ADDR_GRANTED')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,11,'AAA_LOAD_ACCT_DHCP_IP_ADDR_RELEASED')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,12,'AAA_LOAD_ACCT_ACL_TIMERED_ACTION')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,13,'AAA_LOAD_ACCT_ACL_ACTION')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,14,'AAA_LOAD_ACCT_CMD')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 144 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,2352,15,'AAA_LOAD_ACCT_TEST')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 180 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(180,2352,1,'Showtime')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 180 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(180,2352,2,'Idle')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 180 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(180,2352,3,'Silent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 182 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(182,2352,1,'ADSL1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 182 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(182,2352,2,'ADSL2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 182 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(182,2352,3,'ADSL2p')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 182 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(182,2352,4,'VDSL1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 182 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(182,2352,5,'VDSL2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 182 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(182,2352,6,'SDSL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 182 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(182,2352,7,'UNKNOWN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 191 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(191,2352,0,'DE-ACTIVATE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 191 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(191,2352,1,'ACTIVATE-WITH-ACCT')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2352 AND RadAttributeID = 191 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(191,2352,2,'ACTIVATE-WITHOUT-ACCT')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 1958)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(1958,'RedCreek')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1958 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,1958,'RedCreek-Tunneled-IP-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1958 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,1958,'RedCreek-Tunneled-IP-Netmask',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1958 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,1958,'RedCreek-Tunneled-Gateway',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1958 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,1958,'RedCreek-Tunneled-DNS-Server',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1958 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,1958,'RedCreek-Tunneled-WINS-Server1',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1958 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,1958,'RedCreek-Tunneled-WINS-Server2',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1958 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,1958,'RedCreek-Tunneled-HostName',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1958 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,1958,'RedCreek-Tunneled-DomainName',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 1958 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,1958,'RedCreek-Tunneled-Search-List',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 10055)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(10055,'RoaringPenguin')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10055 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,10055,'RP-Upstream-Speed-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10055 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,10055,'RP-Downstream-Speed-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10055 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,10055,'RP-HURL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10055 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,10055,'RP-MOTM',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 10055 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,10055,'RP-Max-Sessions-Per-User',1,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 3199)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(3199,'Shasta')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,3199,'Shasta-User-Privilege',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,3199,'Shasta-Service-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,3199,'Shasta-VPN-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,3199,'Shasta-ISOS-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,3199,'Shasta-L2TP-Ingress-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,3199,'Shasta-L2TP-Egress-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,3199,'Shasta-Home-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,3199,'Shasta-PPPoE-Client-MAC',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,3199,'Shasta-PPPoE-Session-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,3199,'Shasta-IPSec-Split-Tunnel',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,3199,'Shasta-IPSec-Failover-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,3199,'Shasta-Primary-WINS-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,3199,'Shasta-Secondary-WINS-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,3199,'Shasta-PPP-LCP-Echo-Interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,3199,'Shasta-PPP-LCP-Echo-Retry-Interval',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,3199,'Shasta-PPP-LCP-Echo-Max-Retries',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,3199,'Shasta-IPSEC-Allow-Save-Password',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,3199,'Shasta-Policy-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3199 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,3199,'Shasta-User-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3199 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,3199,1,'User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3199 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,3199,2,'Super-User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 3199 AND RadAttributeID = 1 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,3199,3,'SSuper-User')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 166)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(166,'Shiva')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,166,'Shiva-User-Attributes',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,166,'Shiva-Called-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,166,'Shiva-Calling-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,166,'Shiva-Customer-Id',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,166,'Shiva-Type-Of-Service',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,166,'Shiva-Link-Speed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,166,'Shiva-Links-In-Bundle',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,166,'Shiva-Compression-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,166,'Shiva-Link-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,166,'Shiva-Network-Protocols',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 99)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(99,166,'Shiva-Session-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,166,'Shiva-Disconnect-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,166,'Shiva-Acct-Serv-Switch',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,166,'Shiva-Event-Flags',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,166,'Shiva-Function',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 166 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,166,'Shiva-Connect-Reason',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 93 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(93,166,1,'Analog')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 93 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(93,166,2,'Digitized-Analog')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 93 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(93,166,3,'Digital')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 93 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(93,166,4,'Digital-V110')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 93 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(93,166,5,'Digital-V120')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 93 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(93,166,6,'Digital-Leased-Line')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 97 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(97,166,1,'HDLC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 97 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(97,166,2,'ARAV1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 97 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(97,166,3,'ARAV2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 97 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(97,166,4,'SHELL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 97 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(97,166,5,'AALAP')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 97 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(97,166,6,'SLIP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 104 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(104,166,1,'Remote')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 104 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(104,166,2,'Dialback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 104 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(104,166,3,'Virtual-Connection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 104 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(104,166,4,'Bandwidth-On-Demand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,1,'Remote')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,2,'Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,3,'Idle-Timeout')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,4,'Session-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,5,'Admin-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,6,'Dialback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,7,'Virtual-Connection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,8,'Bandwidth-On-Demand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,9,'Failed-Authentication')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,10,'Preempted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,11,'Blocked')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,12,'Tariff-Management')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 100 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,166,13,'Backup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 103 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(103,166,0,'Unknown')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 103 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(103,166,1,'Dialin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 103 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(103,166,2,'Dialout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 166 AND RadAttributeID = 103 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(103,166,3,'Lan-To-Lan')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 7000)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(7000,'Slipstream')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7000 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,7000,'Slip-Stream-Auth',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7000 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,7000,'Slip-Stream-SMTP',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7000 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,7000,'Slip-Stream-Filtered',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7000 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,7000,'Slip-Pass-Through-BB',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7000 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,7000,'Slip-Accelerated',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8741 AND RadAttributeID = 1 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8741,7,'Wireless-Guest')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8741 AND RadAttributeID = 1 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8741,8,'Wireless-Add-ACL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8741 AND RadAttributeID = 1 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,8741,9,'Internet-Access')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8741 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,8741,'SonicWall-User-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 3551)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(3551,'SpringTide')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3551 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,3551,'ST-Acct-VC-Connection-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3551 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,3551,'ST-Service-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3551 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,3551,'ST-Service-Domain',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3551 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,3551,'ST-Policy-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3551 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,3551,'ST-Primary-DNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3551 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,3551,'ST-Secondary-DNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3551 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,3551,'ST-Primary-NBNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 3551 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,3551,'ST-Secondary-NBNS-Server',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 8164)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(8164,'Starent')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,8164,'SN-VPN-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,8164,'SN-VPN-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,8164,'SN-Disconnect-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,8164,'SN-PPP-Progress-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,8164,'SN-Primary-DNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,8164,'SN-Secondary-DNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,8164,'SN-Re-CHAP-Interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,8164,'SN-IP-Pool-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,8164,'SN-PPP-Data-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,8164,'SN-IP-Filter-In',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,8164,'SN-IP-Filter-Out',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,8164,'SN-Local-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,8164,'SN-IP-Source-Validation',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,8164,'SN-PPP-Outbound-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,8164,'SN-PPP-Keepalive',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,8164,'SN-IP-In-ACL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,8164,'SN-IP-Out-ACL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,8164,'SN-PPP-Data-Compression-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,8164,'SN-Subscriber-Permission',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,8164,'SN-Admin-Permission',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,8164,'SN-Simultaneous-SIP-MIP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,8164,'SN-Min-Compress-Size',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,8164,'SN-Service-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,8164,'SN-DNS-Proxy-Use-Subscr-Addr',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,8164,'SN-Tunnel-Password',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,8164,'SN-Tunnel-Load-Balancing',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,8164,'SN-MN-HA-Timestamp-Tolerance',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,8164,'SN-Prepaid-Compressed-Count',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,8164,'SN-Prepaid-Inbound-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,8164,'SN-Prepaid-Outbound-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,8164,'SN-Prepaid-Total-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,8164,'SN-Prepaid-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,8164,'SN-Prepaid-Watermark',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,8164,'SN-NAI-Construction-Domain',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,8164,'SN-Tunnel-ISAKMP-Crypto-Map',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,8164,'SN-Tunnel-ISAKMP-Secret',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,8164,'SN-Ext-Inline-Srvr-Context',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,8164,'SN-L3-to-L2-Tun-Addr-Policy',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,8164,'SN-Long-Duration-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,8164,'SN-Long-Duration-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,8164,'SN-PDSN-Handoff-Req-IP-Addr',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,8164,'SN-HA-Send-DNS-ADDRESS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,8164,'SN-MIP-Send-Term-Verification',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,8164,'SN-Data-Tunnel-Ignore-DF-Bit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,8164,'SN-MIP-AAA-Assign-Addr',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,8164,'SN-Proxy-MIP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,8164,'SN-MIP-Match-AAA-Assign-Addr',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,8164,'SN-IP-Alloc-Method',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,8164,'SN-Gratuitous-ARP-Aggressive',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,8164,'SN-Ext-Inline-Srvr-Up-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,8164,'SN-Ext-Inline-Srvr-Down-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,8164,'SN-Ext-Inline-Srvr-Preference',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,8164,'SN-Ext-Inline-Srvr-Up-VLAN',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,8164,'SN-Ext-Inline-Srvr-Down-VLAN',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,8164,'SN-IP-Hide-Service-Address',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,8164,'SN-PPP-Outbound-Username',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,8164,'SN-GTP-Version',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 63)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(63,8164,'SN-Access-link-IP-Frag',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,8164,'SN-Subscriber-Accounting',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,8164,'SN-Nw-Reachability-Server-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,8164,'SN-Subscriber-IP-Hdr-Neg-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,8164,'SN-GGSN-MIP-Required',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,8164,'SN-Subscriber-Acct-Start',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,8164,'SN-Subscriber-Acct-Interim',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,8164,'SN-Subscriber-Acct-Stop',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,8164,'SN-QoS-Tp-Dnlk',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,8164,'SN-Tp-Dnlk-Committed-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,8164,'SN-Tp-Dnlk-Peak-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,8164,'SN-Tp-Dnlk-Burst-Size',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 77)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(77,8164,'SN-Tp-Dnlk-Exceed-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 78)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(78,8164,'SN-Tp-Dnlk-Violate-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 79)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(79,8164,'SN-QoS-Tp-Uplk',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 80)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(80,8164,'SN-Tp-Uplk-Committed-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 81)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(81,8164,'SN-Tp-Uplk-Peak-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 82)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(82,8164,'SN-Tp-Uplk-Burst-Size',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 83)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(83,8164,'SN-Tp-Uplk-Exceed-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 84)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(84,8164,'SN-Tp-Uplk-Violate-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 85)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(85,8164,'SN-Subscriber-IP-TOS-Copy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 86)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(86,8164,'SN-QoS-Conversation-Class',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 87)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(87,8164,'SN-QoS-Streaming-Class',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 88)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(88,8164,'SN-QoS-Interactive1-Class',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 89)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(89,8164,'SN-QoS-Interactive2-Class',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 90)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(90,8164,'SN-QoS-Interactive3-Class',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 91)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(91,8164,'SN-QoS-Background-Class',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 92)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(92,8164,'SN-PPP-NW-Layer-IPv4',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 93)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(93,8164,'SN-PPP-NW-Layer-IPv6',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,8164,'SN-Virtual-APN-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 95)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(95,8164,'SN-PPP-Accept-Peer-v6Ifid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 96)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(96,8164,'SN-IPv6-rtr-advt-interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 97)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(97,8164,'SN-IPv6-num-rtr-advt',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 98)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(98,8164,'SN-NPU-Qos-Priority',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 99)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(99,8164,'SN-MN-HA-Hash-Algorithm',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,8164,'SN-Subscriber-Acct-Rsp-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 101)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(101,8164,'SN-IPv6-Primary-DNS',5,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,8164,'SN-IPv6-Secondary-DNS',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,8164,'SN-IPv6-Egress-Filtering',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,8164,'SN-Mediation-VPN-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,8164,'SN-Mediation-Acct-Rsp-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,8164,'SN-Home-Sub-Use-GGSN',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,8164,'SN-Visiting-Sub-Use-GGSN',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,8164,'SN-Roaming-Sub-Use-GGSN',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 109)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(109,8164,'SN-Home-Profile',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 110)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(110,8164,'SN-IP-Src-Validation-Drop-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,8164,'SN-QoS-Class-Conversational-PHB',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,8164,'SN-QoS-Class-Streaming-PHB',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,8164,'SN-QoS-Class-Background-PHB',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,8164,'SN-QoS-Class-Interactive-1-PHB',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 115)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(115,8164,'SN-QoS-Class-Interactive-2-PHB',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 116)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(116,8164,'SN-QoS-Class-Interactive-3-PHB',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,8164,'SN-Visiting-Profile',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,8164,'SN-Roaming-Profile',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(119,8164,'SN-Home-Behavior',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 120)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(120,8164,'SN-Visiting-Behavior',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,8164,'SN-Roaming-Behavior',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,8164,'SN-Internal-SM-Index',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,8164,'SN-Mediation-Enabled',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,8164,'SN-IPv6-Sec-Pool',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,8164,'SN-IPv6-Sec-Prefix',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,8164,'SN-IPv6-DNS-Proxy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,8164,'SN-Subscriber-Nexthop-Address',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,8164,'SN-Prepaid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,8164,'SN-Prepaid-Preference',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 130)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(130,8164,'SN-PPP-Always-On-Vse',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 131)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(131,8164,'SN-Voice-Push-List-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 132)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(132,8164,'SN-Unclassify-List-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 133)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(133,8164,'SN-Subscriber-No-Interims',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 134)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(134,8164,'SN-Permit-User-Mcast-PDUs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 135)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(135,8164,'SN-Prepaid-Final-Duration-Alg',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 136)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(136,8164,'SN-IPv6-Min-Link-MTU',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 137)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(137,8164,'SN-Charging-VPN-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 138)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(138,8164,'SN-Chrg-Char-Selection-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 139)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(139,8164,'SN-Cause-For-Rec-Closing',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,8164,'SN-Change-Condition',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 141)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(141,8164,'SN-Dynamic-Addr-Alloc-Ind-Flag',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,8164,'SN-Ntk-Initiated-Ctx-Ind-Flag',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,8164,'SN-Ntk-Session-Disconnect-Flag',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 144)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(144,8164,'SN-Enable-QoS-Renegotiation',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 145)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(145,8164,'SN-QoS-Renegotiation-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 147)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(147,8164,'SN-QoS-Negotiated',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 146)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(146,8164,'SN-Mediation-No-Interims',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 148)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(148,8164,'SN-Primary-NBNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 149)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(149,8164,'SN-Secondary-NBNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 150)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(150,8164,'SN-IP-Header-Compression',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 151)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(151,8164,'SN-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 152)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(152,8164,'SN-Assigned-VLAN-ID',25,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 153)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(153,8164,'SN-Direction',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 154)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(154,8164,'SN-MIP-HA-Assignment-Table',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 156)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(156,8164,'SN-Tun-Addr-Policy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 157)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(157,8164,'SN-DHCP-Lease-Expiry-Policy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 158)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(158,8164,'SN-Subscriber-Template-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 159)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(159,8164,'SN-Subs-IMSA-Service-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 161)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(161,8164,'SN-Traffic-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 162)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(162,8164,'SN-Rad-APN-Name',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 163)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(163,8164,'SN-MIP-Send-Ancid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 164)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(164,8164,'SN-MIP-Send-Imsi',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 165)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(165,8164,'SN-MIP-Dual-Anchor',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 166)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(166,8164,'SN-MIP-ANCID',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 167)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(167,8164,'SN-IMS-AM-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 168)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(168,8164,'SN-IMS-AM-Domain-Name',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 169)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(169,8164,'SN-Service-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 170)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(170,8164,'SN-PDIF-MIP-Required',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 171)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(171,8164,'SN-FMC-Location',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 172)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(172,8164,'SN-PDIF-MIP-Release-TIA',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 173)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(173,8164,'SN-PDIF-MIP-Simple-IP-Fallback',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 174)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(174,8164,'SN-Tunnel-Gn',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 175)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(175,8164,'SN-MIP-Reg-Lifetime-Realm',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 176)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(176,8164,'SN-Ecs-Data-Volume',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 177)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(177,8164,'SN-QoS-Traffic-Policy',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 178)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(178,8164,'SN-ANID',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 187)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(187,8164,'SN-PPP-Reneg-Disc',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 188)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(188,8164,'SN-MIP-Send-Correlation-Info',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 189)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(189,8164,'SN-PDSN-Correlation-Id',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 190)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(190,8164,'SN-PDSN-NAS-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 191)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(191,8164,'SN-PDSN-NAS-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 192)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(192,8164,'SN-Subscriber-Acct-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 193)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(193,8164,'SN-IP-In-Plcy-Grp',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 194)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(194,8164,'SN-IP-Out-Plcy-Grp',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 196)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(196,8164,'SN-IP-Source-Violate-No-Acct',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 198)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(198,8164,'SN-Firewall-Enabled',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 200)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(200,8164,'SNA-PPP-Unfr-data-In-Oct',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 201)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(201,8164,'SNA-PPP-Unfr-data-Out-Oct',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 202)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(202,8164,'SNA-PPP-Unfr-Data-In-Gig',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 203)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(203,8164,'SNA-PPP-Unfr-Data-Out-Gig',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 204)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(204,8164,'SN-Admin-Expiry',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 206)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(206,8164,'SNA-Input-Gigawords',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 207)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(207,8164,'SNA-Output-Gigawords',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 214)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(214,8164,'SN-DNS-Proxy-Intercept-List',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 219)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(219,8164,'SN-Subscriber-Class',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 220)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(220,8164,'SN-CFPolicy-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 221)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(221,8164,'SN-Subs-VJ-Slotid-Cmp-Neg-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 223)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(223,8164,'SN-Primary-DCCA-Peer',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 224)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(224,8164,'SN-Secondary-DCCA-Peer',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 225)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(225,8164,'SN-Subs-Acc-Flow-Traffic-Valid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 226)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(226,8164,'SN-Acct-Input-Packets-Dropped',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 227)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(227,8164,'SN-Acct-Output-Packets-Dropped',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 228)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(228,8164,'SN-Acct-Input-Octets-Dropped',8,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 229)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(229,8164,'SN-Acct-Output-Octets-Dropped',8,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 230)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(230,8164,'SN-Acct-Input-Giga-Dropped',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 231)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(231,8164,'SN-Acct-Output-Giga-Dropped',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 233)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(233,8164,'SN-Overload-Disc-Connect-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 235)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(235,8164,'SN-Overload-Disconnect',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 236)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(236,8164,'SN-Radius-Returned-Username',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 238)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(238,8164,'SN-ROHC-Profile-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 239)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(239,8164,'SN-Firewall-Policy',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 247)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(247,8164,'SN-Transparent-Data',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 248)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(248,8164,'SN-MS-ISDN',6,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 249)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(249,8164,'SN-Routing-Area-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 250)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(250,8164,'SN-Rulebase',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 251)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(251,8164,'SN-Call-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 252)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(252,8164,'SN-IMSI',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 253)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(253,8164,'SN-Long-Duration-Notification',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 254)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(254,8164,'SN-SIP-Method',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 255)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(255,8164,'SN-Event',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 256)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(256,8164,'SN-Role-Of-Node',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 257)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(257,8164,'SN-Session-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 258)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(258,8164,'SN-SIP-Request-Time-Stamp',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 259)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(259,8164,'SN-SIP-Response-Time-Stamp',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 260)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(260,8164,'SN-IMS-Charging-Identifier',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 261)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(261,8164,'SN-Originating-IOI',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 262)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(262,8164,'SN-Terminating-IOI',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 263)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(263,8164,'SN-SDP-Session-Description',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 264)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(264,8164,'SN-GGSN-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 265)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(265,8164,'SN-Sec-IP-Pool-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 266)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(266,8164,'SN-Authorised-Qos',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 267)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(267,8164,'SN-Cause-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 268)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(268,8164,'SN-Node-Functionality',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 269)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(269,8164,'SN-Is-Unregistered-Subscriber',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 270)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(270,8164,'SN-Content-Type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 271)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(271,8164,'SN-Content-Length',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 272)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(272,8164,'SN-Content-Disposition',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 273)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(273,8164,'SN-CSCF-Rf-SDP-Media-Components',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 274)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(274,8164,'SN-ROHC-Flow-Marking-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 275)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(275,8164,'SN-CSCF-App-Server-Info',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 276)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(276,8164,'SN-ISC-Template-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 277)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(277,8164,'SN-CF-Forward-Unconditional',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 278)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(278,8164,'SN-CF-Forward-No-Answer',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 279)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(279,8164,'SN-CF-Forward-Busy-Line',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 280)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(280,8164,'SN-CF-Forward-Not-Regd',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 281)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(281,8164,'SN-CF-Follow-Me',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 282)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(282,8164,'SN-CF-CId-Display',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 283)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(283,8164,'SN-CF-CId-Display-Blocked',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 284)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(284,8164,'SN-CF-Call-Waiting',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 285)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(285,8164,'SN-CF-Call-Transfer',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 287)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(287,8164,'SN-Cscf-Subscriber-Ip-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 288)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(288,8164,'SN-Software-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 290)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(290,8164,'SN-Max-Sec-Contexts-Per-Subs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 291)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(291,8164,'SN-CF-Call-Local',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 292)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(292,8164,'SN-CF-Call-LongDistance',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 293)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(293,8164,'SN-CF-Call-International',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 294)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(294,8164,'SN-CF-Call-Premium',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 295)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(295,8164,'SN-CR-International-Cid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 296)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(296,8164,'SN-CR-LongDistance-Cid',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 297)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(297,8164,'SN-NAT-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 298)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(298,8164,'SN-CF-Call-RoamingInternatnl',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 299)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(299,8164,'SN-PDG-TTG-Required',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 300)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(300,8164,'SN-Bandwidth-Policy',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 301)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(301,8164,'SN-Acs-Credit-Control-Group',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 302)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(302,8164,'SN-CBB-Policy',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 303)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(303,8164,'SN-QOS-HLR-Profile',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 304)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(304,8164,'SN-Fast-Reauth-Username',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 305)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(305,8164,'SN-Pseudonym-Username',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 307)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(307,8164,'SN-TrafficSelector-Class',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 309)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(309,8164,'SN-DHCP-Options',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 310)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(310,8164,'SN-Handoff-Indicator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1001)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1001,8164,'SNA-PPP-Ctrl-Input-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1002)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1002,8164,'SNA-PPP-Ctrl-Output-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1003)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1003,8164,'SNA-PPP-Ctrl-Input-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1004)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1004,8164,'SNA-PPP-Ctrl-Output-Packets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1005)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1005,8164,'SNA-PPP-Framed-Input-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1006)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1006,8164,'SNA-PPP-Framed-Output-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1007)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1007,8164,'SNA-PPP-Discards-Input',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1008)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1008,8164,'SNA-PPP-Discards-Output',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1009)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1009,8164,'SNA-PPP-Errors-Input',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1010)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1010,8164,'SNA-PPP-Errors-Output',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1011)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1011,8164,'SNA-PPP-Bad-Addr',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1012)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1012,8164,'SNA-PPP-Bad-Ctrl',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1013)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1013,8164,'SNA-PPP-Packet-Too-Long',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1014)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1014,8164,'SNA-PPP-Bad-FCS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1015)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1015,8164,'SNA-PPP-Echo-Req-Input',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1016)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1016,8164,'SNA-PPP-Echo-Req-Output',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1017)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1017,8164,'SNA-PPP-Echo-Rsp-Input',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1018)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1018,8164,'SNA-PPP-Echo-Rsp-Output',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1019)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1019,8164,'SNA-RPRRQ-Rcvd-Total',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1020)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1020,8164,'SNA-RPRRQ-Rcvd-Acc-Reg',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1021)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1021,8164,'SNA-RPRRQ-Rcvd-Acc-Dereg',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1022)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1022,8164,'SNA-RPRRQ-Rcvd-Msg-Auth-Fail',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1023)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1023,8164,'SNA-RPRRQ-Rcvd-Mis-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1024)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1024,8164,'SNA-RPRRQ-Rcvd-Badly-Formed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1025)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1025,8164,'SNA-RPRRQ-Rcvd-VID-Unsupported',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1026)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1026,8164,'SNA-RPRRQ-Rcvd-T-Bit-Not-Set',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1027)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1027,8164,'SNA-RPRAK-Rcvd-Total',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1028)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1028,8164,'SNA-RPRAK-Rcvd-Acc-Ack',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1029)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1029,8164,'SNA-RPRAK-Rcvd-Msg-Auth-Fail',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1030)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1030,8164,'SNA-RPRAK-Rcvd-Mis-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1031)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1031,8164,'SNA-RP-Reg-Reply-Sent-Total',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1032)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1032,8164,'SNA-RP-Reg-Reply-Sent-Acc-Reg',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1033)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1033,8164,'SNA-RP-Reg-Reply-Sent-Acc-Dereg',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1034)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1034,8164,'SNA-RP-Reg-Reply-Sent-Bad-Req',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1035)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1035,8164,'SNA-RP-Reg-Reply-Sent-Denied',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1036)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1036,8164,'SNA-RP-Reg-Reply-Sent-Mis-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1037)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1037,8164,'SNA-RP-Reg-Reply-Sent-Send-Err',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1038)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1038,8164,'SNA-RP-Reg-Upd-Sent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1039)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1039,8164,'SNA-RP-Reg-Upd-Re-Sent',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 1040)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1040,8164,'SNA-RP-Reg-Upd-Send-Err',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 8164 AND RadAttributeID = 65530)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65530,8164,'SN-Proxy-MIPV6',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,0,'Not-Defined')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,1,'Admin-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,2,'Remote-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,3,'Local-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,4,'Disc-No-Resource')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,5,'Disc-Excd-Service-Limit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,6,'PPP-LCP-Neg-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,7,'PPP-LCP-No-Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,8,'PPP-LCP-Loopback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,9,'PPP-LCP-Max-Retry')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,10,'PPP-Echo-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,11,'PPP-Auth-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,12,'PPP-Auth-Failed-No-AAA-Resp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,13,'PPP-Auth-No-Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,14,'PPP-Auth-Max-Retry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,15,'Invalid-AAA-Attr')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,16,'Failed-User-Filter')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,17,'Failed-Provide-Service')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,18,'Invalid-IP-Address-AAA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,19,'Invalid-IP-Pool-AAA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,20,'PPP-IPCP-Neg-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,21,'PPP-IPCP-No-Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,22,'PPP-IPCP-Max-Retry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,23,'PPP-No-Rem-IP-Address')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,24,'Inactivity-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,25,'Session-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,26,'Max-Data-Excd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,27,'Invalid-IP-Source-Address')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,28,'MSID-Auth-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,29,'MSID-Auth-Fauiled-No-AAA-Resp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,30,'A11-Max-Retry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,31,'A11-Lifetime-Expired')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,32,'A11-Message-Integrity-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,33,'PPP-lcp-remote-disc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,34,'Session-setup-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,35,'PPP-keepalive-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,36,'Flow-add-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,37,'Call-type-detection-failed')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,38,'Wrong-ipcp-params')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,39,'MIP-remote-dereg')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,40,'MIP-lifetime-expiry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,41,'MIP-proto-error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,42,'MIP-auth-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,43,'MIP-reg-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,44,'Invalid-dest-context')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,45,'Source-context-removed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,46,'Destination-context-removed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,47,'Req-service-addr-unavailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,48,'Demux-mgr-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,49,'Internal-error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,50,'AAA-context-removed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,51,'invalid-service-type')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,52,'mip-relay-req-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,53,'mip-rcvd-relay-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,54,'ppp-restart-inter-pdsn-handoff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,55,'gre-key-mismatch')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 56)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,56,'invalid_tunnel_context')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,57,'no_peer_lns_address')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,58,'failed_tunnel_connect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 59)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,59,'l2tp-tunnel-disconnect-remote')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,60,'l2tp-tunnel-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 61)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,61,'l2tp-protocol-error-remote')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 62)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,62,'l2tp-protocol-error-local')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,63,'l2tp-auth-failed-remote')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,64,'l2tp-auth-failed-local')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 65)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,65,'l2tp-try-another-lns-from-remote')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 66)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,66,'l2tp-no-resource-local')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 67)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,67,'l2tp-no-resource-remote')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 68)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,68,'l2tp-tunnel-disconnect-local')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 69)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,69,'l2tp-admin-disconnect_remote')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 70)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,70,'l2tpmgr-reached-max-capacity')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 71)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,71,'MIP-reg-revocation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 72)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,72,'path-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 73)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,73,'dhcp-relay-ip-validation-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 74)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,74,'gtp-unknown-pdp-addr-or-pdp-type')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 75)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,75,'gtp-all-dynamic-pdp-addr-occupied')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 76)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,76,'gtp-no-memory-is-available')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 77)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,77,'dhcp-relay-static-ip-addr-not-allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 78)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,78,'dhcp-no-ip-addr-allocated')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 79)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,79,'dhcp-ip-addr-allocation-tmr-exp')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 80)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,80,'dhcp-ip-validation-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 81)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,81,'dhcp-static-addr-not-allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 82)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,82,'dhcp-ip-addr-not-available-at-present')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 83)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,83,'dhcp-lease-expired')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 84)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,84,'lpool-ip-validation-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 85)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,85,'lpool-static-ip-addr-not-allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 86)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,86,'static-ip-validation-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 87)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,87,'static-ip-addr-not-present')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 88)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,88,'static-ip-addr-not-allowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 89)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,89,'radius-ip-validation-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 90)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,90,'radius-ip-addr-not-provided')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 91)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,91,'invalid-ip-addr-from-sgsn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 92)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,92,'no-more-sessions-in-aaa')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 93)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,93,'ggsn-aaa-auth-req-failed')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 94)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,94,'conflict-in-ip-addr-assignment')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 95)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,95,'apn-removed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 96)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,96,'credits-used-bytes-in')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 97)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,97,'credits-used-bytes-out')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 98)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,98,'credits-used-bytes-total')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 99)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,99,'prepaid-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 100)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,100,'l2tp-ipsec-tunnel-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,101,'l2tp-ipsec-tunnel-disconnected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,102,'mip-ipsec-sa-inactive')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 103)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,103,'Long-Duration-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 104)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,104,'proxy-mip-registration-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 105)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,105,'proxy-mip-binding-update')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 106)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,106,'proxy-mip-inter-pdsn-handoff-Req-ip-Addr')
-- SIZE WARNING: 'proxy-mip-inter-pdsn-handoff-mismatched-Addr' is 44/40
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 107)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,107,'proxy-mip-inter-pdsn-handoff-mismatched-Addr')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 108)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,108,'Local-purge')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 109)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,109,'failed-update-handoff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 110)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,110,'closed_rp-handoff-complete')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 111)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,111,'closed_rp-duplicate-session')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 112)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,112,'closed_rp-handoff-session-not-found')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 113)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,113,'closed_rp-handoff-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 114)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,114,'pcf-monitor-keep-alive-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 115)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,115,'call-internal-reject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 116)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,116,'call-restarted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 117)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,117,'a11-mn-ha-auth-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 118)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,118,'a11-badly-formed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 119)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,119,'a11-t-bit-not-set')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 120)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,120,'a11-unsupported-vendor-id')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 121)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,121,'a11-mismatched-id')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 122)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,122,'mipha-dup-home-addr-req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 123)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,123,'mipha-dup-imsi-session')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 124)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,124,'ha-unreachable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 125)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,125,'IPSP-addr-in-use')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 126)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,126,'mipfa-dup-home-addr-req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 127)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,127,'mipha-ip-pool-busyout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 128)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,128,'inter-pdsn-handoff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 129)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,129,'active-to-dormant')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 130)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,130,'ppp-renegotiation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 131)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,131,'active-start-param-change')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 132)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,132,'tarrif-boundary')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 133)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,133,'a11-disconnect-no-active-stop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 134)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,134,'nw-reachability-failed-reject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 135)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,135,'nw-reachability-failed-redirect')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 136)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,136,'container-max-exceeded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 137)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,137,'static-addr-not-allowed-in-apn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 138)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,138,'static-addr-required-by-radius')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 139)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,139,'static-addr-not-allowed-by-radius')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 140)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,140,'mip-registration-dropped')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 141)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,141,'counter-rollover')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 142)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,142,'constructed-nai-auth-fail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 143)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,143,'inter-pdsn-Svc-optimize-handoff-disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 144)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,144,'gre-key-collision')
-- SIZE WARNING: 'inter-pdsn-Svc-optimize-handoff-triggered' is 41/40
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 145)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,145,'inter-pdsn-Svc-optimize-handoff-triggered')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 146)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,146,'intra-pdsn-handoff-triggered')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 147)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,147,'delayed-abort-timer-expired')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 148)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,148,'Admin-AAA-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 149)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,149,'Admin-AAA-disconnect-handoff')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 150)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,150,'PPP-IPV6CP-Neg-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 151)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,151,'PPP-IPV6CP-No-Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 152)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,152,'PPP-IPV6CP-Max-Retry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 153)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,153,'PPP-Restart-Invalid-source-IPV4-address')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 154)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,154,'a11-disconnect-handoff-no-active-stop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 155)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,155,'call-restarted-inter-pdsn-handoff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 156)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,156,'call-restarted-ppp-termination')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 157)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,157,'mipfa-resource-conflict')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 158)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,158,'failed-auth-with-charging-svc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 159)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,159,'mipha-dup-imsi-session-purge')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 160)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,160,'mipha-rev-pending-newcall')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 161)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,161,'volume-quota-reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 162)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,162,'duration-quota-reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 163)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,163,'gtp-user-authentication-failed')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 164)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,164,'MIP-reg-revocation-no-lcp-term')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 165)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,165,'MIP-private-ip-no-rev-tunnel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 166)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,166,'Invalid-Prepaid-AAA-attr-in-auth-Resp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 167)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,167,'mipha-prepaid-reset-dynamic-newcall')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 168)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,168,'gre-flow-control-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 169)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,169,'mip-paaa-bc-query-not-found')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 170)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,170,'mipha-dynamic-ip-addr-not-available')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 171)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,171,'a11-mismatched-id-on-handoff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 172)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,172,'a11-badly-formed-on-handoff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 173)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,173,'a11-unsupported-vendor-id-on-handoff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 174)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,174,'a11-t-bit-not-set-on-handoff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 175)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,175,'MIP-reg-revocation-i-bit-on')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 176)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,176,'A11-RRQ-Deny-Max-Count')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 177)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,177,'Dormant-Transition-During-Session-Setup')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 178)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,178,'PPP-Rem-Reneg-Disc-Always-Cfg')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 179)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,179,'PPP-Rem-Reneg-Disc-NAI-MSID-Mismatch')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 180)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,180,'mipha-subscriber-ipsec-tunnel-down')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 181)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,181,'mipha-subscriber-ipsec-tunnel-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 182)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,182,'mipha-subscriber-ipsecmgr-death')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 183)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,183,'flow-is-deactivated')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 184)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,184,'ecsv2-license-exceeded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 185)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,185,'IPSG-Auth-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 186)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,186,'driver-initiated')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 187)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,187,'ims-authorization-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 188)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,188,'service-instance-released')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 189)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,189,'flow-released')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 190)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,190,'ppp-renego-no-ha-addr')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 191)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,191,'intra-pdsn-handoff')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 192)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,192,'overload-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 193)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,193,'css-service-not-found')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 194)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,194,'Auth-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 195)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,195,'dhcp-client-sent-release')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 196)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,196,'dhcp-client-sent-nak')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 197)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,197,'msid-dhcp-chaddr-mismatch')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 198)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,198,'link-broken')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 199)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,199,'prog-end-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 200)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,200,'qos-update-wait-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 201)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,201,'css-synch-cause')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 202)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,202,'Gtp-context-replacement')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 203)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,203,'PDIF-Auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 204)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,204,'l2tp-unknown-apn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 205)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,205,'ms-unexpected-network-reentry')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 206)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,206,'r6-invalid-nai')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 207)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,207,'eap-max-retry-reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 208)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,208,'vbm-hoa-session-disconnected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 209)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,209,'vbm-voa-session-disconnected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 210)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,210,'in-acl-disconnect-on-violation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 211)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,211,'eap-msk-lifetime-expiry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 212)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,212,'eap-msk-lifetime-too-low')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 213)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,213,'mipfa-inter-tech-handoff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 214)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,214,'r6-max-retry-reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 215)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,215,'r6-nwexit-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 216)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,216,'r6-dereg-req-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 217)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,217,'r6-remote-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 218)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,218,'r6r4-protocol-errors')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 219)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,219,'wimax-qos-invalid-aaa-attr')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 220)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,220,'npu-gre-flows-not-available')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 221)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,221,'r4-max-retry-reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 222)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,222,'r4-nwexit-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 223)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,223,'r4-dereg-req-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 224)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,224,'r4-remote-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 225)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,225,'ims-authorization-revoked')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 226)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,226,'ims-authorization-released')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 227)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,227,'ims-auth-decision-invalid')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 228)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,228,'mac-addr-validation-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 229)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,229,'excessive-wimax-pd-flows-cfgd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 230)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,230,'sgsn-canc-loc-sub')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 231)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,231,'sgsn-canc-loc-upd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 232)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,232,'sgsn-mnr-exp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 233)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,233,'sgsn-ident-fail')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 234)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,234,'sgsn-sec-fail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 235)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,235,'sgsn-auth-fail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 236)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,236,'sgsn-glu-fail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 237)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,237,'sgsn-imp-det')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 238)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,238,'sgsn-smgr-purge')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 239)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,239,'sgsn-subs-handed-to-peer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 240)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,240,'sgsn-dns-fail-inter-rau')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 241)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,241,'sgsn-cont-rsp-fail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 242)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,242,'sgsn-hlr-not-found-for-imsi')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 243)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,243,'sgsn-ms-init-det')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 244)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,244,'sgsn-opr-policy-fail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 245)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,245,'sgsn-duplicate-context')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 246)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,246,'hss-profile-update-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 247)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,247,'sgsn-no-pdp-activated')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 248)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,248,'asnpc-idle-mode-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 249)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,249,'asnpc-idle-mode-exit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 250)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,250,'asnpc-idle-mode-auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 251)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,251,'asngw-invalid-qos-configuration')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 252)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,252,'sgsn-dsd-allgprswithdrawn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 253)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,253,'r6-pmk-key-change-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 254)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,254,'sgsn-illegal-me')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 255)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,255,'sess-termination-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 256)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,256,'sgsn-sai-fail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 257)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,257,'sgsn-rnc-removal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 258)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,258,'sgsn-rai-removal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 259)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,259,'sgsn-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 260)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,260,'ggsn-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 261)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,261,'hlr-init-deact')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 262)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,262,'ms-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 263)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,263,'sgsn-detach-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 264)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,264,'sgsn-rab-rel-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 265)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,265,'sgsn-iu-rel-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 266)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,266,'sgsn-gtpu-path-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 267)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,267,'sgsn-gtpc-path-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 268)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,268,'sgsn-local-handoff-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 269)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,269,'sgsn-remote-handoff-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 270)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,270,'sgsn-gtp-no-resource')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 271)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,271,'sgsn-rnc-no-resource')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 272)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,272,'sgsn-odb-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 273)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,273,'sgsn-invalid-ti')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 274)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,274,'sgsn-ggsn-ctxt-non-existent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 275)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,275,'sgsn-apn-restrict-vio')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 276)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,276,'sgsn-regular-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 277)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,277,'sgsn-abnormal-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 278)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,278,'sgsn-actv-rejected-by-peer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 279)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,279,'sgsn-err-ind')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 280)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,280,'asngw-non-anchor-prohibited')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 281)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,281,'asngw-im-entry-prohibited')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 282)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,282,'session-idle-mode-entry-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 283)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,283,'session-idle-mode-exit-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 284)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,284,'asnpc-ms-power-down-nwexit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 285)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,285,'asnpc-r4-nwexit-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 286)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,286,'sgsn-iu-rel-before-call-est')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 287)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,287,'ikev2-subscriber-ipsecmgr-death')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 288)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,288,'All-dynamic-pool-addr-occupied')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 289)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,289,'mip6ha-ip-addr-not-available')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 290)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,290,'bs-monitor-keep-alive-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 291)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,291,'sgsn-att-in-reg-state')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 292)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,292,'sgsn-inbound-srns-in-reg-state')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 293)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,293,'dt-ggsn-tun-reestablish-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 294)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,294,'sgsn-unknown-pdp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 295)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,295,'sgsn-pdp-auth-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 296)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,296,'sgsn-duplicate-pdp-context')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 297)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,297,'sgsn-no-rsp-from-ggsn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 298)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,298,'sgsn-failure-rsp-from-ggsn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 299)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,299,'sgsn-apn-unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 300)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,300,'sgsn-serv-req-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 301)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,301,'sgsn-attach-on-attch-init-abort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 302)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,302,'sgsn-iu-rel-in-israu-init-abort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 303)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,303,'sgsn-smgr-init-abort')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 304)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,304,'sgsn-mm-ctx-cleanup-init-abort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 305)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,305,'sgsn-unknown-abort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 306)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,306,'sgsn-guard-timeout-abort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 307)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,307,'vpn-bounce-dhcpip-validate-req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 308)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,308,'mipv6-id-mismatch')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 309)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,309,'aaa-session-id-not-found')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 310)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,310,'x1-max-retry-reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 311)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,311,'x1-nwexit-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 312)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,312,'x1-dereg-req-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 313)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,313,'x1-remote-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 314)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,314,'x1x2-protocol-errors')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 315)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,315,'x2-max-retry-reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 316)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,316,'x2-nwexit-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 317)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,317,'x2-dereg-req-recd')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 318)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,318,'x2-remote-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 319)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,319,'x1-pmk-key-change-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 320)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,320,'sa-rekeying-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 321)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,321,'sess-sleep-mode-entry-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 322)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,322,'phsgw-non-anchor-prohibited')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 323)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,323,'asnpc-pc-relocation-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 324)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,324,'asnpc-pc-relocation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 325)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,325,'auth_policy_mismatch')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 326)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,326,'sa-lifetime-expiry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 327)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,327,'asnpc-del-ms-entry-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 328)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,328,'phspc-sleep-mode-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 329)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,329,'phspc-sleep-mode-exit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 330)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,330,'phspc-sleep-mode-auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 331)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,331,'phspc-ms-power-down-nwexit')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 332)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,332,'phspc-x2-nwexit-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 333)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,333,'invalid-nat-config')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 334)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,334,'asngw-tid-entry-not-found')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 335)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,335,'No-NAT-IP-Address')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 336)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,336,'excessive-phs-pd-flows-cfgd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 337)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,337,'phsgw-invalid-qos-configuration')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 338)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,338,'Interim-Update')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 339)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,339,'sgsn-attach-abrt-rad-lost')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 340)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,340,'sgsn-inbnd-irau-abrt-rad-lost')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 341)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,341,'ike-keepalive-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 342)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,342,'sgsn-attach-abrt-ms-suspend')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 343)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,343,'sgsn-inbnd-irau-abrt-ms-suspend')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 344)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,344,'duplicate-session-detected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 345)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,345,'sgsn-xid-response-failure')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 346)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,346,'sgsn-nse-cleanup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 347)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,347,'sgsn-gtp-req-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 348)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,348,'sgsn-imsi-mismatch')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 349)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,349,'sgsn-bvc-blocked')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 350)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,350,'sgsn-attach-on-inbound-irau')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 351)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,351,'sgsn-attach-on-outbound-irau')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 352)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,352,'sgsn-incorrect-state')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 353)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,353,'sgsn-t3350-expiry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 354)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,354,'sgsn-page-timer-expiry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 355)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,355,'phsgw-tid-entry-not-found')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 356)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,356,'phspc-del-ms-entry-recd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 357)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,357,'sgsn-pdp-local-purge')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 358)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,358,'phs-invalid-nai')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 359)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,359,'session-sleep-mode-exit-timeout')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 360)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,360,'sgsn-offload-phase2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 361)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,361,'phs-thirdparty-auth-fail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 362)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,362,'remote-error-notify')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 363)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,363,'no-response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 364)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,364,'PDG-Auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 365)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,365,'mme-s1AP-send-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 366)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,366,'mme-egtpc-connection-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 367)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,367,'mme-egtpc-create-session-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 368)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,368,'mme-authentication-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 369)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,369,'mme-ue-detach')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 370)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,370,'mme-mme-detach')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 371)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,371,'mme-hss-detach')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 372)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,372,'mme-pgw-detach')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 373)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,373,'mme-sub-validation-failure')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 374)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,374,'mme-hss-connection-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 375)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,375,'mme-hss-user-unknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 376)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,376,'dhcp-lease-mismatch-detected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 377)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,377,'nemo-link-layer-down')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 378)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,378,'eapol-max-retry-reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 379)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,379,'sgsn-offload-phase3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 380)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,380,'mbms-bearer-service-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 381)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,381,'disconnect-on-violation-odb')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 382)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,382,'disconn-on-violation-focs-odb')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 383)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,383,'CSCF-REG-Admin-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 384)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,384,'CSCF-REG-User-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 385)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,385,'CSCF-REG-Inactivity-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 386)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,386,'CSCF-REG-Network-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 387)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,387,'CSCF-Call-Admin-disconnect')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 388)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,388,'CSCF-CAll-User-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 389)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,389,'CSCF-CALL-Local-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 390)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,390,'CSCF-CALL-No-Resource')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 391)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,391,'CSCF-CALL-No-Respone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 392)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,392,'CSCF-CALL-Inactivity-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 393)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,393,'CSCF-CALL-Media-Auth-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 394)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,394,'CSCF-REG-No-Resource')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 395)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,395,'ms-unexpected-idle-mode-entry')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 396)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,396,'re-auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 397)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,397,'sgsn-pdp-nse-cleanup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 398)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,398,'sgsn-mm-ctxt-gtp-no-resource')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 399)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,399,'unknown-apn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 400)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,400,'gtpc-path-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 401)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,401,'gtpu-path-failure')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 402)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,402,'actv-rejected-by-sgsn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 403)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,403,'sgsn-pdp-gprs-camel-release')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 404)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,404,'sgsn-check-imei-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 405)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,405,'sgsn-sndcp-init-deact')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 406)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,406,'sgsn-pdp-inactivity-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 407)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,407,'fw-and-nat-policy-removed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 408)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,408,'FNG-Auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 409)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,409,'ha-stale-key-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 410)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,410,'No-IPV6-address-for-subscriber')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 411)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,411,'prefix-registration-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 412)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,412,'disconnect-from-policy-server')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 413)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,413,'s6b-auth-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 414)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,414,'gtpc-err-ind')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 415)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,415,'gtpu-err-ind')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 416)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,416,'invalid-pdn-type')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 417)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,417,'aaa-auth-req-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 418)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,418,'apn-denied-no-subscription')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 419)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,419,'Sgw-context-replacement')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 420)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,420,'dup-static-ip-addr-req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 421)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,421,'apn-restrict-violation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 422)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,422,'invalid-wapn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 423)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,423,'ttg-nsapi-allocation-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 424)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,424,'mandatory-gtp-ie-missing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 425)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,425,'aaa-unreachable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 426)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,426,'asngw-service-flow-deletion')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 427)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,427,'CT-PMIP-RRQ-NVSE-Value-Change')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 428)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,428,'tcp-read-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 429)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,429,'tcp-write-failed')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 430)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,430,'ssl-handshake-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 431)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,431,'ssl-renegotiate-failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 432)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,432,'ssl-bad-message')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 433)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,433,'ssl-alert-received')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 434)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,434,'ssl-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 435)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,435,'ssl-migration')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 436)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,436,'sgsn-ard-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 437)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,437,'sgsn-camel-release')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 447)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,447,'Hotlining-Status-Change')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 448)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,448,'ggsn-no-rsp-from-sgsn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 449)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,449,'diameter-protocol-error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 450)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,450,'diameter-request-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 451)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,451,'operator-policy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 452)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,452,'spr-connection-timeout')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 453)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,453,'mipha-dup-wimax-session')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 454)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,454,'invalid-version-attr')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 3 AND Value = 455)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,8164,455,'sgsn-zone-code-failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,0,'Not-Defined')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,1,'Call-Lcp-Down')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,2,'Call-Disconnecting')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,3,'Call-PPP-Renegotiating')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,10,'Call-Lcp-Down_1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,11,'Call-Arrived')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,12,'Call-Lcp-Up')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,13,'Call-Authenticating')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,14,'Call-Authenticated')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,15,'Call-Ipcp-Up')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,16,'Call-Simple-IP-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,17,'Call-Mobile-IP-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,45,'Call-Pdg-Tcp-Connecting')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,46,'Call-Pdg-Ssl-Connecting')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 70)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,70,'Call-Bcmcs-Authenticating')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 85)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,85,'Call-Tunnel-Connecting')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 95)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,95,'Call-Imsa-Authorizing')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 97)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,97,'Call-Imsa-Authorized')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 98)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,98,'Call-MBMS-UE-Authorizing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 99)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,99,'Call-MBMS-Bearer-Authorizing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 115)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,115,'Call-Tunnel-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 120)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,120,'Call-Pdp-Type-IP-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 125)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,125,'Call-Pdp-Type-IPv6-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 130)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,130,'Call-Pdp-Type-PPP-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 140)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,140,'Call-Proxy-Mobile-IP-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 142)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,142,'Call-Pdg-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 141)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,141,'Call-Pdg-Ssl-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 145)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,145,'Call-Ipsg-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 150)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,150,'Call-Bcmcs-Connected')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 155)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,155,'Call-MBMS-UE-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 156)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,156,'Call-MBMS-Bearer-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 160)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,160,'Call-Pending-Addr-From-DHCP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 170)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,170,'Call-Got-Addr-From-DHCP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 180)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,180,'Call-HA-IPSEC-Tunnel-Connecting')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 190)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,190,'Call-HA-IPSEC-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 200)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,200,'Call-ASN-Non-Anchor-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 210)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,210,'Call-ASNPC-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 220)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,220,'Call-Mobile-IPv6-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 221)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,221,'Call-PMIPv6-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 230)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,230,'Call-PHSPC-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 235)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,235,'Call-GTP-IPv4-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 236)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,236,'Call-GTP-IPv6-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 237)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,237,'Call-GTP-IPv4-IPv6-Connected')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 245)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,245,'Call-SGW-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 246)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,246,'Call-MME-Attached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 4 AND Value = 247)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(4,8164,247,'Call-Auth-Only-Connected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 9 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,8164,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 9 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,8164,1,'Stac-LZS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 9 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,8164,2,'MPPC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 9 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,8164,3,'MPCC-Stac-LZS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 9 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,8164,4,'Deflate')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 9 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,8164,5,'Deflate-Stac-LZS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 9 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,8164,6,'Deflate-MPCC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 9 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9,8164,7,'Deflate-MPCC-Stac-LZS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 14 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,8164,0,'No')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 14 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(14,8164,1,'Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,1,'Simple-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,2,'Mobile-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,3,'Simple-IP-Mobile-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,4,'HA-Mobile-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,5,'Simple-IP-HA-Mobile-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,6,'Mobile-IP-HA-Mobile-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,7,'All')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,8,'GGSN-PDP-TYPE-IP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,16,'GGSN-PDP-TYPE-PPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,32,'Network-Mobility')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,38,'FA-HA-NEMO')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 20 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(20,8164,63,'All_New')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,1,'CLI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,2,'FTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,3,'CLI-FTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,4,'Intercept')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,5,'CLI-Intercept')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,7,'CLI-Intercept-FTP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,8,'ECS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,9,'CLI-ECS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,11,'CLI-FTP-ECS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,13,'CLI-Intercept-ECS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 21 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(21,8164,15,'CLI-Intercept-FTP-ECS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 22 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 22 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(22,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 19 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,8164,0,'Normal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 19 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(19,8164,1,'Stateless')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 63 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(63,8164,0,'Normal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 63 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(63,8164,1,'DF-Ignore')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 63 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(63,8164,2,'DF-Fragment-ICMP-Notify')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 267 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(267,8164,0,'Normal_End_Of_Session')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 267 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(267,8164,1,'Successful_Transaction')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 267 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(267,8164,2,'End_Of_Subscriber_Dialog')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 267 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(267,8164,3,'3XX_Redirection')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 267 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(267,8164,4,'4XX_Request_Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 267 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(267,8164,5,'5XX_Server_Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 267 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(267,8164,6,'6XX_Global_Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 267 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(267,8164,7,'Unspecified_Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 267 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(267,8164,8,'Unsuccessful_Session_Setup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 267 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(267,8164,9,'Internal_Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 293 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(293,8164,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 293 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(293,8164,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 291 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(291,8164,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 291 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(291,8164,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 292 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(292,8164,0,'Disable')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 292 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(292,8164,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 294 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(294,8164,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 294 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(294,8164,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 298 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(298,8164,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 298 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(298,8164,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 285 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(285,8164,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 285 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(285,8164,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 284 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(284,8164,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 284 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(284,8164,1,'Enable')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 282 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(282,8164,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 282 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(282,8164,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 283 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(283,8164,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 283 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(283,8164,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 140 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,8164,0,'QOSCHANGE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 140 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,8164,1,'TARIFFTIMECHANGE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 140 AND Value = 500)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(140,8164,500,'SGSNCHANGE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 49 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 49 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 157 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(157,8164,0,'auto-renew')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 157 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(157,8164,1,'disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 153 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,8164,0,'Any')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 153 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,8164,1,'Uplink')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 153 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,8164,2,'Downlink')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 25 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(25,8164,0,'Disable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 25 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(25,8164,1,'Enable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 144 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,8164,0,'No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 144 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(144,8164,1,'Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 198 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(198,8164,0,'False')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 198 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(198,8164,1,'True')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 68 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 68 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(68,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 54 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(54,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 54 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(54,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 62 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(62,8164,0,'GTP_VERSION_0')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 62 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(62,8164,1,'GTP_VERSION_1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 62 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(62,8164,2,'GTP_VERSION_2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 47 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(47,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 47 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(47,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 310 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(310,8164,0,'Active-Handoff')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 310 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(310,8164,1,'Location-Update')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 106 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,8164,0,'Deny')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 106 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,8164,1,'Accept')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 53 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(53,8164,0,'Alloc_Local_Pool')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 53 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(53,8164,1,'Alloc_Dhcp_Client')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 53 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(53,8164,2,'Alloc_Radius')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 53 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(53,8164,3,'Alloc_No_Alloc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 53 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(53,8164,4,'Alloc_Static_Alloc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 53 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(53,8164,5,'Alloc_Dhcp_Relay')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 150 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(150,8164,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 150 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(150,8164,1,'VJ')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 150 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(150,8164,2,'ROHC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 150 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(150,8164,3,'VJ_ROHC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 60 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(60,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 60 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(60,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 196 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 196 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(196,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 126 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(126,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 126 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(126,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 103 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(103,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 103 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(103,8164,1,'Enabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 43 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,8164,0,'no-local-alloc-validate')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 43 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,8164,1,'local-alloc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 43 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(43,8164,2,'local-alloc-validate')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 45 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,8164,1,'Detection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 45 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,8164,2,'Disconnection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 45 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,8164,3,'Dormant-Only-Disconnection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 45 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(45,8164,4,'Dormant-Only-Detection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 253 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(253,8164,0,'Suppress')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 253 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(253,8164,1,'Send')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 105 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,8164,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 105 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,8164,1,'No_Early_PDUs')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 105 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,8164,2,'Delay_GTP_Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 123 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(123,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 123 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(123,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 146 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(146,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 146 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(146,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 50 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 50 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(50,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 165 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(165,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 165 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(165,8164,1,'Enabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 51 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(51,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 51 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(51,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 163 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 163 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(163,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 188 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(188,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 188 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(188,8164,1,'EnabledOrNVSE_Starent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 188 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(188,8164,2,'NVSE_CUstom1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 188 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(188,8164,3,'NVSE_Custom2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 164 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(164,8164,0,'NoneOrDisabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 164 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(164,8164,1,'Starent_NVSE')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 164 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(164,8164,2,'NVSE_Custom1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 164 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(164,8164,3,'NVSE_Custom2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 48 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 48 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,8164,1,'EnabledOrNVSE_Custom1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 48 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,8164,2,'NVSE_Custom2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 48 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48,8164,3,'NVSE_Starent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 99 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,8164,1,'MD5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 99 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,8164,2,'MD5_RFC2002')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 99 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(99,8164,3,'HMAC_MD5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 151 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(151,8164,0,'Reliable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 151 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(151,8164,1,'Optimistic')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 151 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(151,8164,2,'Unidirectional')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 268 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(268,8164,0,'S-CSCF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 268 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(268,8164,1,'P-CSCF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 268 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(268,8164,2,'I-CSCF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 98 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,8164,0,'Best_Effort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 98 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,8164,1,'Bronze')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 98 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,8164,2,'Silver')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 98 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,8164,3,'Gold')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 98 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(98,8164,4,'From_DSCP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 143 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(143,8164,1,'Session-Disconnect')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 299 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(299,8164,0,'No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 299 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(299,8164,1,'Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 172 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(172,8164,0,'No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 172 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(172,8164,1,'Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 170 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(170,8164,0,'No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 170 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(170,8164,1,'Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 173 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(173,8164,0,'No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 173 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(173,8164,1,'Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 46 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(46,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 46 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(46,8164,1,'Enabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 134 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(134,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 134 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(134,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 95 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(95,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 95 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(95,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 130 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(130,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 130 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(130,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 92 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(92,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 92 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(92,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 92 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(92,8164,2,'Passive')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 93 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(93,8164,0,'Disabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 93 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(93,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 93 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(93,8164,2,'Passive')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 187 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(187,8164,0,'Never')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 187 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(187,8164,1,'Always')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 187 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(187,8164,2,'NAI_Prefix_MSID_Mismatch')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 128 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,8164,0,'no_prepaid')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 128 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,8164,1,'custom_prepaid')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 128 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,8164,2,'standard_prepaid')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 128 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(128,8164,4,'wimax_prepaid')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 31 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(31,8164,0,'Uncompressed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 31 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(31,8164,1,'Compressed')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 135 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(135,8164,0,'current_time')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 135 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(135,8164,1,'last-user-layer3-activity-time')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 135 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(135,8164,2,'last-airlink-activity-time')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 135 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(135,8164,3,'last-airlink-activity-time-last-reported')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 129 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(129,8164,0,'prepaid_duration')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 129 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(129,8164,1,'prepaid_volume')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 52 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 52 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(52,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 65530 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65530,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 65530 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(65530,8164,1,'Enabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,0,'Best-Effort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,1,'Pass-Through')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,10,'AF11')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,12,'AF12')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,14,'AF13')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,18,'AF21')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,20,'AF22')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,22,'AF23')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,26,'AF31')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,28,'AF32')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,30,'AF33')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,34,'AF41')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,36,'AF42')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,38,'AF43')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 113 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(113,8164,46,'EF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,0,'Best-Effort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,1,'Pass-Through')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,10,'AF11')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,12,'AF12')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,14,'AF13')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,18,'AF21')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,20,'AF22')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,22,'AF23')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,26,'AF31')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,28,'AF32')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,30,'AF33')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,34,'AF41')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,36,'AF42')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,38,'AF43')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 111 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,8164,46,'EF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,0,'Best-Effort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,1,'Pass-Through')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,10,'AF11')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,12,'AF12')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,14,'AF13')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,18,'AF21')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,20,'AF22')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,22,'AF23')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,26,'AF31')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,28,'AF32')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,30,'AF33')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,34,'AF41')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,36,'AF42')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,38,'AF43')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 114 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(114,8164,46,'EF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,0,'Best-Effort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,1,'Pass-Through')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,10,'AF11')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,12,'AF12')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,14,'AF13')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,18,'AF21')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,20,'AF22')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,22,'AF23')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,26,'AF31')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,28,'AF32')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,30,'AF33')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,34,'AF41')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,36,'AF42')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,38,'AF43')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 115 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(115,8164,46,'EF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,0,'Best-Effort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,1,'Pass-Through')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,10,'AF11')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,12,'AF12')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,14,'AF13')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,18,'AF21')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,20,'AF22')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,22,'AF23')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,26,'AF31')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,28,'AF32')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,30,'AF33')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,34,'AF41')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,36,'AF42')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,38,'AF43')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 116 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(116,8164,46,'EF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,0,'Best-Effort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,1,'Pass-Through')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,10,'AF11')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,12,'AF12')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,14,'AF13')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,18,'AF21')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,20,'AF22')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,22,'AF23')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,26,'AF31')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,28,'AF32')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,30,'AF33')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,34,'AF41')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,36,'AF42')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,38,'AF43')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 112 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,8164,46,'EF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 73 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(73,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 73 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(73,8164,1,'Policing')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 73 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(73,8164,2,'Shaping')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 79 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(79,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 79 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(79,8164,1,'Policing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 79 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(79,8164,2,'Shaping')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 236 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(236,8164,0,'No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 236 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(236,8164,1,'Yes')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 108 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,8164,0,'Deny')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 108 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,8164,1,'Accept')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 274 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(274,8164,0,'False')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 274 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(274,8164,1,'True')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 256 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(256,8164,0,'ORIGINATING_ROLE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 256 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(256,8164,1,'TERMINATING_ROLE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,1,'PDSN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,2,'Management')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,3,'HA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,4,'GGSN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,5,'LNS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,6,'IPSG')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,7,'CSCF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,8,'ASNGW')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,9,'PDIF')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,10,'STANDALONE_FA')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,11,'SGSN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,12,'PHSGW')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,13,'PDG')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,14,'MIPV6HA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,15,'PGW')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,16,'SGW')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,17,'FNG')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,18,'OGW')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,19,'HNBGW')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 24 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(24,8164,20,'BNG')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 225 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(225,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 225 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(225,8164,1,'Enabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 64 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,8164,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 64 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,8164,1,'Radius')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 64 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(64,8164,2,'GTPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 70 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(70,8164,0,'Normal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 70 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(70,8164,1,'Suppress')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 192 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,8164,0,'flow-based-auxilliary')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 192 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,8164,1,'flow-based-all')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 192 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,8164,2,'flow-based-none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 192 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,8164,3,'session-based')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 192 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(192,8164,4,'main-a10-only')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 100 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,8164,0,'None')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 100 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,8164,1,'No_Early_PDUs')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 100 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,8164,2,'Delay_GTP_Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 69 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,8164,0,'Normal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 69 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(69,8164,1,'Suppress')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 71 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(71,8164,0,'Normal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 71 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(71,8164,1,'Suppress')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 219 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,8164,0,'Normal_Subscriber')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 219 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,8164,1,'Ting_100')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 219 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,8164,2,'Ting_500')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 219 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,8164,3,'Ting_Buddy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 219 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,8164,4,'Ting_Star')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 219 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,8164,5,'Ting_Nolimit_SMS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 219 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,8164,6,'Kids_Locator')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 219 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,8164,7,'Ting_2000')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 219 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,8164,8,'Handicapped_Welfare')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 219 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(219,8164,9,'Reserved')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 67 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,8164,0,'Force')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 67 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(67,8164,1,'Detect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 85 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(85,8164,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 85 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(85,8164,1,'Access-Tunnel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 85 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(85,8164,2,'Data-Tunnel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 85 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(85,8164,3,'Both')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 133 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 133 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(133,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 221 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(221,8164,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 221 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(221,8164,1,'Receive')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 221 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(221,8164,2,'Transmit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 221 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(221,8164,3,'Both')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 77 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(77,8164,0,'Transmit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 77 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(77,8164,1,'Drop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 77 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(77,8164,2,'Lower-IP-Precedence')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 77 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(77,8164,3,'Buffer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 77 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(77,8164,4,'Transmit-On-Buffer-Full')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 78 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(78,8164,0,'Transmit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 78 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(78,8164,1,'Drop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 78 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(78,8164,2,'Lower-IP-Precedence')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 78 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(78,8164,3,'Buffer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 78 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(78,8164,4,'Transmit-On-Buffer-Full')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 83 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(83,8164,0,'Transmit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 83 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(83,8164,1,'Drop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 83 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(83,8164,2,'Lower-IP-Precedence')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 83 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(83,8164,3,'Buffer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 83 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(83,8164,4,'Transmit-On-Buffer-Full')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 84 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(84,8164,0,'Transmit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 84 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(84,8164,1,'Drop')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 84 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(84,8164,2,'Lower-IP-Precedence')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 84 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(84,8164,3,'Buffer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 84 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(84,8164,4,'Transmit-On-Buffer-Full')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 156 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(156,8164,0,'no-local-alloc-validate')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 156 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(156,8164,1,'local-alloc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 156 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(156,8164,2,'local-alloc-validate')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 174 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(174,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 174 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(174,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 27 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(27,8164,1,'random')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 27 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(27,8164,2,'balanced')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 27 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(27,8164,3,'prioritized')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 107 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,8164,0,'Deny')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 107 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,8164,1,'Accept')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 206 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(206,8164,0,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 8164 AND RadAttributeID = 206 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(206,8164,1,'Enabled')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 388)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(388,'Symbol')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 388 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,388,'Symbol-User-Privilege',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 388 AND RadAttributeID = 100)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(100,388,'Symbol-User-Login-Source',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 388 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,388,1,'Monitor')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 388 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,388,2,'Helpdesk')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 388 AND RadAttributeID = 1 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,388,4,'Nwadmin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 388 AND RadAttributeID = 1 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,388,8,'Sysadmin')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 388 AND RadAttributeID = 1 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,388,16,'WebAdmin')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 388 AND RadAttributeID = 1 AND Value = 32768)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,388,32768,'Superuser')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 388 AND RadAttributeID = 100 AND Value = 128)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,388,128,'Console')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 388 AND RadAttributeID = 100 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,388,64,'Telnet')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 388 AND RadAttributeID = 100 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,388,32,'SSH')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 388 AND RadAttributeID = 100 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(100,388,16,'Web')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 117)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(117,'Telebit')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 117 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,117,'Telebit-Login-Command',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 117 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,117,'Telebit-Port-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 117 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,117,'Telebit-Activate-Command',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 117 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,117,'Telebit-Accounting-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 6527)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(6527,'Timetra')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,6527,'Timetra-Access',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,6527,'Timetra-Home-Directory',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,6527,'Timetra-Restrict-To-Home',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,6527,'Timetra-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,6527,'Timetra-Default-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,6527,'Timetra-Cmd',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,6527,'Timetra-Action',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,6527,'Timetra-Exec-File',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6527 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,6527,1,'ftp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6527 AND RadAttributeID = 1 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,6527,2,'console')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6527 AND RadAttributeID = 1 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,6527,3,'both')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6527 AND RadAttributeID = 3 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,6527,1,'true')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6527 AND RadAttributeID = 3 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(3,6527,2,'false')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6527 AND RadAttributeID = 5 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(5,6527,1,'permit-all')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6527 AND RadAttributeID = 5 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(5,6527,2,'deny-all')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6527 AND RadAttributeID = 5 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(5,6527,3,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6527 AND RadAttributeID = 7 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,6527,1,'permit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 6527 AND RadAttributeID = 7 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7,6527,2,'deny')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,6527,'Alc-Primary-Dns',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,6527,'Alc-Secondary-Dns',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,6527,'Alc-Subsc-ID-Str',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,6527,'Alc-Subsc-Prof-Str',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,6527,'Alc-SLA-Prof-Str',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,6527,'Alc-Force-Renew',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,6527,'Alc-Create-Host',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,6527,'Alc-ANCP-Str',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,6527,'Alc-Retail-Serv-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,6527,'Alc-Default-Router',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,6527,'Alc-Acct-I-Inprof-Octets-64',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,6527,'Alc-Acct-I-Outprof-Octets-64',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,6527,'Alc-Acct-O-Inprof-Octets-64',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,6527,'Alc-Acct-O-Outprof-Octets-64',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,6527,'Alc-Acct-I-Inprof-Pkts-64',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,6527,'Alc-Acct-I-Outprof-Pkts-64',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,6527,'Alc-Acct-O-Inprof-Pkts-64',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,6527,'Alc-Acct-O-Outprof-Pkts-64',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,6527,'Alc-Client-Hardware-Addr',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 150)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(150,6527,'Timetra-APN-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 6527 AND RadAttributeID = 151)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(151,6527,'Timetra-APN-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 14525)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(14525,'Trapeze')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14525 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,14525,'Trapeze-VLAN-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14525 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,14525,'Trapeze-Mobility-Profile',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14525 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,14525,'Trapeze-Encryption-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14525 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,14525,'Trapeze-Time-Of-Day',0,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 14529)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(14529,'Tropos')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,14529,'Tropos-Unicast-Cipher',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,14529,'Tropos-Layer2-Input-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,14529,'Tropos-Layer2-Output-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,14529,'Tropos-Layer2-Input-Frames',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,14529,'Tropos-Layer2-Output-Frames',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,14529,'Tropos-Layer2-Input-Drops',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,14529,'Tropos-Noise-Floor',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,14529,'Tropos-Noise-Upper-Bound',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,14529,'Tropos-Release',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,14529,'Tropos-Secondary-IP',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,14529,'Tropos-Terminate-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,14529,'Tropos-Average-RSSI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,14529,'Tropos-Channel',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,14529,'Tropos-Retries-Sent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,14529,'Tropos-Retry-Bits',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,14529,'Tropos-Rates-Sent',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,14529,'Tropos-Rates-Received',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,14529,'Tropos-Routed-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,14529,'Tropos-Routless-Since',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,14529,'Tropos-Capability-Info',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,14529,'Tropos-Input-Cap',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,14529,'Tropos-Output-Cap',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,14529,'Tropos-Class-Mult',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,14529,'Tropos-Cell-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,14529,'Tropos-Cell-Location',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,14529,'Tropos-Serial-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,14529,'Tropos-Latitude',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14529 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,14529,'Tropos-Longitude',0,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14529 AND RadAttributeID = 1 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,14529,0,'Legacy-Open')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14529 AND RadAttributeID = 1 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,14529,1,'Legacy-WEP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14529 AND RadAttributeID = 1 AND Value = 5304834)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,14529,5304834,'WPA-TKIP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14529 AND RadAttributeID = 1 AND Value = 5304836)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,14529,5304836,'WPA-AES-CCMP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14529 AND RadAttributeID = 1 AND Value = 1027074)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,14529,1027074,'WPA2-TKIP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 14529 AND RadAttributeID = 1 AND Value = 1027076)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,14529,1027076,'WPA2-AES-CCMP')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 0 AND RadAttributeID = 250)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(250,0,'Char-Noecho',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 29 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(29,0,2,'Manage-Resources')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 126 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(126,0,1,'True')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 0 AND RadAttributeID = 126 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(126,0,0,'False')
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 429)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(429,'USR')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 102)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(102,429,'USR-Last-Number-Dialed-Out',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 232)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(232,429,'USR-Last-Number-Dialed-In-DNIS',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 233)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(233,429,'USR-Last-Callers-Number-ANI',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48952)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48952,429,'USR-Channel',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 49086)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49086,429,'USR-Event-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48943)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48943,429,'USR-Event-Date-Time',3,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 49143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49143,429,'USR-Call-Start-Date-Time',3,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 49142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49142,429,'USR-Call-End-Date-Time',3,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 94)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(94,429,'USR-Default-DTE-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48941)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48941,429,'USR-Initial-Rx-Link-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48940)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48940,429,'USR-Final-Rx-Link-Data-Rate',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 106)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(106,429,'USR-Initial-Tx-Link-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 107)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(107,429,'USR-Final-Tx-Link-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48945)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48945,429,'USR-Chassis-Temperature',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48772)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48772,429,'USR-Chassis-Temp-Threshold',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48946)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48946,429,'USR-Actual-Voltage',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48947)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48947,429,'USR-Expected-Voltage',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48948)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48948,429,'USR-Power-Supply-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48773)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48773,429,'USR-Card-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48953)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48953,429,'USR-Chassis-Slot',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 103)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(103,429,'USR-Sync-Async-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 104)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(104,429,'USR-Originate-Answer-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 108)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(108,429,'USR-Modulation-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 155)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(155,429,'USR-Connect-Term-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 105)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(105,429,'USR-Failure-to-Connect-Reason',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 111)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(111,429,'USR-Equalization-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 112)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(112,429,'USR-Fallback-Enabled',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 49127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49127,429,'USR-Connect-Time-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 49126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49126,429,'USR-Number-of-Rings-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,429,'USR-DTE-Data-Idle-Timout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 113)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(113,429,'USR-Characters-Sent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 114)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(114,429,'USR-Characters-Received',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 117)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(117,429,'USR-Blocks-Sent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 118)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(118,429,'USR-Blocks-Received',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 119)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(119,429,'USR-Blocks-Resent',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 120)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(120,429,'USR-Retrains-Requested',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 121)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(121,429,'USR-Retrains-Granted',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 122)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(122,429,'USR-Line-Reversals',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 123)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(123,429,'USR-Number-Of-Characters-Lost',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 125)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(125,429,'USR-Number-of-Blers',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 126)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(126,429,'USR-Number-of-Link-Timeouts',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 127)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(127,429,'USR-Number-of-Fallbacks',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 128)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(128,429,'USR-Number-of-Upshifts',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 129)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(129,429,'USR-Number-of-Link-NAKs',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 190)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(190,429,'USR-DTR-False-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 191)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(191,429,'USR-Fallback-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 192)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(192,429,'USR-Block-Error-Count-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 218)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(218,429,'USR-DTR-True-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48862)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48862,429,'USR-Security-Login-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48890)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48890,429,'USR-Security-Resp-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48919)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48919,429,'USR-DTE-Ring-No-Answer-Limit',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 124)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(124,429,'USR-Back-Channel-Data-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 153)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(153,429,'USR-Simplified-MNP-Levels',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 199)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(199,429,'USR-Simplified-V42bis-Usage',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 388)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(388,429,'USR-Mbi_Ct_PRI_Card_Slot',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 389)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(389,429,'USR-Mbi_Ct_TDM_Time_Slot',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 390)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(390,429,'USR-Mbi_Ct_PRI_Card_Span_Line',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 391)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(391,429,'USR-Mbi_Ct_BChannel_Used',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48759)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48759,429,'USR-Physical-State',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48916)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48916,429,'USR-Packet-Bus-Session',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 61440)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61440,429,'USR-Server-Time',3,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48733)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48733,429,'USR-Channel-Connected-To',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48734)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48734,429,'USR-Slot-Connected-To',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48735)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48735,429,'USR-Device-Connected-To',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48736)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48736,429,'USR-NFAS-ID',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48737)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48737,429,'USR-Q931-Call-Reference-Value',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48738)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48738,429,'USR-Call-Event-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48739)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48739,429,'USR-DS0',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48740)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48740,429,'USR-DS0s',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48742)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48742,429,'USR-Gateway-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36864)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36864,429,'USR-PW_USR_IFilter_IP',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36865)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36865,429,'USR-PW_USR_IFilter_IPX',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36867)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36867,429,'USR-PW_USR_OFilter_IP',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36868)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36868,429,'USR-PW_USR_OFilter_IPX',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36869)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36869,429,'USR-PW_USR_OFilter_SAP',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36870)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36870,429,'USR-PW_VPN_ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36871)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36871,429,'USR-PW_VPN_Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36872)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36872,429,'USR-PW_VPN_Neighbor',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36873)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36873,429,'USR-PW_Framed_Routing_V2',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36874)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36874,429,'USR-PW_VPN_Gateway',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36875)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36875,429,'USR-PW_Tunnel_Authentication',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36876)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36876,429,'USR-PW_Index',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36877)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36877,429,'USR-PW_Cutoff',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36878)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36878,429,'USR-PW_Packet',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36879)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36879,429,'USR-Primary_DNS_Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36880)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36880,429,'USR-Secondary_DNS_Server',2,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36881)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36881,429,'USR-Primary_NBNS_Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36882)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36882,429,'USR-Secondary_NBNS_Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36883)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36883,429,'USR-Syslog-Tap',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36889)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36889,429,'USR-Chassis-Call-Slot',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36890)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36890,429,'USR-Chassis-Call-Span',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36891)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36891,429,'USR-Chassis-Call-Channel',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36892)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36892,429,'USR-Keypress-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36893)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36893,429,'USR-Unauthenticated-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36899)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36899,429,'USR-Connect-Speed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36900)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36900,429,'USR-Framed_IP_Address_Pool_Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36901)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36901,429,'USR-MP-EDO',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38912)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38912,429,'USR-Bearer-Capabilities',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38913)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38913,429,'USR-Speed-Of-Connection',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38914)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38914,429,'USR-Max-Channels',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38915)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38915,429,'USR-Channel-Expansion',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38916)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38916,429,'USR-Channel-Decrement',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38917)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38917,429,'USR-Expansion-Algorithm',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38918)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38918,429,'USR-Compression-Algorithm',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38919)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38919,429,'USR-Receive-Acc-Map',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38920)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38920,429,'USR-Transmit-Acc-Map',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38922)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38922,429,'USR-Compression-Reset-Mode',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38923)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38923,429,'USR-Min-Compression-Size',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38924)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38924,429,'USR-IP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38925)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38925,429,'USR-IPX',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38926)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38926,429,'USR-Filter-Zones',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38927)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38927,429,'USR-Appletalk',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38928)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38928,429,'USR-Bridging',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38929)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38929,429,'USR-Spoofing',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38930)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38930,429,'USR-Host-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38931)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38931,429,'USR-Send-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38932)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38932,429,'USR-Send-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38933)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38933,429,'USR-Start-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38934)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38934,429,'USR-End-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38935)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38935,429,'USR-Send-Script1',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38936)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38936,429,'USR-Reply-Script1',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38937)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38937,429,'USR-Send-Script2',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38938)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38938,429,'USR-Reply-Script2',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38939)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38939,429,'USR-Send-Script3',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38940)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38940,429,'USR-Reply-Script3',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38941)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38941,429,'USR-Send-Script4',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38942)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38942,429,'USR-Reply-Script4',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38943)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38943,429,'USR-Send-Script5',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38944)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38944,429,'USR-Reply-Script5',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38945)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38945,429,'USR-Send-Script6',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38946)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38946,429,'USR-Reply-Script6',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38947)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38947,429,'USR-Terminal-Type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38948)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38948,429,'USR-Appletalk-Network-Range',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38949)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38949,429,'USR-Local-IP-Address',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38950)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38950,429,'USR-Routing-Protocol',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38951)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38951,429,'USR-Modem-Group',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38978)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38978,429,'USR-Modem-Training-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38979)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38979,429,'USR-Interface-Index',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38959)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38959,429,'USR-MP-MRRU',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36866)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36866,429,'USR-SAP-Filter-In',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36884)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36884,429,'USR-MIC',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36887)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36887,429,'USR-Log-Filter-Packets',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36894)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36894,429,'USR-VPN-Encrypter',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36896)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36896,429,'USR-Re-Chap-Timeout',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39016)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39016,429,'USR-Tunnel-Switch-Endpoint',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39024)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39024,429,'USR-IP-SAA-Filter',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2339)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2339,429,'Initial-Modulation-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38998)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38998,429,'USR-VTS-Session-Key',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38999)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38999,429,'USR-Orig-NAS-Type',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39000)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39000,429,'USR-Call-Arrival-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39001)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39001,429,'USR-Call-End-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39019)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39019,429,'USR-Tunnel-Auth-Hostname',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39020)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39020,429,'USR-Acct-Reason-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39049)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39049,429,'USR-Supports-Tags',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39051)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39051,429,'USR-HARC-Disconnect-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 461)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(461,429,'USR-RMMIE-Status',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2305)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2305,429,'USR-RMMIE-Last-Update-Event',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2313)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2313,429,'USR-RMMIE-x2-Status',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2314)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2314,429,'USR-RMMIE-Planned-Disconnect',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36895)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36895,429,'USR-VPN-GW-Location-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36897)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36897,429,'USR-CCP-Algorithm',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36898)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36898,429,'USR-ACCM-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36902)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36902,429,'USR-Local-Framed-IP-Addr',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38952)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38952,429,'USR-IPX-Routing',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38953)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38953,429,'USR-IPX-WAN',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38954)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38954,429,'USR-IP-RIP-Policies',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38955)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38955,429,'USR-IP-RIP-Simple-Auth-Password',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38956)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38956,429,'USR-IP-RIP-Input-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38957)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38957,429,'USR-IP-Call-Input-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38958)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38958,429,'USR-IPX-RIP-Input-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38960)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38960,429,'USR-IPX-Call-Input-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38961)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38961,429,'USR-AT-Input-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38962)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38962,429,'USR-AT-RTMP-Input-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38963)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38963,429,'USR-AT-Zip-Input-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38964)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38964,429,'USR-AT-Call-Input-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38965)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38965,429,'USR-ET-Bridge-Input-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38966)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38966,429,'USR-IP-RIP-Output-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38967)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38967,429,'USR-IP-Call-Output-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38968)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38968,429,'USR-IPX-RIP-Output-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38969)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38969,429,'USR-IPX-Call-Output-Filter',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38970)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38970,429,'USR-AT-Output-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38971)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38971,429,'USR-AT-RTMP-Output-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38972)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38972,429,'USR-AT-Zip-Output-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38973)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38973,429,'USR-AT-Call-Output-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38974)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38974,429,'USR-ET-Bridge-Output-Filter',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38975)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38975,429,'USR-ET-Bridge-Call-Output-Filte',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38976)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38976,429,'USR-IP-Default-Route-Option',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38977)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38977,429,'USR-MP-EDO-HIPER',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38980)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38980,429,'USR-Tunnel-Security',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38981)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38981,429,'USR-Port-Tap',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38982)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38982,429,'USR-Port-Tap-Format',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38983)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38983,429,'USR-Port-Tap-Output',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38984)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38984,429,'USR-Port-Tap-Facility',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38985)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38985,429,'USR-Port-Tap-Priority',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38986)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38986,429,'USR-Port-Tap-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38987)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38987,429,'USR-MobileIP-Home-Agent-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38988)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38988,429,'USR-Tunneled-MLPP',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38989)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38989,429,'USR-Multicast-Proxy',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38990)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38990,429,'USR-Multicast-Receive',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38992)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38992,429,'USR-Multicast-Forwarding',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38993)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38993,429,'USR-IGMP-Query-Interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38994)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38994,429,'USR-IGMP-Maximum-Response-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38995)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38995,429,'USR-IGMP-Robustness',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38996)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38996,429,'USR-IGMP-Version',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39018)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39018,429,'USR-Callback-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 61441)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61441,429,'USR-Request-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 462)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(462,429,'USR-RMMIE-Num-Of-Updates',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 479)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(479,429,'USR-RMMIE-Manufacturer-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 480)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(480,429,'USR-RMMIE-Product-Code',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 481)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(481,429,'USR-RMMIE-Serial-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 482)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(482,429,'USR-RMMIE-Firmware-Version',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 483)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(483,429,'USR-RMMIE-Firmware-Build-Date',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48722)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48722,429,'USR-Call-Arrival-in-GMT',3,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48721)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48721,429,'USR-Call-Connect-in-GMT',3,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48720)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48720,429,'USR-Call-Terminate-in-GMT',3,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48719)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48719,429,'USR-IDS0-Call-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 48765)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48765,429,'USR-Call-Reference-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 387)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(387,429,'USR-CDMA-Call-Reference-Number',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2190)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2190,429,'USR-Mobile-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 1012)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1012,429,'USR-IWF-IP-Address',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2192)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2192,429,'USR-Called-Party-Number',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2191)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2191,429,'USR-Calling-Party-Number',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2193)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2193,429,'USR-Call-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2194)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2194,429,'USR-ESN',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2195)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2195,429,'USR-IWF-Call-Identifier',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2196)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2196,429,'USR-IMSI',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2197)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2197,429,'USR-Service-Option',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2198)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2198,429,'USR-Disconnect-Cause-Indicator',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2199)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2199,429,'USR-Mobile-NumBytes-Txed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2200)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2200,429,'USR-Mobile-NumBytes-Rxed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2201)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2201,429,'USR-Num-Fax-Pages-Processed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2202)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2202,429,'USR-Compression-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2203)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2203,429,'USR-Call-Error-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2204)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2204,429,'USR-Modem-Setup-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2205)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2205,429,'USR-Call-Connecting-Time',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2206)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2206,429,'USR-Connect-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2304)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2304,429,'USR-RMMIE-Last-Update-Time',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2306)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2306,429,'USR-RMMIE-Rcv-Tot-PwrLvl',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2307)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2307,429,'USR-RMMIE-Rcv-PwrLvl-3300Hz',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2308)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2308,429,'USR-RMMIE-Rcv-PwrLvl-3750Hz',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2309)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2309,429,'USR-RMMIE-PwrLvl-NearEcho-Canc',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2310)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2310,429,'USR-RMMIE-PwrLvl-FarEcho-Canc',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2311)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2311,429,'USR-RMMIE-PwrLvl-Noise-Lvl',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 2312)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2312,429,'USR-RMMIE-PwrLvl-Xmit-Lvl',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36903)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36903,429,'USR-Framed-IPX-Route',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 36904)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36904,429,'USR-MPIP-Tunnel-Originator',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 38997)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38997,429,'USR-IGMP-Routing',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39008)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39008,429,'USR-Rad-Multicast-Routing-Ttl',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39009)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39009,429,'USR-Rad-Multicast-Routing-RtLim',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39010)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39010,429,'USR-Rad-Multicast-Routing-Proto',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39011)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39011,429,'USR-Rad-Multicast-Routing-Bound',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39012)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39012,429,'USR-Rad-Dvmrp-Metric',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39013)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39013,429,'USR-Chat-Script-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39014)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39014,429,'USR-CUSR-hat-Script-Rules',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39015)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39015,429,'USR-Rad-Location-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39017)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39017,429,'USR-OSPF-Addressless-Index',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 39029)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39029,429,'USR-DNIS-ReAuthentication',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 61442)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61442,429,'USR-NAS-Type',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 429 AND RadAttributeID = 61443)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61443,429,'USR-Auth-Mode',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36873 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36873,429,0,'Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36873 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36873,429,1,'On')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36883 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36883,429,0,'Off')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36883 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36883,429,1,'On-Raw')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36883 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36883,429,2,'On-Framed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,6,'Module-Inserted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,7,'Module-Removed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,8,'PSU-Voltage-Alarm')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,9,'PSU-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,10,'HUB-Temp-Out-of-Range')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,11,'Fan-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,12,'Watchdog-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,13,'Mgmt-Bus-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,14,'In-Connection-Est')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,15,'Out-Connection-Est')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,16,'In-Connection-Term')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,17,'Out-Connection-Term')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,18,'Connection-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,19,'Connection-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,20,'DTE-Transmit-Idle')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,21,'DTR-True')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,22,'DTR-False')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,23,'Block-Error-at-Threshold')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,24,'Fallbacks-at-Threshold')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,25,'No-Dial-Tone-Detected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,26,'No-Loop-Current-Detected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,27,'Yellow-Alarm')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,28,'Red-Alarm')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,29,'Loss-Of-Signal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,30,'Rcv-Alrm-Ind-Signal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,31,'Timing-Source-Switch')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,32,'Modem-Reset-by-DTE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,33,'Modem-Ring-No-Answer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,34,'DTE-Ring-No-Answer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,35,'Pkt-Bus-Session-Active')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,36,'Pkt-Bus-Session-Congestion')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,37,'Pkt-Bus-Session-Lost')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,38,'Pkt-Bus-Session-Inactive')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,39,'User-Interface-Reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,40,'Gateway-Port-Out-of-Service')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,41,'Gateway-Port-Link-Active')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,42,'Dial-Out-Login-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,43,'Dial-In-Login-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,44,'Dial-Out-Restricted-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,45,'Dial-Back-Restricted-Number')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,46,'User-Blacklisted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,47,'Attempted-Login-Blacklisted')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,48,'Response-Attempt-Limit-Exceeded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,49,'Login-Attempt-Limit-Exceeded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,50,'Dial-Out-Call-Duration')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,51,'Dial-In-Call-Duration')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,52,'Pkt-Bus-Session-Err-Status')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,53,'NMC-AutoRespnse-Trap')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,54,'Acct-Server-Contact-Loss')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,55,'Yellow-Alarm-Clear')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 56)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,56,'Red-Alarm-Clear')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,57,'Loss-Of-Signal-Clear')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,58,'Rcv-Alrm-Ind-Signal-Clear')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 59)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,59,'Incoming-Connection-Established')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,60,'Outgoing-Connection-Established')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 61)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,61,'Incoming-Connection-Terminated')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 62)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,62,'Outgoing-Connection-Terminated')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,63,'Connection-Attempt-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,64,'Continuous-CRC-Alarm')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 65)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,65,'Continuous-CRC-Alarm-Clear')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 66)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,66,'Physical-State-Change')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 71)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,71,'Gateway-Network-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 72)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,72,'Gateway-Network-Restored')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 73)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,73,'Packet-Bus-Clock-Lost')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 74)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,74,'Packet-Bus-Clock-Restored')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 75)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,75,'D-Channel-In-Service')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 76)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,76,'D-Channel-Out-of-Service')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 77)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,77,'DS0s-In-Service')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 78)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,78,'DS0s-Out-of-Service')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 79)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,79,'T1/T1PRI/E1PRI-Call-Event')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 80)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,80,'Psu-Incompatible')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 85)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,85,'DNS-Contact-Lost')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 86)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,86,'NTP-Contact-Lost')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 87)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,87,'NTP-Contact-Restored')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 88)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,88,'IPGW-Link-Up')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 89)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,89,'IPGW-Link-Down')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 90)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,90,'NTP-Contact-Degraded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 91)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,91,'In-Connection-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 92)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,92,'Out-Connection-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 93)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,93,'Application-ProcessorReset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 94)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,94,'DSP-Reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 95)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,95,'Changed-to-Maint-Srvs-State')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 96)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,96,'Loop-Back-cleared-on-channel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 97)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,97,'Loop-Back-on-channel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 98)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,98,'Telco-Abnormal-Response')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 99)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,99,'DNS-Contact-Restored')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 100)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,100,'DNS-Contact-Degraded')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 101)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,101,'RADIUS-Accounting-Restored')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 102)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,102,'RADIUS-Accounting-Group-Restore')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 103)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,103,'RADIUS-Accounting-Group-Degrade')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 104)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,104,'RADIUS-Accounting-Group-NonOper')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 119)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,119,'T1/T1-E1/PRI-InCall-Fail-Event')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 120)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,120,'T1/T1-E1/PRI-OutCall-Fail-Event')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 121)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,121,'RMMIE-Retrain-Event')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 122)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,122,'RMMIE-Speed-Shift-Event')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 191)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,191,'CDMA-Call-Start')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 49086 AND Value = 192)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(49086,429,192,'CDMA-Call-End')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1,'SlotEmpty')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,2,'SlotUnknown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,3,'NetwMgtCard')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,4,'DualT1NAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,5,'DualModemNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,6,'QuadModemNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,7,'TrGatewayNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,8,'X25GatewayNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,9,'DualV34ModemNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,10,'QuadV32DigitalModemNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,11,'QuadV32AnalogModemNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,12,'QuadV32DigAnlModemNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,13,'QuadV34DigModemNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,14,'QuadV34AnlModemNAC')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,15,'QuadV34DigAnlModemNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,16,'SingleT1NAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,17,'EthernetGatewayNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,18,'AccessServer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,19,'486TrGatewayNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,20,'486EthernetGatewayNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,22,'DualRS232NAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,23,'486X25GatewayNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,25,'ApplicationServerNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,26,'ISDNGatewayNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,27,'ISDNpriT1NAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,28,'ClkedNetMgtCard')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,29,'ModemPoolManagementNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,30,'ModemPoolNetserverNAC')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,31,'ModemPoolV34ModemNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,32,'ModemPoolISDNNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,33,'NTServerNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,34,'QuadV34DigitalG2NAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,35,'QuadV34AnalogG2NAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,36,'QuadV34DigAnlgG2NAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,37,'NETServerFrameRelayNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,38,'NETServerTokenRingNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,39,'X2524ChannelNAC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,42,'WirelessGatewayNac')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,44,'EnhancedAccessServer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,45,'EnhancedISDNGatewayNAC')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1001)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1001,'DualT1NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1002)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1002,'DualAlogMdmNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1003)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1003,'QuadDgtlMdmNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1004)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1004,'QuadAlogDgtlMdmNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1005)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1005,'TokenRingNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1006)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1006,'SingleT1NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1007)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1007,'EthernetNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1008)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1008,'ShortHaulDualT1NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1009)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1009,'DualAlogMgdIntlMdmNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1010)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1010,'X25NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1011)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1011,'QuadAlogNonMgdMdmNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1012)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1012,'QuadAlogMgdIntlMdmNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1013)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1013,'QuadAlogNonMgdIntlMdmNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1014)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1014,'QuadLsdLiMgdMdmNIC')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1015)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1015,'QuadLsdLiNonMgdMdmNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1016)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1016,'QuadLsdLiMgdIntlMdmNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1017)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1017,'QuadLsdLiNonMgdIntlMdmNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1018)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1018,'HSEthernetWithV35NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1019)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1019,'HSEthernetWithoutV35NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1020)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1020,'DualHighSpeedV35NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1021)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1021,'QuadV35RS232LowSpeedNIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1022)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1022,'DualE1NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1023)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1023,'ShortHaulDualE1NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1025)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1025,'BellcoreLongHaulDualT1NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1026)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1026,'BellcoreShrtHaulDualT1NIC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48773 AND Value = 1027)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48773,429,1027,'SCSIEdgeServerNIC')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,1,'110-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,2,'300-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,3,'600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,4,'1200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,5,'2400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,6,'4800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,7,'7200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,8,'9600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,9,'12K-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,10,'14.4K-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,11,'16.8-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,12,'19.2K-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,13,'38.4K-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,14,'75-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,15,'450-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,16,'UNKNOWN-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,17,'57.6K-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,18,'21.6K-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,19,'24K-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,20,'26K-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,21,'28K-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 94 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(94,429,22,'115K-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,1,'110-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,2,'300-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,3,'600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,4,'1200-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,5,'2400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,6,'4800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,7,'7200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,8,'9600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,9,'12000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,10,'14400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,11,'16800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,12,'19200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,13,'38400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,14,'75-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,15,'450-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,16,'UNKNOWN-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,17,'57600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,18,'21600-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,19,'24000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,20,'26400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,21,'28800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,22,'115200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,23,'31200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,24,'33600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,25,'25333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,26,'26666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,27,'28000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,28,'29333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,29,'30666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,30,'32000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,31,'33333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,32,'34666-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,33,'36000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,34,'37333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,35,'38666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,36,'40000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,37,'41333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,38,'42666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,39,'44000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,40,'45333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,41,'46666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,42,'48000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,43,'49333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,44,'50666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,45,'52000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,46,'53333-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,47,'54666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,48,'56000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,49,'57333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,50,'58666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,51,'60000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,52,'61333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,53,'62666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48941 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48941,429,54,'64000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,1,'110-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,2,'300-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,3,'600-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,4,'1200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,5,'2400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,6,'4800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,7,'7200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,8,'9600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,9,'12000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,10,'14400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,11,'16800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,12,'19200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,13,'38400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,14,'75-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,15,'450-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,16,'UNKNOWN-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,17,'57600-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,18,'21600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,19,'24000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,20,'26400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,21,'28800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,22,'115200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,23,'31200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,24,'33600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,25,'25333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,26,'26666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,27,'28000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,28,'29333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,29,'30666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,30,'32000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,31,'33333-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,32,'34666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,33,'36000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,34,'37333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,35,'38666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,36,'40000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,37,'41333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,38,'42666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,39,'44000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,40,'45333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,41,'46666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,42,'48000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,43,'49333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,44,'50666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,45,'52000-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,46,'53333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,47,'54666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,48,'56000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,49,'57333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,50,'58666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,51,'60000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,52,'61333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,53,'62666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48940 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48940,429,54,'64000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,1,'110-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,2,'300-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,3,'600-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,4,'1200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,5,'2400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,6,'4800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,7,'7200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,8,'9600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,9,'12000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,10,'14400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,11,'16800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,12,'19200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,13,'38400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,14,'75-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,15,'450-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,16,'UNKNOWN-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,17,'57600-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,18,'21600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,19,'24000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,20,'26400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,21,'28800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,22,'115200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,23,'31200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,24,'33600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,25,'25333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,26,'26666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,27,'28000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,28,'29333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,29,'30666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,30,'32000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,31,'33333-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,32,'34666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,33,'36000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,34,'37333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,35,'38666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,36,'40000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,37,'41333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,38,'42666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,39,'44000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,40,'45333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,41,'46666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,42,'48000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,43,'49333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,44,'50666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,45,'52000-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,46,'53333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,47,'54666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,48,'56000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,49,'57333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,50,'58666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,51,'60000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,52,'61333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,53,'62666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 106 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(106,429,54,'64000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,1,'110-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,2,'300-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,3,'600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,4,'1200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,5,'2400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,6,'4800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,7,'7200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,8,'9600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,9,'12000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,10,'14400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,11,'16800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,12,'19200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,13,'38400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,14,'75-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,15,'450-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,16,'UNKNOWN-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,17,'57600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,18,'21600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,19,'24000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,20,'26400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,21,'28800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,22,'115200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,23,'31200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,24,'33600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,25,'25333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,26,'26666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,27,'28000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,28,'29333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,29,'30666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,30,'32000-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,31,'33333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,32,'34666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,33,'36000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,34,'37333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,35,'38666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,36,'40000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,37,'41333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,38,'42666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,39,'44000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,40,'45333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,41,'46666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,42,'48000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,43,'49333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,44,'50666-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,45,'52000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,46,'53333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,47,'54666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,48,'56000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,49,'57333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,50,'58666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,51,'60000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,52,'61333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,53,'62666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 107 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(107,429,54,'64000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,1,'NONE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,2,'300-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,3,'1200-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,4,'2400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,5,'4800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,6,'7200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,7,'9600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,8,'12000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,9,'14400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,10,'16800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,11,'19200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,12,'21600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,13,'28800-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,14,'38400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,15,'57600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,16,'115200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,17,'288000-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,18,'75-1200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,19,'1200-75-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,20,'24000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,21,'26400-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,22,'31200-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,23,'33600-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,24,'33333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,25,'37333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,26,'41333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,27,'42666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,28,'44000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,29,'45333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,30,'46666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,31,'48000-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,32,'49333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,33,'50666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,34,'52000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,35,'53333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,36,'54666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,37,'56000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,38,'57333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,39,'64000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,40,'25333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,41,'26666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,42,'28000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,43,'29333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,44,'30666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,45,'32000-BPS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,46,'34666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,47,'36000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,48,'38666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,49,'40000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,50,'58666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,51,'60000-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,52,'61333-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36899 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36899,429,53,'62666-BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 103 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(103,429,1,'Asynchronous')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 103 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(103,429,2,'Synchronous')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 104 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(104,429,1,'Originate_in_Originate_Mode')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 104 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(104,429,2,'Originate_in_Answer_Mode')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 104 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(104,429,3,'Answer_in_Originate_Mode')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 104 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(104,429,4,'Answer_in_Answer_Mode')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,1,'usRoboticsHST')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,2,'ccittV32')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,3,'ccittV22bis')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,4,'bell103')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,5,'ccittV21')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,6,'bell212')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,7,'ccittV32bis')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,8,'ccittV23')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,9,'negotiationFailed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,10,'bell208b')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,11,'v21FaxClass1')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,12,'v27FaxClass1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,13,'v29FaxClass1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,14,'v17FaxClass1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,15,'v21FaxClass2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,16,'v27FaxClass2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,17,'v29FaxClass2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,18,'v17FaxClass2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,19,'v32Terbo')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,20,'v34')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,21,'vFC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,22,'v34plus')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,23,'x2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,24,'v110')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,25,'v120')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,26,'x75')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,27,'asyncSyncPPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,28,'clearChannel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,29,'x2client')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,30,'x2symmetric')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,31,'piafs')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,32,'x2version2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,33,'v90Analog')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,34,'v90Digital')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 108 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(108,429,35,'v90AllDigital')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,1,'usRoboticsHST')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,2,'ccittV32')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,3,'ccittV22bis')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,4,'bell103')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,5,'ccittV21')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,6,'bell212')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,7,'ccittV32bis')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,8,'ccittV23')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,9,'negotiationFailed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,10,'bell208b')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,11,'v21FaxClass1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,12,'v27FaxClass1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,13,'v29FaxClass1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,14,'v17FaxClass1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,15,'v21FaxClass2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,16,'v27FaxClass2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,17,'v29FaxClass2')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,18,'v17FaxClass2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,19,'v32Terbo')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,20,'v34')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,21,'vFC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,22,'v34plus')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,23,'x2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,24,'v110')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,25,'v120')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,26,'x75')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,27,'asyncSyncPPP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,28,'clearChannel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,29,'x2client')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,30,'x2symmetric')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,31,'piafs')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,32,'x2version2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,33,'v90Analogue')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,34,'v90Digital')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2339 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2339,429,35,'v90AllDigital')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,1,'dtrDrop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,2,'escapeSequence')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,3,'athCommand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,4,'carrierLoss')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,5,'inactivityTimout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,6,'mnpIncompatible')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,7,'undefined')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,8,'remotePassword')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,9,'linkPassword')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,10,'retransmitLimit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,11,'linkDisconnectMsgReceived')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,12,'noLoopCurrent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,13,'invalidSpeed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,14,'unableToRetrain')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,15,'managementCommand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,16,'noDialTone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,17,'keyAbort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,18,'lineBusy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,19,'noAnswer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,20,'voice')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,21,'noAnswerTone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,22,'noCarrier')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,23,'undetermined')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,24,'v42SabmeTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,25,'v42BreakTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,26,'v42DisconnectCmd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,27,'v42IdExchangeFail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,28,'v42BadSetup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,29,'v42InvalidCodeWord')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,30,'v42StringToLong')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,31,'v42InvalidCommand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,32,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,33,'v32Cleardown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,34,'dialSecurity')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,35,'remoteAccessDenied')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,36,'loopLoss')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,37,'ds0Teardown')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,38,'promptNotEnabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,39,'noPromptingInSync')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,40,'nonArqMode')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,41,'modeIncompatible')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,42,'noPromptInNonARQ')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,43,'dialBackLink')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,44,'linkAbort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,45,'autopassFailed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,46,'pbGenericError')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,47,'pbLinkErrTxPreAck')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,48,'pbLinkErrTxTardyACK')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,49,'pbTransmitBusTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,50,'pbReceiveBusTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,51,'pbLinkErrTxTAL')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,52,'pbLinkErrRxTAL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,53,'pbTransmitMasterTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,54,'pbClockMissing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,55,'pbReceivedLsWhileLinkUp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 56)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,56,'pbOutOfSequenceFrame')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,57,'pbBadFrame')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,58,'pbAckWaitTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 59)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,59,'pbReceivedAckSeqErr')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,60,'pbReceiveOvrflwRNRFail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 61)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,61,'pbReceiveMsgBufOvrflw')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 62)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,62,'rcvdGatewayDiscCmd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,63,'tokenPassingTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,64,'dspInterruptTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 65)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,65,'mnpProtocolViolation')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 66)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,66,'class2FaxHangupCmd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 67)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,67,'hstSpeedSwitchTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 68)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,68,'tooManyUnacked')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 69)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,69,'timerExpired')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 70)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,70,'t1Glare')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 71)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,71,'priDialoutRqTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 72)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,72,'abortAnlgDstOvrIsdn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 73)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,73,'normalUserCallClear')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 74)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,74,'normalUnspecified')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 75)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,75,'bearerIncompatibility')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 76)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,76,'protocolErrorEvent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 77)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,77,'abnormalDisconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 78)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,78,'invalidCauseValue')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 79)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,79,'resourceUnavailable')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 80)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,80,'remoteHungUpDuringTraining')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 81)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,81,'trainingTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 82)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,82,'incomingModemNotAvailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 83)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,83,'incomingInvalidBearerCap')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 84)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,84,'incomingInvalidChannelID')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 85)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,85,'incomingInvalidProgInd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 86)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,86,'incomingInvalidCallingPty')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 87)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,87,'incomingInvalidCalledPty')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 88)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,88,'incomingCallBlock')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 89)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,89,'incomingLoopStNoRingOff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 90)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,90,'outgoingTelcoDisconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 91)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,91,'outgoingEMWinkTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 92)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,92,'outgoingEMWinkTooShort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 93)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,93,'outgoingNoChannelAvail')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 94)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,94,'dspReboot')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 95)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,95,'noDSPRespToKA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 96)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,96,'noDSPRespToDisc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 97)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,97,'dspTailPtrInvalid')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 155 AND Value = 98)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(155,429,98,'dspHeadPtrInvalid')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,1,'dtrDrop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,2,'escapeSequence')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,3,'athCommand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,4,'carrierLoss')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,5,'inactivityTimout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,6,'mnpIncompatible')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,7,'undefined')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,8,'remotePassword')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,9,'linkPassword')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,10,'retransmitLimit')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,11,'linkDisconnectMsgRec')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,12,'noLoopCurrent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,13,'invalidSpeed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,14,'unableToRetrain')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,15,'managementCommand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,16,'noDialTone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,17,'keyAbort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,18,'lineBusy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,19,'noAnswer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,20,'voice')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,21,'noAnswerTone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,22,'noCarrier')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,23,'undetermined')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,24,'v42SabmeTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,25,'v42BreakTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,26,'v42DisconnectCmd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,27,'v42IdExchangeFail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,28,'v42BadSetup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,29,'v42InvalidCodeWord')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,30,'v42StringToLong')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,31,'v42InvalidCommand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,32,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 33)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,33,'v32Cleardown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 34)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,34,'dialSecurity')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 35)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,35,'remoteAccessDenied')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 36)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,36,'loopLoss')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 37)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,37,'ds0Teardown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 38)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,38,'promptNotEnabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 39)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,39,'noPromptingInSync')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 40)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,40,'nonArqMode')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 41)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,41,'modeIncompatible')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 42)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,42,'noPromptInNonARQ')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 43)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,43,'dialBackLink')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 44)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,44,'linkAbort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 45)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,45,'autopassFailed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 46)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,46,'pbGenericError')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 47)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,47,'pbLinkErrTxPreAck')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 48)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,48,'pbLinkErrTxTardyACK')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 49)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,49,'pbTransmitBusTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 50)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,50,'pbReceiveBusTimeout')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 51)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,51,'pbLinkErrTxTAL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 52)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,52,'pbLinkErrRxTAL')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 53)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,53,'pbTransmitMasterTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 54)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,54,'pbClockMissing')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 55)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,55,'pbReceivedLsWhileLinkUp')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 56)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,56,'pbOutOfSequenceFrame')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 57)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,57,'pbBadFrame')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 58)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,58,'pbAckWaitTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 59)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,59,'pbReceivedAckSeqErr')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 60)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,60,'pbReceiveOvrflwRNRFail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 61)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,61,'pbReceiveMsgBufOvrflw')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 62)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,62,'rcvdGatewayDiscCmd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 63)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,63,'tokenPassingTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 64)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,64,'dspInterruptTimeout')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 65)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,65,'mnpProtocolViolation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 66)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,66,'class2FaxHangupCmd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 67)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,67,'hstSpeedSwitchTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 68)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,68,'tooManyUnacked')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 69)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,69,'timerExpired')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 70)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,70,'t1Glare')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 71)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,71,'priDialoutRqTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 72)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,72,'abortAnlgDstOvrIsdn')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 73)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,73,'normalUserCallClear')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 74)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,74,'normalUnspecified')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 75)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,75,'bearerIncompatibility')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 76)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,76,'protocolErrorEvent')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 77)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,77,'abnormalDisconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 78)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,78,'invalidCauseValue')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 79)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,79,'resourceUnavailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 80)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,80,'remoteHungUpDuringTraining')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 81)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,81,'trainingTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 82)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,82,'incomingModemNotAvailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 83)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,83,'incomingInvalidBearerCap')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 84)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,84,'incomingInvalidChannelID')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 85)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,85,'incomingInvalidProgInd')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 86)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,86,'incomingInvalidCallingPty')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 87)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,87,'incomingInvalidCalledPty')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 88)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,88,'incomingCallBlock')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 89)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,89,'incomingLoopStNoRingOff')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 90)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,90,'outgoingTelcoDisconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 91)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,91,'outgoingEMWinkTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 92)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,92,'outgoingEMWinkTooShort')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 93)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,93,'outgoingNoChannelAvail')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 94)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,94,'dspReboot')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 95)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,95,'noDSPRespToKA')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 96)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,96,'noDSPRespToDisc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 97)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,97,'dspTailPtrInvalid')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 105 AND Value = 98)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(105,429,98,'dspHeadPtrInvalid')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,1,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,2,'mnpLevel3')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,3,'mnpLevel4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,4,'ccittV42')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,5,'usRoboticsHST')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,6,'synchronousNone')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,7,'mnpLevel2')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,8,'mnp10')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,9,'v42Etc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,10,'mnp10Etc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,11,'lapmEtc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,12,'v42Etc2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,13,'v42SRej')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 153 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(153,429,14,'piafs')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 199 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(199,429,1,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 199 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(199,429,2,'ccittV42bis')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 199 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(199,429,3,'mnpLevel5')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 111 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,429,1,'Long')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 111 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(111,429,2,'Short')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 112 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,429,1,'Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 112 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(112,429,2,'Enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 124 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(124,429,1,'450BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 124 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(124,429,2,'300BPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 124 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(124,429,3,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48735 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48735,429,1,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48735 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48735,429,2,'isdnGateway')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48735 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48735,429,3,'quadModem')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,1,'notSupported')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,2,'setup')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,3,'usrSetup')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,4,'telcoDisconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,5,'usrDisconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,6,'noFreeModem')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,7,'modemsNotAllowed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,8,'modemsRejectCall')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,9,'modemSetupTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,10,'noFreeIGW')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,11,'igwRejectCall')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,12,'igwSetupTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,13,'noFreeTdmts')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,14,'bcReject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,15,'ieReject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,16,'chidReject')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,17,'progReject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,18,'callingPartyReject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,19,'calledPartyReject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,20,'blocked')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,21,'analogBlocked')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,22,'digitalBlocked')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,23,'outOfService')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,24,'busy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,25,'congestion')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,26,'protocolError')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,27,'noFreeBchannel')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,28,'inOutCallCollision')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,29,'inCallArrival')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,30,'outCallArrival')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,31,'inCallConnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 48738 AND Value = 32)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(48738,429,32,'outCallConnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,0,'No-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,1,'No-Carrier')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,2,'No-DSR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,3,'Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,4,'Reset')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,5,'Call-Drop-Req')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,6,'Idle-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,7,'Session-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,8,'User-Req-Drop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,9,'Host-Req-Drop')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,10,'Service-Interruption')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,11,'Service-Unavailable')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,12,'User-Input-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,13,'NAS-Drop-For-Callback')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,14,'NAS-Drop-Misc-Non-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,15,'NAS-Internal-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,16,'Line-Busy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,19,'Tunnel-Term-Unreach')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,20,'Tunnel-Refused')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,21,'Tunnel-Auth-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,22,'Tunnel-Session-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,23,'Tunnel-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,25,'Radius-Res-Reclaim')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,26,'DNIS-Auth-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,27,'PAP-Auth-Failure')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 28)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,28,'CHAP-Auth-Failure')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 29)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,29,'PPP-LCP-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 30)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,30,'PPP-NCP-Failed')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39051 AND Value = 31)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39051,429,31,'Radius-Timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36897 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36897,429,1,'NONE')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36897 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36897,429,2,'Stac')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36897 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36897,429,3,'MS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 36897 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36897,429,4,'Any')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38980 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38980,429,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38980 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38980,429,1,'Control-Only')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38980 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38980,429,2,'Data-Only')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38980 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38980,429,3,'Both-Data-and-Control')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 461 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(461,429,1,'notEnabledInLocalModem')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 461 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(461,429,2,'notDetectedInRemoteModem')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 461 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(461,429,3,'ok')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,1,'notOperational')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,2,'operational')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,3,'x2Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,4,'v8Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,5,'remote3200Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,6,'invalidSpeedSetting')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,7,'v8NotDetected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,8,'x2NotDetected')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,9,'incompatibleVersion')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,10,'incompatibleModes')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,11,'local3200Disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,12,'excessHighFrequencyAtten')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,13,'connectNotSupport3200')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2313 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2313,429,14,'retrainBeforeConnection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,1,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,2,'dteNotReady')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,3,'dteInterfaceError')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,4,'dteRequest')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,5,'escapeToOnlineCommandMode')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,6,'athCommand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,7,'inactivityTimeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,8,'arqProtocolError')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,9,'arqProtocolRetransmitLim')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,10,'invalidComprDataCodeword')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,11,'invalidComprDataStringLen')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2314 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2314,429,12,'invalidComprDataCommand')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2305 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2305,429,1,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2305 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2305,429,2,'initialConnection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2305 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2305,429,3,'retrain')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2305 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2305,429,4,'speedShift')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 2305 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(2305,429,5,'plannedDisconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,1,'Access-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,2,'Access-Accept')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,3,'Access-Reject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,4,'Accounting-Request')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,5,'Accounting-Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,7,'Access-Password-Change')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,8,'Access-Password-Ack')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,9,'Access-Password-Reject')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,11,'Access-Challenge')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,12,'Status-Server')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,13,'Status-Client')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,21,'Resource-Free-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,22,'Resource-Free-Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,23,'Resource-Query-Request')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 24)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,24,'Resource-Query-Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 25)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,25,'Disconnect-User')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 26)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,26,'NAS-Reboot-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 27)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,27,'NAS-Reboot-Response')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 253)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,253,'Tacacs-Message')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61441 AND Value = 255)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61441,429,255,'Reserved')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38913 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38913,429,0,'Auto')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38913 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38913,429,1,'56')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38913 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38913,429,2,'64')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38913 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38913,429,3,'Voice')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38917 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38917,429,1,'Constant')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38917 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38917,429,2,'Linear')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38918 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38918,429,0,'None')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38918 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38918,429,1,'Stac')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38918 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38918,429,2,'Ascend')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38918 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38918,429,3,'Microsoft')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38918 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38918,429,4,'Auto')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38922 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38922,429,0,'Auto')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38922 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38922,429,1,'Reset-Every-Packet')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38922 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38922,429,2,'Reset-On-Error')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38926 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38926,429,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38926 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38926,429,2,'disabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38928 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38928,429,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38928 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38928,429,2,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38927 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38927,429,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38927 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38927,429,2,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38929 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38929,429,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38929 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38929,429,2,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38950 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38950,429,1,'Rip1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38950 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38950,429,2,'Rip2')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38952 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38952,429,0,'none')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38952 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38952,429,1,'send')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38952 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38952,429,2,'listen')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38952 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38952,429,3,'respond')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38952 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38952,429,4,'all')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38953 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38953,429,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38953 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38953,429,2,'disabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38976 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38976,429,1,'enabled')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 38976 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(38976,429,2,'disabled')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39018 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39018,429,1,'Normal')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39018 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39018,429,2,'ANI')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39018 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39018,429,3,'Static')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 39018 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(39018,429,4,'Dynamic')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61442 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61442,429,0,'3Com-NMC')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61442 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61442,429,1,'3Com-NETServer')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61442 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61442,429,2,'3Com-HiPerArc')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61442 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61442,429,4,'3Com-SA-Server')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61442 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61442,429,5,'Ascend')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61442 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61442,429,6,'Generic-RADIUS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61442 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61442,429,7,'3Com-NETBuilder-II')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61443 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61443,429,0,'Auth-3Com')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61443 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61443,429,1,'Auth-Ace')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61443 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61443,429,2,'Auth-Safeword')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61443 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61443,429,3,'Auth-UNIX-PW')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61443 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61443,429,4,'Auth-Defender')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61443 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61443,429,5,'Auth-TACACSP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61443 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61443,429,6,'Auth-Netware')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61443 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61443,429,7,'Auth-Skey')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61443 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61443,429,8,'Auth-EAP-Proxy')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 429 AND RadAttributeID = 61443 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(61443,429,9,'Auth-UNIX-Crypt')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 7064)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(7064,'UTStarcom')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 140)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(140,7064,'UTStarcom-VLAN-ID',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 142)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(142,7064,'UTStarcom-CommittedBandwidth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 143)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(143,7064,'UTStarcom-MaxBandwidth',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 145)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(145,7064,'UTStarcom-Priority',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 147)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(147,7064,'UTStarcom-Error-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 152)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(152,7064,'UTStarcom-PrimaryDNS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 153)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(153,7064,'UTStarcom-SecondaryDNS',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 161)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(161,7064,'UTStarcom-MaxBurstSize',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 162)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(162,7064,'UTStarcom-MaxDelay',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 163)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(163,7064,'UTStarcom-MaxJitter',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 165)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(165,7064,'UTStarcom-DeviceId',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 166)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(166,7064,'UTStarcom-Module-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 167)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(167,7064,'UTStarcom-Port-No',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 168)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(168,7064,'UTStarcom-Logical-Port-No',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 169)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(169,7064,'UTStarcom-UNI-MAX-MAC',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 170)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(170,7064,'UTStarcom-Default-Gateway',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 171)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(171,7064,'UTStarcom-CLI-Access-Level',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 180)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(180,7064,'UTStarcom-Act-Input-Octets',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 181)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(181,7064,'UTStarcom-Act-Output-Octets',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 182)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(182,7064,'UTStarcom-Act-Input-Frames',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 183)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(183,7064,'UTStarcom-Act-Output-Frames',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 184)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(184,7064,'UTStarcom-Onu-MC-Filter-Enable',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 185)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(185,7064,'UTStarcom-UNI-Auto-Negotiation',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 186)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(186,7064,'UTStarcom-UNI-Speed',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 187)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(187,7064,'UTStarcom-UNI-Duplex',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 188)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(188,7064,'UTStarcom-ONU-Admin_status',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 7064 AND RadAttributeID = 189)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(189,7064,'UTStarcom-ONU-FW-SC-Upgrade',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 16313)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(16313,'ValemountNetworks')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16313 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,16313,'VNC-PPPoE-CBQ-RX',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16313 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,16313,'VNC-PPPoE-CBQ-TX',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16313 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,16313,'VNC-PPPoE-CBQ-RX-Fallback',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16313 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,16313,'VNC-PPPoE-CBQ-TX-Fallback',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 16313 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,16313,'splash',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 2180)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(2180,'Versanet')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 2180 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,2180,'Versanet-Termination-Cause',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,0,'Normal-Hangup-No-Error-Occurred')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,3,'Call-Waiting-Caused-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,4,'Physical-Carrier-Loss')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,5,'No-err-correction-at-other-end')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,6,'No-resp-to-feature-negotiation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,7,'1st-modem-async-only-2nd-sync')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,8,'No-framing-technique-in-common')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,9,'No-protocol-in-common')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,10,'Bad-resp-to-feature-negotiation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,11,'No-sync-info-from-remote-modem')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,12,'Normal-Hangup-by-Remote-modem')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,13,'Retransmission-limit-reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 14)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,14,'Protocol-violation-occurred')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 15)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,15,'Lost-DTR')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 16)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,16,'Received-GSTN-cleardown')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 17)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,17,'Inactivity-timeout')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 18)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,18,'Speed-not-supported')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 19)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,19,'Long-space-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 20)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,20,'Key-abort-disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 21)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,21,'Clears-previous-disc-reason')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 22)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,22,'No-connection-established')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 2180 AND RadAttributeID = 1 AND Value = 23)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(1,2180,23,'Disconnect-after-three-retrains')
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 24757)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(24757,'WiMAX-Forum')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,24757,'WiMAX-Capability',20,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 257)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(257,24757,'WiMAX-Cap-Release',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 258)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(258,24757,'WiMAX-Cap-Accounting-Capabilities',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 259)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(259,24757,'WiMAX-Cap-Hotlining-Capabilities',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 260)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(260,24757,'WiMAX-Cap-Idle-Mode-Notification-Cap',26,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 258 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(258,24757,0,'No-Accounting')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 258 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(258,24757,1,'IP-Session-Based')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 258 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(258,24757,2,'Flow-Based')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 259 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(259,24757,0,'Not-Supported')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 259 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(259,24757,1,'Hotline-Profile-Id')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 259 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(259,24757,2,'NAS-Filter-Rule')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 259 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(259,24757,4,'HTTP-Redirection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 259 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(259,24757,8,'IP-Redirection')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 260 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(260,24757,0,'Not-Supported')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 260 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(260,24757,1,'Supported')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,24757,'WiMAX-Device-Authentication-Indicator',26,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,24757,'WiMAX-GMT-Timezone-offset',29,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,24757,'WiMAX-AAA-Session-Id',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,24757,'WiMAX-MSK',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,24757,'WiMAX-hHA-IP-MIP4',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,24757,'WiMAX-hHA-IP-MIP6',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,24757,'WiMAX-DHCPv4-Server',24,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,24757,'WiMAX-DHCPv6-Server',24,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,24757,'WiMAX-MN-hHA-MIP4-Key',7,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,24757,'WiMAX-MN-hHA-MIP4-SPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 12)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(12,24757,'WiMAX-MN-hHA-MIP6-Key',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 13)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(13,24757,'WiMAX-MN-hHA-MIP6-SPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 14)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(14,24757,'WiMAX-FA-RK-Key',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 15)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(15,24757,'WiMAX-HA-RK-Key',7,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 16)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(16,24757,'WiMAX-HA-RK-SPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 17)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(17,24757,'WiMAX-HA-RK-Lifetime',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 18)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(18,24757,'WiMAX-RRQ-HA-IP',24,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 19)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(19,24757,'WiMAX-RRQ-MN-HA-Key',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 20)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(20,24757,'WiMAX-RRQ-MN-HA-SPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,24757,'WiMAX-Session-Continue',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,24757,'WiMAX-Beginning-Of-Session',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,24757,'WiMAX-IP-Technology',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 23 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,24757,0,'Reserved-0')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 23 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,24757,1,'Reserved-1')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 23 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,24757,2,'PMIP4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 23 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,24757,3,'CMIP4')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 23 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,24757,4,'CMIP6')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 23 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(23,24757,5,'Ethernet-CS')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,24757,'WiMAX-Hotline-Indicator',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,24757,'WiMAX-Prepaid-Indicator',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,24757,'WiMAX-PDFID',25,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 27)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(27,24757,'WiMAX-SDFID',25,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 28)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(28,24757,'WiMAX-Packet-Flow-Descriptor',20,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7169)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7169,24757,'WiMAX-PFD-Packet-Data-Flow-Id',25,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7170)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7170,24757,'WiMAX-PFD-Service-Data-Flow-Id',25,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7171)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7171,24757,'WiMAX-PFD-Service-Profile-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7172)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7172,24757,'WiMAX-PFD-Direction',26,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7172 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7172,24757,0,'Reserved-0')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7172 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7172,24757,1,'Uplink')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7172 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7172,24757,2,'Downlink')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7172 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7172,24757,3,'Bi-Directional')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7173)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7173,24757,'WiMAX-PFD-Activation-Trigger',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7174)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7174,24757,'WiMAX-PFD-Transport-Type',26,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7174 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7174,24757,0,'Reserved-0')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7174 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7174,24757,1,'IPv4-CS')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7174 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7174,24757,2,'IPv6-CS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7174 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7174,24757,3,'Ethernet')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7175)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7175,24757,'WiMAX-PFD-Uplink-QOS-Id',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7176)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7176,24757,'WiMAX-PFD-Downlink-QOS-Id',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7177)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7177,24757,'WiMAX-PFD-Uplink-Classifier',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7178)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7178,24757,'WiMAX-PFD-Downlink-Classifier',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 29)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(29,24757,'WiMAX-QoS-Descriptor',20,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7425)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7425,24757,'WiMAX-QoSD-QoS-Id',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7426)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7426,24757,'WiMAX-QoSD-Global-Service-Class-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7427)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7427,24757,'WiMAX-QoSD-Service-Class-Name',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7428)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7428,24757,'WiMAX-QoSD-Schedule-Type',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7429)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7429,24757,'WiMAX-QoSD-Traffic-Priority',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7430)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7430,24757,'WiMAX-QoSD-Max-Sustained-Traffic-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7431)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7431,24757,'WiMAX-QoSD-Minimum-Reserved-Traffic-Rate',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7432)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7432,24757,'WiMAX-QoSD-Maximum-Traffic-Burst',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7433)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7433,24757,'WiMAX-QoSD-Tolerated-Jitter',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7434)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7434,24757,'WiMAX-QoSD-Maximum-Latency',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7435)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7435,24757,'WiMAX-QoSD-Reduced-Resources-Code',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7436)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7436,24757,'WiMAX-QoSD-Media-Flow-Type',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7437)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7437,24757,'WiMAX-QoSD-Unsolicited-Grant-Interval',25,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7438)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7438,24757,'WiMAX-QoSD-SDU-Size',25,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7439)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7439,24757,'WiMAX-QoSD-Unsolicited-Polling-Interval',25,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 7440)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7440,24757,'WiMAX-QoSD-Media-Flow-Description-SDP',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7428 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7428,24757,2,'Best-Effort')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7428 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7428,24757,3,'nrtPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7428 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7428,24757,4,'rtPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7428 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7428,24757,5,'Extended-rtPS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7428 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7428,24757,6,'UGS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,1,'VoIP')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,2,'Robust-Browser')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,3,'Secure-Browser-VPN')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,4,'Streaming-Video')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,5,'Streaming-Live-TV')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,6,'Music-Photo-Download')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,7,'Multi-Player-Gaming')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,8,'Location-Based-Services')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,9,'Text-Audio-Books')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,10,'Video-Conversation')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 11)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,11,'Message')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 12)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,12,'Control')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 7436 AND Value = 13)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(7436,24757,13,'Data')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 30)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(30,24757,'WiMAX-Uplink-Granted-QoS',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 31)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(31,24757,'WiMAX-Control-Packets-In',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 32)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(32,24757,'WiMAX-Control-Octets-In',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 33)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(33,24757,'WiMAX-Control-Packets-Out',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 34)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(34,24757,'WiMAX-Control-Octets-Out',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 35)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(35,24757,'WiMAX-PPAC',20,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 8961)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8961,24757,'WiMAX-PPAC-Available-In-Client',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 8961 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8961,24757,1,'Volume-Metering')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 8961 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8961,24757,2,'Duration-Metering')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 8961 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8961,24757,4,'Resource-Metering')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 8961 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(8961,24757,8,'Pools')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 36)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(36,24757,'WiMAX-Session-Termination-Capability',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 36 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(36,24757,1,'Dynamic-Authorization')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 37)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(37,24757,'WiMAX-PPAQ',20,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9473)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9473,24757,'WiMAX-PPAQ-Quota-Identifier',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9474)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9474,24757,'WiMAX-PPAQ-Volume-Quota',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9475)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9475,24757,'WiMAX-PPAQ-Volume-Threshold',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9476)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9476,24757,'WiMAX-PPAQ-Duration-Quota',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9477)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9477,24757,'WiMAX-PPAQ-Duration-Threshold',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9478)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9478,24757,'WiMAX-PPAQ-Resource-Quota',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9479)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9479,24757,'WiMAX-PPAQ-Resource-Threshold',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9480)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9480,24757,'WiMAX-PPAQ-Update-Reason',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9481)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9481,24757,'WiMAX-PPAQ-Prepaid-Server',24,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9482)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9482,24757,'WiMAX-PPAQ-Service-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9483)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9483,24757,'WiMAX-PPAQ-Rating-Group-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9484)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9484,24757,'WiMAX-PPAQ-Termination-Action',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9485)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9485,24757,'WiMAX-PPAQ-Pool-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9486)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9486,24757,'WiMAX-PPAQ-Pool-Multiplier',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9487)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9487,24757,'WiMAX-PPAQ-Requested-Action',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9488)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9488,24757,'WiMAX-PPAQ-Check-Balance-Result',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9489)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9489,24757,'WiMAX-PPAQ-Cost-Information-AVP',6,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9480 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9480,24757,1,'Pre-Initialization')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9480 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9480,24757,2,'Initial-Request')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9480 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9480,24757,3,'Threshold-Reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9480 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9480,24757,4,'Quota-Reached')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9480 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9480,24757,5,'TITSU-Approaching')
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9480 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9480,24757,6,'Remote-Forced-Disconnect')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9480 AND Value = 7)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9480,24757,7,'Client-Service-Termination')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9480 AND Value = 8)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9480,24757,8,'Access-Service-Terminated')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9480 AND Value = 9)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9480,24757,9,'Service-Not-Established')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9480 AND Value = 10)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9480,24757,10,'One-Time-Charging')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9484 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9484,24757,1,'Terminate')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9484 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9484,24757,2,'Request-More-Quota')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9484 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9484,24757,3,'Redirect-Or-Filter')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9487 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9487,24757,1,'Balance-Check')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 9487 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(9487,24757,2,'Price-Enquiry')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 38)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(38,24757,'WiMAX-Prepaid-Tariff-Switching',20,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9729)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9729,24757,'WiMAX-PTS-Prepaid-Quota-Identifier',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9730)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9730,24757,'WiMAX-PTS-Volume-Used-After',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9731)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9731,24757,'WiMAX-PTS-Tariff-Switch-Interval',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 9732)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9732,24757,'WiMAX-PTS-Time-Interval-After',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 39)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(39,24757,'WiMAX-Active-Time-Duration',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 40)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(40,24757,'WiMAX-DHCP-RK',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 41)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(41,24757,'WiMAX-DHCP-RK-Key-Id',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 42)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(42,24757,'WiMAX-DHCP-RK-Lifetime',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 43)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(43,24757,'WiMAX-DHCP-Msg-Server-IP',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 44)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(44,24757,'WiMAX-Idle-Mode-Transition',26,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 45)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(45,24757,'WiMAX-NAP-Id',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 46)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(46,24757,'WiMAX-BS-Id',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 47)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(47,24757,'WiMAX-Location',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 48)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(48,24757,'WiMAX-Acct-Input-Packets-Gigaword',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 49)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(49,24757,'WiMAX-Acct-Output-Packets-Gigaword',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 50)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(50,24757,'WiMAX-Uplink-Flow-Description',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 51)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(51,24757,'WiMAX-Blu-Coa-IPv6',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 52)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(52,24757,'WiMAX-DNS-Server',24,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 53)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(53,24757,'WiMAX-Hotline-Profile-Id',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 54)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(54,24757,'WiMAX-HTTP-Redirection-Rule',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 55)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(55,24757,'WiMAX-IP-Redirection-Rule',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 56)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(56,24757,'WiMAX-Hotline-Session-Timer',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 57)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(57,24757,'WiMAX-NSP-Id',6,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 58)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(58,24757,'WiMAX-HA-RK-Key-Requested',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 58 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(58,24757,0,'No')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 58 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(58,24757,1,'Yes')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 59)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(59,24757,'WiMAX-Count-Type',26,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 60)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(60,24757,'WiMAX-DM-Action-Code',1,3)
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 60 AND Value = 0)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(60,24757,0,'Deregister-MS')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 60 AND Value = 1)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(60,24757,1,'Suspend-MS-Traffic')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 60 AND Value = 2)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(60,24757,2,'Suspend-User-Traffic')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 60 AND Value = 3)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(60,24757,3,'Resume-Traffic')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 60 AND Value = 4)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(60,24757,4,'MS-Terminate')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 60 AND Value = 5)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(60,24757,5,'MS-Idle')
IF NOT EXISTS (SELECT * FROM RadValues WHERE RadVendorID = 24757 AND RadAttributeID = 60 AND Value = 6)
	INSERT INTO RadValues (RadAttributeID,RadVendorID,Value,Name) VALUES(60,24757,6,'MS-Completed-IPv6-Handover')
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 61)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(61,24757,'WiMAX-FA-RK-SPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 62)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(62,24757,'WiMAX-Downlink-Flow-Description',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 64)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(64,24757,'WiMAX-vHA-IP-MIP4',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 65)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(65,24757,'WiMAX-vHA-IP-MIP6',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 66)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(66,24757,'WiMAX-vHA-MIP4-Key',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 67)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(67,24757,'WiMAX-vHA-RK-Key',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 68)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(68,24757,'WiMAX-vHA-RK-SPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 69)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(69,24757,'WiMAX-vHA-RK-Lifetime',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 70)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(70,24757,'WiMAX-MN-vHA-MIP6-Key',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 71)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(71,24757,'WiMAX-MN-vHA-MIP4-SPI',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 72)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(72,24757,'WiMAX-MN-vHA-MIP6-SPI',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 73)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(73,24757,'WiMAX-vDHCPv4-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 74)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(74,24757,'WiMAX-vDHCPv6-Server',5,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 75)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(75,24757,'WiMAX-vDHCP-RK',7,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 76)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(76,24757,'WiMAX-vDHCP-RK-Key-ID',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 24757 AND RadAttributeID = 77)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(77,24757,'WiMAX-vDHCP-RK-Lifetime',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 14122)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(14122,'WISPr')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,14122,'WISPr-Location-ID',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,14122,'WISPr-Location-Name',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,14122,'WISPr-Logoff-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,14122,'WISPr-Redirection-URL',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,14122,'WISPr-Bandwidth-Min-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,14122,'WISPr-Bandwidth-Min-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 7)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(7,14122,'WISPr-Bandwidth-Max-Up',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 8)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(8,14122,'WISPr-Bandwidth-Max-Down',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 9)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(9,14122,'WISPr-Session-Terminate-Time',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 10)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(10,14122,'WISPr-Session-Terminate-End-Of-Day',0,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 11)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(11,14122,'WISPr-Billing-Class-Of-Service',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 21)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(21,14122,'WISPr-Session-Input-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 22)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(22,14122,'WISPr-Session-Input-Gigawords',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 23)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(23,14122,'WISPr-Session-Output-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 24)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(24,14122,'WISPr-Session-Output-Gigawords',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 25)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(25,14122,'WISPr-Session-Octets',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 14122 AND RadAttributeID = 26)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(26,14122,'WISPr-Session-Gigawords',1,3)
IF NOT EXISTS (SELECT * FROM RadVendors WHERE RadVendorID = 838)
	INSERT INTO RadVendors (RadVendorID,Name) VALUES(838,'Xedia')
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 838 AND RadAttributeID = 1)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(1,838,'Xedia-DNS-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 838 AND RadAttributeID = 2)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(2,838,'Xedia-NetBios-Server',2,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 838 AND RadAttributeID = 3)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(3,838,'Xedia-Address-Pool',0,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 838 AND RadAttributeID = 4)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(4,838,'Xedia-PPP-Echo-Interval',1,3)
GO

IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 838 AND RadAttributeID = 5)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(5,838,'Xedia-SSH-Privileges',1,3)
IF NOT EXISTS (SELECT * FROM RadAttributes WHERE RadVendorID = 838 AND RadAttributeID = 6)
	INSERT INTO RadAttributes (RadAttributeID,RadVendorID,Name,RadAttributeType,ReplyType) VALUES(6,838,'Xedia-Client-Access-Network',0,3)
INSERT INTO AccountTypes (AccountType,Description,DNISGroupID,SortOrder) VALUES('PPP', 'PPP Dialup', NULL,1)
go
INSERT INTO AccountTypes (AccountType,Description,DNISGroupID,SortOrder) VALUES('ISDN', 'ISDN', NULL,2)
go
INSERT INTO RadATConfigs (AccountTypeID,RadAttributeID,RadVendorID,Data,Value,RadCheck) VALUES(1, 6, 0, '2', 2, 0)		
go
INSERT INTO RadATConfigs (AccountTypeID,RadAttributeID,RadVendorID,Data,Value,RadCheck) VALUES(1, 7, 0, '1', 1, 0)
go
INSERT INTO RadATConfigs (AccountTypeID,RadAttributeID,RadVendorID,Data,Value,RadCheck) VALUES(2, 6, 0, '2', 2, 0)	
go
INSERT INTO RadATConfigs (AccountTypeID,RadAttributeID,RadVendorID,Data,Value,RadCheck) VALUES(2, 7, 0, '1', 1, 0)	
go
INSERT INTO Servers (Server,IPAddress,Comments,Secret,RadRoamServerID) VALUES('localhost', '127.0.0.1', 'localhost', 'localhost', NULL)
go
INSERT INTO ServerPorts (ServerID,Port,NASIdentifier,IPAddress,MaxSessionTime,UserName,AcctStatusType,CallDate,FramedAddress) VALUES(1, '0', '127.0.0.1', NULL, 0, NULL, 0, NULL, NULL)
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES( 0, 'Generic',    'Start at 0', 	0, NULL)
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES( 1, 'Generic',    'Start at 1',	0, NULL)
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES( 2, 'Livingston', 'Portmaster 2',	0, '.1.3.6.1.4.1.307.3.2.1.1.1.4.')
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES( 3, 'Livingston', 'Portmaster 3',	0, '.1.3.6.1.4.1.307.3.2.1.1.1.4.')
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES( 4, 'Livingston', 'Portmaster 4',	0, '.1.3.6.1.4.1.307.3.2.1.1.1.4.')
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES( 5, 'Ascend',     'Max 4000 T1',	1, '.1.3.6.1.4.1.529.12.3.1.4.')
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES( 6, 'Ascend',     'Max 4000 E1',	1, '.1.3.6.1.4.1.529.12.3.1.4.')
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES( 7, 'Ascend',     'MAX 1800 BRI',	1, '.1.3.6.1.4.1.529.12.3.1.4.')
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES( 8, 'Ascend',     'MAX TNT',	1, '.1.3.6.1.4.1.529.12.3.1.4.')
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES( 9, 'Cisco',      'AS5200',	0, '.1.3.6.1.4.1.9.2.9.2.1.18.')
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES(10, 'USR',        'Total Control',0, NULL)
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES(11, 'CompuTone',  'PowerRack',	0, NULL)
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES(12, 'Microcom',   '6000',		0, NULL)
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES(13, 'USR',	'HyperARC',	0, '.1.3.6.1.4.1.429.4.10.1.1.18.')
go
INSERT INTO ServerTypes(ServerType, Vendor, Model, SNMPType, SNMPUser) VALUES(14, 'Nortel',	'Nortel 5399',	0, '.1.3.6.1.4.1.15.2.16.1.2.1.3.1.')
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(0, 'Generic Message', 0)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(1, 'Generic Error', 5)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(10, 'User Not Found', 3)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(11, 'Bad Password', 3)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(12, 'User Expired', 2)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(13, 'User OverDue', 2)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(14, 'Over Login Limit', 1)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(15, 'No Time Left', 1)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(16, 'Port Not Allowed', 2)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(17, 'Reject Attribute',2)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(18, 'Check Attribute', 2)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(19, 'No Attributes', 3)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(20, 'User Inactive', 2)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(21, 'Start Date not reached',2)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(40, 'SNMP Check Failed', 1)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(50, 'Bad Request', 4)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(51, 'No UserName', 4)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(52, 'No Password', 4)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(53, 'Bad Digest', 6)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(54, 'Bad username', 3)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(60, 'Parse Error', 5)
go
INSERT INTO RadLogMsgs (RadLogMsgID,Description,Severity) VALUES(100, 'Chap Not Allowed', 5)
go
INSERT INTO MasterAccounts (FirstName, LastName, Active, CancelDate, OverDue) VALUES('Test', 'Test', 1, NULL, 0)
go
INSERT INTO SubAccounts (CustomerID, Active, Login, Email, FirstName, LastName, AccountTypeID, Password, ExpireDate, Extension, LoginLimit, TimeLeft) VALUES(1, 1, 'test', NULL, 'Test', 'Test', 1, 'test', NULL, 0, 1, NULL)
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(3,1,0,'Challenge-Response')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(4,1,0,'NAS-IP-Address')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(5,1,0,'NAS-Port')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(6,1,0,'User-Service')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(7,1,0,'Framed-Protocol')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(8,1,0,'Framed-Address')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(9,1,0,'Framed-Netmask')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(10,1,0,'Framed-Routing')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(11,1,0,'Framed-Filter')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(12,1,0,'Framed-MTU')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(13,1,0,'Framed-Compression')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(14,1,0,'Login-Host')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(15,1,0,'Login-Service')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(16,1,0,'Login-TCP-Port')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(17,1,0,'Change-Password')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(18,1,0,'Reply-Message')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(19,1,0,'Callback-Number')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(20,1,0,'Callback-Name')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(21,1,0,'Expiration')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(22,1,0,'Framed-Route')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(23,1,0,'Framed-IPX-Network')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(24,1,0,'State')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(25,1,0,'Class')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(26,1,0,'Vendor-Specific')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(27,1,0,'Session-Timeout')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(28,1,0,'Idle-Timeout')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(29,1,0,'Termination-Action')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(30,1,0,'NAS-Port-DNIS')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(31,1,0,'Caller-Id')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(32,1,0,'NAS-Identifier')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(33,1,0,'Proxy-State')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(34,1,0,'Login-LAT-Service')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(35,1,0,'Login-LAT-Node')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(36,1,0,'Login-LAT-Group')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(37,1,0,'Framed-AppleTalk-Link')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(38,1,0,'Framed-AppleTalk-Network')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(39,1,0,'Framed-AppleTalk-Zone')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(40,1,0,'Acct-Status-Type')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(41,1,0,'Acct-Delay-Time')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(42,1,0,'Acct-Input-Octets')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(43,1,0,'Acct-Output-Octets')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(44,1,0,'Acct-Session-Id')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(45,1,0,'Acct-Authentic')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(46,1,0,'Acct-Session-Time')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(47,1,0,'Acct-Input-Packets')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(48,1,0,'Acct-Output-Packets')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(49,1,0,'Acct-Terminate-Cause')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(50,1,0,'Acct-Multi-Session-ID')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(51,1,0,'Acct-Link-Count')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(52,1,0,'Acct-Input-Gigawords')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(53,1,0,'Acct-Output-Gigawords')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(55,1,0,'Event-Time')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(60,1,0,'CHAP-Challenge')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(61,1,0,'NAS-Port-Type')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(62,1,0,'Port-Limit')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(63,1,0,'Login-LAT-Port')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(64,1,0,'Tunnel-Type')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(65,1,0,'Tunnel-Medium-Type')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(66,1,0,'Tunnel-Client-Endpoint')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(67,1,0,'Tunnel-Server-Endpoirt')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(68,1,0,'Tunnel-Connection')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(69,1,0,'Tunnel-Password')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(70,1,0,'ARAP-Password')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(71,1,0,'ARAP-Features')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(72,1,0,'ARAP-Zone-Access')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(73,1,0,'ARAP-Security')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(74,1,0,'ARAP-Security-Data')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(75,1,0,'Password-Retry')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(76,1,0,'Prompt')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(77,1,0,'Connect-Info')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(78,1,0,'Configuration-Token')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(79,1,0,'EAP-Message')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(80,1,0,'Signature')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(81,1,0,'Tunnel-Private-Group-ID')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(82,1,0,'Tunnel-Assignment-ID')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(83,1,0,'Tunnel-Preference')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(84,1,0,'ARAP-Challenge-Response')
go
INSERT INTO MapAttributes (RadAttributeID,MapType,RadVendorID,MapAttribute) VALUES(85,1,0,'Acct-Interim-Interval')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'User-Service',1,NULL,'Login-User')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'User-Service',2,NULL,'Framed-User')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'User-Service',3,NULL,'Callback-Login')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'User-Service',4,NULL,'Callback-Framed')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'User-Service',5,NULL,'Outbound')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'User-Service',6,NULL,'Administrative')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'User-Service',7,NULL,'NAS-Prompt')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'User-Service',8,NULL,'Authenicate-Only')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'User-Service',9,NULL,'Callback-NAS-Prompt')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'User-Service',129,NULL,'Call-Check-User')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',1,NULL,'PPP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',2,NULL,'SLIP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',3,NULL,'ARA')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',4,NULL,'Gandalf')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',5,NULL,'Xylogics')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',256,NULL,'MPP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',257,NULL,'EURAW')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',258,NULL,'EUUI')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',8,NULL,'X25')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',260,NULL,'COMB')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Protocol',261,NULL,'FR')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Routing',0,NULL,'None')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Routing',1,NULL,'Broadcast')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Routing',2,NULL,'Listen')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Routing',3,NULL,'Broadcast-Listen')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Compression',0,NULL,'None')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Compression',1,NULL,'Van-Jacobsen-TCP-IP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Framed-Compression',2,NULL,'IPX-Header-Compression')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Login-Service',0,NULL,'Telnet')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Login-Service',1,NULL,'Rlogin')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Login-Service',2,NULL,'TCP-Clear')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Login-Service',3,NULL,'PortMaster')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Login-Service',4,NULL,'LAT')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Login-Service',5,NULL,'X25-PAD')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Login-Service',6,NULL,'X25-T3POS')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Termination-Action',0,NULL,'Default')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Termination-Action',1,NULL,'RADIUS-Request')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Status-Type',1,NULL,'Start')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Status-Type',2,NULL,'Stop')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Status-Type',3,NULL,'Alive')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Status-Type',4,NULL,'Modem-Start')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Status-Type',5,NULL,'Modem-Stop')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Status-Type',6,NULL,'Cancel')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Status-Type',7,NULL,'Accounting-On')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Status-Type',8,NULL,'Accounting-Off')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Authentic',0,NULL,'None')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Authentic',1,NULL,'RADIUS')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Authentic',2,NULL,'Local')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',1,NULL,'User-Request')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',2,NULL,'Lost-Carrier')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',3,NULL,'Lost-Service')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',4,NULL,'Idle-Timeout')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',5,NULL,'Session-Timeout')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',6,NULL,'Admin-Reset')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',7,NULL,'Admin-Reboot')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',8,NULL,'Port-Error')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',9,NULL,'NAS-Error')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',10,NULL,'NAS-Request')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',11,NULL,'NAS-Reboot')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',12,NULL,'Port-Unneeded')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',13,NULL,'Port-Preempted')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',14,NULL,'Port-Suspended')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',15,NULL,'Service-Unavailable')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',16,NULL,'Callback')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',17,NULL,'User-Error')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Acct-Terminate-Cause',18,NULL,'Host-Request')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'NAS-Port-Type',0,NULL,'Async')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'NAS-Port-Type',1,NULL,'Sync')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'NAS-Port-Type',2,NULL,'ISDN')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'NAS-Port-Type',3,NULL,'ISDN-V120')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'NAS-Port-Type',4,NULL,'ISDN-V110')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'NAS-Port-Type',5,NULL,'Virtual')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'NAS-Port-Type',6,NULL,'PAIFS')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'NAS-Port-Type',7,NULL,'HDLC-Clear-Channel')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'NAS-Port-Type',8,NULL,'X25')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'NAS-Port-Type',9,NULL,'X75')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',1,NULL,'PPTP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',2,NULL,'L2F')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',3,NULL,'L2TP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',4,NULL,'ATMP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',5,NULL,'VTP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',6,NULL,'AH')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',7,NULL,'IP-IP-Encap')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',8,NULL,'MIN-IP-IP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',9,NULL,'ESP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',10,NULL,'PPTP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',11,NULL,'DVS')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Type',12,NULL,'IP-IP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',1,NULL,'IP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',2,NULL,'IP6')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',3,NULL,'NSAP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',4,NULL,'HDLC')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',5,NULL,'BBN')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',6,NULL,'IEEE-802')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',7,NULL,'E-163')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',8,NULL,'E-164')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',9,NULL,'F-69')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',10,NULL,'X-121')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',11,NULL,'IPX')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',12,NULL,'Appletalk')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',13,NULL,'Decnet4')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',14,NULL,'Vines')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Tunnel-Medium-Type',15,NULL,'E-164-NSAP')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'ARAP-Zone-Access',1,NULL,'Default-Zone')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'ARAP-Zone-Access',2,NULL,'Inclusive-Filter')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'ARAP-Zone-Access',3,NULL,'Exclusive-Filter')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Prompt',1,NULL,'No-Echo')
go
INSERT INTO MapValues (MapType,MapAttribute,RadValue,Description,Value) VALUES(1,'Prompt',2,NULL,'Echo')
go
INSERT INTO RadDestTypes (RadDestType,RadDestTypeID) VALUES('Filter Replace',1)
go
INSERT INTO RadDestTypes (RadDestType,RadDestTypeID) VALUES('Filter Merge',2)
go
INSERT INTO RadDestTypes (RadDestType,RadDestTypeID) VALUES('Nak',3)
go
INSERT INTO RadDestTypes (RadDestType,RadDestTypeID) VALUES('Ignore',4)
go
INSERT INTO RadDestTypes (RadDestType,RadDestTypeID) VALUES('Source Specific',5)
go
INSERT INTO RadFilterTypes (RadFilterTypeID,RadFilterType) VALUES(1,'Client IP')
go
INSERT INTO RadFilterTypes (RadFilterTypeID,RadFilterType) VALUES(2,'Host IP')
go
INSERT INTO RadFilterTypes (RadFilterTypeID,RadFilterType) VALUES(3,'Radius Attribute')
go
INSERT INTO RadFilterTypes (RadFilterTypeID,RadFilterType) VALUES(4,'Destination IP')
go
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Delete',1)
go
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Delete matching',2)
go
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Add',3)
go
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Replace value',4)
go
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Add or replace value',5)
go
INSERT INTO RadMergeTypes (RadMergeType,RadMergeTypeID) VALUES('Add attributes from query',6)
go
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(1,'String')
go
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(2,'Substring')
go
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(3,'Equal')
go
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(4,'Less than')
go
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(5,'Greater than')
go
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(6,'Ends with')
go
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(7,'Starts with')
go
INSERT INTO RadSearchTypes (RadSearchTypeID,RadSearchType) VALUES(8,'Any value')
go
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Disable or Chain Dest',1)
go
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth In',10)
go
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth Out',11)
go
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth Proxy Out',12)
go
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth Proxy Req+Resp',13)
go
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth Req+Resp',14)
go
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Auth Proxy Resp',15)
go
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Acct In',20)
go
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Acct Out',21)
go
INSERT INTO RadSourceTypes (RadSourceType,RadSourceTypeID) VALUES('Acct Proxy Out',22)
go
